#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using DevExpress.XtraEditors;
using DevExpress.Utils;
using DevExpress.Utils.Frames;
using DevExpress.Utils.Controls;
using DevExpress.Utils.Drawing;
namespace DevExpress.Utils.About {
	public class ProductInfo {
		bool isBetaVersion = false;
		string title;
		ProductKind kind;
		Type versionType;
		ProductInfoStage stage = ProductInfoStage.Trial;
		public ProductInfo(string title, Type versionType) :
			this(title, versionType, ProductKind.Default) { }
		public ProductInfo(string title, Type versionType, ProductKind kind, ProductInfoStage stage) { 
			this.stage = stage;
			this.title = title;
			this.kind = kind;
			this.versionType = versionType;
		}
		public ProductInfo(string title, Type versionType, ProductKind kind) : this(title, versionType, kind, ProductInfoStage.Trial) {  }
		public bool IsBetaVersion { get { return isBetaVersion; } }
		public string Title { get { return title; } }
		public ProductKind Kind { get { return kind; } set { kind = value; } }
		public Type VersionType { get { return versionType; } }
		public ProductInfoStage Stage { get { return stage; } }
	}
	public class AboutForm : XtraForm {
		Type controlType = null;
		static ProductKind[] Products = {
					ProductKind.Default, ProductKind.XtraGrid, ProductKind.XtraEditors, ProductKind.XtraBars,
					ProductKind.XtraPrintingSystem, ProductKind.XtraReports, ProductKind.LightSwitchReports, ProductKind.XtraTreeList, 
					ProductKind.XtraVGrid, ProductKind.XtraNavBar, ProductKind.XtraPivotGrid,
					ProductKind.XtraScheduler, ProductKind.XtraRichEdit, ProductKind.XtraCharts, ProductKind.XtraLayout, ProductKind.XtraSpellChecker,
					ProductKind.XPO, ProductKind.ASPxGrid, ProductKind.ASPxNavSuite, ProductKind.DXperienceEnt, 
					ProductKind.XtraWizard, ProductKind.XtraGauges};
		static string[] ImageNames = { 
					 "", "DevExpress.XtraGrid.Images.about", "DevExpress.XtraEditors.Images.abouteditors", "DevExpress.XtraBars.Images.aboutbars",
					 "DevExpress.XtraPrinting.Images.aboutprinting", "DevExpress.XtraReports.Images.aboutreports", "DevExpress.XtraReports.Images.aboutlsreports", 
					 "DevExpress.XtraTreeList.Images.abouttreeList", "DevExpress.XtraVerticalGrid.aboutvertgrid", 
					 "DevExpress.XtraNavBar.aboutnavbar", "DevExpress.XtraPivotGrid.Images.pivotgridabout", 
					 "DevExpress.XtraScheduler.Images.aboutscheduler", "DevExpress.XtraRichEdit.Images.aboutxtrarichtextedit", "DevExpress.XtraCharts.Design.Images.aboutchart",
					 "DevExpress.XtraLayout.Images.about", "DevExpress.XtraSpellChecker.Images.about", "DevExpress.Xpo.about", "", "", "DevExpress.Utils.Images.DXPerience", 
					 "DevExpress.XtraWizard.Images.about", "DevExpress.XtraGauges.Win.about",
					 };
		private System.ComponentModel.Container components = null;
		protected const int HorzTextIndent = 12;
		LabelInfo label = new LabelInfo();
		ProductInfo info = new ProductInfo("", typeof(AboutForm), ProductKind.Default);
		static Color linkColor = Color.FromArgb(27, 101, 250);
		static AppearanceDefault 
			copyRight = new AppearanceDefault(Color.FromArgb(7, 99, 202), Color.Empty, new Font(AppearanceObject.DefaultFont.FontFamily, 7f)),
			version = new AppearanceDefault(Color.White, Color.Empty, HorzAlignment.Far, VertAlignment.Default, new Font(AppearanceObject.DefaultFont.FontFamily, 8.5f)),
			labelAppearance = new AppearanceDefault(Color.Empty, Color.Empty, new Font(AppearanceObject.DefaultFont.FontFamily, 8f));
		static bool checkDpi = false;
		static void CheckDPI(Graphics g) {
			if(checkDpi) return;
			checkDpi = true;
			GraphicsInfo ginfo = new GraphicsInfo();
			try {
				g = ginfo.AddGraphics(g);
				float sc = 96f / g.DpiX;
				copyRight.Font = CheckFontDpi(copyRight, sc);
				version.Font = CheckFontDpi(version, sc);
				labelAppearance.Font = CheckFontDpi(labelAppearance, sc); 
			} finally {
				ginfo.ReleaseGraphics();
			}
		}
		static Font CheckFontDpi(AppearanceDefault appearance, float sc) {
			return new Font(appearance.Font.FontFamily, appearance.Font.Size * sc);
		}
		public Type ControlType {
			get { return controlType; }
			set { controlType = value; }
		}
		public static void Show(Type type, ProductKind kind) {
			Show(new ProductInfo(kind.ToString(), type, kind));
		}
		public static void Show(Type type, ProductKind kind, ProductInfoStage stage) {
			Show(new ProductInfo(kind.ToString(), type, kind, stage));
		}
		public static void Show(ProductInfo info) {
			if(!UpdateInfo(info)) {
				DevExpress.Utils.About.AboutSN.Show(info.Title, info.VersionType, info.Kind, info.Stage);
				return;
			}
			AboutForm form = new AboutForm(info);
			form.ShowDialog();
			form.Dispose();
		}
		static bool UpdateInfo(ProductInfo info) {
			if(info.Kind == ProductKind.Default) return false;
			if(GetImage(info) == "") return false;
			return true;
		}
		static string GetImage(ProductInfo info) {
			int index = Array.IndexOf(Products, info.Kind);
			string name = index >= 0 && index < ImageNames.Length ? ImageNames[index] : "";
			return name;
		}
		Rectangle versionBounds = Rectangle.Empty;
		Rectangle labelInfoBounds = Rectangle.Empty;
		protected virtual void CalcBounds() {
			CalcBounds(new Rectangle(200, 170, 171, 20), new Rectangle(HorzTextIndent, 229, 0, 80));
		}
		protected void CalcBounds(Rectangle verBounds, Rectangle lbBounds) {
			versionBounds = verBounds;
			labelInfoBounds = lbBounds;
		}
		public AboutForm(ProductInfo info)  {
			if(info != null) this.info = info;
			SetStyle(ControlStyles.ResizeRedraw | ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlConstants.DoubleBuffer, true);
			InitializeComponent();
			this.LookAndFeel.UseDefaultLookAndFeel = false;
			this.LookAndFeel.SetSkinStyle("Black");
			CalcBounds();
			ClientSize = Logo.Size;
			CreateLabel();
		}
		protected override bool GetAllowSkin() {
			return true;
		}
		public AboutForm() {
			SetStyle(ControlStyles.ResizeRedraw | ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlConstants.DoubleBuffer, true);
			InitializeComponent();
		}
		protected override void OnKeyDown(KeyEventArgs e) {
			if(e.KeyData == Keys.Escape) {
				e.Handled = true;
				Close();
			}
			base.OnKeyDown(e);
		}
		protected void CreateLabel() {
			CheckDPI(null);
			Rectangle r = labelInfoBounds;
			r.Width = ClientSize.Width - r.X - HorzTextIndent;
			this.label.Font = labelAppearance.Font;
			this.label.BackColor = Color.Transparent;
			this.label.Bounds = r;
			this.label.Parent = this;
			this.label.ItemClick += new LabelInfoItemClickEvent(OnLabelClick);
			UpdateLabelText();
		}
		string GetProcessName(LabelInfoItemClickEventArgs e) {
			if(e.InfoText.Text == Http) return "http://www.devexpress.com/go/DevExpressAbout_Online.aspx";
			if(e.InfoText.Text == HttpKb) return "http://www.devexpress.com/go/DevExpressAbout_KB.aspx";
			if(e.InfoText.Text == Email) return "mailto:support@devexpress.com";
			if(e.InfoText.Text == HttpSC) return "http://www.devexpress.com/go/DevExpressAbout_SC.aspx";
			if(e.InfoText.Text == HttpPurchase) return "http://www.devexpress.com/go/DevExpressAbout_Purchase.aspx";
			if(e.InfoText.Text == PurchaseLink) return "http://www.devexpress.com/purchase";
			if(e.InfoText.Text == RegisterLink) return "http://devexpress.com/Support/Center/KB/p/K18106.aspx";
			return null;
		}
		ContextMenu serialMenu;
		protected ContextMenu SerialMenu {
			get {
				if(serialMenu == null) serialMenu = new ContextMenu(new MenuItem[] { new MenuItem("Copy to clipboard", new EventHandler(OnMenuClick))});
				return serialMenu;
			}
			set {
				if(serialMenu != null) serialMenu.Dispose();
				serialMenu = null;
			}
		}
		void OnMenuClick(object sender, EventArgs e) {
			if(Serial != TrialVersion) Clipboard.SetDataObject(Serial);
		}
		void OnLabelClick(object sender, LabelInfoItemClickEventArgs e) {
			string name = GetProcessName(e);
			if(name == null) {
				SerialMenu.Show(this, PointToClient(Control.MousePosition));
				return;
			}
			System.Diagnostics.Process process = new System.Diagnostics.Process();
			process.StartInfo.FileName = name;
			process.StartInfo.Verb = "Open";
			process.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Normal;
			process.Start();
		}
		protected override void ScaleCore(float x, float y) { }
		protected ProductInfo Info { get { return info; } }
		Image logo = null, logoEx = null;
		Image Logo {
			get {
				if(logo == null) 
					logo = ResourceImageHelper.CreateImageFromResources(GetImageName(), Info.VersionType.Assembly);
				return logo;
			}
		}
		protected virtual Image LogoEx {
			get {
				if(Info != null && Info.IsBetaVersion && logoEx == null) {
					logoEx = ResourceImageHelper.CreateImageFromResources("DevExpress.Utils.rc.png", typeof(AboutForm).Assembly);
				}
				return logoEx;
			}
		}
		protected virtual Image LogoEx2 {
			get {
				return null;
			}
		}
		protected string GetImageName() {
			return string.Format("{0}.png", GetImage(Info));
		}
		protected void UpdateLabelText() {
			this.label.Texts.Add(Info1);
			if(IsTrial)
				this.label.Texts.Add(Serial);
			else
				this.label.Texts.Add(Serial, Info.Stage != ProductInfoStage.Free);
			this.label.Texts.Add("\xd\xa");
			if(IsTrial) {
				this.label.Texts.Add(LabelInfo.HalfLineBreak);
				this.label.Texts.Add(PurchaseInfo);
				this.label.Texts.Add(PurchaseLink, linkColor, true);
				this.label.Texts.Add(LabelInfo.HalfLineBreak);
				this.label.Texts.Add(RegisterInfo);
				this.label.Texts.Add(RegisterLink, linkColor, true);
			}
			else {
				this.label.Texts.Add(LabelInfo.HalfLineBreak);
				this.label.Texts.Add(Info2);
				this.label.Texts.Add(Http, linkColor, true);
				this.label.Texts.Add(LabelInfo.HalfLineBreak);
				this.label.Texts.Add(Info4);
				this.label.Texts.Add(HttpSC, linkColor, true);
			}
		}
		protected override void OnPaint(PaintEventArgs e) {
			GraphicsCache cache = new GraphicsCache(e);
			e.Graphics.DrawImage(Logo, ClientRectangle);
			if(LogoEx != null) 
				e.Graphics.DrawImage(LogoEx, new Rectangle(Point.Empty, LogoEx.Size));
			if(LogoEx2 != null) 
				e.Graphics.DrawImage(LogoEx2, new Rectangle(Point.Empty, LogoEx2.Size));
			DrawVersion(cache);
			DrawCopyRight(cache);
			cache.Dispose();
		}
		protected void DrawCopyRight(GraphicsCache cache) {
			AppearanceObject app = new AppearanceObject(copyRight);
			app.TextOptions.Assign(TextOptions.DefaultOptionsMultiLine);
			Size size = app.CalcTextSize(cache, CopyRight, 0).ToSize();
			size.Width ++; size.Height ++;
			Rectangle bounds = new Rectangle(HorzTextIndent, ClientSize.Height - (size.Height + 10), size.Width, size.Height);
			app.DrawString(cache, CopyRight, bounds);
		}
		protected void DrawVersion(GraphicsCache cache) {
			AppearanceObject app = new AppearanceObject(version);
			Rectangle bounds = versionBounds;
			app.DrawString(cache, Version, bounds);
		}
		protected string Version { 
			get { 
				if(Info.VersionType != null) {
					return string.Format("version {0}", Info.VersionType.Assembly.GetName().Version.ToString());
				}
				return ""; } 
		}
		protected bool IsTrial { get { return Serial == TrialVersion; } }
		protected virtual string Serial { 
			get {
				return GetSerial(Info);
			} 
		}
		public static string GetSerial(ProductInfo pInfo) {
			return TrialVersion;
		}
		public static string CopyRight { get { return string.Format(CopyRightTemplate, DateTime.Now.Year.ToString()); } }
		public const string FreeVersion = "-- FREE VERSION --";
		public const string TrialVersion = "-- TRIAL VERSION --";
		const string CopyRightTemplate = "Copyright  2000-{0} Developer Express Inc.\r\nALL  RIGHTS  RESERVED.";
		const string Info1 = "Serial number: ";
		const string Info2 = "Visit us online at: ";
		const string Http = "www.devexpress.com";
		const string Info2_1 = "Knowledge base: ";
		const string HttpKb = "www.devexpress.com/kb";
		const string HttpPurchase = "Purchase Your Copy from DevExpress.com";
		const string Info4 = "\xd\xaIf you have any questions regarding this product use, please feel free to visit our support center at: ";
		const string HttpSC = "www.devexpress.com/sc";
		const string Email = "support@devexpress.com";
		const string PurchaseInfo = "You have installed DevExpress Products in Evaluation Mode  To purchase a license, please visit us online at: ";
		const string PurchaseLink = "www.devexpress.com/purchase";
		const string RegisterInfo = "\xd\xaIf youve purchased DevExpress Products and need to register your license, please review: ";
		const string RegisterLink = "How to Convert a Trial and Register Your Product";
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		private void InitializeComponent() {
			this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.StartPosition = FormStartPosition.CenterScreen;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.Name = "Form1";
			this.Text = "Information";
			this.ShowInTaskbar = true;
		}
		#endregion
	}
}
