#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Data;
namespace DevExpress.Utils.Design {
	public class TreeViewGetNodeEditTextEventArgs : EventArgs {
		TreeNode node;
		string text;
		bool modified = false;
		public TreeViewGetNodeEditTextEventArgs(TreeNode node) {
			this.node = node;
			this.text = node.Text;
		}
		public string Text { 
			get { return text; } 
			set {
				if(value == null) value = string.Empty;
				if(Text == value) return;
				text = value;
				modified = true;
			} 
		}
		public TreeNode Node { get { return node; } }
		internal bool Modified { get { return modified; } }
	}
	public delegate void TreeViewGetNodeEditTextEventHandler(object sender, TreeViewGetNodeEditTextEventArgs e);
	[ToolboxItem(false)]
	public class DXTreeView : TreeView {
		Hashtable selectedNodes;
		DXTreeSelectionMode selectionMode;
		bool repaintingSelection = false, allowDrag = false, dragAutoScroll, dragAutoExpand, allowHScrollBar;
		public event TreeViewGetNodeEditTextEventHandler GetNodeEditText;
		public event EventHandler SelectionChanged, DragNodeStart;
		public event DragEventHandler DragNodeEnd;
		public event TreeViewGetDragObjectEventHandler DragNodeGetObject;
		public event CancelEventHandler DragNodeAllow;
		int lockSelection = 0;
		Timer mouseUpTimer, dragAutoScrollTimer, dragAutoExpandTimer;
		TreeNode anchorNode;
		public event TreeViewCancelEventHandler AllowMultiSelectNode;
		const int AutoExpandInterval = 1000, AutoScrollTimerStartInterval = 400, AutoScrollHeight = 10;
		public DXTreeView() {
			this.allowHScrollBar = false;
			this.dragAutoScroll = true;
			this.dragAutoExpand = true;
			this.dragAutoScrollTimer = new Timer();
			this.dragAutoScrollTimer.Tick += new EventHandler(OnAutoScrollTimer);
			this.dragAutoExpandTimer = new Timer();
			this.dragAutoExpandTimer.Tick += new EventHandler(OnAutoExpandTimer);
			this.dragAutoExpandTimer.Interval = AutoExpandInterval;
			this.mouseUpTimer = new Timer();
			this.mouseUpTimer.Interval = 50;
			this.mouseUpTimer.Tick += new EventHandler(OnMouseUpTimerTick);
			this.anchorNode = null;
			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlConstants.DoubleBuffer, true);
			this.selectedNodes = new Hashtable();
			this.selectionMode = DXTreeSelectionMode.MultiSelectChildrenSameBranch;
		}
		protected override void Dispose(bool disposing) {
			if(disposing) {
				if(this.mouseUpTimer != null) this.mouseUpTimer.Dispose();
				if(this.dragAutoExpandTimer != null) this.dragAutoExpandTimer.Dispose();
				if(this.dragAutoScrollTimer != null) this.dragAutoScrollTimer.Dispose();
				this.dragAutoExpandTimer = this.dragAutoScrollTimer = null;
				this.mouseUpTimer = null;
				SelectedNodes.Clear();
				this.anchorNode = null;
			}
			base.Dispose(disposing);
		}
		protected TreeNode AnchorNode { get { return anchorNode; } set { anchorNode = value; } }
		protected bool IsSelectionLocked { get { return this.lockSelection != 0; } }
		protected override CreateParams CreateParams {
			get {
				const int TVS_NOHSCROLL = 0x8000;
				CreateParams cp = base.CreateParams;
				if(!AllowHScrollBar) cp.Style |= TVS_NOHSCROLL;
				return cp;
			}
		}
		protected override void OnBeforeSelect(TreeViewCancelEventArgs e) {
			if(IsStandardSelection || this.repaintingSelection) {
				base.OnBeforeSelect(e);
				return;
			}
			if(e.Action == TreeViewAction.ByMouse) {
				e.Cancel = true;
				base.OnBeforeSelect(e);
				return;
			}
			switch(SelectionMode) {
				case DXTreeSelectionMode.MultiSelectChildren :
					OnBeforeSelectMulti(e);
					break;
				case DXTreeSelectionMode.MultiSelectChildrenSameBranch :
					OnBeforeSelectMultiSameBranch(e);
					break;
			}
			base.OnBeforeSelect(e);
		}
		protected void ForceSelectNode(TreeNode node) {
			this.repaintingSelection = true;
			try {
				if(node != null) AnchorNode = node;
				SelectedNode = node;
			}
			finally {
				this.repaintingSelection = false;
			}
		}
		protected virtual void OnBeforeSelectMulti(TreeViewCancelEventArgs e) {
			OnBeforeSelectMultiSameBranch(e); 
		}
		protected virtual void OnBeforeSelectMultiSameBranch(TreeViewCancelEventArgs e) {
			if(e.Node == null) return;
			e.Cancel = true;
			SelectNode(e.Node);
		}
		protected virtual bool AllowNodeBranches(TreeNode checkNode) {
			if(SelectedNodes.Count == 0) return true;
			foreach(TreeNode node in SelectedNodes.Keys) {
				if(node.Parent != checkNode.Parent) return false;
			}
			return true;
		}
		protected virtual bool HasNonMultiSelectNodesSelected {
			get {
				if(SelectedNodes.Count == 0) return false;
				foreach(TreeNode node in SelectedNodes.Keys) {
					if(!CanMultiSelectNode(node)) return true;
				}
				return false;
			}
		}
		protected virtual bool CanMultiSelectNode(TreeNode node) {
			TreeViewCancelEventArgs e = new TreeViewCancelEventArgs(node, node.Nodes.Count != 0, TreeViewAction.Unknown);
			if(AllowMultiSelectNode != null) AllowMultiSelectNode(this, e);
			return !e.Cancel;
		}
		Point mouseDownPoint = new Point(-10000, -10000);
		TreeNode mouseDownNode = null;
		protected override void OnMouseDown(MouseEventArgs e) {
			base.OnMouseDown(e);
			if(e.Button != MouseButtons.Left) return;
			this.mouseDownNode = null;
			this.mouseDownPoint = new Point(e.X, e.Y);
			TreeNode node = GetNodeAt(e.X, e.Y);
			if(node == null) return;
			if(IsStandardSelection) {
				if(AllowDrag) SelectedNode = node;
				this.mouseDownNode = node;
				return;
			}
			if(IsControlPressed) {
				if(!CanMultiSelectNode(node)) SelectNode(node);
				else {
					if(HasNonMultiSelectNodesSelected || !AllowNodeBranches(node)) 
						SelectNode(node);
					else {
						if(!IsNodeSelected(node)) ForceSelectNode(node);
						SetNodeSelection(node, !IsNodeSelected(node));
					}
				}
				Invalidate();
				return;
			}
			if(IsShiftPressed) {
				if(SelectedNodes.Count > 0) {
					SelectRangeCore(AnchorNode, node, true);
					return;
				}
				SelectNode(node);
				return;
			}
			this.mouseDownNode = node;
			if(IsNodeSelected(node) && SelectedNodes.Count > 1) {
				if(AllowDrag) return; 
			}
			SelectNode(node);
		}
		protected override void OnKeyDown(KeyEventArgs e) {
			base.OnKeyDown(e);
			if(e.Handled) return;
			if(IsStandardSelection || AnchorNode == null) return;
			if(e.KeyCode == Keys.Down || e.KeyCode == Keys.Up) {
				DoArrowNavigation(e);
			}
		}
		protected virtual void DoArrowNavigation(KeyEventArgs e) {
			TreeNode actionNode = e.KeyCode == Keys.Down ? AnchorNode.NextVisibleNode : AnchorNode.PrevVisibleNode;
			if(actionNode == null) return;
			if(e.Control) {
				ForceSelectNode(actionNode);
				e.Handled = true;
				return;
			}
			if(e.Shift && CanMultiSelectNode(actionNode) && !HasNonMultiSelectNodesSelected) {
				ForceSelectNode(actionNode);
				AddSelection(actionNode); 
			} else 
				SelectNode(actionNode);
			e.Handled = true;
		}
		private bool allowCustomMouseMoveProcesssingCore = true;
		protected bool AllowCustomMouseMoveProcessing {
			get { return allowCustomMouseMoveProcesssingCore; }
			set { allowCustomMouseMoveProcesssingCore = value; }
		}
		protected override void OnMouseMove(MouseEventArgs e) {
			base.OnMouseMove(e);
			if(!AllowCustomMouseMoveProcessing) return;
			if(e.Button != MouseButtons.Left || this.mouseDownNode == null) return;
			if(Math.Abs(e.X - this.mouseDownPoint.X) > 5 || Math.Abs(e.Y - this.mouseDownPoint.Y) > 5) {
				DoDragDropNodes();
				OnMouseUp(e);
			}
		}
		protected virtual void DoDragDropNodes() {
			if(!AllowDrag) return;
			if((IsStandardSelection && SelectedNode == null) || (!IsStandardSelection && SelCount == 0)) return;
			CancelEventArgs e = new CancelEventArgs(false);
			if(DragNodeAllow != null) DragNodeAllow(this, e);
			if(e.Cancel) return;
			this.mouseDownNode = null; 
			DragDropEffects effect = DragDropEffects.None;
			OnStartDrag();
			try {
				TreeNode[] nodes = IsStandardSelection ? new TreeNode[] { SelectedNode } : SelNodes;
				TreeViewGetDragObjectEventArgs de = new TreeViewGetDragObjectEventArgs(nodes);
				if(DragNodeGetObject != null) DragNodeGetObject(this, de);
				effect = DoDragDrop(de.DragObject, de.AllowEffects);
			}
			finally {
				OnEndDrag(effect);
			}
		}
		protected virtual void OnStartDrag() {
			if(DragNodeStart != null) DragNodeStart(this, EventArgs.Empty);
		}
		void StopTimers() {
			if(this.dragAutoExpandTimer != null) this.dragAutoExpandTimer.Stop();
			if(this.dragAutoScrollTimer != null) this.dragAutoScrollTimer.Stop();
		}
		protected virtual void OnEndDrag(DragDropEffects effect) {
			StopTimers();
			if(DragNodeEnd != null) DragNodeEnd(this, new DragEventArgs(null, 0, 0, 0, DragDropEffects.All, effect));
		}
		protected override void OnDragDrop(DragEventArgs e) {
			StopTimers();
			base.OnDragDrop(e);
		}
		protected override void OnEnter(EventArgs e) {
			SetSelectedNodesColor(NodeColor.Selected);
			base.OnEnter(e);
		}
		protected override void OnLeave(EventArgs e) {
			if(mouseUpTimer != null && this.mouseUpTimer.Enabled) OnMouseUpTimerAction();
			SetSelectedNodesColor(NodeColor.HideSelection);
			base.OnLeave(e);
		}
		Keys autoScrollDirection = Keys.None;
		TreeNode autoExpandNode = null;
		protected override void OnDragOver(DragEventArgs e) {
			base.OnDragOver(e);
			Point pt = PointToClient(new Point(e.X, e.Y));
			if(CheckAutoScroll(pt)) return;
			CheckAutoExpand(pt);
		}
		bool CheckAutoExpand(Point pt) {
			if(this.dragAutoExpandTimer == null) return false;
			if(!DragAutoExpand) {
				this.dragAutoExpandTimer.Stop();
				return false;
			}
			TreeNode node = GetNodeAt(pt);
			if(node == this.autoExpandNode) {
				if(node == null) return false;
				return true;
			} else {
				if(this.autoExpandNode == null) {
					if(GetNodePlusMinusBounds(node).Contains(pt)) {
						this.autoExpandNode = node;
						this.dragAutoExpandTimer.Interval = AutoExpandInterval;
						this.dragAutoExpandTimer.Start();
						return true;
					}
				}
				this.autoExpandNode = null;
				this.dragAutoExpandTimer.Stop();
			}
			return false;
		}
		Rectangle GetNodePlusMinusBounds(TreeNode node) {
			if(node == null) return Rectangle.Empty;
			Rectangle r = node.Bounds;
			r.Width = 20 + (node.ImageIndex > -1 ? 16 : 0);
			r.Offset(-r.Width, 0);
			return r;
		}
		bool CheckAutoScroll(Point pt) {
			if(CheckAutoScrollCore(new Rectangle(0, 0, Width, AutoScrollHeight), pt, Keys.Up)) {
				return true;
			}
			if(CheckAutoScrollCore(new Rectangle(0, ClientRectangle.Height - AutoScrollHeight, Width, AutoScrollHeight), pt, Keys.Down)) {
				return true;
			}
			this.autoScrollDirection = Keys.None;
			if(this.dragAutoScrollTimer != null) this.dragAutoScrollTimer.Stop();
			return false;
		}
		bool CheckAutoScrollCore(Rectangle scroll, Point pt, Keys direction) {
			if(this.dragAutoScrollTimer == null) return false;
			if(!DragAutoScroll) {
				this.dragAutoScrollTimer.Stop();
				return false;
			}
			if(scroll.Contains(pt)) {
				if(this.autoScrollDirection == direction) return true;
				this.autoScrollDirection = direction;
				this.dragAutoScrollTimer.Interval = AutoScrollTimerStartInterval;
				this.dragAutoScrollTimer.Start();
				return true;
			}
			return false;
		}
		void OnAutoScrollTimer(object sender, EventArgs e) {
			if(!IsHandleCreated || !Visible || this.autoScrollDirection == Keys.None) return;
			Rectangle scroll = new Rectangle(0, 0, Width, AutoScrollHeight);
			if(this.autoScrollDirection == Keys.Down) scroll.Y = ClientRectangle.Height - AutoScrollHeight;
			if(scroll.Contains(PointToClient(Control.MousePosition))) {
				if(TopNode != null) {
					TreeNode newNode = null;
					if(this.autoScrollDirection == Keys.Up) {
						newNode = TopNode.PrevVisibleNode;
					} else {
						newNode = GetNodeAt(scroll.Location);
						if(newNode != null) {
							if(newNode.NextVisibleNode == null) newNode.EnsureVisible();
							newNode = newNode.NextVisibleNode;
						}
					}
					if(newNode != null) {
						newNode.EnsureVisible();
						if(this.dragAutoScrollTimer.Interval > 50) this.dragAutoScrollTimer.Interval -= 70;
						this.dragAutoScrollTimer.Start();
					} 
				}
			} else {
				this.autoScrollDirection = Keys.None;
				this.dragAutoScrollTimer.Stop();
			}
		}
		void OnAutoExpandTimer(object sender, EventArgs e) {
			if(this.autoExpandNode == null || !IsHandleCreated || !Visible) return;
			if(GetNodePlusMinusBounds(this.autoExpandNode).Contains(PointToClient(Control.MousePosition))) {
				if(this.autoExpandNode.IsExpanded) this.autoExpandNode.Collapse();
				else this.autoExpandNode.Expand();
				this.dragAutoExpandTimer.Start();
			} else {
				this.autoExpandNode = null;
				this.dragAutoExpandTimer.Stop();
			}
		}
		void OnMouseUpTimerTick(object sender, EventArgs e) {
			if((Control.MouseButtons & MouseButtons.Left) != 0) {
				this.mouseUpTimer.Start();
				return;
			}
			OnMouseUpTimerAction();
		}
		void OnMouseUpTimerAction() {
			this.mouseUpTimer.Stop();
			this.mouseUpTimer.Enabled = false;
			if(!IsHandleCreated) return;
			Point pt = PointToClient(Control.MousePosition);
			OnMouseUp(new MouseEventArgs(MouseButtons.Left, 1, pt.X, pt.Y, 0));
		}
		void StartMouseUpTimer() {
			if(this.mouseUpTimer == null) return;
			this.mouseUpTimer.Start();
		}
		protected override void OnMouseUp(MouseEventArgs e) {
			base.OnMouseUp(e);
			if((Control.MouseButtons & MouseButtons.Left) != 0) {
				StartMouseUpTimer();
				return;
			}
			TreeNode oldDownNode = this.mouseDownNode;
			this.mouseDownNode = null;
			if(IsStandardSelection || e.Button != MouseButtons.Left) return;
			TreeNode node = GetNodeAt(e.X, e.Y);
			if(node == null) return;
			if(node == oldDownNode && IsNodeSelected(node)) SelectNode(node);
		}
		protected bool IsControlPressed { get { return (Control.ModifierKeys & Keys.Control) != 0; } }
		protected bool IsShiftPressed { get { return (Control.ModifierKeys & Keys.Shift) != 0; } }
		protected virtual void SelectRangeCore(TreeNode start, TreeNode end, bool byAction) {
			if(start == null || end == null || IsStandardSelection) return;
			if(!CanMultiSelectNode(end)) {
				SelectNode(end);
				return;
			}
			BeginSelection();
			try {
				if(start.Nodes.Contains(end)) {
					RemoveSelection(start);
					start = start.Nodes[0];
				}
				if(end.Parent == start.Parent && end.Parent != null) {
					if(byAction) ForceSelectNode(end);
					int s = end.Parent.Nodes.IndexOf(start), e = end.Parent.Nodes.IndexOf(end);
					for(int n = Math.Min(s, e); n <= Math.Max(s, e); n++) {
						AddSelection(end.Parent.Nodes[n]);						
					}
				} else {
					SelectNode(end);
				}
			}
			finally {
				EndSelection();
			}
		}
		protected void SetNodeSelection(TreeNode node, bool selected) {
			if(selected) AddSelection(node);
			else RemoveSelection(node);
		}
		protected virtual void AddSelection(TreeNode node) {
			if(SelectedNodes.ContainsKey(node)) return;
			if(AnchorNode == null) this.anchorNode = node;
			if(SelectedNode == null) ForceSelectNode(node);
			SelectedNodes.Add(node, 0);
			SetNodeColor(node, GetSelectionColor());
			OnSelectionChanged();
		}
		protected virtual void RemoveSelection(TreeNode node) {
			if(!SelectedNodes.ContainsKey(node)) return;
			if(node == AnchorNode) this.anchorNode = null;
			SelectedNodes.Remove(node);
			if(node == SelectedNode) {
				ForceSelectNode(FindNearestSelectedNode(node));
			}
			SetNodeColor(node, NodeColor.Default);
			OnSelectionChanged();
		}
		TreeNodeCollection GetCollection(TreeNode node) {
			if(node.Parent == null) return node.TreeView.Nodes;
			return node.Parent.Nodes;
		}
		TreeNode FindNearestSelectedNode(TreeNode source) {
			TreeNode res = null, first = null;
			int indexDelta = 100000, i = GetCollection(source).IndexOf(source);
			foreach(TreeNode node in SelectedNodes.Keys) {
				first = node;
				if(node.Parent == source.Parent) {
					int delta = Math.Abs(GetCollection(node).IndexOf(node) - i);
					if(delta < indexDelta) {
						res = node;
						indexDelta = delta;
					}
				}
			}
			if(res != null) return res;
			return first;
		}
		protected NodeColor GetSelectionColor() {
			if(!Focused) return NodeColor.HideSelection;
			return NodeColor.Selected;
		}
		protected virtual void ClearSelection() {
			this.anchorNode = null;
			SetSelectedNodesColor(NodeColor.Default);
			SelectedNodes.Clear();
		}
		protected virtual void SetSelectedNodesColor(NodeColor color) {
			if(SelectedNodes.Count > 1) BeginUpdate();
			try {
				foreach(TreeNode node in SelectedNodes.Keys) {
					SetNodeColor(node, color);
				}
			}
			finally {
				if(SelectedNodes.Count > 1) EndUpdate();
			}
		}
		protected virtual void SetNodeColor(TreeNode node, NodeColor color) {
			Color fore = Color.Empty, back = Color.Empty;
			switch(color) {
				case NodeColor.HideSelection :
					fore = SystemColors.ControlText;
					back = SystemColors.ControlLight;
					break;
				case NodeColor.Selected:
					fore = SystemColors.HighlightText;
					back = SystemColors.Highlight;
					break;
			}
			node.ForeColor = fore;
			node.BackColor = back;
		}
		const int TVM_GETEDITCONTROL = (0x1100 + 15), WM_SETTEXT = 0x000C;
		protected override void OnBeforeLabelEdit(NodeLabelEditEventArgs e) {
			base.OnBeforeLabelEdit(e);
			if(e.CancelEdit || e.Node == null || GetNodeEditText == null) return;
			IntPtr handle = SendMessage(Handle, TVM_GETEDITCONTROL, IntPtr.Zero, IntPtr.Zero);
			if(handle != IntPtr.Zero) {
				TreeViewGetNodeEditTextEventArgs et = new TreeViewGetNodeEditTextEventArgs(e.Node);
				GetNodeEditText(this, et);
				if(et.Modified) SendMessage(handle, WM_SETTEXT, IntPtr.Zero, et.Text);
			}
		}
		[DllImport("USER32.dll")]
		static extern IntPtr SendMessage(IntPtr hWnd, int Msg, IntPtr wParam, IntPtr lParam);
		[DllImport("USER32.dll", CharSet = CharSet.Auto)]
		static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, string lParam);
		protected virtual void OnSelectionChanged() {
			if(IsSelectionLocked) return;
			if(SelectionChanged != null) SelectionChanged(this, EventArgs.Empty);
		}
		protected Hashtable SelectedNodes { get { return selectedNodes; } }
		protected bool IsStandardSelection { get { return SelectionMode == DXTreeSelectionMode.Standard; } }
		public virtual void BeginSelection() {
			this.lockSelection ++;
		}
		public virtual void EndSelection() {
			if(--this.lockSelection == 0) {
				OnSelectionChanged();
			}
		}
		public virtual void UpdateSelection() {
			if(Nodes.Count == 0) {
				if(SelectedNodes.Count > 0) {
					ClearSelection();
					OnSelectionChanged();
				}
				return;
			}
			if(SelCount == 0) return;
			bool fireChanged = false;
			TreeNode[] selNodes = SelNodes;
			foreach(TreeNode node in selNodes) {
				if(node.TreeView == null) {
					SelectedNodes.Remove(node);
					fireChanged = true;
				}
			}
			if(fireChanged) OnSelectionChanged();
		}
		public virtual void SelectNode(TreeNode node) {
			if(IsStandardSelection) SelectedNode = node;
			else {
				if(SelCount != 1 || !IsNodeSelected(node)) {
					ClearSelection();
					AddSelection(node);
				}
				if(SelectedNode != node) ForceSelectNode(node);
			}
		}
		[DefaultValue(true)]
		public bool DragAutoScroll { get { return dragAutoScroll; } set { dragAutoScroll = value; } }
		[DefaultValue(true)]
		public bool DragAutoExpand { get { return dragAutoExpand; } set { dragAutoExpand = value; } }
		[DefaultValue(false)]
		public bool AllowDrag { get { return allowDrag; } set { allowDrag = value; } }
		[DefaultValue(false)]
		public bool AllowHScrollBar { 
			get { return allowHScrollBar; } 
			set { 
				if(AllowHScrollBar == value) return;
				allowHScrollBar = value; 
				if(IsHandleCreated) UpdateStyles();
			} 
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public TreeNode[] SelNodes {
			get { 
				if(SelectedNodes.Count == 0) return new TreeNode[] { };
				TreeNode[] nodes = new TreeNode[SelectedNodes.Count];
				SelectedNodes.Keys.CopyTo(nodes, 0);
				if(nodes.Length > 1) Array.Sort(nodes, new TreeNodeComparer());
				return nodes;
			}
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public TreeNode SelNode {
			get {
				if(SelCount == 0) return null;
				return SelNodes[0];
			} 
			set {
				SelectNode(value);
			}
		}
		public virtual bool IsNodeSelected(TreeNode node) { return SelectedNodes.ContainsKey(node); }
		public void SelectRange(TreeNode start, TreeNode end) {
			SelectRangeCore(start, end, false);
		}
		[Browsable(false)]
		public int SelCount { get { return SelectedNodes.Count; } }
		public DXTreeSelectionMode SelectionMode { 
			get { return selectionMode; }
			set {
				if(SelectionMode == value) return;
				selectionMode = value;
				OnSelectionModeChanged();
			}
		}
		protected virtual void OnSelectionModeChanged() {
		}
		protected enum NodeColor { Default, Selected, HideSelection }
		class TreeNodeComparer : IComparer {
			public int Compare(object a, object b) {
				TreeNode node1 = a as TreeNode, node2 = b as TreeNode;
				return Comparer.Default.Compare(node1.Index, node2.Index);
			}
		}
	}
	public class TreeViewGetDragObjectEventArgs : EventArgs {
		object dragObject;
		DragDropEffects allowEffects;
		public TreeViewGetDragObjectEventArgs(object dragObject) {
			this.dragObject = dragObject;
			this.allowEffects = DragDropEffects.All;
		}
		public object DragObject { get { return dragObject; } set { dragObject = value; } }
		public DragDropEffects AllowEffects { get { return allowEffects; } set { allowEffects = value; } }
	}
	public delegate void TreeViewGetDragObjectEventHandler(object sender, TreeViewGetDragObjectEventArgs e);
	public enum DXTreeSelectionMode { Standard, MultiSelectChildren, MultiSelectChildrenSameBranch }
}
