#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using DevExpress.Utils.Drawing.Helpers;
using System.Drawing;
using System.Runtime.InteropServices;
using DevExpress.Skins;
using DevExpress.Utils.Drawing;
using DevExpress.Utils;
using DevExpress.LookAndFeel;
namespace DevExpress.Utils.Internal {
	public class DXLayeredWindow : NativeWindow {
		const int WS_CAPTION = 0x00C00000, WS_BORDER = 0x00800000, WS_DLGFRAME = 0x00400000, WS_VSCROLL = 0x00200000,
			WS_HSCROLL = 0x00100000, WS_SYSMENU = 0x00080000, WS_THICKFRAME = 0x00040000,
			WS_EX_TOOLWINDOW = 0x00000080, WS_POPUP = unchecked((int)0x80000000),
			WS_EX_WINDOWEDGE = 0x00000100,
			WS_EX_LAYERED = 0x80000,
			WS_EX_TRANSPARENT = 0x20,
			LWA_COLORKEY = 1, LWA_ALPHA = 2;
		Rectangle bounds = Rectangle.Empty;
		bool visible = false;
		byte alpha = 255;
		Color transparencyKey = Color.Magenta;
		public Color TransparencyKey {
			get { return transparencyKey; }
			set { transparencyKey = value; }
		}
		public byte Alpha {
			get { return alpha; }
			set { alpha = value; }
		}
		public void Create(IWin32Window topLevel) {
			if(IsCreated) return;
			CreateParams cp = new CreateParams();
			cp.Parent = topLevel.Handle;
			cp.Style = WS_POPUP;
			cp.ExStyle = WS_EX_LAYERED | WS_EX_TRANSPARENT;
			cp.Caption = null;
			CreateHandle(cp);
			if(TransparencyKey.IsEmpty) {
				if(alpha != 255)
					SetLayeredWindowAttributes(Handle, 0, alpha, LWA_ALPHA);
			}
			else {
				SetLayeredWindowAttributes(Handle, ColorTranslator.ToWin32(TransparencyKey), alpha, LWA_ALPHA | LWA_COLORKEY);
			}
		}
		public static bool IsAllowLayeredWindow {
			get {
				return OSFeature.Feature.IsPresent(OSFeature.LayeredWindows);
			}
		}
		public bool IsCreated { get { return Handle != IntPtr.Zero; } }
		public bool Visible {
			get { return visible; }
			set {
				if(Visible == value) return;
				if(value)
					Show();
				else
					Hide();
				this.visible = value;
			}
		}
		internal void Reset() {
			lastSize = Size.Empty;
		}
		public void Show() { ShowCore(); }
		public void Show(Point location) {
			if(IsCreated && Visible && location == Bounds.Location) return;
			this.bounds.Location = location;
			ShowCore();
		}
		public Rectangle Bounds { get { return bounds; } }
		protected void SetSizeCore(Size size) {
			bounds.Size = size;
		}
		Size lastSize = Size.Empty;
		protected void ShowCore() {
			if(!IsCreated) return;
			this.visible = true;
			int flags = NativeMethods.SWP.SWP_NOACTIVATE | NativeMethods.SWP.SWP_SHOWWINDOW | NativeMethods.SWP.SWP_DRAWFRAME;
			if(this.lastSize == Size) {
				flags |= NativeMethods.SWP.SWP_NOSIZE | NativeMethods.SWP.SWP_NOZORDER;
			}
			this.lastSize = Size;
			NativeMethods.SetWindowPos(Handle, new IntPtr(-1), Bounds.X, Bounds.Y, Size.Width, Size.Height, flags);
		}
		public void Hide() {
			if(!IsCreated || !Visible) return;
			this.visible = false;
			NativeMethods.SetWindowPos(Handle, new IntPtr(-1), 0, 0, 0, 0,
				NativeMethods.SWP.SWP_NOACTIVATE | NativeMethods.SWP.SWP_HIDEWINDOW | NativeMethods.SWP.SWP_NOSIZE | NativeMethods.SWP.SWP_NOMOVE);
		}
		protected override void WndProc(ref Message m) {
			switch(m.Msg) {
				case MSG.WM_NCHITTEST:
					NCHitTest(ref m);
					return;
				case MSG.WM_PAINT:
					WMPaint(ref m);
					return;
				case MSG.WM_ERASEBKGND:
					WMEraseBkgnd(ref m);
					return;
			}
			base.WndProc(ref m);
		}
		void NCHitTest(ref Message m) {
			m.Result = new IntPtr(NativeMethods.HT.HTTRANSPARENT);
		}
		void WMPaint(ref Message m) {
			NativeMethods.PAINTSTRUCT ps = new NativeMethods.PAINTSTRUCT();
			IntPtr dc = NativeMethods.BeginPaint(Handle, ref ps);
			OnPaint(dc, ps.rcPaint.ToRectangle());
			NativeMethods.EndPaint(Handle, ref ps);
			m.Result = IntPtr.Zero;
		}
		void WMEraseBkgnd(ref Message m) {
			OnPaint(m.WParam, Rectangle.Empty);
			m.Result = IntPtr.Zero;
		}
		void OnPaint(IntPtr dc, Rectangle clipRect) {
			using(Graphics g = Graphics.FromHdc(dc)) {
				OnDraw(g);
			}
		}
		protected virtual bool UseDoubleBuffer { get { return false; } }
		protected void OnDraw(Graphics g) {
			if(UseDoubleBuffer) {
				using(XtraBufferedGraphics bg = XtraBufferedGraphicsManager.Current.Allocate(g, new Rectangle(Point.Empty, Size))) {
					DrawBackground(bg.Graphics);
					DrawForeground(bg.Graphics);
					bg.Render();
				}
			}
			else {
				DrawBackground(g);
				DrawForeground(g);
			}
		}
		protected virtual void DrawForeground(Graphics g) { }
		protected virtual void DrawBackground(Graphics g) {
			g.Clear(TransparencyKey);
		}
		public virtual Size Size {
			get { return bounds.Size; }
			set {
				if(Size == value) return;
				bounds.Size = value;
				OnWindowChanged();
			}
		}
		protected virtual void OnWindowChanged() {
			if(!IsCreated || !Visible) return;
			ShowCore();
		}
		public void Invalidate() {
			if(!IsCreated || !Visible) return;
			InvalidateRgn(Handle, IntPtr.Zero, true);
		}
		[DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
		static extern bool InvalidateRgn(IntPtr hWnd, IntPtr hrgn, bool erase);
		[DllImport("user32.dll")]
		static extern bool SetLayeredWindowAttributes(IntPtr hwnd, int crKey, byte bAlpha, uint dwFlags);
	}
	public class DXSkinLayeredWindow : DXLayeredWindow {
		SkinImage skinImage = null;
		int imageIndex;
		public DXSkinLayeredWindow() { }
		public SkinImage SkinImage {
			get { return skinImage; }
			set {
				skinImage = value;
				CreateSkinElement();
				SetSizeCore(Size);
				OnWindowChanged();
			}
		}
		public int ImageIndex {
			get { return imageIndex; }
			set {
				if(ImageIndex == value) return;
				imageIndex = value;
			}
		}
		XSkinElement element;
		void CreateSkinElement() {
			SkinBuilderElementInfo bi = new SkinBuilderElementInfo();
			bi.Image = SkinImage;
			this.element = new XSkinElement(bi);
		}
		public override Size Size {
			get { return SkinImage.GetImageBounds(0).Size; }
		}
		protected override void DrawForeground(Graphics g) {
			if(element == null) return;
			SkinElementInfo info = new SkinElementInfo(this.element, new Rectangle(Point.Empty, Size));
			info.ImageIndex = ImageIndex;
			using(GraphicsCache cache = new GraphicsCache(g)) {
				ObjectPainter.DrawObject(cache, SkinElementPainter.Default, info);
			}
		}
		internal class XSkinElement : SkinElement {
			internal XSkinElement(SkinBuilderElementInfo info) : base(null, "Dummy", info) { }
		}
	}
	public abstract class DXLayeredWindowEx : NativeWindow, IDisposable {
		const int
			WS_POPUP = unchecked((int)0x80000000),
			WS_EX_LAYERED = 0x80000,
			ULW_ALPHA = 0x02;
		byte AC_SRC_OVER = 0x00;
		byte AC_SRC_ALPHA = 0x01;
		byte alphaCore = 255;
		Rectangle bounds = Rectangle.Empty;
		bool isVisibleCore = false;
		Control ownerCore;
		bool isDisposing;
		public void Dispose() {
			if(!isDisposing) {
				isDisposing = true;
				OnDisposing();
			}
		}
		protected virtual void OnDisposing() {
			if(IsCreated)
				DestroyHandle();
		}
		public byte Alpha {
			get { return alphaCore; }
			set { alphaCore = value; }
		}
		public void Create(IWin32Window topLevel) {
			if(IsCreated) return;
			ownerCore = topLevel as Control;
			CreateParams cp = new CreateParams();
			cp.Parent = topLevel.Handle;
			cp.Style = WS_POPUP;
			cp.ExStyle = WS_EX_LAYERED;
			cp.Caption = null;
			CreateHandle(cp);
		}
		public bool IsCreated {
			get { return Handle != IntPtr.Zero; }
		}
		public bool IsVisible {
			get { return isVisibleCore; }
		}
		public Rectangle Bounds {
			get { return bounds; }
		}
		public void Show(Point location) {
			if(IsCreated && IsVisible && location == Bounds.Location) return;
			this.bounds.Location = location;
			ShowCore();
		}
		Size lastSize = Size.Empty;
		protected void ShowCore() {
			if(!IsCreated) return;
			this.isVisibleCore = true;
			int flags = NativeMethods.SWP.SWP_NOACTIVATE | NativeMethods.SWP.SWP_SHOWWINDOW | NativeMethods.SWP.SWP_DRAWFRAME;
			if(this.lastSize == Size) {
				flags |= NativeMethods.SWP.SWP_NOSIZE | NativeMethods.SWP.SWP_NOZORDER;
			}
			this.lastSize = Size;
			NativeMethods.SetWindowPos(Handle, new IntPtr(-1), Bounds.X, Bounds.Y, Size.Width, Size.Height, flags);
		}
		public void Hide() {
			if(!IsCreated || !IsVisible) return;
			NativeMethods.SetWindowPos(Handle, new IntPtr(-1), 0, 0, 0, 0,
				NativeMethods.SWP.SWP_NOACTIVATE | NativeMethods.SWP.SWP_HIDEWINDOW | NativeMethods.SWP.SWP_NOSIZE | NativeMethods.SWP.SWP_NOMOVE);
			this.isVisibleCore = false;
		}
		protected override void WndProc(ref Message m) {
			switch(m.Msg) {
				case MSG.WM_NCHITTEST:
					NCHitTest(ref m);
					return;
			}
			base.WndProc(ref m);
		}
		void NCHitTest(ref Message m) {
			m.Result = new IntPtr(NativeMethods.HT.HTTRANSPARENT);
		}
		public virtual Size Size {
			get { return bounds.Size; }
			set {
				if(Size == value) return;
				bounds.Size = value;
				OnWindowChanged();
			}
		}
		protected virtual void OnWindowChanged() {
			if(!IsCreated || !IsVisible) return;
			ShowCore();
		}
		protected virtual Point GetPaintOffset() {
			return new Point(0, 0);
		}
		protected virtual Control CheckOwner() {
			return ownerCore;
		}
		volatile int updateRequested = 0;
		public void Invalidate() {
			if(updateRequested > 0) return;
			Control owner = CheckOwner();
			if(owner != null) {
				updateRequested++;
				owner.BeginInvoke(new MethodInvoker(UpdateAsync));
			}
		}
		void UpdateAsync() {
			Update();
			updateRequested = 0;
		}
		public void Update() {
			IntPtr screenDC = GetDC(IntPtr.Zero);
			IntPtr backBufferDC = NativeMethods.CreateCompatibleDC(screenDC);
			IntPtr hBufferBitmap = Create32bppDIBSection(screenDC, Bounds.Width, Bounds.Height);
			IntPtr tmp = IntPtr.Zero;
			try {
				tmp = NativeMethods.SelectObject(backBufferDC, hBufferBitmap);
				Rectangle rect = new Rectangle(0, 0, Bounds.Width, Bounds.Height);
				using(XtraBufferedGraphics bg = XtraBufferedGraphicsManager.Current.Allocate(backBufferDC, rect)) {
					Point paintOffset = GetPaintOffset();
					bg.Graphics.TranslateTransform(paintOffset.X, paintOffset.Y);
					using(GraphicsCache cache = new GraphicsCache(bg.Graphics))
						DrawCore(cache);
					bg.Render();
				}
				NativeMethods.POINT newLocation = new NativeMethods.POINT(Bounds.Location);
				NativeMethods.SIZE newSize = new NativeMethods.SIZE(Bounds.Size);
				NativeMethods.POINT sourceLocation = new NativeMethods.POINT(0, 0);
				BLENDFUNCTION blend = new BLENDFUNCTION();
				blend.BlendOp = AC_SRC_OVER;
				blend.BlendFlags = 0;
				blend.SourceConstantAlpha = Alpha;
				blend.AlphaFormat = AC_SRC_ALPHA;
				UpdateLayeredWindow(Handle, screenDC, ref newLocation, ref newSize, backBufferDC, ref sourceLocation, 0, ref blend, ULW_ALPHA);
			}
			finally {
				NativeMethods.SelectObject(backBufferDC, tmp);
				NativeMethods.DeleteObject(hBufferBitmap);
				NativeMethods.DeleteDC(backBufferDC);
				NativeMethods.ReleaseDC(IntPtr.Zero, screenDC);
			}
		}
		static IntPtr Create32bppDIBSection(IntPtr hDC, int w, int h) {
			NativeMethods.BITMAPINFO_SMALL bitmapInfo = new NativeMethods.BITMAPINFO_SMALL();
			bitmapInfo.biSize = Marshal.SizeOf(bitmapInfo);
			bitmapInfo.biBitCount = 0x20;
			bitmapInfo.biPlanes = 1;
			bitmapInfo.biWidth = w;
			bitmapInfo.biHeight = h;
			return NativeMethods.CreateDIBSection(hDC, ref bitmapInfo, 0, 0, IntPtr.Zero, 0);
		}
		protected abstract void DrawCore(GraphicsCache cache);
		[DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
		static extern IntPtr GetDC(IntPtr hWnd);
		[System.Security.SuppressUnmanagedCodeSecurity, System.Security.SecurityCritical]
		[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true, ExactSpelling = true)]
		static extern bool UpdateLayeredWindow(IntPtr hwnd,
			IntPtr hdcDst, ref NativeMethods.POINT pptDst, ref NativeMethods.SIZE pSizeDst,
			IntPtr hdcSrc, ref NativeMethods.POINT pptSrc,
			int crKey, ref BLENDFUNCTION pBlend, int dwFlags);
		[StructLayout(LayoutKind.Sequential)]
		struct BLENDFUNCTION {
			public byte BlendOp;
			public byte BlendFlags;
			public byte SourceConstantAlpha;
			public byte AlphaFormat;
		}
	}
	public class DragArrowsHelper : IDisposable {
		Control owner;
		DXSkinLayeredWindow arrow1, arrow2;
		bool visible = false;
		bool isVertical = true;
		Point position1, position2;
		SkinImage arrows;
		public DragArrowsHelper(UserLookAndFeel lookAndFeel, Control owner)
			: this(owner, CheckArrowsSkinImage(GetArrowsSkinElementImageByLF(lookAndFeel))) {
		}
		public DragArrowsHelper(ISkinProvider provider, Control owner)
			: this(owner, CheckArrowsSkinImage(GetArrowsSkinElementImage(provider))) {
		}
		public DragArrowsHelper(Control owner, Bitmap bitmap) : this(owner, CheckArrowsImage(bitmap)) {
		}
		protected DragArrowsHelper(Control owner, SkinImage arrows) {
			this.arrows = CheckArrowsSkinImage(arrows);
			this.position2 = this.position1 = Point.Empty;
			this.owner = owner;
			this.arrow1 = new DXSkinLayeredWindow();
			this.arrow2 = new DXSkinLayeredWindow();
			this.arrow1.SkinImage = arrows;
			this.arrow2.SkinImage = arrows;
		}
		[ThreadStatic]
		static Bitmap defaultDragArrows;
		static Bitmap DefaultDragArrows {
			get {
				if(defaultDragArrows == null) {
					defaultDragArrows = (Bitmap)ResourceImageHelper.CreateImageFromResources("DevExpress.Utils.DragArrows.png", typeof(DragArrowsHelper).Assembly);
					defaultDragArrows.MakeTransparent();
				}
				return defaultDragArrows;
			}
		}
		static SkinImage CheckArrowsImage(Image bitmap) {
			if(bitmap == null) bitmap = DefaultDragArrows;
			SkinImage i = new SkinImage(bitmap);
			i.ImageCount = 4;
			i.Layout = SkinImageLayout.Horizontal;
			return i;
		}
		static SkinImage CheckArrowsSkinImage(SkinImage image) {
			if(image == null || image.Image == null) {
				return CheckArrowsImage(null);
			}
			return image;
		}
		public static SkinImage GetArrowsSkinElementImageByLF(UserLookAndFeel lookAndFeel) {
			if(lookAndFeel.ActiveStyle != ActiveLookAndFeelStyle.Skin) return null;
			return GetArrowsSkinElementImage(lookAndFeel);
		}
		public static SkinImage GetArrowsSkinElementImage(ISkinProvider provider) {
			Skin skin = CommonSkins.GetSkin(provider);
			if(skin == null) return null;
			SkinElement res = skin[CommonSkins.SkinDropArrows];
			return res == null ? null : res.Image;
		}
		public static bool IsAllow {
			get { return SkinManager.AllowArrowDragIndicators && DXLayeredWindow.IsAllowLayeredWindow; } 
		}
		Size GetImageSize() {
			Size res = this.arrows.GetImageBounds(0).Size;
			if(res.Width < 2) res.Width = 2;
			if(res.Height < 2) res.Height = 2;
			return res;
		}
		protected Control Owner { get { return owner; } }
		protected void Create() {
			if(!Owner.IsHandleCreated) return;
			if(arrow1.IsCreated) return;
			this.arrow1.Create(Owner);
			this.arrow2.Create(Owner);
		}
		public void Show(bool isVertical, Point pos1, Point pos2) {
			Create();
			this.isVertical = isVertical;
			this.position1 = UpdatePosition(pos1, true);
			this.position2 = UpdatePosition(pos2, false);
			arrow1.ImageIndex = isVertical ? 1 : 3;
			arrow2.ImageIndex = isVertical ? 0 : 2;
			Show();
		}
		Point UpdatePosition(Point pos, bool isBottom) {
			Point res = pos;
			if(IsVertical) {
				res.X -= GetImageSize().Width / 2;
				if(isBottom) res.Y -= GetImageSize().Height;
			}
			else {
				res.Y -= GetImageSize().Height / 2;
				if(isBottom) res.X -= GetImageSize().Width;
			}
			return res;
		}
		protected void Show() {
			arrow1.Show(position1);
			arrow2.Show(position2);
		}
		public void Reset() {
			if(arrow1 != null) arrow1.Reset();
			if(arrow2 != null) arrow2.Reset();
		}
		public bool IsVertical { get { return isVertical; } }
		public bool IsHorizontal { get { return !IsVertical; } }
		public bool IsVisible { get { return visible; } }
		public void Hide() {
			this.arrow2.Hide();
			this.arrow1.Hide();
			this.visible = false;
		}
		public virtual void Dispose() {
			if(arrow1 != null) {
				if(arrow1.IsCreated) arrow1.DestroyHandle();
			}
			if(arrow2 != null) {
				if(arrow2.IsCreated) arrow2.DestroyHandle();
			}
			this.owner = null;
		}
	}
}
