#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Collections.Generic;
using System.Text;
using DevExpress.Utils;
using System.Drawing;
using DevExpress.Utils.Drawing;
using DevExpress.Utils.Paint;
using DevExpress.Utils.Text.Internal;
namespace DevExpress.Utils.Text {
	public static class StringParser {
		static ColorConverter converter;
		static ColorConverter Converter {
			get {
				if (converter == null)
					converter = new ColorConverter();
				return converter;
			}
		}
		public static StringCommand ParseCommand(ref int n, string text) {
			StringCommand res = new StringCommand();
			int pos = n + 1;
			if (string.IsNullOrEmpty(text) || text[n] != '<') return res;
			int len = text.Length;
			if (len < 3) return res;
			if(text[pos] == '<') {
				n++;
				return res;
			}
			if (text[pos] == '/') {
				pos++;
				res.SetEnd();
			}
			else {
				res.SetStart();
			}
			int commandEnd = FindCommandEnd(text, pos, len);
			if (commandEnd == -1) return res;
			string cmd = text.Substring(pos, commandEnd - pos - 1).ToLowerInvariant();
			if (cmd.Length < 1) return res;
			if(!CheckValidCommand(cmd)) return res;
			res.Command = cmd;
			n = commandEnd;
			return res;
		}
		static bool CheckValidCommand(string cmd) {
			switch(cmd) {
				case "br":
				case "b":
				case "i":
				case "u":
				case "s": return true;
			}
			if(cmd.StartsWith("color")) return true;
			if(cmd.StartsWith("size")) return true;
			return false;
		}
		public static List<StringBlock> Parse(AppearanceObject appearance, string text) {
			return Parse(Color.Empty, appearance.Font.Size, text, false);
		}
		public static List<StringBlock> Parse(float fontSize, string text, bool allowNewLineSymbols) {
			return Parse(Color.Empty, fontSize, text, allowNewLineSymbols);
		}
		public static List<StringBlock> Parse(Color foreColor, float fontSize , string text, bool allowNewLineSymbols) {
			if(text == null) text = string.Empty;
			List<StringBlock> res = new List<StringBlock>();
			StringBlock block = new StringBlock();
			block.FontSettings.SetColor(foreColor);
			block.FontSettings.SetSize(fontSize);
			StringFontSettings defaultSettings = block.FontSettings.Clone();
			int len = text.Length;
			StringBuilder sb = new StringBuilder();
			for (int n = 0; n < len; n++) {
				char ch = text[n];
				if (ch == '<' && n < len - 1) {
					StringCommand command = ParseCommand(ref n, text);
					if (command.IsValid) {
						n--; 
						if (command.Command == "br") sb.Append("\xd");
						StringFontSettings current = block.FontSettings.Clone();
						ApplyCommandSettings(current, defaultSettings, command);
						if (block.FontSettings.IsEquals(current)) continue;
						if (sb.Length == 0) {
							block.FontSettings.Assign(current);
							continue;
						}
						block.Text = sb.ToString();
						res.Add(block);
						block = new StringBlock();
						block.FontSettings.Assign(current);
						sb = new StringBuilder();
						continue;
					}
				}
				sb.Append(ch);
			}
			if (sb.Length > 0) block.Text = sb.ToString();
			if (!block.IsEmpty) res.Add(block);
			if (allowNewLineSymbols) 
				UpdateLineNumbers(res);
			return res;
		}
		static void ApplyCommandSettings(StringFontSettings current, StringFontSettings defaultSettings, StringCommand command) {
			if (IsFontStyleCommand(command)) {
				FontStyle options = GetFontStyleCommandOptions(command);
				if (command.IsStart)
					current.SetStyle(current.Style | options);
				else
					current.SetStyle(current.Style & (~options));
				return;
			}
			if (IsColorCommand(command)) {
				current.SetColor(command.IsEnd ? Color.Empty: GetFontColorCommandOptions(command, current.Color));
				return;
			}
			if (IsFontSizeCommand(command)) {
				current.SetSize(command.IsEnd ? defaultSettings.Size : GetFontSizeCommandOptions(command, current.Size));
				return;
			}
		}
		static void UpdateLineNumbers(List<StringBlock> stringBlocks) {
			int lineNumber = 0;
			for (int i = 0; i < stringBlocks.Count; i++) {
				stringBlocks[i].LineNumber = lineNumber;
				stringBlocks[i].Text = stringBlocks[i].Text.Replace("\r\n", "\r");
				stringBlocks[i].Text = stringBlocks[i].Text.Replace('\n', '\r');
				string[] strings = stringBlocks[i].Text.Split('\r');
				if (strings.Length > 1) {
					stringBlocks[i].Text = strings[0];
					for (int j = 1; j < strings.Length; j++) {
						lineNumber++;
						StringBlock stringBlock = new StringBlock();
						stringBlock.SetBlock(stringBlocks[i]);
						stringBlock.Text = strings[j];
						stringBlock.LineNumber = lineNumber;
						i++;
						stringBlocks.Insert(i, stringBlock);
					}
				}
			}
			for (int i = 1; i < stringBlocks.Count; i++)
				if (stringBlocks[i].Text == string.Empty && stringBlocks[i].LineNumber == stringBlocks[i - 1].LineNumber)
					stringBlocks.RemoveAt(i--);
			for (int i = 0; i < stringBlocks.Count - 1; i++)
				if (stringBlocks[i].Text == string.Empty && stringBlocks[i].LineNumber == stringBlocks[i + 1].LineNumber)
					stringBlocks.RemoveAt(i--);
			for (int i = 0; i < stringBlocks.Count; i++)
				if (stringBlocks[i].Text == string.Empty)
					stringBlocks[i].Text = "\r";
		}
		static bool IsColorCommand(StringCommand command) {
			return command.Command.StartsWith("color");
		}
		static bool IsFontSizeCommand(StringCommand command) {
			return command.Command.StartsWith("size");
		}
		static bool IsFontStyleCommand(StringCommand command) {
			return "bius".Contains(command.Command);
		}
		static Color GetColorFromString(string str) {
			Color color = Color.Empty;
			try {
				return (Color)Converter.ConvertFromInvariantString(str);
			}
			catch {
			}
			return color;
		}
		static Color GetFontColorCommandOptions(StringCommand command, Color current) {
			Color color = current;
			if (command.Command.StartsWith("color=")) {
				Color res = GetColorFromString(command.Command.Substring(6));
				if (res == Color.Empty) return current;
				return res;
			}
			return color;
		}
		static FontStyle GetFontStyleCommandOptions(StringCommand command) {
			FontStyle res = FontStyle.Regular;
			switch (command.Command) {
				case "b": res |= FontStyle.Bold; break;
				case "i": res |= FontStyle.Italic; break;
				case "u": res |= FontStyle.Underline; break;
				case "s": res |= FontStyle.Strikeout; break;
			}
			return res;
		}
		static float GetFontSizeCommandOptions(StringCommand command, float current) {
			if (command.Command.StartsWith("size=")) {
				float res;
				int sizeStart = 5;
				if (command.Command[5] == '+' || command.Command[5] == '-') sizeStart = 6;
				if (float.TryParse(command.Command.Substring(sizeStart), System.Globalization.NumberStyles.Any, System.Globalization.NumberFormatInfo.InvariantInfo, out res) && res > 0)
					return sizeStart == 6 ? current + (command.Command[5] == '-' ? -res : res) : res;
			}
			return current;
		}
		static int FindCommandEnd(string text, int pos, int len) {
			int commandEnd = pos + 1;
			bool found = false;
			while (commandEnd < len) {
				if (text[commandEnd++] == '>') {
					found = true;
					break;
				}
			}
			if (!found) return -1;
			return commandEnd;
		}
	} 
	public class StringInfo {
		AppearanceObject appearance;
		List<StringBlock> blocks;
		List<Rectangle> blockBounds;
		Rectangle bounds;
		internal Rectangle originalBounds;
		string sourceString;
		public StringInfo() : this(null, null, string.Empty) { }
		public StringInfo(List<StringBlock> blocks, List<Rectangle> blockBounds, string sourceString) {
			this.appearance = null;
			this.originalBounds = Rectangle.Empty;
			this.sourceString = sourceString;
			this.blocks = blocks;
			this.blockBounds = blockBounds;
			this.bounds = Rectangle.Empty;
		}
		protected internal void Assign(List<StringBlock> blocks, List<Rectangle> blockBounds, string sourceString) {
			this.sourceString = sourceString;
			this.blocks = blocks;
			this.blockBounds = blockBounds;
		}
		public void UpdateAppearanceColors(AppearanceObject source) {
			Appearance.ForeColor = source.GetForeColor();
			Appearance.BackColor = source.GetBackColor();
			Appearance.BackColor2 = source.GetBackColor2();
			Appearance.GradientMode = source.GetGradientMode();
			Appearance.Image = source.GetImage();
			Appearance.BorderColor = source.GetBorderColor();
		}
		protected internal AppearanceObject Appearance { 
			get {
				if(appearance == null) appearance = new AppearanceObject();
				return appearance; 
			} 
		}
		public Rectangle Bounds { get { return bounds; } }
		public string SourceString { get { return sourceString; } }
		internal List<StringBlock> Blocks { get { return blocks; } }
		internal List<Rectangle> BlocksBounds { get { return blockBounds; } }
		public bool IsEmpty { get { return Blocks == null || Blocks.Count == 0; } }
		internal void SetBounds(Rectangle bounds) { this.bounds = bounds; }
		internal void UpdateLocation(Point pt) {
			if (BlocksBounds == null) return;
			Point delta = new Point(pt.X - Bounds.X, pt.Y - Bounds.Y);
			for(int i = 0; i < BlocksBounds.Count; i++) {
				BlocksBounds[i] = new Rectangle(new Point(BlocksBounds[i].X + delta.X, BlocksBounds[i].Y + delta.Y), BlocksBounds[i].Size);
			}
		}
		internal void UpdateXLocation(HorzAlignment align, Rectangle bounds) {
			if(!StringPainter.IsValidSize(bounds) || align == HorzAlignment.Near || align == HorzAlignment.Default) return;
			for(int n = 0; n < Blocks.Count; ) {
				int blockCount = GetLineBlockCount(n);
				int lineWidth = GetBlocksWidth(n, blockCount);
				int newX = StringPainter.CalcXByAlignment(bounds, lineWidth, align);
				for(int b = n; b < n + blockCount; b++) {
					Rectangle blockBounds = BlocksBounds[b];
					blockBounds.X = newX;
					BlocksBounds[b] = blockBounds;
					newX += blockBounds.Width;
				}
				n += blockCount;
			}
		}
		internal int GetLineBlockCount(int startIndex) {
			int count = 0;
			int lineNo = 0;
			for(int n = startIndex; n < Blocks.Count; n++, count++) {
				if(n == startIndex) {
					lineNo = Blocks[n].LineNumber;
					continue;
				}
				if(Blocks[n].LineNumber != lineNo) break;
			}
			return count;
		}
		internal int GetBlocksWidth(int startIndex, int count) {
			if(count < 2) return BlocksBounds[startIndex].Width;
			return BlocksBounds[startIndex + count - 1].Right - BlocksBounds[startIndex].Left;
		}
		internal void Assign(StringInfo info) {
			Blocks.Clear();
			BlocksBounds.Clear();
			this.appearance = info.Appearance.Clone() as AppearanceObject;
			this.sourceString = info.SourceString;
			this.bounds = info.Bounds;
			this.originalBounds = info.originalBounds;
			for(int n = 0; n < info.Blocks.Count; n++) {
				Blocks.Add(info.Blocks[n]);
				BlocksBounds.Add(info.BlocksBounds[n]);
			}
		}
		public void SetLocation(Point pt) {
			SetLocation(pt.X, pt.Y);
		}
		public void SetLocation(int x, int y) {
			int dx = x - Bounds.X;
			int dy = y - Bounds.Y;
			Offset(dx, dy);
		}
		public void Offset(int x, int y) {
			this.originalBounds.Offset(x, y);
			if (BlocksBounds == null) return;
			for(int n = 0; n < BlocksBounds.Count; n++) {
				Rectangle r = BlocksBounds[n];
				r.Offset(x, y);
				BlocksBounds[n] = r;
			}
		}
		internal void SetEmpty() {
			Assign(new List<StringBlock>(), new List<Rectangle>(), string.Empty);
		}
	}
	public class StringPainter {
		static StringPainter defaultPainter;
		public static StringPainter Default {
			get {
				if(defaultPainter == null) defaultPainter = new StringPainter();
				return defaultPainter; 
			}
		}
		public string RemoveFormat(string text) {
			if(string.IsNullOrEmpty(text)) return string.Empty;
			List<StringBlock> strings = StringParser.Parse(12, text, false);
			if(strings == null || strings.Count == 0) return string.Empty;
			StringBuilder sb = new StringBuilder();
			foreach(StringBlock block in strings) sb.Append(block.Text);
			return sb.ToString();
		}
		public StringInfo Calculate(Graphics graphics, AppearanceObject appearance, string text, int maxWidth, XPaint painter) {
			return Calculate(graphics, appearance, text, new Rectangle(0, 0, maxWidth, 0), painter);
		}
		public StringInfo Calculate(Graphics graphics, AppearanceObject appearance, string text, int maxWidth) {
			return Calculate(graphics, appearance, text, new Rectangle(0, 0, maxWidth, 0));
		}
		public StringInfo DrawString(GraphicsCache cache, AppearanceObject appearance, string text, Rectangle bounds) {
			return DrawString(cache, appearance, text, bounds, null);
		}
		public StringInfo DrawString(GraphicsCache cache, AppearanceObject appearance, string text, Rectangle bounds, TextOptions defaultOptions) {
			if(bounds.Width < 1 || bounds.Height < 1) {
				StringInfo res = new StringInfo();
				res.SetEmpty();
				return res;
			}
			StringInfo info = Calculate(cache.Graphics, appearance, defaultOptions, text, bounds, cache.Paint);
			DrawString(cache, info, defaultOptions);
			return info;
		}
		public void DrawString(GraphicsCache cache, StringInfo info) {
			DrawString(cache, info, null);
		}
		public void DrawString(GraphicsCache cache, StringInfo info, TextOptions defaultOptions) {
			if(info.IsEmpty) return;
			StringFormat sf = info.Appearance.GetStringFormat(defaultOptions);
			StringAlignment prevAlignment = sf.Alignment;
			sf.Alignment = StringAlignment.Near;
			try {
				for(int n = 0; n < info.Blocks.Count; n++) {
					StringBlock sb = info.Blocks[n];
					Rectangle r = info.BlocksBounds[n];
					Color color = sb.FontSettings.Color;
					if(color.IsEmpty) color = info.Appearance.GetForeColor();
					DrawStringBlock(cache, sf, sb, r, color);
				}
			}
			finally {
				sf.Alignment = prevAlignment;
			}
		}
		protected virtual SizeF CalcStringBlockSize(StringBlock block, Graphics g, XPaint painter, string drawText, StringFormat format) {
			return painter.CalcTextSize(g, drawText, block.Font, format, 0);
		}
		protected virtual void DrawStringBlock(GraphicsCache cache, StringFormat format, StringBlock sb, Rectangle rect, Color color) {
			cache.Paint.DrawString(cache, sb.Text, sb.Font, cache.GetSolidBrush(color), rect, format);
		}
		public StringInfo Calculate(Graphics graphics, AppearanceObject appearance, string text, Rectangle bounds) {
			return Calculate(graphics, appearance, appearance.GetTextOptions(), text, bounds);
		}
		public StringInfo Calculate(Graphics graphics, AppearanceObject appearance, string text, Rectangle bounds, XPaint painter) {
			return Calculate(graphics, appearance, appearance.GetTextOptions(), text, bounds, painter);
		}
		public StringInfo Calculate(Graphics graphics, AppearanceObject appearance, TextOptions textOptions, string text, Rectangle bounds) {
			return Calculate(graphics, appearance, textOptions, text, bounds, null);
		}
		public virtual StringInfo Calculate(Graphics graphics, AppearanceObject appearance, TextOptions textOptions, string text, Rectangle bounds, XPaint painter) {
			StringInfo info = new StringInfo();
			info.Appearance.Assign(appearance);
			if(textOptions != null) info.Appearance.TextOptions.UpdateDefaultOptions(textOptions);
			List<StringBlock> strings = Parse(info.Appearance, text);
			UpdateFont(graphics, strings, info.Appearance.Font);
			List<StringBlock> blocks = new List<StringBlock>();
			List<Rectangle> blocksBounds = new List<Rectangle>();
			info.Assign(blocks, blocksBounds, text);
			info.originalBounds = bounds;
			if(strings.Count == 0) return info;
			TextProcessInfo te = new TextProcessInfo();
			te.AllowMultiLine = info.Appearance.TextOptions.WordWrap == WordWrap.Wrap;
			te.CurrentPosition = bounds.Location;
			te.Graphics = graphics;
			te.Info = info;
			te.Bounds = bounds;
			te.Paint = painter == null ? XPaint.Graphics : painter;
			foreach(StringBlock str in strings) {
				if(te.LineHeight == 0) te.LineHeight = str.FontHeight;
				te.Block = str;
				ProcessText(te);
			}
			UpdateBaseLine(info);
			UpdateMaximumBounds(info, bounds);
			info.SetBounds(Rectangle.FromLTRB(bounds.X, bounds.Y, te.End.X, te.End.Y));
			UpdateBoundsByAppearance(info, bounds);
			return info;
		}
		public class TextProcessInfo {
			public TextProcessInfo() {
				this.LineHeight = this.LineNumber = 0;
				this.End = Point.Empty;
				this.IsNewLine = true;
				this.Paint = XPaint.Graphics;
			}
			public XPaint Paint;
			public StringInfo Info;
			public StringBlock Block;
			public Graphics Graphics;
			public bool AllowMultiLine;
			public Rectangle Bounds;
			public Point End;
			public int LineHeight;
			public int LineNumber;
			public bool IsNewLine;
			public Point CurrentPosition;
			public int CurrentX {
				get { return CurrentPosition.X; }
				set { CurrentPosition.X = value; }
			}
			public int CurrentY {
				get { return CurrentPosition.Y; }
				set { CurrentPosition.Y = value; }
			}
		}
		void UpdateMaximumBounds(StringInfo info, Rectangle bounds) {
			if(!IsValidSize(bounds)) return;
			for(int n = info.BlocksBounds.Count - 1; n >= 0; n--) {
				Rectangle block = info.BlocksBounds[n];
				if(block.Right > bounds.Right) {
					block.Width = bounds.Right - block.X;
					if(block.Width < 1) block = Rectangle.Empty;
				}
				if(block.Bottom > bounds.Bottom) {
					block.Height = bounds.Bottom - block.Y;
					if(block.Height < 1) block = Rectangle.Empty;
				}
				if(block.IsEmpty) {
					info.BlocksBounds.RemoveAt(n);
					info.Blocks.RemoveAt(n);
					continue;
				}
				info.BlocksBounds[n] = block;
			}
		}
		public void UpdateLocation(StringInfo info, Rectangle bounds) {
			if(bounds.Size != info.originalBounds.Size) {
				if(bounds.Size == info.Bounds.Size) {
					info.originalBounds.Size = bounds.Size;
					UpdateBoundsByAppearance(info, bounds);
				}
				else {
					Graphics g = GraphicsInfo.Default.AddGraphics(null);
					try {
						StringInfo newInfo = Calculate(g, info.Appearance, null, info.SourceString, bounds);
						info.Assign(newInfo);
					}
					finally {
						GraphicsInfo.Default.ReleaseGraphics();
					}
				}
			}
			UpdateLocation(info, bounds.Location);
		}
		public void UpdateLocation(StringInfo info, Point location) {
			if(location != info.originalBounds.Location) {
				UpdateBoundsByAppearance(info, new Rectangle(location, info.originalBounds.Size));
			}
		}
		void UpdateBoundsByAppearance(StringInfo info, Rectangle bounds) {
			int x = CalcXByAlignment(bounds, info.Bounds.Width, info.Appearance.HAlignment);
			int y = CalcYByAppearance(info, info.Appearance.TextOptions.VAlignment, bounds);
			if(bounds.Width < 1) x = bounds.X;
			if(bounds.Height < 1) y = bounds.Y;
			info.UpdateLocation(new Point(x, y));
			if(bounds.Width > 0) info.UpdateXLocation(info.Appearance.HAlignment, bounds);
			info.originalBounds = bounds;
		}
		int CalcYByAppearance(StringInfo info, VertAlignment align, Rectangle bounds) {
			int delta = bounds.Height - info.Bounds.Height;
			int y = bounds.Y;
			if(delta < 1) return y;
			if(align == VertAlignment.Center || align == VertAlignment.Default) y += delta / 2 + (delta % 2);
			if(align == VertAlignment.Bottom) y += delta;
			return y;
		}
		internal static int CalcXByAlignment(Rectangle totalBounds, int textWidth, HorzAlignment align) {
			int delta = totalBounds.Width - textWidth;
			int x = totalBounds.X;
			if(delta < 1) return x;
			if(align == HorzAlignment.Center) x += delta / 2 + (delta % 2);
			if(align == HorzAlignment.Far) x += delta;
			return x;
		}
		internal void ProcessText(TextProcessInfo te) {
			int maxWidth = te.Bounds.Width;
			if(maxWidth <= 0) maxWidth = int.MaxValue;
			string drawText = te.Block.Text;
			while(drawText.Length > 0 && drawText[0] != '\0') {
				te.IsNewLine = te.Bounds.X == te.CurrentX || !te.AllowMultiLine;
				int textWidth = 0;
				string words = GetNextWords(te, drawText, maxWidth - (te.CurrentX - te.Bounds.X), out textWidth);
				if(te.AllowMultiLine) {
					if(words == "" || words[0] < ' ') {
						drawText = ProcessEmptyWord(te, drawText);
						continue;
					}
				}
				if(te.IsNewLine)
					te.LineHeight = te.Block.FontHeight;
				else
					te.LineHeight = Math.Max(te.Block.FontHeight, te.LineHeight);
				Rectangle lastBlockBounds = CreateNewBlock(te, textWidth, words);
				if(!te.AllowMultiLine && maxWidth != int.MaxValue && te.CurrentX >= te.Bounds.X + maxWidth) {
					lastBlockBounds.Width = Math.Max(0, (te.Bounds.X + maxWidth) - lastBlockBounds.X);
					te.End.X = lastBlockBounds.Right;
					te.CurrentX = lastBlockBounds.Right;
					te.Info.BlocksBounds[te.Info.BlocksBounds.Count - 1] = lastBlockBounds;
					break;
				}
				te.End.X = Math.Min(te.End.X, te.Bounds.X + maxWidth);
				if(words.Length == drawText.Length) break;
				drawText = drawText.Substring(words.Length, drawText.Length - words.Length);
			}
		}
		Rectangle CreateNewBlock(TextProcessInfo te, int textWidth, string words) {
			StringBlock newBlock = new StringBlock();
			newBlock.Text = words;
			newBlock.SetBlock(te.Block);
			newBlock.LineNumber = te.LineNumber;
			te.Info.Blocks.Add(newBlock);
			Rectangle lastBlock = new Rectangle(te.CurrentPosition, new Size(textWidth, te.Block.FontHeight));
			te.Info.BlocksBounds.Add(lastBlock);
			te.CurrentX += textWidth;
			te.End.Y = Math.Max(te.End.Y, te.CurrentY + te.LineHeight);
			te.End.X = Math.Max(te.End.X, te.CurrentX);
			return lastBlock;
		}
		void RemoveControlChars(ref string drawText) {
			while(drawText.Length > 0 && drawText[0] != (char)13 && drawText[0] < ' ')
				drawText = drawText.Remove(0, 1);
		}
		string ProcessEmptyWord(TextProcessInfo te, string drawText) {
			bool trimText = true;
						if(drawText.Length > 0 && drawText[0] == (char)13) {
							while(drawText.Length > 0 && drawText[0] == (char)13) {
								drawText = drawText.Remove(0, 1);
					RemoveControlChars(ref drawText);
					te.CurrentY += te.LineHeight;
							}
				trimText = false;
						}
						else {
				te.CurrentY += te.LineHeight;
						}
			RemoveControlChars(ref drawText);
			te.LineNumber++;
			te.CurrentX = te.Bounds.X;
			te.IsNewLine = true;
			if(trimText) drawText = drawText.TrimStart();
			return drawText;
		}
		void UpdateBaseLine(StringInfo info) {
			if(info.Blocks.Count < 2) return;
			int count;
			for(int n = 0; n < info.Blocks.Count;) {
				int ascentHeight;
				int lineHeight = GetLineHeight(info, n, out count, out ascentHeight);
				for(int l = n; l < n + count; l++) {
					Rectangle bounds = info.BlocksBounds[l];
					bounds.Y += ascentHeight - info.Blocks[l].FontAscentHeight; 
					info.BlocksBounds[l] = bounds;
				}
				n += count;
			}
		}
		int GetLineHeight(StringInfo info, int startIndex, out int count, out int ascentHeight) {
			count = 1;
			ascentHeight = info.Blocks[startIndex].FontAscentHeight;
			int lineHeight = info.Blocks[startIndex].FontHeight;
			int lineNo = info.Blocks[startIndex].LineNumber;
			for(int n = startIndex + 1; n < info.Blocks.Count; n++) {
				if(info.Blocks[n].LineNumber != lineNo) break;
				count++;
				lineHeight = Math.Max(lineHeight, info.Blocks[n].FontHeight);
				ascentHeight = Math.Max(ascentHeight, info.Blocks[n].FontAscentHeight);
			}
			return lineHeight;
		}
		string GetNextWords(TextProcessInfo te, string drawText, int wordsWidth, out int width) {
			width = 0;
			if(!te.AllowMultiLine) {
				width = GetStringWidth(te, drawText);
				return drawText;
			}
			string res = te.IsNewLine ? GetNextWord(ref drawText) : "";
			string nextWord = "";
			while(drawText != "") {
				nextWord = GetNextWord(ref drawText);
				if(nextWord == "" || nextWord[0] < ' ') break;
				int textWidth = GetStringWidth(te, res + nextWord);
				if(textWidth > wordsWidth)
					break;
				width = textWidth;
				res += nextWord;
			}
			if(width == 0 && res.Length > 0) width = GetStringWidth(te, res);
			return res;
		}
		int GetStringWidth(TextProcessInfo te, string drawText) {
			if(drawText.Length == 0) return 0;
			StringFormat format = te.Info.Appearance.GetStringFormat();
			StringAlignment halign = format.Alignment;
			StringFormatFlags oldFlags = format.FormatFlags;
			try {
				format.Alignment = StringAlignment.Near;
				format.FormatFlags |= StringFormatFlags.MeasureTrailingSpaces;
				SizeF textSize = CalcStringBlockSize(te.Block, te.Graphics, te.Paint, drawText, format);
				return XPaint.TextSizeRound(textSize.Width);
			}
			finally {
				format.FormatFlags = oldFlags;
				format.Alignment = halign;
			}
		}
		string GetNextWord(ref string drawText) {
			if(drawText.Length == 0) return "";
			int i = 1;
			while(i < drawText.Length && !char.IsWhiteSpace(drawText[i]) && drawText[i] >= ' ')
				i++;
			string word = drawText.Substring(0, i);
			drawText = drawText.Substring(i, drawText.Length - i);
			return word;
		}
		public List<StringBlock> Parse(AppearanceObject appearance, string text) {
			return ParseCore(appearance, text);
		}
		protected void UpdateFont(Graphics graphics, List<StringBlock> strings, Font font) {
			foreach(StringBlock str in strings) {
				str.SetFont(graphics, str.FontSettings.GetFont(font));
			}
		}
		protected internal virtual List<StringBlock> ParseCore(AppearanceObject appearance, string text) {
			return StringParser.Parse(appearance, text);
		}
		internal static bool IsValidSize(Rectangle bounds) {
			return bounds.Width > 0 && bounds.Height > 0;
		}
	}
}
namespace DevExpress.Utils.Text.Internal {
	public class StringCommand {
		bool isStart = true;
		string command = string.Empty;
		public bool IsValid { get { return !string.IsNullOrEmpty(command); } }
		public string Command { get { return command; } set { command = value; } }
		public bool IsStart { get { return isStart; } }
		public bool IsEnd { get { return !isStart; } }
		internal void SetStart() { this.isStart = true; }
		internal void SetEnd() { this.isStart = false; }
	}
	public class StringFontSettings {
		bool styleSet = false;
		FontStyle style;
		Color color;
		float size;
		public StringFontSettings() {
			this.style = FontStyle.Regular;
			this.color = Color.Empty;
			this.size = 0;
		}
		public FontStyle Style { get { return style; } }
		public Color Color { get { return color; } }
		public float Size { get { return size; } }
		public bool IsEquals(StringFontSettings settings) {
			return settings.Style == this.Style && settings.Color == this.Color && settings.Size == this.Size;
		}
		internal bool IsStyleSet { get { return styleSet; } }
		internal void SetStyle(FontStyle style) { 
			this.style = style;
			this.styleSet = true;
		}
		internal void SetColor(Color color) { this.color = color; }
		internal void SetSize(float size) { this.size = size; }
		internal void Assign(StringFontSettings source) {
			this.styleSet = source.styleSet;
			this.style = source.style;
			this.color = source.color;
			this.size = source.size;
		}
		internal Font GetFont(Font font) {
			return ResourceCache.DefaultCache.GetFont(font, Size < 1 ? font.Size : Size, styleSet ? Style : font.Style);
		}
		internal StringFontSettings Clone() {
			StringFontSettings res = new StringFontSettings();
			res.Assign(this);
			return res;
		}
	}
	public class StringBlock {
		string text;
		Font font;
		int fontHeight = -1;
		int fontAscentHeight = -1;
		int lineNumber = -1;
		StringFontSettings fontSettings;
		public StringBlock() {
			this.fontSettings = new StringFontSettings();
			this.text = null;
		}
		public Font Font { get { return font; } }
		public bool IsEmpty { get { return string.IsNullOrEmpty(this.text); } }
		public string Text {
			get { return text; }
			set {
				text = value;
			}
		}
		public int LineNumber { get { return lineNumber; } set { lineNumber = value; } }
		internal int FontHeight { get { return fontHeight; } }
		internal int FontAscentHeight { get { return fontAscentHeight; } }
		public StringFontSettings FontSettings { get { return fontSettings; } }
		internal void SetFont(Graphics graphics, Font font) {
			Graphics g = GraphicsInfo.Default.AddGraphics(graphics);
			try {
				this.font = font;
				Dummy.Font = font;
				this.fontHeight = Dummy.CalcDefaultTextSize(g).Height;
				this.fontAscentHeight = DevExpress.Utils.Text.TextUtils.GetFontAscentHeight(g, font);
			}
			finally {
				GraphicsInfo.Default.ReleaseGraphics();
			}
		}
		internal void SetBlock(StringBlock source) {
			this.font = source.font;
			this.fontHeight = source.fontHeight;
			this.fontAscentHeight = source.fontAscentHeight;
			this.FontSettings.Assign(source.FontSettings);
		}
		[ThreadStatic]
		static AppearanceObject dummy;
		protected static AppearanceObject Dummy {
			get {
				if(dummy == null) dummy = new AppearanceObject();
				return dummy;
			}
		}
	}
}
