#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.IO;
using System.Runtime.Serialization;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Design;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Globalization;
using System.ComponentModel.Design;
using DevExpress.Utils.Drawing;
using System.Collections.Generic;
using System.ComponentModel.Design.Serialization;
using System.CodeDom;
using System.Collections.Specialized;
namespace DevExpress.Utils {
	[Editor("DevExpress.Utils.Design.ImageInfoImageEditor, " + AssemblyInfo.SRAssemblyDesign, typeof(UITypeEditor))]
	public class ImageInfo {
		string name;
		Image image;
		byte[] imdata;
		internal ImageInfo(byte[] imdata) {
			this.name = String.Empty;
			this.imdata = imdata;
		}
		public ImageInfo(string name, Image image) {
			this.name = name;
			this.image = image;
		}
		internal void Assign(ImageInfo imageInfo) {
			imdata = imageInfo.imdata;
			image = imageInfo.image;
		}
		public string Name { get { return name; } set { name = value; } }
		[Browsable(false)]
		public Image Image {
			get {
				if(imdata != null) {
					MemoryStream temp = new MemoryStream(imdata);
					image = Image.FromStream(temp);
					imdata = null;
				}
				return image;
			}
			set { image = value; imdata = null; }
		}
		public float HorizontalResolution { get { return Image.HorizontalResolution; } }
		public Size PhysicalDimension { get { return Image.Size; } }
		public PixelFormat PixelFormat { get { return Image.PixelFormat; } }
		public ImageFormat RawFormat { get { return Image.RawFormat; } }
		public Size Size { get { return Image.Size; } }
		public float VerticalResolution { get { return Image.VerticalResolution; } }
	}
	public class InnerImagesList : List<ImageInfo>, IList {
		Images owner;
		public InnerImagesList(Images owner) {
			this.owner = owner;
		}
		public void SetKeyName(int index, string name) {
			if(index < 0 || index >= Count) {
				return;
			}
			this[index].Name = name;
		}
		int IList.Add(object obj) {
			ImageInfo info = obj as ImageInfo;
			info.Image = owner.Resize(info.Image);
			Add(obj as ImageInfo);
			return Count;
		}
	}
	[ListBindable(false), Editor("DevExpress.Utils.Design.ImageCollectionEditor, " + AssemblyInfo.SRAssemblyDesign, typeof(System.Drawing.Design.UITypeEditor))]
	public class Images : IList {
		int lockUpdate;
		ImageCollection owner;
		InnerImagesList innerImages;
		bool allowModifyImages;
		public Images(ImageCollection owner) : this(owner, true) {
		}
		public Images(ImageCollection owner, bool allowModifyImages) {
			this.innerImages = new InnerImagesList(this);
			this.lockUpdate = 0;
			this.owner = owner;
			this.allowModifyImages = allowModifyImages;
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public InnerImagesList InnerImages { get { return innerImages; } }
		internal bool ReCreateImages() {
			if(Count > 0)
				if(DialogResult.No == MessageBox.Show("The collection will be cleared. Do you want to continue?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question))
					return false;
			InnerImages.Clear();
			return true;
		}
		protected ImageCollection Owner { get { return owner; } }
		protected virtual void BeginUpdate() {
			lockUpdate++;
		}
		protected virtual void EndUpdate() {
			if(--lockUpdate == 0)
				OnCollectionChanged(new CollectionChangeEventArgs(CollectionChangeAction.Refresh, null));
		}
		public virtual Image this[int index] { 
			get { return InnerImages[index].Image; }
			set {
				if(value == null) throw new ArgumentNullException();
				InnerImages[index].Image = value;
				OnInsertComplete(index, value);
			}
		}
		public Image this[string name] {
			get {
				ImageInfo ii = Find(name);
				if(ii != null) return ii.Image;
				return null;
			}
			set {
				if(value == null) throw new ArgumentNullException();
				ImageInfo ii = Find(name);
				if(ii == null) return;
				ii.Image = value;
				OnInsertComplete(InnerImages.IndexOf(ii), value);
			}
		}
		ImageInfo Find(string name) {
			for(int n = 0; n < InnerImages.Count; n++) {
				ImageInfo ii = InnerImages[n];
				if(string.Compare(name, ii.Name, true, CultureInfo.InvariantCulture) == 0) return ii;
			}
			return null;
		}
		public Image GetImageFromCollection(Image iml, int width, int height, int wIndex, int hIndex) {
			Bitmap res = new Bitmap(width, height);
			res.SetResolution(iml.HorizontalResolution, iml.VerticalResolution);
			Graphics g = Graphics.FromImage(res);
			g.DrawImage(iml, 0, 0, new Rectangle(width * wIndex, height * hIndex, width, height), GraphicsUnit.Pixel);
			g.Dispose();
			MakeTransparent(res);
			return res;
		}
		protected virtual void AddImageStrip(Image sourceImage, bool vertical) {
			BeginUpdate();
			try {
				Size size = Owner.ImageSize;
				int index = 0;
				for(int i = vertical ? sourceImage.Height : sourceImage.Width; i > 0; i -= vertical ? size.Height : size.Width) {
					InnerImages.Add(new ImageInfo("", GetImageFromCollection(sourceImage, size.Width, size.Height, vertical ? 0 : index, vertical ? index : 0)));
					index++;
				}
			}
			finally {
				EndUpdate();
			}
		}
		public void AddImageStrip(Image sourceImage) {
			AddImageStrip(sourceImage, false);
		}
		public void AddImageStripVertical(Image sourceImage) {
			AddImageStrip(sourceImage, true);
		}
		public void SetKeyName(int index, string name) {
			if(index < 0 || index >= Count)
				throw new IndexOutOfRangeException();
			InnerImages[index].Name = name;
		}
		internal void Assign(ImageCollectionStreamer stream) {
			if(stream == null || stream.Collection == null || stream.Collection.Images.Count == 0) {
				InnerImages.Clear();
				return;
			}
			InnerImages.Clear();
			for(int n = 0; n < stream.Collection.Images.Count; n++) {
				if(n >= InnerImages.Count) InnerImages.Add(new ImageInfo("", null));
				InnerImages[n].Assign(stream.Collection.Images.InnerImages[n]);
			}
		}
		internal void AddCore(byte[] imdata) {
			if(imdata == null) return;
			InnerImages.Add(new ImageInfo(imdata));
		}
		public void AddRange(object[] images) {
			foreach(object image in images) {
				if(image is Image)
					Add(image as Image);
			}
		}
		public void AddRange(IList images) {
			foreach(object image in images) {
				if(image is Image)
					Add(image as Image);
			}
		}
		public virtual int Add(Image image) {
			return Add(image, string.Empty);
		}
		public virtual int Add(Image image, string name) {
			if(image == null || image.Size.IsEmpty) return -1;
			InnerImages.Add(new ImageInfo(name, image));
			int res = InnerImages.Count - 1;
			OnInsertComplete(res, image);
			return res;
		}
		protected virtual void OnInsertComplete(int index, object item) {
			Image image = item as Image;
			if(image == null) throw new ArgumentException("image");
			if(allowModifyImages) {
				ImageInfo i = InnerImages[index];
				i.Image = Resize(image);
			}
			OnCollectionChanged(new CollectionChangeEventArgs(CollectionChangeAction.Add, item));
		}
		protected internal virtual Image Resize(Image image) {
			if(allowModifyImages) {
				if(image.Size != Owner.ImageSize) {
					image = new Bitmap(image, Owner.ImageSize);
				}
				MakeTransparent(image);
			}
			return image;
		}
		protected virtual void OnRemoveComplete(int index, object item) {
			OnCollectionChanged(new CollectionChangeEventArgs(CollectionChangeAction.Remove, item));
		}
		protected void MakeTransparent(Image image) {
			if(Owner.TransparentColor == Color.Empty) return;
			Bitmap bitmap = image as Bitmap;
			if(bitmap != null) bitmap.MakeTransparent(Owner.TransparentColor);
		}
		protected virtual void OnClear() {
			if(Count == 0) return;
			BeginUpdate();
			try {
				InnerImages.Clear();
			}
			finally {
				EndUpdate();
			}
		}
		protected virtual void OnCollectionChanged(CollectionChangeEventArgs e) {
			if(this.lockUpdate != 0) return;
			Owner.OnChanged();
		}
		#region IList Members
		int IList.Add(object obj) {
			ImageInfo imageInfo = obj as ImageInfo;
			if(imageInfo != null)
				return Add(imageInfo.Image, imageInfo.Name);
			return Add(obj as Image);
		}
		object IList.this[int index] { get { return this[index]; } set { } }
		public void Clear() {
			OnClear();
		}
		public bool Contains(object value) { return IndexOf(value) != -1; }
		public int IndexOf(object value) {
			for(int n = 0; n < InnerImages.Count; n++) {
				if(object.ReferenceEquals(InnerImages[n].Image, value)) return n;
			}
			return -1;
		}
		public void Insert(int index, object value) {
			Insert(index, value, "");
		}
		public void Insert(int index,object value,string name) {
			Image image = value as Image;
			if(image == null) return;
			InnerImages.Insert(index, new ImageInfo(name, image));
			OnInsertComplete(index, image);
		}
		[Description("Gets a value indicating whether the collection has a fixed size.")]
public bool IsFixedSize { get { return false; } }
		[Description("Gets a value indicating whether the collection is read-only.")]
public bool IsReadOnly { get { return false; } }
		public void Remove(object value) {
			RemoveAt(IndexOf(value));
		}
		public void RemoveByName(string name) {
			ImageInfo ii = Find(name);
			RemoveAt(InnerImages.IndexOf(ii));
		}
		public void RemoveAt(int index) {
			if(index < 0) throw new IndexOutOfRangeException();
			ImageInfo i = InnerImages[index];
			InnerImages.RemoveAt(index);
			OnRemoveComplete(index, i.Image);
		}
		public void CopyTo(Array array, int index) {
			GetImageArray().CopyTo(array, index);
		}
		public Image[] ToArray() {
			return (Image[])GetImageArray().ToArray(typeof(Image));
		}
		[Description("Gets the number of items in the collection.")]
public int Count { get { return InnerImages.Count; } }
		[Description("Gets a value indicating whether access to the collection is synchronized (thread safe).")]
public bool IsSynchronized { get { return true; } }
		[Description("Gets an object that can be used to synchronize access to the collection.")]
public object SyncRoot { get { return this; } }
		public IEnumerator GetEnumerator() {
			return GetImageArray().GetEnumerator();
		}
		[Description("Gets the collection of names assigned to the images in the current collection.")]
public StringCollection Keys {
			get {
				StringCollection strings = new StringCollection();
				for(int i = 0; i < InnerImages.Count; i++) {
					ImageInfo info = InnerImages[i];
					strings.Add(info.Name);
				}
				return strings;
			}
		}
		ArrayList GetImageArray() {
			ArrayList list = new ArrayList();
			for(int n = 0; n < InnerImages.Count; n++) {
				list.Add(InnerImages[n].Image);
			}
			return list;
		}
		#endregion
	}
	public interface IImageCollectionHelper {
		Control OwnerControl { get; }
	}
	[Designer("DevExpress.Utils.Design.ImageCollectionDesigner, " + AssemblyInfo.SRAssemblyDesign, typeof(IDesigner)),
	 Description("Represents a collection of System.Drawing.Image objects and supports alpha channels in images."),
   DesignerSerializer("DevExpress.Utils.Design.ImageCollectionCodeDomSerializer, " + AssemblyInfo.SRAssemblyDesign, "System.ComponentModel.Design.Serialization.CodeDomSerializer, System.Design"),
	 ToolboxTabName(AssemblyInfo.DXTabComponents)
	]
	public class ImageCollection : Component, ISupportInitialize, ICloneable {
		Size imageSize;
		Color transparentColor = Color.Empty;
		Images images;
		private static readonly object changed = new object();
		static ImageCollection() {
			DevExpress.Utils.Design.DXAssemblyResolverEx.Init();
		}
		public ImageCollection(IContainer container) : this() {
			container.Add(this);
		}
		public ImageCollection() : this(true) {
		}
		public ImageCollection(bool allowModifyImages) {
			this.images = CreateImages(allowModifyImages);
			this.imageSize = DefaultSize;
			ResetTransparentColor();
		}
		object ICloneable.Clone() {
			ImageCollection res = new ImageCollection();
			res.imageSize = ImageSize;
			res.transparentColor = TransparentColor;
			foreach(Image image in Images) {
				res.Images.Add(image.Clone() as Image);
			}
			return res;
		}
		internal static Size DefaultSize = new Size(16, 16);
		void ResetImageSize() { ImageSize = Size.Empty;	}
		bool ShouldSerializeImageSize() { return ImageSize != DefaultSize; 	}
		[Description("Gets or sets the size of the images in the image list."),
Localizable(true), RefreshProperties(RefreshProperties.All)]
		public Size ImageSize {
			get { return imageSize; }
			set {
				if(value == Size.Empty) value = DefaultSize;
				if(ImageSize == value) return;
				if(this._lockInit == 0 && DesignMode)
					if(!Images.ReCreateImages()) return;
				imageSize = value;
				OnChanged();
			}
		}
		void ResetTransparentColor() {	TransparentColor = Color.Empty; }
		bool ShouldSerializeTransparentColor() { return TransparentColor != Color.Empty;}
		[Description("Gets or sets the color to treat as transparent."),
Localizable(true)]
		public Color TransparentColor {
			get { return transparentColor; }
			set {
				if(TransparentColor == value) return;
				transparentColor = value;
				OnChanged();
			}
		}
		[Description("Provides access to the collection of images."),
RefreshProperties(RefreshProperties.All), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Images Images { get { return images; } }
		[Browsable(false)]
		public ImageCollectionStreamer ImageStream {
			get { return new ImageCollectionStreamer(this); }
			set {
				Images.Assign(value);
				OnChanged();
			}
		}
		int _lockInit = 0;
		public void BeginInit() {
			_lockInit ++;
		}
		public void EndInit() {
			_lockInit = 0;
		}
		public void AddImageStrip(Image image) {
			Images.AddImageStrip(image);
		}
		public void AddImageStripVertical(Image image) {
			Images.AddImageStripVertical(image);
		}
		public void AddImage(Image image) {
			AddImage(image, string.Empty);
		}
		public void AddImage(Image image, string name) {
			Images.Add(image, name);
		}
		public void Clear() {
			Images.Clear();
		}
		protected internal virtual void OnChanged() {
			if(this._lockInit != 0) return;
			FireChanged();
			RaiseChanged(EventArgs.Empty);
		}
		protected void FireChanged() {
			if(!DesignMode || this._lockInit != 0) return;
			System.ComponentModel.Design.IComponentChangeService srv = GetService(typeof(System.ComponentModel.Design.IComponentChangeService)) as System.ComponentModel.Design.IComponentChangeService;
			if(srv == null) return;
			srv.OnComponentChanged(this, null, null, null);
		}
		protected internal virtual void RaiseChanged(EventArgs e) {
			EventHandler handler = (EventHandler)this.Events[changed];
			if(handler != null) handler(this, e);
		}
		protected virtual Images CreateImages(bool allowModifyImages) {
			return new Images(this, allowModifyImages);
		}
		[Category("Events")]
		public event EventHandler Changed {
			add { this.Events.AddHandler(changed, value); }
			remove { this.Events.RemoveHandler(changed, value); }
		}
		public static Image ImageFromFile(string fileName) {
			if(!File.Exists(fileName)) return null;
			FileInfo fInfo = new FileInfo(fileName);
			if(fInfo.Extension.ToLower().Equals(".ico")) {
				try {
				using(Icon icon = new Icon(fileName))
					return icon.ToBitmap(); 
				} catch {}
			}
			try {
				return Image.FromFile(fileName); 
			} catch {}
			return null;
		}
		public static void AddImage(object images, Image image) {
			if(images == null || image == null) return;
			ImageList list = images as ImageList;
			if(list != null) {
				list.Images.Add(image);
				return;
			}
			ImageCollection coll = images as ImageCollection;
			if(coll != null) coll.AddImage(image);
		}
		public static void DrawImageListImage(GraphicsCache cache, object images, int index, Rectangle bounds) {
			if(images == null) return;
			ImageList list = images as ImageList;
			if(list != null) {
				bounds = cache.CalcRectangle(bounds);
				list.Draw(cache.Graphics, bounds.X, bounds.Y, bounds.Width, bounds.Height, index);
				return;
			}
			Image image = GetImageListImage(images, index);
			if(image == null) return;
			cache.Paint.DrawImage(cache.Graphics, image, bounds);
		}
		public static void DrawImageListImage(GraphicsCache cache, Image image, object images, int index, Rectangle bounds, bool enabled) {
			if(image != null) {
				cache.Paint.DrawImage(cache.Graphics, image, bounds, new Rectangle(Point.Empty, image.Size), enabled);
				return;
			}
			if(IsImageExists(null, images, index))
				DrawImageListImage(cache, images, index, bounds, enabled);
		}
		public static void DrawImageListImage(GraphicsCache cache, object images, int index, Rectangle bounds, bool enabled) {
			if(enabled) {
				DrawImageListImage(cache, images, index, bounds);
				return;
			}
			if(images == null) return;
			ImageList list = images as ImageList;
			if(list != null) {
				cache.Paint.DrawImage(cache, list, index, bounds, false);
				return;
			}
			Image image = GetImageListImage(images, index);
			if(image == null) return;
			cache.Paint.DrawImage(cache.Graphics, image, bounds, new Rectangle(Point.Empty, image.Size), false);
		}
		public static void DrawImageListImage(GraphicsInfoArgs info, object images, int index, Rectangle bounds, bool enabled) {
			DrawImageListImage(info.Cache, images, index, bounds, enabled);
		}
		public static Size GetImageListSize(Image image, object images, int index) {
			if(image != null) return image.Size;
			if(IsImageListImageExists(images, index)) return GetImageListSize(images);
			return Size.Empty;
		}
		public static Size GetImageListSize(object images) {
			if(images == null) return Size.Empty;
			ImageList list = images as ImageList;
			if(list != null) return list.ImageSize;
			ImageCollection ic = images as ImageCollection;
			if(ic != null) return ic.ImageSize;
			return Size.Empty;
		}
		public static bool IsImageExists(Image image, object images, int index) {
			return image != null || IsImageListImageExists(images, index);
		}
		public static bool IsImageListImageExists(object images, int index) {
			if(images == null || index < 0) return false;
			ImageList list = images as ImageList;
			if(list != null) return index < list.Images.Count;
			ImageCollection ic = images as ImageCollection;
			if(ic != null) return index < ic.Images.Count;
			return false;
		}
		public static int GetImageListImageCount(object images) {
			if(images == null) return 0;
			ImageList list = images as ImageList;
			if(list != null) return list.Images.Count;
			ImageCollection ic = images as ImageCollection;
			if(ic != null) return ic.Images.Count;
			return 0;
		}
		public static Image GetImageListImage(object images, int index) {
			if(images == null || index < 0) return null;
			ImageList list = images as ImageList;
			if(list != null) return index < list.Images.Count ? list.Images[index] : null;
			ImageCollection ic = images as ImageCollection;
			if(ic != null) return index < ic.Images.Count ? ic.Images[index] : null;
			return null;
		}
		public void ExportToFile(string fileName) {
			if(this.Images.Count == 0) return;
			try {
				Bitmap images = new Bitmap(this.ImageSize.Width * this.Images.Count, this.ImageSize.Height);
				for(int i = 0; i < this.Images.Count; i++) {
					Graphics g = Graphics.FromImage(images);
					g.DrawImage(this.Images[i], new Point(i * this.ImageSize.Width, 0));
					g.Dispose();
				}
				if(System.IO.File.Exists(fileName)) {
					System.IO.FileInfo fi = new FileInfo(fileName);
					fi.Delete();
				}
				images.Save(fileName, this.Images[0].RawFormat);
			}
			catch(Exception e) {
				MessageBox.Show(e.Message, e.Source);
			}
		}
	}
	[Serializable, TypeConverter(typeof(DevExpress.Utils.Design.BinaryTypeConverter))]
	public class ImageCollectionStreamer : ISerializable {
		ImageCollection collection = null;
		static ImageCollectionStreamer() {
			DevExpress.Utils.Design.DXAssemblyResolverEx.Init();
		}
		public ImageCollectionStreamer(ImageCollection collection) {
			this.collection = collection;
		}
		ImageCollectionStreamer(SerializationInfo si, StreamingContext context) {
			if(si.MemberCount == 0) return;
			byte[] data = si.GetValue("Data", typeof(object)) as byte[];
			if(data == null) return;
			Size size = (Size)si.GetValue("ImageSize", typeof(Size));
			this.collection = new ImageCollection();
			this.collection.ImageSize = size;
			int position = 0;
			while(position < data.Length) {
				int imSize = GetSize(data, position);
				position += 4;
				if(imSize == 0) break;
				byte[] imdata = new byte[imSize];
				Array.Copy(data, position, imdata, 0, imSize);
				this.collection.Images.AddCore(imdata);
				position += imSize;
			}
		}
		static ImageFormat GetFormat(Image image) {
			ImageFormat res = image.RawFormat;
			if(res == ImageFormat.Jpeg || res == ImageFormat.MemoryBmp)
				res = ImageFormat.Png;
			return res;
		}
		static ImageCodecInfo FindCodec(ImageFormat format) {
			ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();
			foreach(ImageCodecInfo codec in codecs) {
				if(codec.FormatID.Equals(format.Guid))
					return codec;
			}
			return null;
		}
		public static ImageCodecInfo GetCodec(Image image) {
			ImageFormat format = GetFormat(image);
			ImageCodecInfo codec = FindCodec(format);
			if(codec == null) codec = FindCodec(ImageFormat.Png);
			return codec;
		}
		public ImageCollection Collection { get { return collection; } }
		void ISerializable.GetObjectData(SerializationInfo si, StreamingContext context) {
			if(collection == null || collection.Images.Count == 0) return;
			using(MemoryStream stream = GetSerializationStream()) {
				si.AddValue("ImageSize", collection.ImageSize);
				si.AddValue("Data", stream.ToArray());
			}
		}
		MemoryStream GetSerializationStream() {
			MemoryStream stream = new MemoryStream();
			foreach(Image image in collection.Images) {
				MemoryStream tempStream = new MemoryStream();
				image.Save(tempStream, GetCodec(image), null);
				tempStream.Seek(0, SeekOrigin.Begin);
				byte[] size = GetSize((int)tempStream.Length);
				stream.Write(size, 0, size.Length);
				tempStream.WriteTo(stream);
				tempStream.Close();
			}
			stream.Seek(0, SeekOrigin.Begin);
			return stream;
		}
		byte[] GetSize(int size) {
			byte[] res = new byte[4];
			res[0] = (byte)(size & 0xff);
			res[1] = (byte)((size >> 8) & 0xff);
			res[2] = (byte)((size >> 16) & 0xff);
			res[3] = (byte)((size >> 24) & 0xff);
			return res;
		}
		int GetSize(byte[] size, int position) {
			return size[position + 0] | (size[position + 1] << 8) | (size[position + 2] << 16) | (size[position + 3] << 24);
		}
	}
}
