#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.ComponentModel;
using DevExpress.Utils.Paint;
using System.Globalization;
using System.ComponentModel.Design.Serialization;
using System.Reflection;
using DevExpress.Utils.Serializing;
using DevExpress.Utils.Drawing.Helpers;
using System.Collections.Generic;
using DevExpress.LookAndFeel;
using DevExpress.Skins;
namespace DevExpress.Utils.Drawing {
	public class BackgroundPaintHelper {
		static Color defaultDisabledColorDefaultValue = Color.FromArgb(0xb0, 207, 207, 207);
		public static Color DefaultDisabledColor = defaultDisabledColorDefaultValue;
		public delegate void PaintInvoke(Control control, PaintEventArgs e);
		public static void PaintDisabledControl(Control control, PaintEventArgs e) {
			PaintDisabledControl(e, new Rectangle(Point.Empty, control.Size));
		}
		public static void PaintDisabledControl(PaintEventArgs e, Rectangle bounds) {
			using(Brush brush = new SolidBrush(DefaultDisabledColor)) {
				e.Graphics.FillRectangle(brush, bounds);
			}
		}
		public static void PaintDisabledControl(Control control, GraphicsCache cache) {
			PaintDisabledControl(cache, new Rectangle(Point.Empty, control.Size));
		}
		public static void PaintDisabledControl(GraphicsCache cache, Rectangle bounds) {
			Brush brush = cache.GetSolidBrush(DefaultDisabledColor);
			cache.FillRectangle(brush, bounds);
		}
		public static void PaintDisabledControl(UserLookAndFeel lookAndFeel, GraphicsCache cache, Rectangle bounds) {
			Brush brush = cache.GetSolidBrush(GetDisabledColor(lookAndFeel));
			cache.FillRectangle(brush, bounds);
		}
		public static void PaintDisabledControl(UserLookAndFeel lookAndFeel, PaintEventArgs e, Rectangle bounds) {
			using(Brush brush = new SolidBrush(GetDisabledColor(lookAndFeel))) {
				e.Graphics.FillRectangle(brush, bounds);
			}
		}
		static Color GetDisabledColor(UserLookAndFeel lookAndFeel) {
			if(lookAndFeel == null) return DefaultDisabledColor;
			if(lookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin && DefaultDisabledColor == defaultDisabledColorDefaultValue) {
				Color res = CommonSkins.GetSkin(lookAndFeel).Colors[CommonColors.DisabledControl];
				return Color.FromArgb(200, res);
			}
			return DefaultDisabledColor;
		}
		public static bool PaintTransparentBackground(Control control, PaintEventArgs e, Rectangle bounds,
			PaintInvoke paint, PaintInvoke paintBackground) {
			Graphics g = e.Graphics;
			Control parent = control.Parent;
			if(parent == null) return false;
			NativeMethods.POINT p = new NativeMethods.POINT();
			p.X = p.Y = 0;
			NativeMethods.MapWindowPoints(control.Handle, parent.Handle, ref p, 1);
			bounds.Offset(p.X, p.Y);
			PaintEventArgs parentArgs = new PaintEventArgs(g, bounds);
			GraphicsState state = g.Save();
			try {
				g.TranslateTransform(-p.X, -p.Y);
				paintBackground(parent, parentArgs);
				g.Restore(state);
				state = g.Save();
				g.TranslateTransform(-p.X, -p.Y);
				paint(parent, parentArgs);
			}
			finally {
				g.Restore(state);
			}
			return true;
		}
	}
	public class DXPaintEventArgs : EventArgs {
		Rectangle clipRectangle;
		Graphics graphics;
		PaintEventArgs paintArgs;
		public DXPaintEventArgs(PaintEventArgs paintArgs) {
			this.paintArgs = paintArgs;
		}
		public DXPaintEventArgs(Graphics graphics) {
			this.graphics = graphics;
		}
		public DXPaintEventArgs(Graphics graphics, Rectangle clipRectangle)
			: this(graphics) {
			this.clipRectangle = clipRectangle;
		}
		public DXPaintEventArgs() {
		}
		public PaintEventArgs PaintArgs { get { return paintArgs; }	}
		public Graphics Graphics {
			get { return PaintArgs == null ? graphics : PaintArgs.Graphics; }
		}
		public Rectangle ClipRectangle {
			get { return PaintArgs == null ? clipRectangle : PaintArgs.ClipRectangle; }
		}
	}
	public class ResourceCache : IDisposable {
		Dictionary<Color, Brush> solidBrushes;
		Dictionary<PenKey, Pen> pens;
		Dictionary<GradientKey, Brush> gradientBrushes;
		Hashtable fonts;
		class ColorComparer : IEqualityComparer<Color> {
			public bool Equals(Color x, Color y) {
				return x == y;
			}
			public int GetHashCode(Color obj) {
				return obj.GetHashCode();
			}
		}
		public ResourceCache() {
			this.solidBrushes = new Dictionary<Color, Brush>(new ColorComparer());
			this.pens = new Dictionary<PenKey, Pen>(new PenKeyComparer());
		}
		[ThreadStatic]
		static ResourceCache defaultCache;
		public static ResourceCache DefaultCache {
			get { 
				if(defaultCache == null) defaultCache = new ResourceCache();
				return defaultCache;
			}
		}
		const int MaxCount = 1000;
		public virtual void CheckCache() {
			if(Pens.Count > MaxCount) ClearHashtable(Pens);
			if(SolidBrushes.Count > MaxCount) ClearHashtable(SolidBrushes);
			if(this.gradientBrushes != null && gradientBrushes.Count > MaxCount) ClearHashtable(this.gradientBrushes);
			if(this.fonts != null && fonts.Count > MaxCount) ClearHashtable(this.fonts);
		}
		public virtual void Dispose() {
			Clear();
		}
		public virtual void Clear() {
			ClearHashtable(SolidBrushes);
			ClearHashtable(Pens);
			ClearHashtable(this.gradientBrushes);
			ClearHashtable(this.fonts);
		}
		public Font GetFont(Font font, FontStyle style) {
			if(font.Style == style) return font;
			return GetFont(font, font.Size, style);
		}
		public virtual Font GetFont(Font font, float size, FontStyle style) {
			if(font.Style == style && font.Size == size) return font;
			string key = GetFontKey(font, style, size);
			object res = Fonts[key];
			if(res == null) {
				font = new Font(font.FontFamily, size, style, font.Unit);
				Fonts[key] = font;
				return font;
			}
			return res as Font;
		}
		protected virtual string GetFontKey(Font font, FontStyle style, float size) {
			return string.Format("{0}: {1} {2} {3}", new object[] {font.FontFamily, size, font.Unit, style});
		}
		protected virtual Brush GetSystemBrush(Color color) {
			return SystemBrushes.FromSystemColor(color);
		}
		protected virtual Pen GetSystemPen(Color color) {
			return SystemPens.FromSystemColor(color);
		}
		public virtual Brush GetSolidBrush(Color color) {
			Brush brush = null;
			if(color.IsSystemColor) brush = GetSystemBrush(color);
			if(brush != null) return brush;
			if(SolidBrushes.TryGetValue(color, out brush))
				return brush;
			brush = new SolidBrush(color);
			SolidBrushes.Add(color, brush);
			return brush;
		}
		public virtual Pen GetPen(Color color) {
			return GetPen(color, 1);
		}
		public virtual Pen GetPen(Color color, int width) {
			Pen pen = null;
			if(width == 1 && color.IsSystemColor) pen = GetSystemPen(color);
			if(pen != null) return pen;
			PenKey penHash = GetPenHash(color, width);
			if(Pens.TryGetValue(penHash, out pen)) return pen;
			pen = new Pen(color, width);
			Pens.Add(penHash, pen);
			return pen;
		}
		public Brush GetGradientBrush(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode) {
			rect.Inflate(1, 1);
			return GetGradientBrush(rect, startColor, endColor, mode, 1);
		}
		public virtual Brush GetGradientBrush(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode, int blendCount) {
			GradientKey key = GetGradientHash(rect, startColor, endColor, mode, blendCount);
			Brush brush;
			if(GradientBrushes.TryGetValue(key, out brush))
				return brush;
			if(startColor == endColor || endColor == Color.Empty || rect.Width < 1 || rect.Height < 1) 
				brush = new SolidBrush(startColor);
			else
				brush = new LinearGradientBrush(rect, startColor, endColor, mode);
			GradientBrushes.Add(key, brush);
			return brush;
		}
		class PenKeyComparer : IEqualityComparer<PenKey> {
			public bool Equals(PenKey x, PenKey y) {
				return x.Color == y.Color && x.Width == y.Width;
			}
			public int GetHashCode(PenKey obj) {
				return obj.Color.GetHashCode() ^ obj.Width;
			}
		}
		protected struct PenKey {
			public Color Color;
			public int Width;
			public PenKey(Color color, int width) {
				this.Color = color;
				this.Width = width;
			}
		}
		protected PenKey GetPenHash(Color color, int width) {
			return new PenKey(color, width);
		}
		class GradientKeyComparer : IEqualityComparer<GradientKey> {
			public bool Equals(GradientKey x, GradientKey y) {
				return x.Mode == y.Mode && x.StartColor == y.StartColor && x.EndColor == y.EndColor && x.BlendCount == y.BlendCount && x.Rect == y.Rect;
			}
			public int GetHashCode(GradientKey obj) {
				return obj.Rect.GetHashCode() ^ (int)obj.Mode ^ obj.StartColor.GetHashCode() ^ obj.EndColor.GetHashCode() ^ obj.BlendCount;
			}
		}
		protected struct GradientKey {
			public Rectangle Rect;
			public Color StartColor;
			public Color EndColor;
			public LinearGradientMode Mode;
			public int BlendCount;
			public GradientKey(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode, int blendCount) {
				this.Rect = rect;
				this.StartColor = startColor;
				this.EndColor = endColor;
				this.Mode = mode;
				this.BlendCount = blendCount;
			}
		}
		protected GradientKey GetGradientHash(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode, int blendCount) {
			return new GradientKey(rect, startColor, endColor, mode, blendCount);
		}
		protected Dictionary<Color, Brush> SolidBrushes {
			get { return solidBrushes; } 
		}
		protected Dictionary<PenKey, Pen> Pens {
			get { return pens; } 
		}
		protected Dictionary<GradientKey, Brush> GradientBrushes {
			get {
				if(gradientBrushes == null)
					this.gradientBrushes = new Dictionary<GradientKey, Brush>(new GradientKeyComparer());
				return gradientBrushes; 
			} 
		}
		protected Hashtable Fonts { 
			get { 
				if(fonts == null) fonts = new Hashtable();
				return fonts; 
			} 
		}
		protected void ClearHashtable(IDictionary hash) {
			if(hash == null) return;
			foreach(IDisposable obj in hash.Values) {
				obj.Dispose();
			}
			hash.Clear();
		}
		[Description("")]
		public int ResourceCount {
			get {
				return SolidBrushes.Count + GradientBrushes.Count + Pens.Count;
			}
		}
	}
	public class GraphicsCache : IDisposable {
		XPaint paint;
		DXPaintEventArgs paintArgs;
		GraphicsClip clipInfo;
		ResourceCache cache;
		Matrix matrix;
		Point offset, offsetEx;
		bool matrixReady;
		public GraphicsCache(Graphics g) : this(new DXPaintEventArgs(g)) { }
		public GraphicsCache(PaintEventArgs e, XPaint paint) : this(new DXPaintEventArgs(e), paint) { }
		public GraphicsCache(DXPaintEventArgs e, XPaint paint) {
			this.paint = paint;
			this.clipInfo = new GraphicsClip();
			SetPaintArgs(e);
		}
		[Description("This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.")]
		public ResourceCache Cache {
			get {
				if(cache == null) return ResourceCache.DefaultCache;
				return cache;
			}
			set {
				cache = value;
			}
		}
		public GraphicsCache(PaintEventArgs e) : this(new DXPaintEventArgs(e)) { }
		public GraphicsCache(DXPaintEventArgs e) : this(e, XPaint.Graphics) { }
		public virtual void Dispose() {
			Clear();
			if(this.cache != null) cache.Dispose();
			this.cache = null;
			if(this.clipInfo != null) this.clipInfo.Dispose();
		}
		public virtual void Clear() {
			this.paintArgs = null;
			this.matrix = null;
			this.matrixReady = false;
			this.offset = Point.Empty;
			this.offsetEx = Point.Empty;
		}
		[Description("This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.")]
		public virtual Matrix TransformMatrix { get { return matrix; } }
		[Description("This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.")]
		public virtual Point Offset {
			get {
				if(!IsMatrixReady) UpdateMatrix();
				return offset;
			}
		}
		[Description("This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.")]
		public virtual Point OffsetEx {
			get {
				if(!IsMatrixReady) UpdateMatrix();
				return offsetEx;
			}
		}
		public virtual Rectangle CalcRectangle(Rectangle r) {
			r.Offset(Offset); 
			return r;
		}
		public virtual Rectangle CalcClipRectangle(Rectangle r) {
			r.Offset(Offset); 
#if DXWhidbey
			r.Offset(OffsetEx); 
#endif
			return r;
		}
		public void ResetMatrix() {
			this.matrixReady = false;
		}
		protected virtual bool IsMatrixReady { get { return matrixReady; } }
		protected virtual void UpdateMatrix() {
			this.matrix = null;
			this.offset = Point.Empty;
			this.offsetEx = Point.Empty;
			if(Graphics != null) {
				this.matrix = Graphics.Transform;
				IntPtr hdc = Graphics.GetHdc();
				try {
					NativeMethods.POINT pt = new NativeMethods.POINT();
					bool res = NativeMethods.GetViewportOrgEx(hdc, ref pt);
					this.offsetEx = new Point(pt.X, pt.Y);
				} finally {
					Graphics.ReleaseHdc(hdc);
				}
			}
			if(TransformMatrix != null) {
				offset = new Point((int)TransformMatrix.OffsetX, (int)TransformMatrix.OffsetY);
			}
			this.matrixReady = true;
		}
		internal void SetGraphics(Graphics g) { SetPaintArgs(new DXPaintEventArgs(g, Rectangle.Empty)); }
		internal void SetPaintArgs(DXPaintEventArgs e) {
			this.matrixReady = false;
			this.paintArgs = e;
			if(this.paintArgs == null) return;
			this.clipInfo.ReleaseGraphics();
			this.clipInfo.MaximumBounds = e.ClipRectangle;
		}
		public bool IsNeedDrawRect(Rectangle r) {
			if(r.IsEmpty) return false;
			if(PaintArgs == null) return true;
			if(PaintArgs.ClipRectangle.IsEmpty) return true;
			if(PaintArgs.ClipRectangle.IntersectsWith(r))
				return Graphics.IsVisible(CalcRectangle(r));
			return false;
		}
		[Description("This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.")]
		public GraphicsClip ClipInfo {
			get {
				if(clipInfo.Graphics != Graphics) clipInfo.Initialize(this);
				return clipInfo;
			}
		}
		[Description("Gets or sets a DevExpress.Utils.Paint.XPaint object.")]
		public XPaint Paint {
			get { return paint; }
			set {
				if(value == null) value = XPaint.Graphics;
				paint = value;
			}
		}
		[Description("Gets an object providing information for painting.")]
		public DXPaintEventArgs PaintArgs {
			get { return paintArgs; }
			set {
				if(PaintArgs == value) return;
				SetPaintArgs(value);
			}
		}
		[Description("Gets an object that serves as the painting surface and provides painting facilities.")]
		public Graphics Graphics { get { return PaintArgs == null ? null : PaintArgs.Graphics; } }
		public Font GetFont(Font font, FontStyle fontStyle) { return Cache.GetFont(font, fontStyle); }
		public Brush GetSolidBrush(Color color) { return Cache.GetSolidBrush(color); }
		public Pen GetPen(Color color) { return Cache.GetPen(color); }
		public Pen GetPen(Color color, int width) { return Cache.GetPen(color, width); }
		public Brush GetGradientBrush(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode) {
			return Cache.GetGradientBrush(rect, startColor, endColor, mode);
		}
		public Brush GetGradientBrush(Rectangle rect, Color startColor, Color endColor, LinearGradientMode mode, int blendCount) {
			return Cache.GetGradientBrush(rect, startColor, endColor, mode, blendCount);
		}
		public void FillRectangle(Brush brush, Rectangle rect) {
			Paint.FillRectangle(Graphics, brush, rect);
		}
		public void FillRectangle(Brush brush, RectangleF rect) {
			Paint.FillRectangle(Graphics, brush, rect);
		}
		public void FillRectangle(Color color, Rectangle rect) {
			FillRectangle(GetSolidBrush(color), rect);
		}
		public void DrawVString(string text, Font font, Brush foreBrush, Rectangle bounds, StringFormat strFormat, int angle) {
			Paint.DrawVString(this, text, font, foreBrush, bounds, strFormat, angle);
		}
		public void DrawString(string text, Font font, Brush foreBrush, Rectangle bounds, StringFormat strFormat) {
			Paint.DrawString(this, text, font, foreBrush, bounds, strFormat);
		}
		public SizeF CalcTextSize(string text, Font font, StringFormat strFormat, int maxWidth) {
			return Paint.CalcTextSize(Graphics, text, font, strFormat, maxWidth);
		}
		public void DrawRectangle(Pen pen, Rectangle r) {
			Paint.DrawRectangle(Graphics, pen, r);
		}
	}
	public class GraphicsInfoArgs {
		GraphicsCache cache;
		Rectangle bounds;
		Graphics graphics;
		public GraphicsInfoArgs(GraphicsInfoArgs info, Rectangle bounds) {
			this.graphics = info.graphics;
			this.cache = info.cache;
			this.bounds = bounds;
		}
		public GraphicsInfoArgs(GraphicsCache cache, Rectangle bounds) {
			this.cache = cache;
			this.bounds = bounds;
		}
		public virtual XPaint Paint { get { return Cache.Paint; } }
		public virtual GraphicsCache Cache { 
			get { return cache; } 
			set { 
				cache = value; 
			}
		}
		public Rectangle CalcRectangle(Rectangle bounds) {
			if(Cache != null) return Cache.CalcRectangle(bounds);
			return bounds;
		}
		public DXPaintEventArgs PaintArgs {
			get { return Cache != null ? Cache.PaintArgs : null; }
		}
		public virtual Graphics Graphics { 
			get { return (Cache != null ? Cache.Graphics : graphics); } 
			set { 
				if(value != null) {
					if(Cache == null) 
						Cache = new GraphicsCache(value);
					else 
						Cache.SetGraphics(value);
				}
				graphics = value; 
			}
		}
		public virtual Rectangle Bounds { 
			get { return bounds; }
			set { bounds = value; }
		}
	}
	public class GraphicsInfo {
		[ThreadStatic]
		static Graphics sharedGraphics;
		static Graphics SharedGraphics { 
			get { 
				if(sharedGraphics == null) {
					sharedGraphics = Graphics.FromHwndInternal(IntPtr.Zero);
				}
				return sharedGraphics;
			}
		}
		GraphicsCache cache;
		int lockGraphics;
		[ThreadStatic]
		static GraphicsInfo defaultInfo;
		public static GraphicsInfo Default {
			get { 
				if(defaultInfo == null) defaultInfo = new GraphicsInfo();
				return defaultInfo;
			}
		}
		public void CreateCache() {
			if(this.cache != null) return;
			this.cache = new GraphicsCache((Graphics)null);
		}
		protected virtual int LockGraphics { 
			get { return lockGraphics; }
			set { lockGraphics = value; }
		}
		public virtual GraphicsCache Cache { get { return cache; } }
		public virtual Graphics Graphics { get { return Cache == null ? null : Cache.Graphics; } }
		public virtual Graphics AddGraphics(Graphics g) {
			if(LockGraphics ++ != 0) return Graphics;
			if(g == null) g = SharedGraphics;
			g = g == null ? Graphics.FromHwnd(IntPtr.Zero) : g;
			if(this.cache == null) 
				this.cache = new GraphicsCache(g);
			else {
				this.cache.SetGraphics(g);
			}
			return Graphics;
		}
		public virtual void ReleaseGraphics() {
			if(--LockGraphics == 0) {
				if(cache != null) {
					cache.Clear();
				}
			}
		}
	}
}
