#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using DevExpress.Skins;
using DevExpress.Utils;
using DevExpress.Utils.Drawing;
using DevExpress.Utils.WXPaint;
using DevExpress.LookAndFeel;
using DevExpress.LookAndFeel.Helpers;
using DevExpress.XtraEditors.Controls;
using DevExpress.XtraEditors.ViewInfo;
namespace DevExpress.XtraEditors.ViewInfo {
	public enum ScrollBarState {Normal, IncButtonPressed, DecButtonPressed, ThumbPressed, 
		IncAreaPressed, DecAreaPressed, IncButtonHot, DecButtonHot, ThumbHot}
	public class ScrollBarArrowArgs: ObjectInfoArgs {
		Color normalColor, disabledColor;
		ScrollButton scrollButton;
		int pressedShift;
		public ScrollBarArrowArgs(GraphicsCache cache, 
			Rectangle bounds, ObjectState state, ScrollButton scrollButton): base(cache, bounds, state) {
			this.normalColor = SystemColors.ControlText;
			this.disabledColor = SystemColors.GrayText;
			this.scrollButton = scrollButton;
			this.pressedShift = 1;
		}
		public Color NormalColor { get { return this.normalColor; } set { this.normalColor = value; } }
		public Color DisabledColor { get { return this.disabledColor; } set { this.disabledColor = value; } }
		public ScrollButton ScrollButton { get { return this.scrollButton; } set { this.scrollButton = value; } }
		public int PressedShift { get { return this.pressedShift; } set { this.pressedShift = value; } }
		public Brush GetBrush() {
			return Cache.GetSolidBrush(ObjectState.Disabled == State ? DisabledColor : NormalColor);
		}
		public Pen GetPen() {
			return Cache.GetPen(ObjectState.Disabled == State ? DisabledColor : NormalColor);
		}
	}
	public class ScrollBarArrowPainter: ObjectPainter {
		static Pen lightLightPen = new Pen(SystemBrushes.ControlLightLight);
		public override void DrawObject (ObjectInfoArgs e) {
			ScrollBarArrowArgs sArgs = e as ScrollBarArrowArgs;
			Point[] points = CalculateArrowPoints(sArgs);
			if(ObjectState.Disabled == e.State) {
				for (int i = 0; i < points.Length; i ++)
					points[i].Offset(1, 1);
				DrawArrow(e, SystemBrushes.ControlLightLight, lightLightPen, points);
				for (int i = 0; i < points.Length; i ++)
					points[i].Offset(-1, -1);
			}
			DrawArrow(e, sArgs.GetBrush(), sArgs.GetPen(), points);
		}
		private void DrawArrow(ObjectInfoArgs e, Brush brush, Pen pen, Point[] points) {
			e.Graphics.DrawPolygon(pen, points);
			e.Graphics.FillPolygon(brush, points);
		}
		private Point[] CalculateArrowPoints(ScrollBarArrowArgs e) {
			Rectangle bounds = e.Bounds;
			Point[] points = {new Point(0), new Point(0), new Point(0)};
			if(ObjectState.Pressed == e.State)
				bounds.Offset(e.PressedShift, e.PressedShift);
			int delta = bounds.Width - bounds.Height;
			if(delta > 0) 
				bounds.Inflate(-delta / 2, 0);
			else bounds.Inflate(0, delta / 2);
			bounds = GetArrowBounds(bounds, e.ScrollButton);
			int middleLeftRight = bounds.Left + bounds.Width / 2;
			int middleTopBottom = bounds.Top + bounds.Height / 2;
			switch (e.ScrollButton) {
				case ScrollButton.Up: 
					points[0].X = middleLeftRight; points[0].Y = bounds.Top;
					points[1].X = bounds.Right; points[1].Y = bounds.Bottom;
					points[2].X = bounds.Left; points[2].Y = bounds.Bottom;
					break;
				case ScrollButton.Down:
					bounds.Offset(0, 1);
					points[0].X = middleLeftRight; points[0].Y = bounds.Bottom;
					points[1].X = bounds.Right; points[1].Y = bounds.Top;
					points[2].X = bounds.Left; points[2].Y = bounds.Top;
					break;
				case ScrollButton.Left:
					points[0].X = bounds.Left; points[0].Y = middleTopBottom;
					points[1].X = bounds.Right; points[1].Y = bounds.Top;
					points[2].X = bounds.Right; points[2].Y = bounds.Bottom;
					break;
				case ScrollButton.Right:
					bounds.Offset(1, 0);
					points[0].X = bounds.Right; points[0].Y = middleTopBottom;
					points[1].X = bounds.Left; points[1].Y = bounds.Top;
					points[2].X = bounds.Left; points[2].Y = bounds.Bottom;
					break;
			}
			return points;
		}
		private Rectangle GetArrowBounds(Rectangle bounds, ScrollButton button) {
			Rectangle r = bounds;
			Size size = GetArrowSize(bounds, button);
			r.X += (r.Width - size.Width) / 2 - 1;
			r.Y += (r.Height - size.Height) / 2 - 1;
			r.Width = size.Width; r.Height = size.Height;
			if((ScrollButton.Up == button) || (ScrollButton.Down == button)) {
				r.Width ++;
			} else {
				r.Height ++;
			}
			return r;
		}
		private Size GetArrowSize(Rectangle bounds, ScrollButton button) {
			Size size = new Size(0, 0);
			if((ScrollButton.Up == button) || (ScrollButton.Down == button)) {
				size.Width = (bounds.Width - 1) / 2 - 3;
				if(size.Width % 2 == 0) size.Width ++;
				size.Height = size.Width / 2 + 1;
			} else {
				size.Height = (bounds.Height - 1) / 2 - 3;
				if( size.Height % 2 == 0) size.Height ++;
				size.Width = size.Height / 2 + 1;
			}
			return size;
		}
	}
	public class ScrollBarButtonInfoArgs : StyleObjectInfoArgs {
		public ScrollButton ScrollButton;
		public ScrollBarButtonInfoArgs(GraphicsCache cache, Rectangle bounds, AppearanceObject style, AppearanceObject backstyle,
			ScrollButton scrollButton, ObjectState state): base(cache, bounds, style, backstyle, state) {
			this.ScrollButton = scrollButton;
		}
	}
	public abstract class ScrollBarButtonPainterBase: ObjectPainter {
		ObjectPainter buttonPainter;
		ScrollBarArrowPainter arrowPainter;
		public ScrollBarButtonPainterBase() {
			this.arrowPainter = new ScrollBarArrowPainter();
		}
		public override void DrawObject(ObjectInfoArgs e) {
			ScrollBarButtonInfoArgs sbe = (e as ScrollBarButtonInfoArgs);
			ObjectInfoArgs bArgs = CreateInfoArgs(sbe);
			ButtonPainter.DrawObject(bArgs);
			if(NeedToDrawArrow) {
				ScrollBarArrowArgs aArgs = new ScrollBarArrowArgs(e.Cache, e.Bounds, e.State, sbe.ScrollButton);
				this.arrowPainter.DrawObject(aArgs);
			}
		}
		public ObjectPainter ButtonPainter { 
			get { 
				if(buttonPainter == null) buttonPainter = CreateButtonPainter();
				return buttonPainter;
			} 
		}
		protected abstract ObjectPainter CreateButtonPainter();
		protected virtual ObjectInfoArgs CreateInfoArgs(ScrollBarButtonInfoArgs e)  {
			return new StyleObjectInfoArgs(e.Cache, e.Bounds, e.Appearance, e.BackAppearance, e.State);
		}
		protected virtual bool NeedToDrawArrow { get { return true; } }
	}
	public class ScrollBarThumbInfoArgs: StyleObjectInfoArgs {
		bool isFlashing;
		ScrollBarType scrollBarType;
		public ScrollBarThumbInfoArgs(GraphicsCache cache, Rectangle bounds, AppearanceObject style, AppearanceObject backStyle,
			ObjectState state, bool isFlashing, ScrollBarType scrollBarType): base(cache, bounds, style, backStyle, state) {
			this.isFlashing = isFlashing;
			this.scrollBarType = scrollBarType;
		}
		public bool IsFlashing { get { return this.isFlashing; } set { this.isFlashing = value; } }
		public ScrollBarType ScrollBarType { get { return this.scrollBarType; } }
	}
	public class ScrollBarThumbButtonInfoArgs: StyleObjectInfoArgs {
		static Brush flashingBrush = new HatchBrush(HatchStyle.Percent50, Color.Black, Color.White);
		bool isFlashing;
		ScrollBarType scrollBarType;
		public ScrollBarThumbButtonInfoArgs(GraphicsCache cache, Rectangle bounds, 
			AppearanceObject style, AppearanceObject backStyle,
			ObjectState state, bool isFlashing, ScrollBarType scrollBarType): base(cache, bounds, style, backStyle, state) {
			this.isFlashing = isFlashing;
			this.scrollBarType = ScrollBarType;
		}
		public bool IsFlashing { get { return this.isFlashing; } set { this.isFlashing = value; } }
		public Brush FlashingBrush { get { return flashingBrush; } }
		public ScrollBarType ScrollBarType { get { return this.scrollBarType; } }
	}
	public abstract class ScrollBarThumbPainterBase: ObjectPainter {
		ObjectPainter buttonPainter = null;
		public ScrollBarThumbPainterBase() { }
		public override void DrawObject(ObjectInfoArgs e) {
			ObjectInfoArgs bArgs = CreateInfoArgs(e);
			if((! CanBePressed) && (ObjectState.Pressed == bArgs.State))
				bArgs.State = ObjectState.Normal;
			ButtonPainter.DrawObject(bArgs);
		}
		public ObjectPainter ButtonPainter { 
			get { 
				if(buttonPainter == null) buttonPainter = CreateButtonPainter();
				return buttonPainter; 
			}
		}
		protected abstract ObjectPainter CreateButtonPainter();
		protected virtual ObjectInfoArgs CreateInfoArgs(ObjectInfoArgs e) {
			return new ScrollBarThumbButtonInfoArgs(e.Cache, e.Bounds, 
				(e as ScrollBarThumbInfoArgs).Appearance, (e as ScrollBarThumbInfoArgs).BackAppearance,
				e.State, (e as ScrollBarThumbInfoArgs).IsFlashing, 
				(e as ScrollBarThumbInfoArgs).ScrollBarType);
		}
		protected virtual bool CanBePressed { get { return false; } }
	}
	public class ScrollBarInfoArgs: StyleObjectInfoArgs {
		IScrollBar scrollBar;
		public ScrollBarInfoArgs(GraphicsCache cache, Rectangle bounds, AppearanceObject style, AppearanceObject backStyle, IScrollBar scrollBar): 
			base(cache, bounds, style, backStyle, ObjectState.Normal) {
			this.scrollBar = scrollBar;
		}
		public ISkinProvider SkinProvider { get { return ScrollBar.LookAndFeel.ActiveLookAndFeel; } }
		public IScrollBar ScrollBar { get { return this.scrollBar; } }
		public ScrollBarViewInfo ViewInfo { get { return ScrollBar.ViewInfo; } }
		public ScrollBarState ScrollBarState { get { return ViewInfo.State; } }
		public Rectangle DecButtonBounds { get { return ViewInfo.DecButtonBounds; } }
		public Rectangle IncButtonBounds { get { return ViewInfo.IncButtonBounds; } }
		public Rectangle ThumbButtonBounds { get { return ViewInfo.ThumbButtonBounds; } }
	}
	public abstract class ScrollBarPainterBase: ObjectPainter {
		Brush scrollAreaBrush, pressedScrollAreaBrush;
		ObjectPainter _buttonPainter = null;
		ObjectPainter _thumbPainter = null;
		public ScrollBarPainterBase() {
			this.scrollAreaBrush = new HatchBrush(HatchStyle.Percent50, SystemColors.Control, SystemColors.Window);
			this.pressedScrollAreaBrush = new HatchBrush(HatchStyle.Percent50, SystemColors.ControlDarkDark, SystemColors.WindowText);
		}
		public void DestroyBrushes() {
			if(scrollAreaBrush != null) scrollAreaBrush.Dispose();
			if(pressedScrollAreaBrush != null) pressedScrollAreaBrush.Dispose();
		}
		public virtual int ThumbMinWidth { get { return System.Windows.Forms.SystemInformation.HorizontalScrollBarThumbWidth; } }
		public override void DrawObject (ObjectInfoArgs e) {
			ScrollBarInfoArgs sbe = e as ScrollBarInfoArgs;
			if(! sbe.IncButtonBounds.IsEmpty)
				DrawArrowButtons(sbe);
			if(! sbe.ThumbButtonBounds.IsEmpty)
				DrawThumbButton(sbe);
			DrawScrollArea(sbe);
		}
		protected ObjectPainter ThumbPainter {
			get {
				if(_thumbPainter == null) _thumbPainter = CreateScrollBarThumbPainter();
				return _thumbPainter;
			}
		}
		protected ObjectPainter ButtonPainter {
			get {
				if(_buttonPainter == null) _buttonPainter = CreateScrollBarButtonPainter();
				return _buttonPainter;
			}
		}
		protected abstract ObjectPainter CreateScrollBarButtonPainter();
		protected abstract ObjectPainter CreateScrollBarThumbPainter();
		protected virtual Color ButtonBackColor { get { return SystemColors.Control; } }
		protected virtual void DrawNormalScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			e.Cache.Paint.FillRectangle(e.Graphics, ScrollAreaBrush, bounds);
		}
		protected virtual void DrawPressedScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			e.Cache.Paint.FillRectangle(e.Graphics, PressedScrollAreaBrush, bounds);
		}
		protected virtual Brush ScrollAreaBrush { get {	return scrollAreaBrush;	} }
		protected virtual Brush PressedScrollAreaBrush { get {	return pressedScrollAreaBrush;	} }
		private void DrawArrowButtons(ScrollBarInfoArgs e) {
			ScrollBarButtonInfoArgs incArgs = new ScrollBarButtonInfoArgs(e.Cache, e.IncButtonBounds, 
				e.Appearance, e.BackAppearance, GetIncScrollButton(e), GetIncButtonState(e));
			e.Appearance.BackColor = ButtonBackColor;
			ScrollBarButtonInfoArgs decArgs = new ScrollBarButtonInfoArgs(e.Cache, e.DecButtonBounds, 
				e.Appearance, e.BackAppearance, GetDecScrollButton(e), GetDecButtonState(e));
			ButtonPainter.DrawObject(incArgs);
			ButtonPainter.DrawObject(decArgs);
		}
		private void DrawThumbButton(ScrollBarInfoArgs e) {
			ScrollBarThumbInfoArgs thumbArgs = new ScrollBarThumbInfoArgs(e.Cache, e.ThumbButtonBounds, e.Appearance, e.BackAppearance,
				GetThumbButtonState(e), e.ViewInfo.IsThumbFlashing, e.ScrollBar.ScrollBarType);
			ThumbPainter.DrawObject(thumbArgs);
		}
		private void DrawScrollAreaOld(ScrollBarInfoArgs e) {
			Rectangle decR, incR;
			Rectangle decButtonBounds = e.ViewInfo.VisibleDecButtonBounds;
			Rectangle incButtonBounds = e.ViewInfo.VisibleIncButtonBounds;
			if(e.ThumbButtonBounds.IsEmpty) {
				if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal) 
					decR = new Rectangle(decButtonBounds.Right, 0, 
						incButtonBounds.X - decButtonBounds.Right, decButtonBounds.Height);
				else decR = new Rectangle(0, decButtonBounds.Bottom, 
						 decButtonBounds.Width, incButtonBounds.Y - decButtonBounds.Bottom);
				DrawNormalScrollArea(e, decR, true);
			} else {
				if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal) 
					decR = new Rectangle(decButtonBounds.Right, 0, 
						e.ThumbButtonBounds.X - decButtonBounds.Right, decButtonBounds.Height);
				else decR = new Rectangle(0, decButtonBounds.Bottom, 
						 decButtonBounds.Width, e.ThumbButtonBounds.Y - decButtonBounds.Bottom);
				if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal)
					incR = new Rectangle(e.ThumbButtonBounds.Right, 0, 
						incButtonBounds.X - e.ThumbButtonBounds.Right, e.ThumbButtonBounds.Height);
				else incR = new Rectangle(0, e.ThumbButtonBounds.Bottom, 
						 e.ThumbButtonBounds.Width, incButtonBounds.Y - e.ThumbButtonBounds.Bottom);
				if(e.ViewInfo.IsRightToLeft){
					Rectangle r = decR;
					decR = incR;
					incR = r;
				}
				if(e.ScrollBarState == ScrollBarState.DecAreaPressed) 
					DrawPressedScrollArea(e, decR, false);
				else DrawNormalScrollArea(e, decR, false);
				if(e.ScrollBarState == ScrollBarState.IncAreaPressed) 
					DrawPressedScrollArea(e, incR, true);
				else DrawNormalScrollArea(e, incR, true);
			}
		}
		private void DrawScrollArea(ScrollBarInfoArgs e) {
			Rectangle decR, incR;
			Rectangle decButtonBounds = e.ViewInfo.VisibleDecButtonBounds;
			Rectangle incButtonBounds = e.ViewInfo.VisibleIncButtonBounds;
			if (e.ThumbButtonBounds.Width == 0 && e.ThumbButtonBounds.Height == 0) {
				if (e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal)
					decR = new Rectangle(decButtonBounds.Right, decButtonBounds.Y,
						incButtonBounds.X - decButtonBounds.Right, decButtonBounds.Height);
				else decR = new Rectangle(decButtonBounds.X, decButtonBounds.Bottom,
						 decButtonBounds.Width, incButtonBounds.Y - decButtonBounds.Bottom);
				DrawNormalScrollArea(e, decR, true);
			}
			else {
				if (e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal)
					decR = new Rectangle(decButtonBounds.Right, decButtonBounds.Y,
						e.ThumbButtonBounds.X - decButtonBounds.Right, decButtonBounds.Height);
				else decR = new Rectangle(decButtonBounds.X, decButtonBounds.Bottom,
						 decButtonBounds.Width, e.ThumbButtonBounds.Y - decButtonBounds.Bottom);
				if (e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal)
					incR = new Rectangle(e.ThumbButtonBounds.Right, decButtonBounds.Y,
						incButtonBounds.X - e.ThumbButtonBounds.Right, e.ThumbButtonBounds.Height);
				else incR = new Rectangle(decButtonBounds.X, e.ThumbButtonBounds.Bottom,
						 e.ThumbButtonBounds.Width, incButtonBounds.Y - e.ThumbButtonBounds.Bottom);
				if (e.ViewInfo.IsRightToLeft) {
					Rectangle r = decR;
					decR = incR;
					incR = r;
				}
				if (e.ScrollBarState == ScrollBarState.DecAreaPressed)
					DrawPressedScrollArea(e, decR, false);
				else DrawNormalScrollArea(e, decR, false);
				if (e.ScrollBarState == ScrollBarState.IncAreaPressed)
					DrawPressedScrollArea(e, incR, true);
				else DrawNormalScrollArea(e, incR, true);
			}
		}
		private ScrollButton GetIncScrollButton(ScrollBarInfoArgs e) {
			if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal) {
				if( ! e.ViewInfo.IsRightToLeft)
					return ScrollButton.Right;
				else return ScrollButton.Left;
			}
			else return ScrollButton.Down;
		}
		private ScrollButton GetDecScrollButton(ScrollBarInfoArgs e) {
			if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal) {
				if( ! e.ViewInfo.IsRightToLeft)
					return ScrollButton.Left;
				else return ScrollButton.Right;
			}
			else return ScrollButton.Up;
		}
		private ObjectState GetDecButtonState(ScrollBarInfoArgs e) {
			if( ! e.ViewInfo.Enabled)
				return ObjectState.Disabled;
			if(e.ScrollBarState == ScrollBarState.DecButtonHot)
				return ObjectState.Hot;
			if(e.ScrollBarState == ScrollBarState.DecButtonPressed)
				return ObjectState.Pressed;
			return ObjectState.Normal;
		}
		private ObjectState GetIncButtonState(ScrollBarInfoArgs e) {
			if( ! e.ViewInfo.Enabled)
				return ObjectState.Disabled;
			if(e.ScrollBarState == ScrollBarState.IncButtonHot)
				return ObjectState.Hot;
			if(e.ScrollBarState == ScrollBarState.IncButtonPressed)
				return ObjectState.Pressed;
			return ObjectState.Normal;
		}
		private ObjectState GetThumbButtonState(ScrollBarInfoArgs e) {
			if( ! e.ViewInfo.Enabled)
				return ObjectState.Disabled;
			if(e.ScrollBarState == ScrollBarState.ThumbHot) 
				return ObjectState.Hot;
			if(e.ScrollBarState == ScrollBarState.ThumbPressed)
				return ObjectState.Pressed;
			return ObjectState.Normal;
		}
		protected internal virtual int GetButtonCornerSize(ScrollBarType scrollType) {
			return 0;
		}
	}
	public class Style3DScrollBarButtonPainter: ScrollBarButtonPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new Style3DButtonObjectPainter();
		}
	}
	public class Style3DScrollBarThumbButtonPainter: Style3DButtonObjectPainter {
		protected override void DrawButtonBackground(ObjectInfoArgs e, AppearanceObject style, Rectangle r) {
			base.DrawButtonBackground(e, style, r);
			if((e as ScrollBarThumbButtonInfoArgs).IsFlashing) 
				e.Cache.Paint.FillRectangle(e.Graphics, (e as ScrollBarThumbButtonInfoArgs).FlashingBrush, r);
		}
	}
	public class Style3DScrollBarThumbPainter: ScrollBarThumbPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new Style3DScrollBarThumbButtonPainter();
		}
	}
	public class Style3DScrollBarPainter: ScrollBarPainterBase {
		protected override ObjectPainter CreateScrollBarButtonPainter() {
			return new Style3DScrollBarButtonPainter();
		}
		protected override ObjectPainter CreateScrollBarThumbPainter() {
			return new Style3DScrollBarThumbPainter();
		}
	}
	public class FlatScrollBarButtonPainter: ScrollBarButtonPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new FlatButtonObjectPainter();
		}
	}
	public class FlatScrollBarThumbButtonPainter: FlatButtonObjectPainter {
		protected override void DrawButtonBackground(ObjectInfoArgs e, AppearanceObject style, Rectangle r) {
			base.DrawButtonBackground(e, style, r);
			if((e as ScrollBarThumbButtonInfoArgs).IsFlashing) 
				e.Cache.Paint.FillRectangle(e.Graphics, (e as ScrollBarThumbButtonInfoArgs).FlashingBrush, r);
		}
	}
	public class FlatScrollBarThumbPainter: ScrollBarThumbPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new FlatScrollBarThumbButtonPainter();
		}
	}
	public class FlatScrollBarPainter: ScrollBarPainterBase {
		protected override ObjectPainter CreateScrollBarButtonPainter() {
			return new FlatScrollBarButtonPainter();
		}
		protected override ObjectPainter CreateScrollBarThumbPainter() {
			return new FlatScrollBarThumbPainter();
		}
	}
	public class UltraFlatScrollBarButtonPainter: ScrollBarButtonPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new UltraFlatButtonObjectPainter();
		}
	}
	public class UltraFlatScrollBarThumbButtonPainter: UltraFlatButtonObjectPainter {
		protected override void DrawButtonBackground(ObjectInfoArgs e, AppearanceObject style, Rectangle r) {
			base.DrawButtonBackground(e, style, r);
			if((e as ScrollBarThumbButtonInfoArgs).IsFlashing) 
				e.Cache.Paint.FillRectangle(e.Graphics, (e as ScrollBarThumbButtonInfoArgs).FlashingBrush, r);
		}
	}
	public class UltraFlatScrollBarThumbPainter: ScrollBarThumbPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new UltraFlatScrollBarThumbButtonPainter();
		}
	}
	public class UltraFlatScrollBarPainter: ScrollBarPainterBase {
		protected override ObjectPainter CreateScrollBarButtonPainter() {
			return new UltraFlatScrollBarButtonPainter();
		}
		protected override ObjectPainter CreateScrollBarThumbPainter() {
			return new UltraFlatScrollBarThumbPainter();
		}
	}
	public class SkinScrollBarPainter : ScrollBarPainterBase {
		ISkinProvider provider;
		public SkinScrollBarPainter(ISkinProvider provider) { this.provider = provider; }
		public ISkinProvider Provider { get { return provider; } }
		public override int ThumbMinWidth { get { return 14; } }
		protected override ObjectPainter CreateScrollBarButtonPainter() {
			return new SkinScrollBarButtonPainter(Provider);
		}
		protected override ObjectPainter CreateScrollBarThumbPainter() {
			return new SkinScrollBarThumbPainter(Provider);
		}
		SkinElement GetSkinElement(ScrollBarInfoArgs e) {
			return CommonSkins.GetSkin(Provider)[e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal ?
				CommonSkins.SkinScrollShaft : CommonSkins.SkinScrollShaftVert];
		}
		protected override void DrawNormalScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			SkinElementInfo info = new SkinElementInfo(GetSkinElement(e), bounds);
			info.State = e.State;
			info.ImageIndex = -1;
			ObjectPainter.DrawObject(e.Cache, SkinElementPainter.Default, info);
		}
		protected override void DrawPressedScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			SkinElementInfo info = new SkinElementInfo(GetSkinElement(e), bounds);
			info.State = ObjectState.Pressed;
			info.ImageIndex = -1;
			ObjectPainter.DrawObject(e.Cache, SkinElementPainter.Default, info);
		}
		protected internal override int GetButtonCornerSize(ScrollBarType scrollType) {
			SkinElement skin = CommonSkins.GetSkin(Provider)[scrollType == ScrollBarType.Horizontal ? CommonSkins.SkinScrollButtonThumb : CommonSkins.SkinScrollButtonThumbVert];
			return skin.Properties.GetInteger(CommonSkins.OptScrollButtonCorner);
		}
	}
	public class SkinScrollBarButtonPainter : SkinButtonObjectPainter {
		public SkinScrollBarButtonPainter(ISkinProvider provider) : base(provider) { }
		protected override SkinElementInfo UpdateInfo(ObjectInfoArgs e) {
			ScrollBarButtonInfoArgs ee = e as ScrollBarButtonInfoArgs;
			SkinElementInfo info = new SkinElementInfo(CommonSkins.GetSkin(Provider)[CommonSkins.SkinScrollButton], e.Bounds);
			info.State = e.State;
			info.Cache = e.Cache;
			info.ImageIndex = SkinElementPainter.Default.CalcDefaultImageIndex(info.Element.Image, e.State);
			switch(ee.ScrollButton) {
				case ScrollButton.Down : info.ImageIndex += 4; break;
				case ScrollButton.Left : info.ImageIndex += 8; break;
				case ScrollButton.Right : info.ImageIndex += 12; break;
			}
			return info;
		}
		public override void DrawObject(ObjectInfoArgs e) {
			e.Cache.FillRectangle(e.Cache.GetSolidBrush(LookAndFeelHelper.GetSystemColorEx(Provider, SystemColors.Control)), e.Bounds);
			base.DrawObject(e);
		}
	}
	public class SkinScrollBarThumbPainter : SkinButtonObjectPainter {
		public SkinScrollBarThumbPainter(ISkinProvider provider) : base(provider) { }
		protected override SkinElementInfo UpdateInfo(ObjectInfoArgs e) {
			ScrollBarThumbInfoArgs ee = e as ScrollBarThumbInfoArgs;
			SkinElementInfo info = new SkinElementInfo(CommonSkins.GetSkin(Provider)[ee.ScrollBarType == ScrollBarType.Horizontal ? CommonSkins.SkinScrollButtonThumb : CommonSkins.SkinScrollButtonThumbVert], e.Bounds);
			info.State = e.State;
			info.Cache = e.Cache;
			info.ImageIndex = SkinElementPainter.Default.CalcDefaultImageIndex(info.Element.Image, e.State);
			return info;
		}
	}
	public class XPScrollBarButtonPainter : ScrollBarButtonPainterBase {
		protected override ObjectPainter CreateButtonPainter() {
			return new XPAdvButtonPainter();
		}
		protected override ObjectInfoArgs CreateInfoArgs(ScrollBarButtonInfoArgs e)  {
			XPAdvButtonInfoArgs args = new XPAdvButtonInfoArgs(GetButton(e.ScrollButton));
			args.Cache = e.Cache;
			args.Bounds = e.Bounds;
			args.State = e.State;
			return args;
		}
		protected override bool NeedToDrawArrow { get { return false; } }
		private ButtonPredefines GetButton(ScrollButton scrollButton) {
			switch(scrollButton) {
				case ScrollButton.Up: return ButtonPredefines.Up;
				case ScrollButton.Left: return ButtonPredefines.Left;
				case ScrollButton.Right: return ButtonPredefines.Right;
			}
			return ButtonPredefines.Down;
		}
	}
	public class XPScrollBarPartInfoArgs: ObjectInfoArgs {
		int partId;
		public XPScrollBarPartInfoArgs(GraphicsCache cache, Rectangle bounds, 
			ObjectState state, int partId): base(cache, bounds, state) {
			this.partId = partId;
		}
		public int PartId { get { return this.partId; } }
	}
	public class XPScrollBarPartPainter: XPObjectPainter {
		public XPScrollBarPartPainter() {
			DrawArgs = new WXPPainterArgs("scrollbar", 0, 0);
		}
		protected override void UpdateDrawArgs(ObjectInfoArgs e) {
			base.UpdateDrawArgs(e);
			DrawArgs.PartId = (e as XPScrollBarPartInfoArgs).PartId;
		}
		protected override int CalcThemeStateId(ObjectInfoArgs e) {
			return CalcButtonStateId(e);
		}
	}
	public class XPScrollBarThumbPainter: ScrollBarThumbPainterBase {
		public override void DrawObject(ObjectInfoArgs e) {
			base.DrawObject(e);
			int partId = (e as ScrollBarThumbInfoArgs).ScrollBarType == ScrollBarType.Horizontal ?
				XPConstants.SBP_GRIPPERHORZ : XPConstants.SBP_GRIPPERVERT;
			XPScrollBarPartInfoArgs args = new XPScrollBarPartInfoArgs(e.Cache, e.Bounds, e.State, partId);
			Rectangle minBounds = ButtonPainter.CalcObjectMinBounds(args);
			if((e.Bounds.Width > minBounds.Width) && (e.Bounds.Height > minBounds.Height))
				ButtonPainter.DrawObject(args);
		}
		protected override ObjectPainter CreateButtonPainter() {
			return new XPScrollBarPartPainter();
		}
		protected override ObjectInfoArgs CreateInfoArgs(ObjectInfoArgs e) {
			int partId = (e as ScrollBarThumbInfoArgs).ScrollBarType == ScrollBarType.Horizontal ?
				XPConstants.SBP_THUMBBTNHORZ: XPConstants.SBP_THUMBBTNVERT;
			return new XPScrollBarPartInfoArgs(e.Cache, e.Bounds, e.State, partId);
		}
		protected override bool CanBePressed { get { return true; } }
	}
	public class XPScrollBarPainter: ScrollBarPainterBase {
		XPScrollBarPartPainter partPainter;
		public XPScrollBarPainter() {
			partPainter = new XPScrollBarPartPainter();
		}
		protected override ObjectPainter CreateScrollBarButtonPainter() {
			return new XPScrollBarButtonPainter();
		}
		protected override ObjectPainter CreateScrollBarThumbPainter() {
			return new XPScrollBarThumbPainter();
		}
		protected override void DrawNormalScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			XPScrollBarPartInfoArgs args = new XPScrollBarPartInfoArgs(e.Cache, bounds, 
				ObjectState.Normal, GetScrollBarPartId(e, isInc));
			partPainter.DrawObject(args);
		}
		protected override void DrawPressedScrollArea(ScrollBarInfoArgs e, Rectangle bounds, bool isInc) {
			XPScrollBarPartInfoArgs args = new XPScrollBarPartInfoArgs(e.Cache, bounds, 
				ObjectState.Pressed, GetScrollBarPartId(e, isInc));
			partPainter.DrawObject(args);
		}
		private int GetScrollBarPartId(ScrollBarInfoArgs e, bool isInc) {
			if(e.ScrollBar.ScrollBarType == ScrollBarType.Horizontal) {
				if(isInc) 
					return XPConstants.SBP_LOWERTRACKHORZ;
				else return XPConstants.SBP_UPPERTRACKHORZ;
			} else {
				if(isInc) 
					return XPConstants.SBP_LOWERTRACKVERT;
				else return XPConstants.SBP_UPPERTRACKVERT;
			}
		}
	}
	public class LookAndFeelScrollBarPainter {
		public static ScrollBarPainterBase GetPainter(UserLookAndFeel lookAndFeel) {
			switch(lookAndFeel.ActiveStyle) {
				case ActiveLookAndFeelStyle.Style3D:
					return new Style3DScrollBarPainter();
				case ActiveLookAndFeelStyle.UltraFlat:
					return new UltraFlatScrollBarPainter();
				case ActiveLookAndFeelStyle.Office2003:
					if(DevExpress.Utils.WXPaint.WXPPainter.Default.ThemesEnabled) return new XPScrollBarPainter();
					return new FlatScrollBarPainter();
				case ActiveLookAndFeelStyle.WindowsXP:
					return new XPScrollBarPainter();
				case ActiveLookAndFeelStyle.Skin:
					return new SkinScrollBarPainter(lookAndFeel.ActiveLookAndFeel);
			} 
			return new FlatScrollBarPainter();
		}
	}
	public class LookAndFeelScrollBarButtonPainter {
		public static ObjectPainter GetPainter(UserLookAndFeel lookAndFeel) {
			switch(lookAndFeel.ActiveStyle) {
				case ActiveLookAndFeelStyle.Style3D:
					return new Style3DScrollBarButtonPainter();
				case ActiveLookAndFeelStyle.UltraFlat:
					return new UltraFlatScrollBarButtonPainter();
				case ActiveLookAndFeelStyle.WindowsXP:
					return new XPScrollBarButtonPainter();
				case ActiveLookAndFeelStyle.Office2003:
					if(DevExpress.Utils.WXPaint.WXPPainter.Default.ThemesEnabled) return new XPScrollBarButtonPainter();
					return new FlatScrollBarButtonPainter();
				case ActiveLookAndFeelStyle.Skin:
					return new SkinScrollBarButtonPainter(lookAndFeel.ActiveLookAndFeel);
			} 
			return new FlatScrollBarButtonPainter();
		}
	}
	public class LookAndFeelScrollBarThumbPainter {
		public static ScrollBarThumbPainterBase GetPainter(UserLookAndFeel lookAndFeel) {
			switch(lookAndFeel.ActiveStyle) {
				case ActiveLookAndFeelStyle.Style3D:
					return new Style3DScrollBarThumbPainter();
				case ActiveLookAndFeelStyle.UltraFlat:
					return new UltraFlatScrollBarThumbPainter();
				case ActiveLookAndFeelStyle.Office2003:
					if(DevExpress.Utils.WXPaint.WXPPainter.Default.ThemesEnabled) return new XPScrollBarThumbPainter();
					return new FlatScrollBarThumbPainter();
				case ActiveLookAndFeelStyle.WindowsXP:
					return new XPScrollBarThumbPainter();
			} 
			return new FlatScrollBarThumbPainter();
		}
	}
	public enum ScrollBarHitTest {IncButton, DecButton, Thumb, IncArea, DecArea, None}
	public class ScrollBarViewInfo {
		IScrollBar scrollBar;
		ScrollBarState state;
		bool isReady;
		bool isThumbFlashing;
		Rectangle incButtonBounds, decButtonBounds, thumbButtonBounds;
		float thumbTrackValue;
		int oldTrackingPos;
		public ScrollBarViewInfo(IScrollBar scrollBar) {
			this.scrollBar = scrollBar;
			this.state = ScrollBarState.Normal;
			this.isThumbFlashing = false;
			this.thumbTrackValue = 0;
			this.oldTrackingPos = 0;
			Reset();
		}
		public ScrollBarPainterBase Painter { get { return ScrollBar.Painter; } }
		public IScrollBar ScrollBar { get { return this.scrollBar; } }
		public bool IsReady { 
			get	  { return this.isReady;  } 
			set {
				if(value == IsReady) return;
				if(! value) 
					Reset();
				else Calculate(); 
			}
		}
		public Rectangle IncButtonBounds { get { Calculate(); return this.incButtonBounds; } }
		public Rectangle DecButtonBounds { get { Calculate(); return this.decButtonBounds; } }
		public Rectangle ThumbButtonBounds { get { Calculate(); return this.thumbButtonBounds; } }
		public bool Enabled { get { return ScrollBar.GetEnabled() && (VisibleMaximum - Minimum > 0); }}
			public ScrollBarState State {
			get { return this.state; }
			set { this.state = value; }
		}
		public void Calculate() {
			if(IsReady) return;
			CalculateCore();
			this.isReady = true;
		}
		public void Reset() {
			this.isReady = false;
			ResetCore();
		}
		public bool IsRightToLeft { get { return (ScrollBar.RightToLeft == RightToLeft.Yes) && (ScrollBarType == ScrollBarType.Horizontal); } } 
		public bool IsThumbFlashing { get { return this.isThumbFlashing; } set { this.isThumbFlashing = value; } }
		protected void CalculateCore() {
			this.incButtonBounds = CalcIncButtonBounds;
			this.decButtonBounds = CalcDecButtonBounds;
			this.thumbButtonBounds = CalcThumbButtonBounds;
		}
		protected void ResetCore() {
			this.incButtonBounds = Rectangle.Empty;
			this.decButtonBounds = Rectangle.Empty;
			this.thumbButtonBounds = Rectangle.Empty;
		}
		protected Rectangle CalcIncButtonBounds {
			get {
				if(! IsRightToLeft)
					return VisibleIncButtonBounds;
				else return this.incButtonBounds = VisibleDecButtonBounds;
			}
		}
		protected Rectangle CalcDecButtonBounds {
			get {
				if(! IsRightToLeft)
					return VisibleDecButtonBounds;
				else return VisibleIncButtonBounds;
			}
		}
		protected virtual Rectangle CalcThumbButtonBounds {
			get {
				if(!IsThumbVisible || Maximum - Minimum + 1 == 0) return Rectangle.Empty;
				int thumbWidth = ScrollAreaWidth * LargeChange / (Maximum - Minimum + 1);
				if(LargeChange > 1 &&
				   (thumbWidth > ScrollAreaWidth - ScrollAreaWidth / LargeChange) &&
				   LargeChange <= Maximum) 
					thumbWidth = ScrollAreaWidth - ScrollAreaWidth / LargeChange;
				if(thumbWidth < Painter.ThumbMinWidth)
					thumbWidth = Painter.ThumbMinWidth;
				if(thumbWidth > ScrollAreaWidth) thumbWidth = ScrollAreaWidth;
				Int64 startPos;
				if((ScrollBarState.ThumbPressed == state) && (ThumbTrackIntValue == ScrollBar.Value)) {
					float thumbTrackVisibleValue = (!IsRightToLeft) ? this.ThumbTrackIntValue - Minimum : (float)(VisibleMaximum - Minimum) - this.thumbTrackValue;
					startPos = (ScrollAreaWidth - thumbWidth) * (Int64)thumbTrackVisibleValue;
				}
				else {
					startPos = (ScrollAreaWidth - thumbWidth) * (Int64)(VisibleValue - Minimum);
				}
				startPos /=(VisibleMaximum - Minimum);
				startPos = Math.Min(startPos, ScrollAreaWidth - thumbWidth) + ButtonWidth - ButtonCornerSize;
				if(ScrollBarType == ScrollBarType.Horizontal)
					return new Rectangle ((int)startPos, 0, thumbWidth, ScrollBarHeight);
				else return new Rectangle (0, (int)startPos, ScrollBarHeight, thumbWidth);
			}
		}
		public void BeforeThumbTracking(int pos) {
			this.oldTrackingPos = pos;
			this.thumbTrackValue = ScrollBar.Value;
		}
		public int ThumbTrackIntValue { get { return (int)Math.Round(this.thumbTrackValue); } }
		internal int PointToValue(Point p) {
			int pos = 0;
			if(ScrollBarType == ScrollBarType.Horizontal) 
				pos = p.X;
			else 
				pos = p.Y;
			return (int)(Minimum + (float)(pos - ButtonWidth) / ScrollAreaWidth *(Maximum - Minimum));
		}
		public int GetValueByPos(int pos) {
			float oldThumbTrackValue = this.thumbTrackValue;
			if(pos < ButtonWidth)
				this.thumbTrackValue = (!IsRightToLeft) ? Minimum : VisibleMaximum;
			else 
				if(pos > ScrollBarWidth - ButtonWidth)
					this.thumbTrackValue = (!IsRightToLeft) ? VisibleMaximum: Minimum;
			else {
				int thumbWidth;
				if(ScrollBarType == ScrollBarType.Horizontal)
					thumbWidth = ThumbButtonBounds.Width;
				else thumbWidth = ThumbButtonBounds.Height;
				float dif;
				dif = (float)(pos - this.oldTrackingPos) * (float)(VisibleMaximum - Minimum) / (float)(ScrollAreaWidth - thumbWidth);
				if(! IsRightToLeft) 
					this.thumbTrackValue += dif;
				else this.thumbTrackValue -= dif;
			}
			if(this.thumbTrackValue < Minimum)
				this.thumbTrackValue = Minimum;
			if(this.thumbTrackValue > VisibleMaximum)
				this.thumbTrackValue = VisibleMaximum;
			if(oldThumbTrackValue != this.thumbTrackValue)
				this.oldTrackingPos = pos;
			return ThumbTrackIntValue;
		}
		public ScrollBarState GetScrollBarState(Point pt, bool pressed) {
			ScrollBarHitTest hTest = GetHitTest(pt);
			if(pressed && hTest == ScrollBarHitTest.IncButton) 
				return ScrollBarState.IncButtonPressed;
			if(pressed && hTest == ScrollBarHitTest.DecButton) 
				return ScrollBarState.DecButtonPressed;
			if(pressed && hTest == ScrollBarHitTest.IncArea) 
				return ScrollBarState.IncAreaPressed;
			if(pressed && hTest == ScrollBarHitTest.DecArea) 
				return ScrollBarState.DecAreaPressed;
			if(pressed && hTest == ScrollBarHitTest.Thumb) 
				return ScrollBarState.ThumbPressed;
			return ScrollBarState.Normal;
		}
		public virtual ScrollBarHitTest GetHitTest(Point pt) {
			if(IncButtonBounds.IsEmpty)
				return ScrollBarHitTest.None;
			if(IncButtonBounds.Contains(pt))
				return ScrollBarHitTest.IncButton;
			if(DecButtonBounds.Contains(pt))
				return ScrollBarHitTest.DecButton;
			Rectangle thumbR = ThumbButtonBounds;
			if(thumbR.IsEmpty)
				return ScrollBarHitTest.None;
			if(thumbR.Contains(pt)) 
				return ScrollBarHitTest.Thumb;
			if(ScrollBarType == ScrollBarType.Horizontal) {
				if(! IsRightToLeft) {
					if(pt.X < thumbR.X)
						return ScrollBarHitTest.DecArea;
					else return ScrollBarHitTest.IncArea;
				} else {
					if(pt.X > thumbR.Right)
						return ScrollBarHitTest.DecArea;
					else return ScrollBarHitTest.IncArea;
				}
			} else {
				if(pt.Y < thumbR.Y) 
					return ScrollBarHitTest.DecArea;
				else return ScrollBarHitTest.IncArea;
			}
		}
		public void UpdateHotState(Point pt) {
			if(IsNormalOrHotState(State)) {
				ScrollBarState oldState = State;
				ScrollBarHitTest hitTest = GetHitTest(pt);
				this.state = ScrollBarState.Normal;
				if(ScrollBarHitTest.IncButton == hitTest)
					this.state = ScrollBarState.IncButtonHot;
				if(ScrollBarHitTest.DecButton == hitTest)
					this.state = ScrollBarState.DecButtonHot;
				if(ScrollBarHitTest.Thumb == hitTest)
					this.state = ScrollBarState.ThumbHot;
				if(oldState != State) {
					InvalidateByState(oldState);
					InvalidateByState(State);
				}
			}
		}
		public bool IsNormalOrHotState(ScrollBarState state) {
			return (ScrollBarState.Normal == state) || (ScrollBarState.DecButtonHot == state) 
				|| (ScrollBarState.IncButtonHot == state) || (ScrollBarState.ThumbHot == state);
		}
		public int LargeChange { get { return ScrollBar.LargeChange; } }
		public int Maximum { get { return ScrollBar.Maximum; } }
		public int Minimum { get { return ScrollBar.Minimum; } }
		public ScrollBarType ScrollBarType { get { return ScrollBar.ScrollBarType; } } 
		public int VisibleValue { get { return ScrollBar.VisibleValue; } }
		public int VisibleMaximum { get { return Math.Max(Maximum - LargeChange + 1, 1); } }
		public Rectangle IncAreaBounds {
			get {
				return GetAreaBounds(!IsRightToLeft);
			}
		}
		public Rectangle DecAreaBounds {
			get {
				return GetAreaBounds(IsRightToLeft);
			}
		}
		Rectangle GetAreaBounds(bool far) {
			Calculate();
			Rectangle thumbR = ThumbButtonBounds;
			if(thumbR.IsEmpty) return Rectangle.Empty;
			Rectangle res = thumbR;
			if(ScrollBarType == ScrollBarType.Horizontal) {
				if(!far) {
					res.X = DecButtonBounds.Right;
					res.Width = thumbR.X - res.X;
				} else {
					res.X = thumbR.Right;
					res.Width = IncButtonBounds.Left - res.X;
				}
			} else {
				if(!far) {
					res.Y = DecButtonBounds.Bottom;
					res.Height = thumbR.Y - res.Y;
				} else {
					res.Y = thumbR.Bottom;
					res.Height = IncButtonBounds.Y - res.Y;
				}
			}
			if(res.Width == 0 || res.Height == 0) return Rectangle.Empty;
			return res;
		}
		public virtual Rectangle VisibleIncButtonBounds {
			get { 
				if(ButtonWidth == 0)
					return Rectangle.Empty;
				if(ScrollBarType == ScrollBarType.Horizontal)
					return new Rectangle(ScrollBarWidth - ButtonWidth, 0, ButtonWidth, ScrollBarHeight); 
				else return new Rectangle(0, ScrollBarWidth - ButtonWidth, ScrollBarHeight, ButtonWidth); 
			}
		}
		public virtual Rectangle VisibleDecButtonBounds {
			get {
				if(ButtonWidth == 0)
					return Rectangle.Empty;
				if(ScrollBarType == ScrollBarType.Horizontal)
					return new Rectangle(0, 0, ButtonWidth, ScrollBarHeight);
				else return new Rectangle (0, 0, ScrollBarHeight, ButtonWidth);
			}
		}
		public bool IsThumbVisible { get { return Enabled && (Maximum != Minimum) && (ScrollAreaWidth >= Painter.ThumbMinWidth + 2); } }
		private int ScrollBarHeight {
			get {
				if(ScrollBarType == ScrollBarType.Horizontal)
					return ScrollBar.GetHeight();
				else return ScrollBar.GetWidth();
			}
		}
		public int ScrollBarWidth {
			get {
				if(ScrollBarType == ScrollBarType.Horizontal)
					return ScrollBar.GetWidth();
				else return ScrollBar.GetHeight();
			}
		}
		public virtual int ButtonWidth {
			get {
				int buttonWidth;
				if(ScrollBarType == ScrollBarType.Horizontal)
					buttonWidth = SystemInformation.HorizontalScrollBarArrowWidth;
				else buttonWidth = SystemInformation.VerticalScrollBarArrowHeight;
				if(buttonWidth > ScrollBarWidth * 2)
					return 0;
				if(buttonWidth > ScrollBarWidth / 2)
					buttonWidth = ScrollBarWidth / 2;
				return buttonWidth;
			}
		}
		public int ScrollAreaWidth { 
			get { 
				if(ButtonWidth == 0) return 0;
				return (ScrollBarWidth - ButtonWidth * 2) + ButtonCornerSize * 2; 
			}	
		}
		public int ButtonCornerSize { 
			get {
				if(ButtonWidth == 0) return 0;
				return Painter.GetButtonCornerSize(ScrollBarType); 
			} 
		}
		private void InvalidateByState(ScrollBarState state) {
			Rectangle bounds = Rectangle.Empty;
			if((ScrollBarState.DecButtonHot == state) || (ScrollBarState.DecButtonPressed == state))
				bounds = DecButtonBounds;
			if((ScrollBarState.IncAreaPressed == state) || (ScrollBarState.IncButtonHot == state) 
				|| (ScrollBarState.IncButtonPressed == state))
				bounds = IncButtonBounds;
			if((ScrollBarState.DecButtonHot == state) || (ScrollBarState.DecButtonPressed == state))
				bounds = DecButtonBounds;
			if((ScrollBarState.ThumbHot == state) || (ScrollBarState.ThumbPressed == state))
				bounds = ThumbButtonBounds;
			if(! bounds.IsEmpty)
				ScrollBar.Invalidate(bounds);
		}
	}
}
namespace DevExpress.XtraEditors {
	public enum ScrollBarType {Horizontal, Vertical}
	public interface IScrollBar {
		bool GetEnabled();
		RightToLeft RightToLeft { get; }
		int LargeChange { get; }
		int Maximum { get; }
		int Minimum { get; }
		ScrollBarType ScrollBarType { get; }
		int GetWidth();
		int GetHeight();
		ScrollBarPainterBase Painter { get; }
		int VisibleValue { get; }
		int Value { get; }
		void Invalidate(Rectangle rect);
		UserLookAndFeel LookAndFeel { get; }
		ScrollBarViewInfo ViewInfo { get; }
		void ChangeValueBasedByState(ScrollBarState state);
	}
	public abstract class ScrollBarBase : Control, ISupportLookAndFeel, IScrollBar {
		protected internal const int DefaultLargeChangeCore = 10;
		protected internal const int DefaultMaximumCore = 100;
		protected internal const int DefaultMinimumCore = 0;
		protected internal const int DefaultSmallChangeCore = 1;
		const int DefaultFirstScrollInterval = 200;
		const int DefaultSecondScrollInterval = 50;
		ScrollBarPainterBase painter;
		ScrollBarViewInfo viewInfo;
		AppearanceObject style, backStyle;
		UserLookAndFeel lookAndFeel;
		int updateCount;
		int beforeThumbDraggingValue;
		Timer scrollTimer;
		Timer keyBoardFocusTimer;
		bool scrollBarAutoSize;
		public ScrollBarBase() {
			SetStyle(ControlConstants.DoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
			base.TabStop = false;
			this.painter = null;
			this.viewInfo = CreateScrollBarViewInfo(); 
			this.lookAndFeel = new ControlUserLookAndFeel(this);
			this.lookAndFeel.StyleChanged += new EventHandler(OnLookAndFeelStyleChanged);
			SetDefaultScrollBarHeight();
			this.updateCount = 0;
			this.scrollTimer = new Timer();
			this.scrollTimer.Enabled = false;
			this.scrollTimer.Tick += new EventHandler(ScrollTimerEventProcessor);
			this.keyBoardFocusTimer = new Timer();
			this.keyBoardFocusTimer.Enabled = false;
			this.keyBoardFocusTimer.Tick += new EventHandler(KeyBoardTimerEventProcessor);
			this.keyBoardFocusTimer.Interval = 500;
			this.scrollBarAutoSize = false;
			this.style = new AppearanceObject("ScrollBar");
			this.backStyle = new AppearanceObject("ScrollBar Back");
			this.backStyle.BackColor = SystemColors.WindowText;
		}
		[EditorBrowsable(EditorBrowsableState.Never)]
		public DevExpress.Accessibility.BaseAccessible GetAccessible() { return DXAccessible; }
		DevExpress.Accessibility.BaseAccessible dxAccessible;
		protected virtual DevExpress.Accessibility.BaseAccessible DXAccessible { 
			get {
				if(dxAccessible == null) dxAccessible = new DevExpress.Accessibility.ScrollBarAccessible(this);
				return dxAccessible;
			}
		}
		protected virtual ScrollBarViewInfo CreateScrollBarViewInfo() {
			return new ScrollBarViewInfo(this);
		}
		protected override AccessibleObject CreateAccessibilityInstance() {
			return DXAccessible.Accessible;
		}
		protected override void Dispose(bool disposing){
			if(disposing) {
				DelayedTimerKillIfExists();
				if(this.scrollTimer != null) {
					this.scrollTimer.Enabled = false;
					this.scrollTimer.Dispose();
					this.scrollTimer = null;
				}
				if(this.keyBoardFocusTimer != null) {
					this.keyBoardFocusTimer.Enabled = false;
					this.keyBoardFocusTimer.Dispose();
					this.keyBoardFocusTimer = null;
				}
				if(this.lookAndFeel != null) {
					this.lookAndFeel.StyleChanged -= new EventHandler(OnLookAndFeelStyleChanged);
					this.lookAndFeel.Dispose();
					this.lookAndFeel = null;
				}
				if(painter != null) {
					painter.DestroyBrushes();
					painter = null;
				}
			}
			base.Dispose(disposing);
		}
		bool ISupportLookAndFeel.IgnoreChildren { get { return false; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Obsolete("Don't use the AutoSize property, use the ScrollBarAutoSize property instead")]
		public new bool AutoSize { 
			get { return this.ScrollBarAutoSize; }
			set { ScrollBarAutoSize = value; }
		}
		[Description("Gets or sets a value specifying whether the scroll bar is automatically sized."), DefaultValue(false)]
		public bool ScrollBarAutoSize {
			get { return this.scrollBarAutoSize; }
			set {
				if(value == ScrollBarAutoSize) return;
				this.scrollBarAutoSize = value;
				SetDefaultScrollBarHeight();
				OnScrollBarScrollBarAutoSizeChanged(new EventArgs());
			}
		}
		[Description("Gets or sets the increment applied to the ScrollBarBase.Value property when the scroll box is moved by a 'page'."), DefaultValue(DefaultLargeChangeCore)]
		public int LargeChange {
			get { return LargeChangeCore; }
			set {
				if(!IsHandleCreated) {
					LargeChangeCore = value;
					return;
				}
				if((value <= 0) || (value > Maximum - Minimum + 1) || (value == LargeChangeCore)) return;
				LargeChangeCore = value;
				LayoutChanged();
			}
		}
		[Description("Gets or sets the upper limit of values of the scrollable range."), DefaultValue(DefaultMaximumCore)]
		public int Maximum {
			get { return MaximumCore; }
			set {
				if(!IsHandleCreated) {
					MaximumCore = value;
					return;
				}
				if(value == Maximum) return;
				if(value < Minimum) Minimum = value;
				MaximumCore = value;
				if(Maximum < Value)
					Value = Maximum;
				LayoutChanged();
			}
		}
		[Description("Gets or sets the lower limit of the scrollable range."), DefaultValue(DefaultMinimumCore)]
		public int Minimum {
			get {
				return MinimumCore;
			}
			set {
				if(!IsHandleCreated) {
					MinimumCore = value;
					return;
				}
				if(value == Minimum) return;
				if(value > Maximum) Maximum = value;
				MinimumCore = value;
				if(Minimum > Value)
					Value = value;
				LayoutChanged();
			}
		}
		bool ShouldSerializeLookAndFeel() { return LookAndFeel.ShouldSerialize(); }
		[Description("Gets a value providing access to settings controlling the control's look and feel."), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public virtual UserLookAndFeel LookAndFeel { get { return lookAndFeel; } }
		[Browsable(false)]
		public abstract ScrollBarType ScrollBarType {get;}
		[Description("Gets or sets the value by which the ScrollBarBase.Value property changes when the user presses one of the arrow keys or clicks one of the scroll-bar buttons."), DefaultValue(DefaultSmallChangeCore)]
		public int SmallChange {
			get { return SmallChangeCore; }
			set {
				if(!IsHandleCreated) {
					SmallChangeCore = value;
					return;
				}
				if((value <= 0) && (value >= Maximum - Minimum)) return;
				SmallChangeCore = value;
			}
		}
		[Description("Gets or sets a value specifying the scroll box's current position."), DefaultValue(DefaultMinimumCore)]
		public int Value {
			get { return ValueCore; }
			set {
				if(!IsHandleCreated) {
					ValueCore = value;
					return;
				}
				if(value > Maximum) value = Maximum;
				if(value < Minimum) value = Minimum;
				if((value < Minimum) || (value > Maximum) || (value == Value)) return;
				int oldValue = Value;
				ValueCore = value;
				LayoutChanged();
				if(oldValue != Value)
					OnValueChanged(EventArgs.Empty);
			}
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color BackColor { get { return base.BackColor; } set { base.BackColor = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Image BackgroundImage { get { return base.BackgroundImage; } set { base.BackgroundImage = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Font Font { get { return base.Font; } set { base.Font = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color ForeColor { get { return base.ForeColor; } set { base.ForeColor = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new ImeMode ImeMode { get { return base.ImeMode; } set { base.ImeMode = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Text { get { return base.Text; } set { base.Text = value; } }
		[Description("Gets or sets a value indicating whether a user can focus the scroll bar control using the TAB key."), DefaultValue(false)]
		public new bool TabStop { get { return base.TabStop; } set { base.TabStop = value; } }
		public void BeginUpdate() { updateCount ++; }
		public void CancelUpdate() {
			updateCount --;
			if(updateCount < 0)
				updateCount = 0;
		}
		public void EndUpdate() {
			CancelUpdate();
			LayoutChanged(true);
		}
		private static readonly object scroll = new object();
		private static readonly object valueChanged = new object();
		private static readonly object scrollBarScrollBarAutoSizeChanged = new object();
		[Description("Fires when the scroll thumb has been moved either by a mouse or keyboard action.")]
		public event ScrollEventHandler Scroll {
			add { Events.AddHandler(scroll, value); }
			remove { Events.RemoveHandler(scroll, value); }
		}
		[Description("Fires immediately after the ScrollBarBase.Value property has been changed.")]
		public event EventHandler ValueChanged {
			add { Events.AddHandler(valueChanged, value); }
			remove { Events.RemoveHandler(valueChanged, value); }
		}
		[Description("Fires after the ScrollBarBase.ScrollBarAutoSize property's value has been changed.")]
		public event EventHandler ScrollBarAutoSizeChanged {
			add { Events.AddHandler(scrollBarScrollBarAutoSizeChanged, value); }
			remove { Events.RemoveHandler(scrollBarScrollBarAutoSizeChanged, value); }
		}
		protected override void OnGotFocus(EventArgs e) {
			KeyBoardTimerEnabled = true;
			base.OnGotFocus(e);
		}
		protected override void OnLostFocus(EventArgs e) {
			KeyBoardTimerEnabled = false;
			base.OnLostFocus(e);
		}
		protected override void OnHandleCreated(EventArgs e) {
			myHandle = this.Handle;
			base.OnHandleCreated(e);
		}
		protected override void OnHandleDestroyed(EventArgs e) {
			myHandle = IntPtr.Zero;
			base.OnHandleDestroyed(e);
		}
		protected override void OnEnabledChanged(EventArgs e) {
			base.OnEnabledChanged(e);
			LayoutChanged();
		}
		protected internal ScrollBarPainterBase Painter { 
			get { 
				if(painter == null)
					painter = CreateScrollBarPainter();
				return painter; 
			} 
		}
		protected override void OnPaint(PaintEventArgs e) {
			GraphicsCache cache = new GraphicsCache(e.Graphics);
			ScrollBarInfoArgs args = new ScrollBarInfoArgs(cache, ClientRectangle, this.style, this.backStyle, this);
			args.State = (Enabled ? ObjectState.Normal : ObjectState.Disabled);
			Painter.DrawObject(args);
		}
		protected override bool IsInputKey(Keys keyData) {
			return (keyData == Keys.Up) || (keyData == Keys.Left) || (keyData == Keys.Down)
				|| (keyData == Keys.Right) || (keyData == Keys.PageUp) || (keyData == Keys.PageDown)
				|| (keyData == Keys.Home) || (keyData == Keys.End);
		}
		protected override void OnKeyDown(KeyEventArgs e) {
			base.OnKeyDown(e);
			if (e.Handled) return;
			if((e.KeyData == Keys.Up) || (e.KeyData == Keys.Left))
				((IScrollBar)this).ChangeValueBasedByState(ScrollBarState.DecButtonPressed);
			if((e.KeyData == Keys.Down) || (e.KeyData == Keys.Right))
				((IScrollBar)this).ChangeValueBasedByState(ScrollBarState.IncButtonPressed);
			if(e.KeyData == Keys.PageUp)
				((IScrollBar)this).ChangeValueBasedByState(ScrollBarState.DecAreaPressed);
			if(e.KeyData == Keys.PageDown)
				((IScrollBar)this).ChangeValueBasedByState(ScrollBarState.IncAreaPressed);
			if(e.KeyData == Keys.Home)
				Value = this.Minimum;
			if(e.KeyData == Keys.End)
				Value = ViewInfo.VisibleMaximum;
		}
		protected override void OnMouseDown(MouseEventArgs e) {
			base.OnMouseDown(e);
			if(e.Button != MouseButtons.Left) return;
			if(TabStop)
				Focus();
			State = ViewInfo.GetScrollBarState(new Point(e.X, e.Y), true);
			if(State == ScrollBarState.ThumbPressed) {
				this.beforeThumbDraggingValue = Value;
				ViewInfo.BeforeThumbTracking((ScrollBarType.Horizontal == ScrollBarType) ? e.X : e.Y);
			}
			if(Control.ModifierKeys == Keys.Shift)
				ChangeValueBasedByStateOnShiftPressed(State, new Point(e.X, e.Y));
			else
				((IScrollBar)this).ChangeValueBasedByState(State);
			this.scrollTimer.Enabled = IsScrollingState(State);
			if(this.scrollTimer.Enabled)
				this.scrollTimer.Interval = DefaultFirstScrollInterval;
		}
		protected override void OnMouseMove(MouseEventArgs e) {
			base.OnMouseMove(e);
			if(State == ScrollBarState.ThumbPressed) {
				int pos = (ScrollBarType.Horizontal == ScrollBarType) ? e.X: e.Y;
				if((pos < - 3 * ViewInfo.ButtonWidth) || (pos - ViewInfo.ScrollBarWidth > 2 * ViewInfo.ButtonWidth))
					SetScrollBarValue(ScrollEventType.ThumbTrack, this.beforeThumbDraggingValue);
				else { 
					int iPos = ViewInfo.GetValueByPos(pos);
					SetScrollBarValue(ScrollEventType.ThumbTrack, iPos);
					LayoutChanged();
				}
			}
			ViewInfo.UpdateHotState(new Point(e.X, e.Y));
		}
		protected override void OnMouseUp(MouseEventArgs e) {
			this.scrollTimer.Enabled = false;
			SetNormalState();
			ViewInfo.UpdateHotState(new Point(e.X, e.Y));
			base.OnMouseUp(e);
		}
		protected virtual void SetNormalState() {
			if(State == ScrollBarState.Normal) return;
				if(IsScrollingState(State) || (State == ScrollBarState.ThumbPressed))  { 
					if(State == ScrollBarState.ThumbPressed)
						SetScrollBarValue(ScrollEventType.ThumbPosition, Value);
					State = ScrollBarState.Normal; 
					SetScrollBarValue(ScrollEventType.EndScroll, Value);
				}
				State = ScrollBarState.Normal;
				LayoutChanged();
			}
		const int WM_CAPTURECHANGED = 0x215;
		protected override void WndProc ( ref System.Windows.Forms.Message m ) {
			if(m.Msg == WM_CAPTURECHANGED) {
				if(m.LParam == this.Handle) 
					OnGotCapture();
				else
					OnLostCapture();
			}
			base.WndProc(ref m);
			DoDelayedUpdateMessageProcessing();
		}
		protected virtual void OnLostCapture() {
			SetNormalState();
		}
		protected virtual void OnGotCapture() { }
		protected override void OnMouseLeave(EventArgs e) {
			ViewInfo.UpdateHotState(new Point(-10000, -10000));
			base.OnMouseLeave(e);
		}
		protected override void OnResize(EventArgs e) {
			base.OnResize(e);
			if(ScrollBarAutoSize) 
				SetDefaultScrollBarHeight();
			LayoutChanged(true);
		}
		protected override void OnRightToLeftChanged(EventArgs e) {
			base.OnRightToLeftChanged(e);
			LayoutChanged();
		}
		internal protected ScrollBarViewInfo ViewInfo { get { return this.viewInfo; } }
		protected abstract int LargeChangeCore { get; set; }
		protected abstract int MaximumCore { get; set; }
		protected abstract int MinimumCore { get; set; }
		protected abstract int SmallChangeCore { get; set; }
		protected abstract int ValueCore { get; set; }
		protected virtual ScrollBarPainterBase CreateScrollBarPainter() {
			return LookAndFeelScrollBarPainter.GetPainter(LookAndFeel);
		}
		protected virtual void OnScrollBarScrollBarAutoSizeChanged(EventArgs e) {
			EventHandler handler = (EventHandler)this.Events[scrollBarScrollBarAutoSizeChanged];
			if(handler != null) handler(this, e);
		}
		protected virtual void OnScroll(ScrollEventArgs e) {
			if(lockEvents) return;
			ScrollEventHandler handler = (ScrollEventHandler)this.Events[scroll];
			if(handler != null) handler(this, e);
		}
		protected virtual void OnValueChanged(EventArgs e) {
			if(lockEvents) return;
			EventHandler handler = (EventHandler)this.Events[valueChanged];
			if(handler != null) handler(this, e);
			AccessibilityNotifyClients(AccessibleEvents.ValueChange, -1);
		}
		protected ScrollBarState State {
			get { return ViewInfo.State; }
			set {
				if(value == State) return;
				ViewInfo.State = value;
				KeyBoardTimerEnabled = ViewInfo.IsNormalOrHotState(value) && Focused;
			}
		}
		protected internal int VisibleValue {
			get {
				if(ViewInfo.IsRightToLeft)
					return (ViewInfo.VisibleMaximum - Minimum) - Value;
				else return Value;
			}
			set {
				if(ViewInfo.IsRightToLeft)
					Value = (ViewInfo.VisibleMaximum - Minimum) - value;
				else Value = value;
			}
		}
		private bool IsScrollingState(ScrollBarState state) {
			return (state == ScrollBarState.IncButtonPressed) || 
				(state == ScrollBarState.DecButtonPressed) ||
				(state == ScrollBarState.IncAreaPressed) || 
				(state == ScrollBarState.DecAreaPressed);
		}
		private void SetScrollBarValue(ScrollEventType scrollType, int value) {
			value = GetScrollBarValue(value);
			ScrollEventArgs e = new ScrollEventArgs(scrollType, value);
			OnScroll(e);
			Value = e.NewValue;
		}
		private int GetScrollBarValue(int value) {
			if(value < Minimum) return Minimum;
			if(value > ViewInfo.VisibleMaximum) return ViewInfo.VisibleMaximum;
			return value;
		}
		void IScrollBar.ChangeValueBasedByState(ScrollBarState state) {
			int newValue = Value;
			ScrollEventType scrollType = ScrollEventType.EndScroll;
			switch(state) {
				case ScrollBarState.IncButtonPressed:
					newValue += SmallChange; 
					scrollType = ScrollEventType.SmallIncrement;
					break;
				case ScrollBarState.DecButtonPressed:
					newValue -= SmallChange; 
					scrollType = ScrollEventType.SmallDecrement;
					break;
				case ScrollBarState.IncAreaPressed:
					newValue += LargeChange; 
					scrollType = ScrollEventType.LargeIncrement;
					break;
				case ScrollBarState.DecAreaPressed:
					newValue -= LargeChange; 
					scrollType = ScrollEventType.LargeDecrement;
					break;
			}
			if(scrollType != ScrollEventType.EndScroll)
				SetScrollBarValue(scrollType, newValue);
		}
		protected virtual void ChangeValueBasedByStateOnShiftPressed(ScrollBarState state, Point point) {
			if(state == ScrollBarState.DecAreaPressed || state == ScrollBarState.IncAreaPressed) {
				int newValue = ViewInfo.PointToValue(point);
				SetScrollBarValue(ScrollEventType.ThumbPosition, newValue);
			}
		}
		private void ScrollTimerEventProcessor(Object item, EventArgs e) {
			this.scrollTimer.Interval = DefaultSecondScrollInterval;
			ScrollBarState state = GetScrollBarState();
			if(state == State)
				((IScrollBar)this).ChangeValueBasedByState(state);
		}
		protected virtual ScrollBarState GetScrollBarState(){
			return ViewInfo.GetScrollBarState(PointToClient(MousePosition), true);
		}
		private bool KeyBoardTimerEnabled {
			get { return this.keyBoardFocusTimer.Enabled; }
			set {
				if (keyBoardFocusTimer == null) return;
				keyBoardFocusTimer.Enabled = value;
				if(! value) {
					bool oldIsThumbFlashing = ViewInfo.IsThumbFlashing;
					ViewInfo.IsThumbFlashing = false;
					if(oldIsThumbFlashing)
						Invalidate(ViewInfo.ThumbButtonBounds);
				}
			}
		}
		private void KeyBoardTimerEventProcessor(Object item, EventArgs e) {
			if(ViewInfo.IsThumbVisible) {
				ViewInfo.IsThumbFlashing = ! ViewInfo.IsThumbFlashing;
				Invalidate(ViewInfo.ThumbButtonBounds);
			}
		}
		void LayoutChanged() {
			LayoutChanged(false);
		}
		private void LayoutChanged(bool allowDelayed) {
			if(updateCount == 0) {
				ViewInfo.Reset();
				Invalidate();
				if(IsHandleCreated) {
					if(allowDelayed)
						DelayedUpdate();
					else
						Update();
				}
			}
		}
		private void SetDefaultScrollBarHeight() {
			if(ScrollBarType == ScrollBarType.Horizontal)
				Height = SystemInformation.HorizontalScrollBarHeight;
			else Width = SystemInformation.VerticalScrollBarWidth;
		}
		private void OnLookAndFeelStyleChanged(object sender, System.EventArgs e) {
			if(painter != null) painter.DestroyBrushes();
			painter = CreateScrollBarPainter();
			ViewInfo.Reset();
			Invalidate();
		}
		ScrollBarPainterBase IScrollBar.Painter { get { return this.Painter; } }
		int IScrollBar.VisibleValue { get { return this.VisibleValue; } }
		ScrollBarViewInfo IScrollBar.ViewInfo { get { return this.ViewInfo; } }
		#region IScrollBar Members
		public bool GetEnabled() {
			return Enabled;
		}
		public int GetWidth() {
			return Width;
		}
		public int GetHeight() {
			return Height;
		}
		#endregion
		bool lockEvents = false;
		internal void Assign(ScrollArgs scrollArgs) {
			BeginUpdate();
			try {
				this.lockEvents = !scrollArgs.FireEventsOnAssign;
				Enabled = scrollArgs.Enabled;
				MinimumCore = scrollArgs.Minimum;
				MaximumCore = scrollArgs.Maximum;
				LargeChangeCore = scrollArgs.LargeChange;
				SmallChangeCore = scrollArgs.SmallChange;
				Value = Math.Min(Maximum, scrollArgs.Value);
			}
			finally {
				this.lockEvents = false;
				EndUpdate();
			}
		}
		System.Threading.Timer delayedTimer;
		bool delayedUpdateFlag;
		IntPtr myHandle;
		const int DelayedInterval = 100;
		void DelayedUpdate() {
			delayedUpdateFlag = false;
			if(this.delayedTimer != null) {
				this.delayedTimer.Change(DelayedInterval, -1);
			} else {
				this.delayedTimer = new System.Threading.Timer(delayedTimerHandler, null, DelayedInterval, -1);
			}
		}
		void delayedTimerHandler(object state) {
			try {
				IntPtr _myHandle = myHandle;
				if(_myHandle == IntPtr.Zero)
					return;
				if(!InvokeRequired)
					return;
				delayedUpdateFlag = true;
				DevExpress.Utils.Drawing.Helpers.NativeMethods.PostMessage(_myHandle, 0, IntPtr.Zero, IntPtr.Zero);
			} catch { }
		}
		static void OnDelayedUpdateStatic(object thisObjectWeakReference) {
			WeakReference weakRef = thisObjectWeakReference as WeakReference;
			if (weakRef == null)
				return;
			if (!weakRef.IsAlive)
				return;
			ScrollBarBase thisObject = weakRef.Target as ScrollBarBase;
			if (thisObject != null)
				thisObject.OnDelayedUpdate();
		}
		void OnDelayedUpdate() {
			if(delayedTimer == null)
				return;
			DelayedTimerKillIfExists();
			if(IsHandleCreated)
				Update();
			delayedUpdateFlag = false;
		}
		void DelayedTimerKillIfExists() {
			if(this.delayedTimer != null) {
				this.delayedTimer.Dispose();
				this.delayedTimer = null;
			}
		}
		void DoDelayedUpdateMessageProcessing() {
			if(delayedUpdateFlag) {
				delayedUpdateFlag = false;
				MethodInvoker call = delegate() { OnDelayedUpdateStatic(new WeakReference(this)); };
				BeginInvoke(call);
			}
		}
	}
	public abstract class  HScrollBarBase: ScrollBarBase {
		[Description("Gets the value indicating whether the scroll bar is horizontal or vertical.")]
		public override ScrollBarType ScrollBarType { get { return ScrollBarType.Horizontal; } }
	}
	public abstract class  VScrollBarBase: ScrollBarBase {
		[Description("Gets the value indicating whether the scroll bar is horizontal or vertical.")]
		public override ScrollBarType ScrollBarType { get { return ScrollBarType.Vertical; } }
	}
	[Description("Allows you to implement horizontal scrolling in containers that do not provide their own scroll bars."),
	 ToolboxTabName(AssemblyInfo.DXTabNavigation),
	  ToolboxBitmap(typeof(RootNS), "HScrollBar")
	]
	public class  HScrollBar: HScrollBarBase {
		int largeChangeCore, maximumCore, minimumCore, smallChangeCore, valueCore;
		public HScrollBar() {
			largeChangeCore = DefaultLargeChangeCore;
			maximumCore = DefaultMaximumCore;
			minimumCore = DefaultMinimumCore;
			smallChangeCore = DefaultSmallChangeCore;
			valueCore = minimumCore;
		}
		protected override int LargeChangeCore { get { return largeChangeCore; } set { largeChangeCore = value; } }
		protected override int MaximumCore { get { return maximumCore; } set { maximumCore = value; } }
		protected override int MinimumCore { get { return minimumCore; } set { minimumCore = value; } }
		protected override int SmallChangeCore { get { return smallChangeCore; } set { smallChangeCore = value; } }
		protected override int ValueCore { get { return valueCore; } set { valueCore = value; } }
		protected override Size DefaultSize { get { return new Size(80, 17); } }
	}
	[Description("Allows you to implement vertical scrolling in containers that do not provide their own scroll bars."),
	 ToolboxTabName(AssemblyInfo.DXTabNavigation),
	  ToolboxBitmap(typeof(RootNS), "VScrollBar")
	]
	public class  VScrollBar: VScrollBarBase {
		int largeChangeCore, maximumCore, minimumCore, smallChangeCore, valueCore;
		public VScrollBar() {
			largeChangeCore = DefaultLargeChangeCore;
			maximumCore = DefaultMaximumCore;
			minimumCore = DefaultMinimumCore;
			smallChangeCore = DefaultSmallChangeCore;
			valueCore = minimumCore;
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override RightToLeft RightToLeft { get { return base.RightToLeft; } set { base.RightToLeft = value; } }
		protected override int LargeChangeCore { get { return largeChangeCore; } set { largeChangeCore = value; } }
		protected override int MaximumCore { get { return maximumCore; } set { maximumCore = value; } }
		protected override int MinimumCore { get { return minimumCore; } set { minimumCore = value; } }
		protected override int SmallChangeCore { get { return smallChangeCore; } set { smallChangeCore = value; } }
		protected override int ValueCore { get { return valueCore; } set { valueCore = value; } }
		protected override Size DefaultSize { get { return new Size(17, 80); } }
	}
	public class ScrollArgs {
		int _largeChange, _smallChange, fValue, _maximum, _minimum;
		bool fEnabled, fireEventsOnAssign = true;
		public ScrollArgs(DevExpress.XtraEditors.ScrollBarBase scroll) {
			this.fEnabled = scroll.Enabled;
			this._maximum = scroll.Maximum;
			this._minimum = scroll.Minimum;
			this._largeChange = scroll.LargeChange;
			this._smallChange = scroll.SmallChange;
			this.fValue = scroll.Value;
		}
		public ScrollArgs() {
			this._largeChange = 1;
			this._smallChange = 1;
			this.fValue = 0;
			this._maximum = 0;
			this._minimum = 0;
			this.fEnabled = true;
		}
		public bool FireEventsOnAssign {
			get { return fireEventsOnAssign; }
			set { fireEventsOnAssign = value; }
		}
		public virtual bool IsEquals(ScrollArgs args) {
			if(this.Enabled == args.Enabled && this.LargeChange == args.LargeChange && 
				this.SmallChange == args.SmallChange && this.Maximum == args.Maximum && 
				this.Value == args.Value && this.Minimum == args.Minimum) return true;
			return false;
		}
		public int LargeChange { 
			get { return _largeChange; } 
			set { 
				if(value < 0) value = 0;
				_largeChange = value; 
			} 
		}
		public int SmallChange { 
			get { return _smallChange; } 
			set { 
				if(value < 0) value = 0;
				_smallChange = value; 
			} 
		}
		public int Value { get { return fValue; } set { fValue = value; } }
		public int Minimum { 
			get { return _minimum; } 
			set { 
				_minimum = value; 
			} 
		}
		public int Maximum { 
			get { return _maximum; } 
			set { 
				_maximum = value; 
			} 
		}
		public bool Enabled { get { return fEnabled; } set { fEnabled = value; } }
		public virtual void Check() {
			if(!Enabled) {
				Minimum = LargeChange = SmallChange = Value = Maximum = 0;
			} else {
				if(Value < Minimum) Value = Minimum;
				if(Maximum < Minimum) Maximum = Minimum;
				if(Value > Maximum) Value = Maximum;
				if(Value + LargeChange > Maximum)
					Value = Maximum - LargeChange;
			}
		}
		public virtual void AssignTo(DevExpress.XtraEditors.ScrollBarBase scroll) {
			scroll.Assign(this);
		}
	}
}
