#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using DevExpress.LookAndFeel;
using DevExpress.Utils;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Reflection;
using System.Resources;
using System.Windows.Forms;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Xml;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using DevExpress.Skins.Info;
using DevExpress.Utils.Drawing;
namespace DevExpress.Skins {
	public static class SkinLoader {
		readonly static string SkinAssemblyFormat = "DevExpress.{0}" + AssemblyInfo.VSuffix;
		readonly static string UserSkinsNamespace = "DevExpress.UserSkins.";
		readonly static string BonusSkins = "BonusSkins";
		readonly static string OfficeSkins = "OfficeSkins";
		public enum DevExpressSkins { Bonus, Office };
		public static bool RegisterSkins(DevExpressSkins skins) {
			string skinsName = (skins == DevExpressSkins.Bonus) ? BonusSkins : OfficeSkins;
			string asmName = String.Format(SkinAssemblyFormat, skinsName);
			Assembly skinAssembly = GetAssembly(asmName);
			if(skinAssembly == null) {
				skinAssembly = AppDomain.CurrentDomain.Load(new AssemblyName(asmName));
			}
			return InvokeRegister(skinAssembly, UserSkinsNamespace + skinsName);
		}
		static bool InvokeRegister(Assembly skinAssembly, string typeName) {
			Type skinsType = skinAssembly.GetType(typeName);
			if(skinsType != null) {
				MethodInfo mInfo = skinsType.GetMethod("Register");
				if(mInfo != null) {
					mInfo.Invoke(null, new object[] { });
					return true;
				}
			}
			return false;
		}
		static Assembly GetAssembly(string asmName) {
			return Array.Find(
				AppDomain.CurrentDomain.GetAssemblies(),
				delegate(Assembly assembly) {
					return assembly.GetName().Name == asmName;
				}
			);
		}
	}
	public interface ISkinProvider {
		string SkinName { get; }
	}
	public enum SkinProductId { NavBar, Common, Editors, Tab, Grid, Bars, Docking, VGrid, Scheduler, NavPane, Form, Reports, Ribbon, Printing, RichEdit, Chart, Gauges } 
	public class SkinContainer {
		bool loaded;
		Hashtable products;
		string skinName;
		SkinCreator creator;
		internal bool embedded;
		public SkinContainer(string skinName) {
			this.skinName = skinName;
			this.loaded = true;
		}
		public SkinContainer(SkinCreator creator) {
			this.creator = creator;
			this.skinName = creator.SkinName;
		}
		public SkinContainer Clone(string skinName) {
			return Clone(skinName, true);
		}
		public SkinContainer Clone(string skinName, bool register) {
			SkinContainer container = null;
			if(!register || Creator == null) {
				if(register) SkinManager.Default.Skins.Remove(skinName);
				container = new SkinContainer(skinName);
				foreach(SkinProductId product in Enum.GetValues(typeof(SkinProductId))) {
					Skin skin = GetSkin(product);
					if(skin != null) {
						SkinBuilder builder = new SkinBuilder(skinName);
						builder.skin = skin.Clone(skinName) as Skin;
						container.RegisterSkin(product, builder);
						if(register) SkinManager.Default.Skins.Add(container);
					}
				}
				return container;
			}
			container = new SkinContainer(Creator.Clone(skinName));
			if(SkinManager.Default.Skins[skinName] == null)
				SkinManager.Default.Skins.Add(container);
			container.Load();
			return container;
		}
		public bool IsEmbedded { get { return embedded; } }
		public IDictionaryEnumerator GetDictionaryEnumerator() { 
			foreach(DictionaryEntry entry in Products) {
				GetSkin(entry.Key);
			}
			return Products.GetEnumerator();
		}
		public Skin[] GetSkins() {
			ArrayList list = new ArrayList();
			foreach(SkinProductId product in Enum.GetValues(typeof(SkinProductId))) {
				Skin skin = GetSkin(product);
				if(skin != null) list.Add(skin);
			}
			return list.ToArray(typeof(Skin)) as Skin[];
		}
		protected SkinCreator Creator { get { return creator; } }
		public bool Loaded { get { return loaded; } }
		protected void SetLoaded() { this.loaded = true; } 
		public string SkinName { 
			get { return skinName; }
		}
		protected internal void SetSkinName(string name) {
			this.skinName = name;
			if(Loaded) {
				foreach(Skin skin in GetSkins()) skin.SetSkinName(name);
			}
		}
		class ProductsKeyComparer : IEqualityComparer {
			bool IEqualityComparer.Equals(object x, object y) {
				return x.Equals(y);
			}
			int IEqualityComparer.GetHashCode(object obj) {
				if(obj is SkinProductId) {
					return (int)(SkinProductId)obj;
				} else
					return obj.GetHashCode();
			}
		}
		protected Hashtable Products {
			get {
				if(products == null) products = new Hashtable(new ProductsKeyComparer());
				return products;
			}
		}
		public Skin GetSkin(object productId) {
			Load();
			if(this.products == null) return null;
			return Products[productId] as Skin;
		}
		public Skin CommonSkin {
			get {
				return GetSkin(SkinProductId.Common);
			}
		}
		public Skin RegisterSkin(object productId, SkinBuilder builder) {
			if(productId == null || builder == null || builder.Skin.Name != SkinName) return null;
			Load();
			Products[productId] = builder.Skin;
			builder.Skin.SetContainer(this);
			return builder.Skin;
		}
		internal void SetCreator(SkinCreator creator) {
			this.loaded = false;
			this.creator = creator;
		}
		public virtual void Load() {
			if(Loaded) return;
			SetLoaded();
			if(Creator == null) return;
			Creator.Load();
		}
	}
	public class SkinContainerCollection : CollectionBase {
		Hashtable nameHash;
		protected Hashtable NameHash { 
			get {
				if(nameHash == null) nameHash = new Hashtable();
				return nameHash;
			}
		}
		protected internal void Add(SkinContainer skinContainer) {
			if(this[skinContainer.SkinName] != null) return;
			List.Add(skinContainer);
		}
		protected internal void Remove(string skinName) {
			SkinContainer container = this[skinName];
			if(container != null) List.Remove(container);
		}
		public void Remove(SkinContainer container) {
			if(container == null || container.IsEmbedded) return;
			List.Remove(container);
		}
		public SkinContainer this[int index] { get { return (SkinContainer)List[index]; } }
		public SkinContainer this[string name] { get { return Count == 0 ? null : (SkinContainer)NameHash[name]; } }
		protected override void OnInsertComplete(int position, object item) {
			SkinContainer sc = (SkinContainer)item;
			NameHash[sc.SkinName] = sc;
		}
		protected override void OnRemoveComplete(int position, object item) {
			SkinContainer sc = (SkinContainer)item;
			NameHash[sc.SkinName] = null;
		}
		protected override void OnClear() {
			NameHash.Clear();
			InnerList.Clear();
		}
	}
	public class SkinRegistrator {
		public const string DesignTimeSkinName = "DevExpress Style";
		public const string DefaultSkinName = "DevExpress Style";
		public const string DefaultDemoSkin = "DevExpress Style";
		public static void Register() {
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("DevExpress Style", "DevExpress.Utils.SkinData."));
/*
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("Caramel", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("Money Twins", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("Lilian", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("DevExpress Dark Style", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("iMaginary", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("Black", "DevExpress.Utils.SkinData."));
			SkinManager.Default.RegisterEmbeddedSkin(new SkinBlobXmlCreator("Blue", "DevExpress.Utils.SkinData."));
*/
		}
	}
	public class SkinManagerHelper {
		public static void RenameContainer(string oldName, string newName) {
			SkinContainer container = SkinManager.Default.Skins[oldName];
			if(container == null || container.IsEmbedded) return;
			SkinManager.Default.Skins.Remove(container);
			container.SetSkinName(newName);
			SkinManager.Default.Skins.Add(container);
		}
	}
	public class SkinManager {
		public static readonly string DefaultSkinName = SkinRegistrator.DefaultSkinName;
		static bool allowFormSkins, allowArrowDragIndicators = true, allowWindowGhosting = false;
		[Description("Gets whether the form title bar skinning feature is enabled.")]
		public static bool AllowFormSkins { get { return allowFormSkins; } }
		public static bool AllowWindowGhosting { get { return allowWindowGhosting; } set { allowWindowGhosting = value; } }
		public static void EnableFormSkinsIfNotVista() {
			if(DevExpress.Utils.Drawing.Helpers.NativeVista.IsVista) return;
			EnableFormSkins();
		}
		public static void EnableFormSkins() {
			allowFormSkins = true;
			if(!AllowWindowGhosting) DevExpress.Utils.Drawing.Helpers.NativeVista.DisableWindowGhosting();
		}
		public static void DisableFormSkins() {
			allowFormSkins = false;
		}
		static bool allowMdiFormSkins;
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public static bool AllowMdiFormSkins { get { return allowMdiFormSkins; } }
		public static void EnableMdiFormSkins() {
			allowMdiFormSkins = true;
		}
		public static bool AllowArrowDragIndicators {
			get { return allowArrowDragIndicators; }
			set { allowArrowDragIndicators = value; }
		}
		public static void DisableMdiFormSkins() {
			allowMdiFormSkins = false;
		}
		[ThreadStatic]
		static SkinManager defaultManager;
		[Description("Provides access to the default skin manager.")]
		public static SkinManager Default {
			get {
				if(defaultManager == null) {
					defaultManager = new SkinManager();
					SkinRegistrator.Register();
				}
				return defaultManager;
			}
		}
		SkinContainerCollection skins;
		public SkinManager() { 	
		}
		public bool RegisterAssembly(Assembly asm) {
			if(!IsSkinAssembly(asm)) return false;
			bool res = false;
			if(asm.GetName().Name == "DevExpress.BonusSkins" + AssemblyInfo.VSuffix) {
				Type type = asm.GetType("DevExpress.UserSkins.BonusSkins");
				if(type != null) {
					MethodInfo mi = type.GetMethod("Register", BindingFlags.Public | BindingFlags.Static);
					if(mi != null) mi.Invoke(null, null);
					return true;
				}
			}
			if(asm.GetName().Name == "DevExpress.OfficeSkins" + AssemblyInfo.VSuffix) {
				Type type = asm.GetType("DevExpress.UserSkins.OfficeSkins");
				if(type != null) {
					MethodInfo mi = type.GetMethod("Register", BindingFlags.Public | BindingFlags.Static);
					if(mi != null) mi.Invoke(null, null);
					return true;
				}
			}
			res |= RegisterBySkinInfo(asm, "SkinInfo", false);
			res |= RegisterBySkinInfo(asm, "SkinInfoBlob", true);
			return res;
		}
		internal bool RegisterBySkinInfo(Assembly asm, string resourceName, bool isBlob) {
			bool res = false;
			ResourceManager rm = new ResourceManager(resourceName, asm);
			if(rm == null) return false;
			for(int n = 1; ; n++) {
				object infoRes = null;
				try {
					infoRes = rm.GetObject(resourceName + n.ToString());
				if(infoRes == null) break;
				}
				catch {
					break;
				}
				string[] info = infoRes as string[];
				if(info == null) info = infoRes.ToString().Split(';');
				if(info == null) break;
				if(isBlob) {
					RegisterSkin(new SkinBlobXmlCreator(info[0], info[1], asm, null));
				}
				else {
					RegisterSkin(new SkinEmbeddedXmlCreator(info[0], info[1], asm));
				}
				res = true;
			}
			return res;
		}
		internal bool IsSkinAssembly(Assembly asm) {
			return Compare(asm.GetName().GetPublicKeyToken(), SkinAssemblyHelper.PublicKeyToken);
		}
		bool registrationProcessed;
		internal void RegisterDTUserSkins(IComponent comp) {
			if(comp == null || comp.Site == null || !comp.Site.DesignMode) return;
			RegisterDTUserSkins(comp.Site.GetService(typeof(IDesignerHost)) as IDesignerHost);
		}
		internal void RegisterDTUserSkins(IDesignerHost host) {
			if(host == null) return;
			if(host != null && !this.registrationProcessed) {
				try {
					if(host != null) host.GetType(host.RootComponentClassName);
				}
				catch {
				}
			}
			RegisterDTUserSkins();
		}
		bool Compare(byte[] b1, byte[] b2) {
			if(b1 == null || b2 == null) return false;
			if(b1.Length != b2.Length) return false;
			for(int n = 0; n < b1.Length; n++) {
				if(b1[n] != b2[n]) return false;
			}
			return true;
		}
		[Obsolete("You should use the RegisterAssembly method instead of that")]
		public bool RegisterSkinAssembly(Assembly asm) {
			object[] obj = asm.GetCustomAttributes(typeof(SkinAssemblyAttribute), true);
			if(obj == null || obj.Length == 0) return false;
			if(!IsSkinAssembly(asm)) return false;
			Type[] types = asm.GetTypes();
			foreach(Type type in types) {
				MethodInfo[] ms = type.GetMethods(BindingFlags.Public | BindingFlags.Static);
				foreach(MethodInfo mi in ms) {
					mi.Invoke(null, null);
				}
			}
			return true;
		}
		public bool RegisterSkins(Assembly asm) {
			object[] obj = asm.GetCustomAttributes(typeof(SkinAssemblyAttribute), true);
			if(obj == null || obj.Length == 0) return false;
			if(!IsSkinAssembly(asm)) return false;
			Type[] types = asm.GetTypes();
			foreach(Type type in types) {
				MethodInfo[] ms = type.GetMethods(BindingFlags.Public | BindingFlags.Static);
				foreach(MethodInfo mi in ms) {
					mi.Invoke(null, null);
				}
			}
			return true;
		}
		internal void RegisterDTUserSkins() {
			if(this.registrationProcessed) return;
			this.registrationProcessed = true;
			try {
				Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
				if(assemblies == null || assemblies.Length == 0) return;
				foreach(Assembly asm in assemblies) {
					RegisterAssembly(asm);
					try {
						if(asm.Location.Contains("ProjectAssemblies")) {
							AssemblyName[] asmName = asm.GetReferencedAssemblies();
							foreach (AssemblyName refAsm in asmName) {
								RegisterAssembly(Assembly.Load(refAsm));
							}
						}
					}
					catch { }
				}
			} catch {
			}
		}
		[Description("Gets the collection of skins that are currently available for use in Developer Express controls.")]
		public SkinContainerCollection Skins {
			get {
				if(skins == null) skins = new SkinContainerCollection();
				return skins;
			}
		}
		public string GetValidSkinName(string skinName) {
			if(skinName == null || Skins[skinName] == null) return DefaultSkinName;
			return skinName;
		}
		static object[] skinProductIds = new object[] { SkinProductId.NavBar, SkinProductId.Common, SkinProductId.Editors, SkinProductId.Tab, SkinProductId.Grid, SkinProductId.Bars, SkinProductId.Docking, SkinProductId.VGrid, SkinProductId.Scheduler, SkinProductId.NavPane, SkinProductId.Form, SkinProductId.Reports, SkinProductId.Ribbon, SkinProductId.Printing, SkinProductId.RichEdit, SkinProductId.Chart, SkinProductId.Gauges };
		public Skin GetSkin(SkinProductId productId, ISkinProvider provider) {
			return GetSkin(skinProductIds[(int)productId], provider);
		}
		public Skin GetSkin(object productId, ISkinProvider provider) {
			if(provider == null) provider = UserLookAndFeel.Default;
			return GetSkin(productId, provider.SkinName);
		}
		public Skin GetSkin(object productId) { return GetSkin(productId, UserLookAndFeel.Default); }
		public Skin GetSkin(object productId, string skinName) {
			SkinContainer container = Skins[skinName];
			if(container == null) container = Skins[DefaultSkinName];
			if(container != null) return container.GetSkin(productId);
			return null;
		}
		internal void RegisterEmbeddedSkin(SkinCreator creator) {
			RegisterSkinCore(creator, true);
		}
		public Skin RegisterSkin(object productId, SkinBuilder skinBuilder) {
			if(productId == null || skinBuilder == null) return null;
			SkinContainer container = Skins[skinBuilder.Skin.Name];
			if(container == null) {
				container = new SkinContainer(skinBuilder.Skin.Name);
				Skins.Add(container);
			}
			return container.RegisterSkin(productId, skinBuilder);
		}
		public void RegisterSkin(SkinCreator creator) {
			RegisterSkinCore(creator, false);
		}
		protected void RegisterSkinCore(SkinCreator creator, bool embedded) {
			if(creator == null) return;
			SkinContainer container = Skins[creator.SkinName];
			if(container == null) {
				container = new SkinContainer(creator);
				Skins.Add(container);
			} else 
				container.SetCreator(creator);
			container.embedded = embedded;
		}
	}
	public abstract class SkinCreator {
		string skinName;
		bool loaded;
		public SkinCreator(string skinName) {
			this.skinName = skinName;
		}
		protected void SetLoaded() { this.loaded = true; }
		public abstract void Load();
		public abstract SkinCreator Clone(string skinName); 
		public bool Loaded { get { return loaded; } }
		public string SkinName { get { return skinName; } set { skinName = value; } }
	}
	[AttributeUsage(AttributeTargets.Assembly)]
	public class SkinAssemblyAttribute : Attribute {
		public SkinAssemblyAttribute() { }
	}
	public enum SkinImageStretch { Tile, NoResize, Stretch };
	public enum SkinImageLayout { Horizontal, Vertical }
	public enum SkinOffsetKind { Default, Center, Near, Far }
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public class SkinOffset : ICloneable {
		Point offset = Point.Empty;
		SkinOffsetKind kind = SkinOffsetKind.Default;
		public SkinOffset(SkinOffsetKind kind, int x, int y) {
			this.kind = kind;
			this.offset = new Point(x, y);
		}
		public SkinOffset() { }
		public virtual object Clone() {
			return MemberwiseClone();
		}
		[Browsable(false)]
		public bool IsCenter { get { return Kind == SkinOffsetKind.Center; } }
		[RefreshProperties(RefreshProperties.All)]
		public SkinOffsetKind Kind { get { return kind; } set { kind = value; } }
		[RefreshProperties(RefreshProperties.All)]
		public Point Offset { get { return offset; } set { offset = value; } }
		public Rectangle GetBounds(Rectangle dest, Size size, SkinOffsetKind defaultOffset) {
			return RectangleHelper.GetBounds(dest, size, this, defaultOffset);
		}
		[Browsable(false)]
		public bool IsEmpty { get { return Kind == SkinOffsetKind.Default && Offset.IsEmpty; } }
		public override string ToString() {
			if(IsEmpty) return string.Empty;
			return string.Format("({0}, {1}", Kind, Offset);
		}
	}
	public class SkinGlyph : SkinImage {
		public SkinGlyph(Image image, string imageName) : base(image, imageName) { }
		public SkinGlyph(string imageName) : base(imageName) { }
		public SkinGlyph(string imageName, SkinPaddingEdges sizingMargins) : base(imageName, sizingMargins) { }
		public SkinGlyph(string imageName, SkinPaddingEdges sizingMargins, Color transparentColor) : base(imageName, sizingMargins, transparentColor) { }
		public SkinGlyph(string imageName, Color transparentColor) : base(imageName, null, transparentColor) { }
		public SkinGlyph(Image image, SkinPaddingEdges sizingMargins, Color transparentColor) : base(image, sizingMargins, transparentColor) { }
		public SkinGlyph(Image image, SkinPaddingEdges sizingMargins) : base(image, sizingMargins) { }
		public SkinGlyph(Image image) : this(image, new SkinPaddingEdges(0)) { }
		public SkinGlyph(Image image, Color transparentColor) : this(image, new SkinPaddingEdges(0), transparentColor) { }
		public override SkinImageStretch Stretch { get { return SkinImageStretch.NoResize; } set { } }
	}
	public interface ISkinImageProvider {
		Image GetImage();
		bool CheckReload();
		void UpdateProviderInfo(string newPath);
	}
	public interface ISkinImageLoader {
		Image Load(SkinImage image, string imageName);
	}
	public class SkinImage : ICloneable {
		Image image;
		Stream rawData;
		SkinImageStretch stretch = SkinImageStretch.Stretch;
		SkinImageLayout layout = SkinImageLayout.Horizontal;
		SkinPaddingEdges sizingMargins;
		ImageCollection images;
		int imageCount = 1;
		string imageName = "";
		Color transparentColor;
		internal ISkinImageLoader imageLoader;
		ISkinImageProvider imageProvider;
		bool useOwnImage = false;
		public SkinImage(Image image, string imageName) : this((Image)null) { 
			this.imageName = imageName;
			this.image = image;
		}
		public SkinImage(string imageName) : this(imageName, null) { }
		public SkinImage(string imageName, SkinPaddingEdges sizingMargins) : this(imageName, sizingMargins, Color.Empty) { }
		public SkinImage(string imageName, SkinPaddingEdges sizingMargins, Color transparentColor) {
			SetImage(imageName, transparentColor);
			this.sizingMargins = sizingMargins == null ? new SkinPaddingEdges() : sizingMargins;
		}
		public SkinImage(Image image, SkinPaddingEdges sizingMargins) : this(image, sizingMargins, Color.Empty) { }
		public SkinImage(Image image, SkinPaddingEdges sizingMargins, Color transparentColor) {
			SetImage(image, transparentColor);
			this.sizingMargins = sizingMargins == null ? new SkinPaddingEdges() : sizingMargins;
		}
		public void SetImageProvider(ISkinImageProvider provider) {
			this.imageProvider = provider;
		}
		internal void SetRawData(Stream data) {
			this.rawData = data;
		}
		[Browsable(false)]
		public ISkinImageProvider ImageProvider { get { return imageProvider; } }
		public SkinImage(Image image) : this(image, new SkinPaddingEdges(0)) { }
		public SkinPaddingEdges SizingMargins { get { return sizingMargins; } set { sizingMargins = value; } }
		public int ImageCount { get { return imageCount; } set { imageCount = value; } }
		public SkinImageLayout Layout { get { return layout; } set { layout = value; } }
		public virtual SkinImageStretch Stretch { get { return stretch; } set { stretch = value; } }
		[Browsable(false)]
		public bool UseOwnImage { get { return useOwnImage; } set { useOwnImage = value; } }
		[Browsable(false)]
		public Image Image { 
			get { 
				if(!UseOwnImage && ImageProvider != null) return ImageProvider.GetImage();
				if(rawData != null) {
					SetImage(CreateImageFromStream(rawData), TransparentColor);
					rawData = null;
				}
				return image; 
			}
			set {
				if(image == value)
					return;
				image = value;
				this.images = null;
			}
		}
		Image CreateImageFromStream(Stream ms) {
			Bitmap b = (Bitmap)Bitmap.FromStream(ms);
			if(object.Equals(b.RawFormat, ImageFormat.Gif)) return b;
			Bitmap bmp = new Bitmap(b.Width, b.Height, (b.PixelFormat & PixelFormat.Alpha) != 0 ? PixelFormat.Format32bppArgb : PixelFormat.Format24bppRgb);
			bmp.SetResolution(b.HorizontalResolution, b.VerticalResolution);
			Graphics g = Graphics.FromImage(bmp);
			g.CompositingMode = CompositingMode.SourceCopy;
			g.DrawImageUnscaled(b, Point.Empty);
			b.Dispose();
			g.Dispose();
			return bmp;
		}
		[Browsable(false), DefaultValue("")]
		public string ImageName {
			get { return imageName; }
			set {
				if(value == null) value = "";
				if(ImageName == value) return;
				if(value == "") Image = null;
				else
					SetImage(GetImage(value), TransparentColor);
				this.imageName = value;
			}
		}
		public void SetImage(Image image, string imageName) {
			if(imageName == null) imageName = "";
			this.imageName = imageName;
			this.image = image;
			if(Image != null) SetImage(Image, TransparentColor);
		}
		bool ShouldSerializeTransparentColor() { return TransparentColor != Color.Empty; } 
		public Color TransparentColor { 
			get { return transparentColor; }
			set {
				if(TransparentColor == value) return;
				transparentColor = value;
				SetImage(Image, TransparentColor);
			}
		}
		public void SetImageNameCore(string imageName) {
			this.imageName = imageName;
		}
		public void SetImage(string imageName, Color transparentColor) {
			this.imageName = imageName;
			SetImage(GetImage(imageName), transparentColor);
		}
		protected Image GetImage(string imageName) {
			if(imageLoader == null) return Image.FromFile(imageName);
			return imageLoader.Load(this, imageName);
		}
		public void SetImage(Image image, Color transparentColor) {
			if(image != null)
				this.rawData = null;
			Image = image;
			if(transparentColor != Color.Empty) {
				Bitmap bmp = image as Bitmap;
				if(bmp != null) bmp.MakeTransparent(transparentColor);
			}
			this.transparentColor = transparentColor;
		}
		public ImageCollection GetImages() {
			if(this.images != null || Image == null) return this.images;
			this.images = new ImageCollection();
			int count = (ImageCount > 0 ? ImageCount : 1);
			if(Layout == SkinImageLayout.Horizontal) {
				this.images.ImageSize = new Size(Image.Width / count, Image.Height);
				this.images.AddImageStrip(Image);
			} else {
				this.images.ImageSize = new Size(Image.Width, Image.Height / count);
				this.images.AddImageStripVertical(Image);
			}
			return images;
		}
		internal Size GetImageSize() {
			if(Image == null || ImageCount == 0) return Size.Empty;
			Size size = Image.Size;
			if(ImageCount == 1) return size;
			if(Layout == SkinImageLayout.Horizontal) 
				size.Width /= ImageCount;
			else 
				size.Height /= ImageCount;
			return size;
		}
		public Rectangle GetImageBounds(int index) {
			if(Image == null || index < 0 || index >= ImageCount) return Rectangle.Empty;
			Size size = GetImageSize();
			Rectangle res = new Rectangle(Point.Empty, size);
			if(ImageCount == 1) return res;
			if(Layout == SkinImageLayout.Horizontal) 
				res.X = size.Width * index;
			else
				res.Y = size.Height * index;
			return res;
		}
		public virtual object Clone() { return Copy(); }
		public SkinImage Copy() {
			return MemberwiseClone() as SkinImage;
		}
		public bool CheckReload() {
			if(ImageProvider != null) return ImageProvider.CheckReload();
			if(Image == null || ImageName == "" || imageLoader == null) return false;
			Image img = null;
			try {
				img = imageLoader.Load(this, ImageName);
				SetImage(img, TransparentColor);
			} catch { }
			return img != null && rawData != null;
		}
	}
	public class SkinColor : ICloneable {
		Color backColor, backColor2, foreColor, solidImageCenterColor, solidImageCenterColor2;
		bool fontBold;
		LinearGradientMode gradientMode;
		LinearGradientMode solidImageCenterGradientMode;
		Skin owner;
		public SkinColor() {
			this.solidImageCenterGradientMode = this.gradientMode = LinearGradientMode.Horizontal;
		}
		object ICloneable.Clone() { return MemberwiseClone(); }
		[Browsable(false)]
		public Skin Owner { get { return owner; } }
		protected internal void SetOwner(Skin owner) { this.owner = owner; }
		public Color GetBackColor() {
			return GetColor(BackColor);
		}
		public Color GetBackColor2() {
			return GetColor(BackColor2);
		}
		public Color GetForeColor() {
			return GetColor(ForeColor);
		}
		public Color GetSolidImageCenterColor() {
			return GetColor(SolidImageCenterColor);
		}
		public Color GetSolidImageCenterColor2() {
			return GetColor(SolidImageCenterColor2);
		}
		protected Color GetColor(Color clr) {
			if(clr == Color.Empty) return clr;
			if(Owner != null && clr.IsSystemColor) return Owner.GetSystemColor(clr);
			return clr;
		}
		[DefaultValue(false)]
		public bool FontBold { get { return fontBold; } set { fontBold = value; } }
		[DefaultValue(LinearGradientMode.Horizontal)]
		public LinearGradientMode GradientMode { get { return gradientMode; } set { gradientMode = value; } }
		[DefaultValue(LinearGradientMode.Horizontal)]
		public LinearGradientMode SolidImageCenterGradientMode { get { return solidImageCenterGradientMode; } set { solidImageCenterGradientMode = value; } }
		bool ShouldSerializeBackColor() { return BackColor != Color.Empty; }
		public Color BackColor { get { return backColor; } set { backColor = value; } }
		bool ShouldSerializeBackColor2() { return BackColor2 != Color.Empty; }
		public Color BackColor2 { get { return backColor2; } set { backColor2 = value; } }
		bool ShouldSerializeForeColor() { return ForeColor != Color.Empty; }
		public Color ForeColor { get { return foreColor; } set { foreColor = value; } }
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public bool IsEmpty {
			get {
				return BackColor.IsEmpty && ForeColor.IsEmpty && BackColor2.IsEmpty && SolidImageCenterColor.IsEmpty && SolidImageCenterColor2.IsEmpty;
			}
		}
		bool ShouldSerializeSolidImageCenterColor() { return SolidImageCenterColor != Color.Empty; }
		public Color SolidImageCenterColor {
			get { return solidImageCenterColor; }
			set { solidImageCenterColor = value; }
		}
		bool ShouldSerializeSolidImageCenterColor2() { return SolidImageCenterColor2 != Color.Empty; }
		public Color SolidImageCenterColor2 {
			get { return solidImageCenterColor2; }
			set { solidImageCenterColor2 = value; }
		}
	}
	public class SkinBorder : ICloneable {
		Color left, right, top, bottom;
		SkinPaddingEdges thin;
		Skin owner;
		public SkinBorder() {
			this.thin = new SkinPaddingEdges();
			left = right = top = bottom = Color.Empty;
		}
		public virtual object Clone() { return MemberwiseClone(); }
		[Browsable(false)]
		public Skin Owner { get { return owner; } }
		protected internal void SetOwner(Skin owner) { this.owner = owner; }
		[Browsable(false)]
		public bool IsEmpty {
			get { 
				return 
				  All.IsEmpty && Top.IsEmpty && Bottom.IsEmpty &&
					Left.IsEmpty && Right.IsEmpty &&
					  Thin.Left == 0 && Thin.Top == 0 && Thin.Right == 0 && Thin.Bottom == 0; 
			}
		}
		public void SetBorderColor(Color color) {
			left = right = top = bottom = color;
		}
		public Color All { 
			get { 
				return Left == Right && Left == Top && Left == Bottom ? GetLeft() : Color.Empty; 
			}
			set {
				Left = Right = Top = Bottom = value;
			}
		}
		public SkinPaddingEdges Thin { get { return thin; } set { thin = value; } }
		public Color Left { get { return left; } set { left = value; } }
		public Color Right { get { return right; } set { right = value; } }
		public Color Top { get { return top; } set { top = value; } }
		public Color Bottom { get { return bottom; } set { bottom = value; } }
		public Color GetLeft() { return GetColor(Left); }
		public Color GetRight() { return GetColor(Right); }
		public Color GetTop() { return GetColor(Top); }
		public Color GetBottom() { return GetColor(Bottom); }
		protected Color GetColor(Color clr) {
			if(clr == Color.Empty) return clr;
			if(Owner != null && clr.IsSystemColor) return Owner.GetSystemColor(clr);
			return clr;
		}
		bool ShouldSerializeLeft() { return Left != Color.Empty; }
		bool ShouldSerializeRight() { return Right != Color.Empty; }
		bool ShouldSerializeTop() { return Top != Color.Empty; }
		bool ShouldSerializeBottom() { return Bottom != Color.Empty; }
	}
	public class SkinElement {
		SkinBuilderElementInfo info;
		string elementName;
		Skin owner;
		public SkinElement(Skin skin, string elementName, SkinBuilderElementInfo info) {
			this.owner = skin;
			this.elementName = elementName;
			this.info = info == null ? new SkinBuilderElementInfo() : info;
			UpdateOwner();
		}
		public SkinElement Copy(Skin newSkin) {
			SkinBuilderElementInfo info = Info.Copy();
			SkinElement element = new SkinElement(newSkin, elementName, info);
			element.IsCustomElement = IsCustomElement;
			return element;
		}
		public SkinElement Copy(Skin newSkin, string newName) {
			SkinBuilderElementInfo info = Info.Copy();
			SkinElement element = new SkinElement(newSkin, newName, info);
			element.IsCustomElement = IsCustomElement;
			return element;
		}
		bool isCustomElement;
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public bool IsCustomElement { get { return isCustomElement; } set { isCustomElement = value; } }
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public void Assign(SkinElement sourceElement) {
			this.info = sourceElement.Info.Copy();
		}
		protected virtual void UpdateOwner() {
			Info.Color.SetOwner(Owner);
			Info.Border.SetOwner(Owner);
			Info.Properties.SetOwner(Owner);
		}
		[Browsable(false)]
		public Skin Owner { get { return owner; } }
		[Browsable(false)]
		public string ElementName { get { return elementName; } }
		protected internal SkinBuilderElementInfo Info { get { return info; } }
		public SkinOffset Offset { get { return info.Offset; } }
		[Browsable(false)]
		public SkinImage Image { get { return info.Image; } }
		[Browsable(false)]
		public SkinGlyph Glyph { get { return info.Glyph; } }
		[Browsable(false)]
		public SkinBorder Border { get { return info.Border; } }
		[Browsable(false)]
		public SkinColor Color { get { return info.Color; } }
		public SkinSize Size { get { return info.Size; } }
		[Browsable(false)]
		public SkinProperties Properties { get { return info.Properties; } }
		public SkinPaddingEdges ContentMargins { get { return info.ContentMargins; } }
		public AppearanceDefault GetAppearanceDefault() {
			AppearanceDefault appDefault = new AppearanceDefault();
			Apply(appDefault);
			return appDefault;
		}
		[Browsable(false)]
		public bool IsRequireTransparentBackground {
			get {
				if(Image != null && Image.Image != null) return true;
				if(Color.GetBackColor2() != Color.GetBackColor() && !Color.GetBackColor2().IsEmpty) return true;
				return false;
			}
		}
		public Color GetForeColor(ObjectState state) {
			if((state & ObjectState.Pressed) != 0) {
				Color pressed = Properties.GetColor("ForeColorPressed");
				if(!pressed.IsEmpty) return pressed;
			}
			if((state & ObjectState.Hot) != 0) {
				Color hot = Properties.GetColor("ForeColorHot");
				if(!hot.IsEmpty) return hot;
			}
			if((state & ObjectState.Disabled) != 0) {
				Color disabled = Properties.GetColor("ForeColorDisabled");
				if(!disabled.IsEmpty) return disabled;
			}
			if((state & ObjectState.Selected) != 0) {
				Color selected = Properties.GetColor("ForeColorSelected");
				if(!selected.IsEmpty) return selected;
			}
			return Properties.GetColor("ForeColor");
		}
		public Color GetForeColor(AppearanceObject appearance, ObjectState state) {
			Color res = GetForeColor(state);
			if(res.IsEmpty) return appearance.ForeColor;
			return res;
		}
		public AppearanceObject GetForeColorAppearance(AppearanceObject appearance, ObjectState state) {
			Color clr = GetForeColor(state);
			if(clr.IsEmpty) return appearance;
			AppearanceObject res = (AppearanceObject)appearance.Clone();
			res.ForeColor = clr;
			return res;
		}
		public AppearanceDefault Apply(AppearanceDefault appDefault) {
			ApplyForeColorAndFont(appDefault);
			if(Color.GetBackColor() != System.Drawing.Color.Empty) {
				appDefault.BackColor = Color.GetBackColor();
				appDefault.BackColor2 = Color.GetBackColor2();
				appDefault.GradientMode = Color.GradientMode;
			}
			if(Border.All != System.Drawing.Color.Empty)
				appDefault.BorderColor = Border.All;
			if(Border.GetLeft() != System.Drawing.Color.Empty) appDefault.BorderColor = Border.GetLeft();
			return appDefault;
		}
		public AppearanceDefault ApplyForeColorAndFont(AppearanceDefault appDefault) {
			if(Color.GetForeColor() != System.Drawing.Color.Empty) 
				appDefault.ForeColor = Color.GetForeColor();
			if(Color.FontBold) {
				if(appDefault.Font == null) appDefault.Font = AppearanceObject.DefaultFont;
				appDefault.Font = new Font(appDefault.Font, FontStyle.Bold);
			}
			int fontDelta = Properties.GetInteger("FontDelta");
			if(fontDelta != 0) {
				if(appDefault.Font == null) appDefault.Font = AppearanceObject.DefaultFont;
				try {
					appDefault.Font = new Font(appDefault.Font.FontFamily, appDefault.Font.Size + fontDelta, appDefault.Font.Style);
				}
				catch { }
			}
			return appDefault;
		}
		public override string ToString() { return ElementName; } 
	}
	public class Skin {
		SkinProperties properties;
		SkinColors colors;
		Hashtable elements;
		SkinContainer container;
		string name;
		string cloned;
		public Skin(string name) {
			this.name = name;
			this.properties = new SkinProperties();
			this.properties.SetOwner(this);
			this.colors = new SkinColors(this);
		}
		public string Cloned {
			get { return cloned; }
			set {
				cloned = value;
			}
		}
		public virtual object Clone(string skinName) {
			Skin res = new Skin(skinName);
			res.Properties.Clone(Properties);
			res.Colors.Clone(Colors);
			foreach(SkinElement element in GetElements()) {
				SkinElement newElement = element.Copy(res);
				res.AddElement(newElement);
			}
			return res;
		}
		public void AddElement(SkinElement newElement) {
			newElement.Color.SetOwner(this);
			newElement.Properties.SetOwner(this);
			Elements.Add(newElement.ElementName, newElement);
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public void RemoveElement(string elementName) {
			Elements.Remove(elementName);
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public void RemoveElement(SkinElement element) {
			Elements.Remove(element.ElementName);
		}
		public SkinProductId GetProductId() {
			if(Container == null) return SkinProductId.Common;
			foreach(SkinProductId product in Enum.GetValues(typeof(SkinProductId))) {
				if(Container.GetSkin(product) == this) return product;
			}
			return SkinProductId.Common;
		}
		public SkinContainer Container { get { return container; } }
		protected internal void SetContainer(SkinContainer container) { this.container = container; }
		public Skin CommonSkin {
			get {
				if(Container == null) return null;
				return Container.CommonSkin;
			}
		}
		protected internal Color GetSystemColorCore(Color color, bool always) {
			Skin common = CommonSkin;
			if(common == null || (common == this && !always) || !color.IsKnownColor) return color;
			KnownColor known = color.ToKnownColor();
			return common.Colors.GetColor(CommonColors.GetSystemColorName(known));
		}
		public Color TranslateColor(Color color) {
			Skin common = CommonSkin;
			if(common == null || !color.IsKnownColor) return color;
			KnownColor known = color.ToKnownColor();
			string res = CommonColors.GetSystemColorName(known, false);
			if(res == null) return color;
			return common.Colors.GetColor(res);
		}
		public Color GetSystemColor(Color color) {
			return GetSystemColorCore(color, true);
		}
		public SkinColors Colors { get { return colors; } }
		public SkinProperties Properties { get { return properties; } }
		public string Name { get { return name; } }
		public SkinElement this[string name] { get { return Elements[name] as SkinElement; } }
		protected internal Hashtable Elements {
			get { 
				if(elements == null) elements = new Hashtable();
				return elements;
			}
		}
		internal void SetSkinName(string name) { this.name = name; }
		public ICollection GetElements() {
			if(elements == null) return null;
			return Elements.Values;
		}
		protected internal SkinElement Add(string elementName, SkinBuilderElementInfo elementInfo) {
			SkinElement element = this[elementName];
			if(element == null) {
				element = new SkinElement(this, elementName, elementInfo);
				Elements[elementName] = element;
			}
			return element;
		}
	}
	public class SkinBuilder {
		internal Skin skin;
		public SkinBuilder(string name) { 
			this.skin = new Skin(name);
		}
		public Skin Skin { get { return skin; } }
		public SkinProperties Properties { get { return Skin.Properties; } }
		public SkinElement CreateElement(string elementName, SkinBuilderElementInfo elementInfo) {
			SkinElement element = Skin.Add(elementName, elementInfo);
			return element;
		}
	}
	public class SkinColors : SkinCustomProperties {
		Skin owner;
		public SkinColors(Skin owner) {
			this.owner = owner;
		}
		[Browsable(false)]
		public Skin Owner { get { return owner; } }
		public Color GetColor(object name) {
			if(this.customPropertiesHash == null) return Color.Black;
			object obj = customPropertiesHash[name];
			if(obj == null) return Color.Black;
			Color clr  = (Color)obj;
			if(clr.IsSystemColor) return Owner.GetSystemColorCore(clr, false);
			return clr;
		}
		public Color this[object name] {
			get {
				return GetColor(name);
			}
			set {
				CustomProperties[name] = value;
			}
		}
		public bool Contains(string name) { return CustomProperties.ContainsKey(name); }
		protected internal override void SetProperty(object name, object val) {
			CustomProperties[name] = val;
		}
	}
	public class SkinProperties : SkinCustomProperties {
		Skin owner;
		public object this[object name] {
			get {
				if(this.customPropertiesHash == null) return null;
				return CustomProperties[name];
			}
			set {
				CustomProperties[name] = value;
			}
		}
		public Skin Owner { get { return owner; } }
		internal void SetOwner(Skin skin) { this.owner = skin; }
		public Color GetColor(string name) {
			object obj = this[name];
			if(obj == null) return Color.Empty;
			Color res = (Color)obj;
			if(Owner != null && res.IsSystemColor) return Owner.GetSystemColor(res);
			return res;
		}
		public bool GetBoolean(string name) {
			object obj = this[name];
			if(obj == null) return false;
			return (bool)obj;
		}
		public int GetInteger(string name) {
			object obj = this[name];
			if(obj == null) return 0;
			return (int)obj;
		}
		protected internal override void SetProperty(object name, object val) { 
			this[name] = val;
		}
	}
	public abstract class SkinCustomProperties {
		protected Hashtable customPropertiesHash;
		protected internal Hashtable CustomProperties { 
			get { 
				if(this.customPropertiesHash == null) this.customPropertiesHash = new Hashtable();
				return customPropertiesHash; 
			} 
		}
		protected internal abstract void SetProperty(object name, object val);
		public void Clone(SkinCustomProperties source) {
			this.customPropertiesHash = null;
			if(source.customPropertiesHash == null) return;
			foreach(DictionaryEntry entry in source.customPropertiesHash) {
				CustomProperties.Add(entry.Key, entry.Value);
			}
		}
	}
	public class SkinBuilderElementInfo {
		SkinImage image;
		SkinGlyph glyph;
		SkinSize size;
		SkinColor color;
		SkinPaddingEdges contentMargins;
		SkinBorder border;
		SkinOffset offset;
		SkinProperties properties;
		public SkinBuilderElementInfo() {
			this.offset = new SkinOffset();
			this.size = new SkinSize();
			this.properties = new SkinProperties();
			this.contentMargins = new SkinPaddingEdges();
			this.color = new SkinColor();
			this.border = new SkinBorder();
		}
		public SkinBuilderElementInfo Copy() {
			SkinBuilderElementInfo res = (SkinBuilderElementInfo)MemberwiseClone();
			if(Glyph != null) res.glyph = Glyph.Copy() as SkinGlyph;
			if(Image != null) res.image = Image.Copy();
			res.properties = new SkinProperties();
			res.properties.Clone(Properties);
			return res;
		}
		public SkinProperties Properties { get { return properties; } }
		public SkinOffset Offset { get { return offset; } set { offset = value; } }
		public SkinImage Image { get { return image; } set { image = value; } }
		public SkinGlyph Glyph { get { return glyph; } set { glyph = value; } }
		public SkinSize Size { get { return size; } set { size = value; } }
		public SkinPaddingEdges ContentMargins { get { return contentMargins; } set { contentMargins = value; } }
		public SkinColor Color { get { return color; } set { color = value; } }
		public SkinBorder Border { get { return border; } set { border = value; } }
	}
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public class SkinSize : ICloneable {
		Size minSize;
		internal static Size DefaultSize = new Size(10, 10);
		bool allowHGrow, allowVGrow;
		public SkinSize() : this(DefaultSize, true, true) { }
		public SkinSize(Size minSize) : this(minSize, true, true) { }
		public SkinSize(Size minSize, bool allowGrow) : this(minSize, allowGrow, allowGrow) { }
		public SkinSize(Size minSize, bool allowHGrow, bool allowVGrow) {
			this.minSize = minSize;
			this.allowHGrow = allowHGrow;
			this.allowVGrow = allowVGrow;
		}
		public virtual object Clone() {
			return MemberwiseClone();
		}
		internal bool ShouldSerialize() { return ShouldSerializeMinSize() || !AllowHGrow || !AllowVGrow; }
		bool ShouldSerializeMinSize() { return MinSize != DefaultSize; } 
		[RefreshProperties(RefreshProperties.All)]
		public Size MinSize { 
			get { return minSize; } 
			set { 
				if(value.Width < 0) value.Width = 0;
				if(value.Height < 0) value.Height = 0;
				minSize = value; 
			} 
		}
		[DefaultValue(true)]
		public bool AllowHGrow { get { return allowHGrow; } set { allowHGrow = value; } }
		[DefaultValue(true)]
		public bool AllowVGrow { get { return allowVGrow; } set { allowVGrow = value; } }
		public override string ToString() { 
			if(ShouldSerializeMinSize()) return string.Format("({0})", MinSize);
			return string.Empty; 
		}
	}
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public class SkinPaddingEdges : ICloneable {
		bool all = true;
		int top, right, left, bottom;
		public SkinPaddingEdges() { }
		public SkinPaddingEdges(int all) {
			this.all = true;
			this.top = all;
		}
		public SkinPaddingEdges(int left, int top, int right, int bottom) {
			this.left = left;
			this.top = top;
			this.right = right;
			this.bottom = bottom;
			this.all = false;
			OnChanged();
		}
		protected virtual void OnChanged() {
			if(left == top && left == right && left == bottom) {
				if(top == 0) all = false;
				else all = true;
			}
		}
		[Browsable(false)]
		public bool IsEmpty { get { return left == 0 && top == 0 && right == 0 && bottom == 0; } }
		[RefreshProperties(RefreshProperties.All), DefaultValue(0)]
		public int All {
			get { return all ? top : 0; }
			set {
				if (all != true || top != value) {
					all = true;
					top = left = right = bottom = value;
					OnChanged();
				}
			}
		}
		[RefreshProperties(RefreshProperties.All), DefaultValue(0)]
		public int Bottom {
			get { return all ? top : bottom; }
			set {
				if (all || bottom != value) {
					all = false;
					bottom = value;
					OnChanged();
				}
			}
		}
		[RefreshProperties(RefreshProperties.All), DefaultValue(0)]
		public int Top {
			get { return top; }
			set {
				if (all || top != value) {
					all = false;
					top = value;
					OnChanged();
				}
			}
		}
		[RefreshProperties(RefreshProperties.All), DefaultValue(0)]
		public int Left {
			get { return all ? top : left; }
			set {
				if (all || left != value) {
					all = false;
					left = value;
					OnChanged();
				}
			}
		}
		[RefreshProperties(RefreshProperties.All), DefaultValue(0)]
		public int Right {
			get { return all ? top : right; }
			set {
				if (all || right != value) {
					all = false;
					right = value;
					OnChanged();
				}
			}
		}
		public int Width { get { return all ? top * 2: left + right; } }
		public int Height { get { return all ? top * 2 : top + bottom; } }
		public override string ToString() {	
			if(IsEmpty) return string.Empty;
			if(All != 0) return string.Format("(All={0})", All);
			string text = GetText("", "Left", Left);
			text = GetText(text, "Top", Top);
			text = GetText(text, "Right", Right);
			text = GetText(text, "Bottom", Bottom);
			return string.Format("({0})", text); 
		}
		string GetText(string currentText, string name, int val) {
			if(val == 0) return currentText;
			if(currentText.Length > 0) currentText += ", ";
			return string.Format("{0}{1}={2}", currentText, name, val);
		}
		public override int GetHashCode() { return base.GetHashCode();	}
		public override bool Equals(object obj) {
			SkinPaddingEdges edges = obj as SkinPaddingEdges;
			if(edges == null) return false;
			return edges.all == all && edges.top == top && edges.left == left && 
				edges.bottom == bottom && 
				edges.right == right;
		}
		public void Scale(float dx, float dy) {
			top = (int)((float)top * dy);
			left = (int)((float)left * dx);
			right = (int)((float)right * dx);
			bottom = (int)((float)bottom * dy);
		}
		bool ShouldSerializeAll() {	return all && top != 0;}
		bool ShouldSerializeBottom() { return !all && bottom != 0;}
		bool ShouldSerializeLeft() { return !all && left != 0; }
		bool ShouldSerializeRight() { return !all && right != 0; }
		bool ShouldSerializeTop() {	return !all && top != 0; }
		void ResetAll() { All = 0;}
		void ResetBottom() { Bottom = 0;}
		void ResetLeft() { Left = 0; }
		void ResetRight() { Right = 0; }
		void ResetTop() { Top = 0; }
		public object Clone() {
			SkinPaddingEdges edges = new SkinPaddingEdges();
			edges.all = all;
			edges.top = top;
			edges.right = right;
			edges.bottom = bottom;
			edges.left = left;
			return edges;
		}
		public Rectangle Deflate(Rectangle bounds) {
			if(IsEmpty) return bounds;
			Rectangle res = bounds;
			if(res.Width < Left + Right) {
				if(res.Width < Left) res.X += res.Width;
				res.Width = 0;
			} else {
				res.X += Left; res.Width -= (Left + Right);
			}
			if(res.Height < Top + Bottom) {
				if(res.Height < Top) res.Y += res.Height;
				res.Height = 0;
			} else {
				res.Y += Top; res.Height -= (Top + Bottom);
			}
			return res;
		}
		public Rectangle Inflate(Rectangle bounds) {
			bounds.X -= Left; bounds.Width += Left + Right;
			bounds.Y -= Top; bounds.Height += Top + Bottom;
			return bounds;
		}
	}
	public class RectangleHelper {
		public static Rectangle GetCenterBounds(Rectangle dest, Size size) {
			if(dest.Width < size.Width) size.Width = dest.Width;
			if(dest.Height < size.Height) size.Height = dest.Height;
			return new Rectangle(dest.X + (dest.Width - size.Width) / 2, dest.Y + (dest.Height - size.Height) / 2, size.Width, size.Height);
		}
		public static Rectangle GetBounds(Rectangle dest, Size size, SkinOffset offset, SkinOffsetKind defaultOffset) {
			SkinOffsetKind kind = offset.Kind;
			if(kind == SkinOffsetKind.Default) kind = defaultOffset;
			Rectangle res = dest;
			if(dest.Width < size.Width) size.Width = dest.Width;
			if(dest.Height < size.Height) size.Height = dest.Height;
			switch(kind) {
				case SkinOffsetKind.Near : 
					res.Location = new Point(dest.X, dest.Y + (dest.Height - size.Height) / 2);
					break;
				case SkinOffsetKind.Far : 
					res.Location = new Point(dest.Right - size.Width, dest.Y + (dest.Height - size.Height) / 2);
					break;
				default:
					res.Location = new Point(dest.X + (dest.Width - size.Width) / 2, dest.Y + (dest.Height - size.Height) / 2);
					break;
			}
			res.Size = size;
			res.Offset(offset.Offset.X, offset.Offset.Y);
			return res;
		}
	}
	public interface ISkinInfoProvider {
		SkinInfo GetSkinInfo(Point point);
	}
	public class SkinInfo {
		Point location;
		Rectangle elementBounds;
		object hitInfo;
		SkinProductId productId;
		SkinElementInfo sampleInfo;
		SkinElementPainter samplePainter;
		string elementName;
		public SkinInfo() {
			this.productId = SkinProductId.Editors;
			this.elementName = string.Empty;
		}
		public SkinElementInfo SampleInfo { get { return sampleInfo; } set { sampleInfo = value; } }
		public SkinElementPainter SamplePainter { get { return samplePainter; } set { samplePainter = value; } }
		public Rectangle ElementBounds { get { return elementBounds; } set { elementBounds = value; } }
		public Point Location { get { return location; } set { location = value; } }
		public object HitInfo { get { return hitInfo; } set { hitInfo = value; } }
		public SkinProductId ProductId { get { return productId; } set { productId = value; } }
		public string ElementName { get { return elementName; } set { elementName = value; } }
	}
}
