#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Text;
using System.Runtime.InteropServices;
using System.Collections.Generic;
namespace DevExpress.Utils.Text {
	public class TextHighLight {
		int startIndex;
		int length;
		Color backColor;
		Color foreColor;
		public TextHighLight(int startIndex, int length, Color backColor, Color foreColor) {
			this.startIndex = startIndex;
			this.length = length;
			this.backColor = backColor;
			this.foreColor = foreColor;
		}
		public int StartIndex { get { return startIndex; } set { startIndex = value; } }
		public int Length { get { return length; } set { length = value; } }
		public int EndIndex { get { return StartIndex + Length; } }
		public Color BackColor { get { return backColor; } }
		public Color ForeColor { get { return foreColor; } }
		public bool IsTextHighLighted(int textPos, int textLen) {
			int textEndPos = textPos + textLen;
			return (textPos <= StartIndex && textEndPos >= StartIndex)
				|| (textEndPos >= EndIndex && textPos <= EndIndex)
				|| (textPos >= StartIndex && textEndPos <= EndIndex);
		}
	}
	public class TextUtils {
		[ThreadStatic]
		static FontsCache fontsCache;
		static int tabStopSpacesCount;
		static int leftOffset, rightOffset, topOffset, bottomOffset;
		static bool useKerning;
		static TextUtils() {
			tabStopSpacesCount = 4;
			ResetOffsets();
		}
		public static int TabStopSpacesCount { get { return tabStopSpacesCount; } set { tabStopSpacesCount = value; } }
		public static int LeftOffset { get { return leftOffset; } set { leftOffset = value; } }
		public static int RightOffset { get { return rightOffset; } set { rightOffset = value; } }
		public static int TopOffset { get { return topOffset; } set { topOffset = value; } }
		public static int BottomOffset { get { return bottomOffset; } set { bottomOffset = value; } }
		public static void ResetOffsets() {
			SetOffsets(0, 0, 0, 0);
		}
		public static void SetOffsets(int left, int top, int right, int bottom) {
			LeftOffset = left;
			TopOffset = top;
			RightOffset = right;
			BottomOffset = bottom;
		}
		public static bool UseKerning { get { return useKerning; } set { useKerning = value; } }
		private static FontsCache Fonts { 
			get {
				if(fontsCache == null) fontsCache = new FontsCache();
				return fontsCache; 
			} 
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Point location) {
			DrawString(g, text, font, foreColor, location, Rectangle.Empty, StringFormat.GenericDefault);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Point location, Rectangle clipBounds) {
			DrawString(g, text, font, foreColor, location, clipBounds, StringFormat.GenericDefault);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Point location, StringFormat stringFormat) {
			DrawString(g, text, font, foreColor, location, Rectangle.Empty, stringFormat);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Point location, Rectangle clipBounds, StringFormat stringFormat) {
			DrawString(g, text, font, foreColor, new Rectangle(location, GetStringSize(g, text, font, stringFormat)), clipBounds, stringFormat);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds) {
			DrawString(g, text, font, foreColor, drawBounds, Rectangle.Empty, StringFormat.GenericDefault);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds, Rectangle clipBounds) {
			DrawString(g, text, font, foreColor, drawBounds, clipBounds, StringFormat.GenericDefault);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds, StringFormat stringFormat) {
			DrawString(g, text, font, foreColor, drawBounds, Rectangle.Empty, stringFormat);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds, Rectangle clipBounds, StringFormat stringFormat) {
			DrawString(g, text, font, foreColor, drawBounds, clipBounds, stringFormat, null);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds, StringFormat stringFormat, TextHighLight hightLight) {
			DrawString(g, text, font, foreColor, drawBounds, Rectangle.Empty, stringFormat, hightLight);
		}
		public static void DrawString(Graphics g, string text, Font font, Color foreColor, Rectangle drawBounds, Rectangle clipBounds, StringFormat stringFormat, TextHighLight hightLight) {
			Fonts.DrawString(g, text, font, foreColor, drawBounds, clipBounds, stringFormat, hightLight);
		}
		public static Size GetStringSize(Graphics g, string text, Font font) {
			return GetStringSize(g, text, font, StringFormat.GenericDefault);
		}
		public static Size GetStringSize(Graphics g, string text, Font font, StringFormat stringFormat) {
			if(text == null) return Size.Empty;
			return Fonts.GetStringSize(g, text, font, stringFormat);
		}
		public static Size GetStringSize(Graphics g, string text, Font font, StringFormat stringFormat, int maxWidth) {
			if(text == null) return Size.Empty;
			return Fonts.GetStringSize(g, text, font, stringFormat, maxWidth);
		}
		public static int[] GetMeasureString(Graphics g, string text, Font font) {
			return GetMeasureString(g, text, font, StringFormat.GenericDefault);
		}
		public static int[] GetMeasureString(Graphics g, string text, Font font, StringFormat stringFormat) {
			return Fonts.GetMeasureString(g, text, font, stringFormat);
		}
		public static int GetStringHeight(Graphics g, string text, Font font, int width) {
			return GetStringHeight(g, text, font, width, StringFormat.GenericDefault);
		}
		public static int GetStringHeight(Graphics g, string text, Font font, int width, StringFormat stringFormat) {
			return Fonts.GetStringHeight(g, text, font, width, stringFormat);
		}
		public static int GetFontHeight(Graphics g, Font font) {
			return Fonts.GetFontHeight(g, font);
		}
		public static int GetFontAscentHeight(Graphics g, Font font) {
			return Fonts.GetFontAscentHeight(g, font);
		}
		public static Size GetScaleSize(Size size) {
			Size ret = Size.Empty;
			DevExpress.Utils.Drawing.GraphicsInfo.Default.AddGraphics(null);
			IntPtr hdc = DevExpress.Utils.Drawing.GraphicsInfo.Default.Graphics.GetHdc();
			try {
				ret = new Size(
					Convert.ToInt32(size.Width * Win32Util.GetLogicPixelPerInchX(hdc) / 96.0),
					Convert.ToInt32(size.Height * Win32Util.GetLogicPixelPerInchY(hdc) / 96.0));
			}
			finally {
				DevExpress.Utils.Drawing.GraphicsInfo.Default.Graphics.ReleaseHdc(hdc);
				DevExpress.Utils.Drawing.GraphicsInfo.Default.ReleaseGraphics();
			}
			return ret;
		}
	}
	public class FontCache : IDisposable {
		const int InitialCharCount = 1024;
		FontStyle fontStyle;
		IntPtr fontHandle;
		IntPtr fontUnderlineHandle;
		bool underline;
		int height;
		int ascentHeight;
		int averageCharWidth;
		int[] charsWidth;
		int[] italicOffset;
		char firstChar;
		char lastChar;
		char currentLastChar;
		Dictionary<uint, int> kerningPairs;
		public FontCache(Graphics graphics, Font font) {
			this.fontStyle = font.Style;
			this.fontHandle = CreateGdiFont(font);
			this.underline = font.Underline;
			if(!Underline) {
				Font fontUnderline = new Font(font, fontStyle | FontStyle.Underline);
				this.fontUnderlineHandle = CreateGdiFont(fontUnderline);
				fontUnderline.Dispose();
			}
			Calculate(graphics);
		}
		public virtual void Dispose() {
			Win32Util.Win32API.DeleteObject(this.fontHandle);
			if(this.fontUnderlineHandle != IntPtr.Zero)
				Win32Util.Win32API.DeleteObject(this.fontUnderlineHandle);
		}
		public static char TabStopChar { get { return '\t'; } }
		public static char ReturnChar { get { return '\n'; } }
		public static char SpaceChar { get { return ' '; } }
		public static bool IsTabStop(char ch) { return ch == TabStopChar; }
		public static bool IsReturn(char ch) { return ch == ReturnChar; }
		public static bool IsSpace(char ch) { return ch == SpaceChar; }
		public int Height { get { return height; } }
		public int AscentHeight { get { return ascentHeight; } }
		public bool IsItalic { get { return (fontStyle & FontStyle.Italic) == FontStyle.Italic; } }
		IntPtr CreateGdiFont(Font font) {
			Win32Util.LOGFONT lf = new Win32Util.LOGFONT();
			font.ToLogFont(lf);
			if (font.Unit != GraphicsUnit.Point)
				lf.lfHeight = (int)-font.Size;
			IntPtr hFont = Win32Util.Win32API.CreateFont(lf.lfHeight, lf.lfWidth, lf.lfEscapement, lf.lfOrientation, lf.lfWeight, lf.lfItalic, lf.lfUnderline, lf.lfStrikeOut, lf.lfCharSet, lf.lfOutPrecision, lf.lfClipPrecision, lf.lfQuality, lf.lfPitchAndFamily, lf.lfFaceName);
			return hFont;
		}
		bool IsMultiLine(StringFormat stringFormat) {
			return (stringFormat.FormatFlags & StringFormatFlags.NoWrap) == 0;
		}
		public void DrawString(Graphics graphics, Color foreColor, string text, Rectangle drawBounds, 
			Rectangle clipBounds, StringFormat stringFormat, TextHighLight hightLight) {
			TextOutDraw draw = new TextOutDraw(this, graphics, ValidateString(text, hightLight, IsMultiLine(stringFormat)), drawBounds, clipBounds, stringFormat, hightLight);
			IntPtr hdc = graphics.GetHdc();
			IntPtr oldFontHandle = Win32Util.SelectObject(hdc, FontHandle);
			Win32Util.SetTextColor(hdc, foreColor);
			Win32Util.SetBkMode(hdc, Win32Util.TRANSPARENT);
			draw.DrawString(hdc);
			Win32Util.SelectObject(hdc, oldFontHandle);
			graphics.ReleaseHdc(hdc);
		}
		public Size GetStringSize(Graphics graphics, string text, StringFormat stringFormat) {
			return GetStringSize(graphics, text, stringFormat, int.MaxValue);
		}
		public Size GetStringSize(Graphics graphics, string text, StringFormat stringFormat, int maxWidth) {
			if(maxWidth <= 0) return GetStringSize(graphics, text, stringFormat);
			text = ValidateString(text, IsMultiLine(stringFormat));
			TextOutDraw draw = new TextOutDraw(this, graphics, text, new Rectangle(0, 0, maxWidth, int.MaxValue), Rectangle.Empty, stringFormat, null);
			return new Size(draw.MaxDrawWidth, draw.LineCount * Height);
		}
		public int[] GetMeasureString(Graphics graphics, string text, StringFormat stringFormat) {
			int[] widths = GetCharactersWidth(graphics, ValidateString(text, true, IsMultiLine(stringFormat)), stringFormat);
			return widths;
		}
		public int GetStringHeight(Graphics graphics, string text, int width, StringFormat stringFormat) {
			text = ValidateString(text, IsMultiLine(stringFormat));
			TextOutDraw draw = new TextOutDraw(this, graphics, text, new Rectangle(0, 0, width, int.MaxValue), Rectangle.Empty, stringFormat, null);
			return draw.LineCount * Height;
		}
		internal bool Underline { get { return underline; } }
		internal IntPtr FontHandle { get { return this.fontHandle; } }
		internal IntPtr FontUnderlineHandle { get { return this.fontUnderlineHandle != IntPtr.Zero ? this.fontUnderlineHandle : this.fontHandle; } }
		string ValidateString(string text, bool isMultiLine) {
			return ValidateString(text, false, null, isMultiLine);
		}
		string ValidateString(string text, bool removeReturn, bool isMultiLine) {
			return ValidateString(text, removeReturn, null, isMultiLine);
		}
		string ValidateString(string text, TextHighLight highLight, bool isMultiLine) {
			return ValidateString(text, false, highLight, isMultiLine);
		}
		public static int MaxSingleLineChars = 10000; 
		public static int MaxMultiLineChars = 10000;
		string ValidateString(string text, bool removeReturn, TextHighLight highLight, bool isMultiLine) {
			if(text == null) return string.Empty;
			int i = 0;
			if(isMultiLine) {
				if(text.Length > MaxMultiLineChars)
					text = text.Substring(0, MaxMultiLineChars);
			} else {
				if(text.Length > MaxSingleLineChars)
					text = text.Substring(0, MaxSingleLineChars);
			}
			while(i < text.Length) {
				if(IsReturn(text[i]) && removeReturn) {
					text = text.Remove(i, 1);
					if(highLight != null) {
						if(i <= highLight.EndIndex) {
							if(i <= highLight.StartIndex)
								highLight.StartIndex --;
							else highLight.Length --;
						}
					}
				}
				else i ++;
			}
			return text;
		}
		public int[] GetCharactersWidth(Graphics graphics, string text, StringFormat stringFormat) {
			PrepareWidthAndKerningArrays(graphics, text);
			int[] widths = new int[text.Length];
			int tabIndex = 0;
			for(int i = 0; i < text.Length; i ++) {
				if(!IsFontChar(text[i])) {
					widths[i] = this.averageCharWidth;
					if(IsTabStop(text[i])) {
						widths[i] = GetTabWidth(stringFormat, tabIndex ++);
					}
					if(IsReturn(text[i])) {
						widths[i] = 0;
					}
				} else {
					widths[i] = this.charsWidth[text[i] - this.firstChar];
					if(TextUtils.UseKerning) {
						if((i < text.Length - 1) && IsFontChar(text[i]) && IsFontChar(text[i + 1])) {
							widths[i] += GetKerningPairAmount(text[i], text[i + 1]);
						}
					}
				}
			}
			return widths;
		}
		public int GetItalicOffset(char ch) {
			if(this.italicOffset == null) return 0;
			if(ch < this.firstChar) return 0;
			int index = ch - this.firstChar;
			if(this.italicOffset.Length <= index) return 0;
			return this.italicOffset[index];
		}
		bool IsFontChar(char ch) {
			return ch >= this.firstChar && ch <= this.lastChar;
		}
		int GetTabWidth(StringFormat stringFormat, int tabIndex) {
			float tabOffSet = 0;
			float[] offSets = stringFormat.GetTabStops(out tabOffSet);
			if(tabIndex < offSets.Length)
				tabOffSet = offSets[tabIndex];
			if(tabOffSet > 0)
				return (int)tabOffSet;
			else return this.charsWidth[SpaceChar - this.firstChar] * TextUtils.TabStopSpacesCount;
		}
		void Calculate(Graphics graphics) {
			IntPtr hdc = graphics.GetHdc();
			IntPtr oldFontHandle = Win32Util.SelectObject(hdc, FontHandle);
			Win32Util.TEXTMETRIC textMetric = Win32Util.GetTextMetrics(hdc);
			this.height = textMetric.tmHeight;
			this.ascentHeight = textMetric.tmAscent;
			this.firstChar = textMetric.tmFirstChar;
			this.lastChar = textMetric.tmLastChar;
			this.currentLastChar = this.firstChar + InitialCharCount < this.lastChar ? (char)(this.firstChar + InitialCharCount) : this.lastChar;
			this.averageCharWidth = textMetric.tmAveCharWidth;
			CreateWidthAndKerningArrays(hdc);
			Win32Util.SelectObject(hdc, oldFontHandle);
			graphics.ReleaseHdc(hdc);
		}
		void CreateWidthAndKerningArrays(IntPtr hdc) {
			this.charsWidth = Win32Util.GetCharWidth(hdc, this.firstChar, this.currentLastChar);
			if(IsItalic)
				this.italicOffset = Win32Util.GetItalicOffset(hdc, this.firstChar, this.currentLastChar);
		}
		void CreateKernings(IntPtr hdc) {
			Win32Util.KerningPair[] pairs = Win32Util.GetKerningPairs(hdc);
			if(pairs != null) {
				kerningPairs = new Dictionary<uint, int>(pairs.Length);
				for(int i = 0; i < pairs.Length; i++) {
					this.kerningPairs[GetKerningPairHashCode(pairs[i].wFirst, pairs[i].wSecond)] = pairs[i].iKernAmount;
				}
			} else
				kerningPairs = new Dictionary<uint, int>(0);
		}
		void PrepareWidthAndKerningArrays(Graphics graphics, string text) {
			char newLastChar = this.currentLastChar;
			for(int i = 0; i < text.Length; i ++) {
				if(text[i] > newLastChar) 
					newLastChar = text[i];
			}
			if(newLastChar > this.currentLastChar) {
				this.currentLastChar = newLastChar + InitialCharCount < this.lastChar ? (char)(newLastChar + InitialCharCount) : this.lastChar;
				IntPtr hdc = graphics.GetHdc();
				IntPtr oldFontHandle = Win32Util.SelectObject(hdc, FontHandle);
				CreateWidthAndKerningArrays(hdc);
				Win32Util.SelectObject(hdc, oldFontHandle);
				graphics.ReleaseHdc(hdc);
			}
			if(TextUtils.UseKerning && kerningPairs == null) {
				IntPtr hdc = graphics.GetHdc();
				IntPtr oldFontHandle = Win32Util.SelectObject(hdc, FontHandle);
				CreateKernings(hdc);
				Win32Util.SelectObject(hdc, oldFontHandle);
				graphics.ReleaseHdc(hdc);
			}
		}
		uint GetKerningPairHashCode(char wFirst, char wSecond) {
			return (uint)wFirst * 0x10000 + wSecond;
		}
		int GetKerningPairAmount(char wFirst, char wSecond) {
			uint pairHashCode = GetKerningPairHashCode(wFirst, wSecond);
			int amount;
			return this.kerningPairs.TryGetValue(pairHashCode, out amount) ? amount : 0;
		}
	}
	internal class TextLine {
		int position;
		int length;
		int position2;
		int length2;
		TextLines lines;
		string elipsisText;
		int elipsisWidth;
		int[] elipsisWidths; 
		static int[] emptyWidths = new int[0];
		public TextLine(TextLines lines) {
			this.lines = lines;
			elipsisText = "";
			elipsisWidths = emptyWidths;
		}
		public int Position { get { return position; } set {position = value;} }
		public int Length { get { return length; } set {length = value;}}
		public int Position2 { get { return position2; } set {position2 = value;} }
		public int Length2 { get { return length2; } set {length2 = value;}}
		public int TextWidth { get { return GetTextWidth(position, length); } }
		public int Text2Width { get { return GetTextWidth(position2, length2); } }
		public string ElipsisText { get { return elipsisText; } }
		public int ElipsisWidth { get { return elipsisWidth; } }
		public int[] ElipsisWidths { get { return elipsisWidths; } }
		public int LineWidth { get { return TextWidth + ElipsisWidth + Text2Width; } }
		public int GetCharWidth(int index) {
			return GetCharWidth(index, false);
		}
		public int GetCharWidth(int index, bool includeItalic) {
			index += Position;
			return Lines.Widths[index] + (includeItalic ? Lines.GetItalicOffset(index) : 0);
		}
		public bool HasElipsis { 
			get { return elipsisText.Length > 0; }
			set {
				if(HasElipsis == value) return;
				elipsisWidth = 0;
				elipsisWidths = emptyWidths;
				if(!value) {
					elipsisText = "";
				} else {
					elipsisText = "...";
					elipsisWidths = GetCharactersWidth(elipsisText);
					for(int i = 0; i < elipsisWidths.Length; i ++)
						elipsisWidth += elipsisWidths[i];
				}
			}
		}
		public void UpdateTrimmingLine(StringTrimming trimming, int drawBoundsWidth) {
			if (trimming == StringTrimming.None || TextWidth <= drawBoundsWidth) return;
			int len = 0;
			int lineLength = Length;
			HasElipsis = GetIsTrimmingElipsis(trimming);
			int drawWidth = drawBoundsWidth - ElipsisWidth;
			switch(trimming) {
				case StringTrimming.Character:
				case StringTrimming.EllipsisCharacter: {
					int lineWidth = 0;
					while(lineWidth + GetCharWidth(len, true) <= drawWidth) { 
						lineWidth += GetCharWidth(len++);
					}
					break;
				}
				case StringTrimming.Word:
				case StringTrimming.EllipsisWord: {
					int wordLength = GetNextWord(Position, lineLength);
					int lineWidth = GetTextWidth(Position, wordLength);
					while(lineWidth <= drawWidth) {
						len += wordLength;
						wordLength = GetNextWord(Position + len, lineLength);
						lineWidth = GetTextWidth(Position, len + wordLength);
					}
					break;
				}
				case StringTrimming.EllipsisPath: {
					int firstWidth = 0;				
					int lastWidth = 0;
					int pos2 = lineLength - 1;
					int charWidth = GetCharWidth(0);
					while (firstWidth + lastWidth + charWidth < drawWidth) { 
						if(firstWidth <= lastWidth) 
							firstWidth += GetCharWidth(len ++);
						else lastWidth += GetCharWidth(pos2 --);
						charWidth = firstWidth <= lastWidth ? GetCharWidth(len) : GetCharWidth(pos2);
					}
					pos2 ++;
					Position2 = pos2;
					Length2 = lineLength - pos2;
					break;
				}
			}
			Length = len;
		}
		protected bool GetIsTrimmingElipsis(StringTrimming trimming) {
				return trimming == StringTrimming.EllipsisCharacter
					|| trimming == StringTrimming.EllipsisPath
					|| trimming == StringTrimming.EllipsisWord;
		}
		protected TextLines Lines { get { return lines; } }
		protected int[] Widths { get { return lines.Widths; } }
		protected int[] GetCharactersWidth(string text) { return lines.GetCharacterWidths(text);  }
		protected int GetTextWidth(int pos, int len) {
			return Lines.Draw.GetTextWidth(pos, len);
		}
		protected int GetNextWord(int startIndex, int textLength) {
			return Lines.Draw.GetNextWord(startIndex, textLength);
		}
	}
	internal class TextLines : List<TextLine> {
		TextOutDraw draw;
		public TextLines(TextOutDraw draw) {
			this.draw = draw;
		}
		public void Add(int position, int length) {
			TextLine line = AddLine();
			line.Position = position;
			line.Length = length;
		}
		public int Width {
			get {
				int width = 0;
				for(int i = 0; i < Count; i ++) {
					int tWidth = this[i].LineWidth;
					if(width < tWidth)
						width = tWidth;
				}
				return width;
			}
		}
		public int[] Widths { get { return Draw.Widths; } }
		public int[] GetCharacterWidths(string text) {
			return Draw.GetCharactersWidth(text);
		}
		public int GetItalicOffset(int position) {
			return Draw.GetItalicOffset(position);
		}
		public TextOutDraw Draw { get { return draw; } }
		protected TextLine AddLine() {
			TextLine line = new TextLine(this);
			Add(line);
			return line;
		}
	}
	public class TextOutDraw {
		Graphics graphics;
		string text;
		Rectangle drawBounds;
		Rectangle clipedBounds;
		int[] widths;
		bool[] hotPrefixes;
		StringFormat format;
		TextHighLight highLight;
		FontCache fontCache;
		int pos;
		int drawTop;
		int linepos;
		bool isCliped;
		bool measureTrailingSpaces;
		bool isNoWrap;
		bool trimming;
		TextLines lines;
		public TextOutDraw(FontCache fontCache, Graphics graphics, string text, 
			Rectangle drawBounds, Rectangle clipBounds, 
			StringFormat format, TextHighLight highLight) {
			this.fontCache = fontCache;
			this.graphics = graphics;
			this.format = format;
			this.highLight = highLight;
			this.text = ValidateHotPrefix(text);
			this.drawBounds = ApplyTextUtilsOffsets(drawBounds);
			if(clipBounds.IsEmpty) 
				this.clipedBounds = drawBounds;
			else {
				this.isCliped = true;
				this.clipedBounds = clipBounds;
				this.clipedBounds.Intersect(drawBounds);
			}
			this.widths = GetCharactersWidth(Text);
			this.isNoWrap = (format.FormatFlags & StringFormatFlags.NoWrap) == StringFormatFlags.NoWrap;
			this.trimming = format.Trimming != StringTrimming.None;
			this.isCliped |= ((format.FormatFlags & StringFormatFlags.NoClip) == 0) ||
				(IsNoWrap && !trimming);
			this.measureTrailingSpaces = (format.FormatFlags & StringFormatFlags.MeasureTrailingSpaces) == StringFormatFlags.MeasureTrailingSpaces;
			CreateLines();
		}
		public FontCache FontCache { get { return fontCache; } }
		public virtual int FontHeight { get { return FontCache.Height; } }
		public virtual int[] GetCharactersWidth(string text) { return FontCache.GetCharactersWidth(this.graphics, text, Format); }
		public string Text { get { return text; } }
		public StringFormat Format { get { return format; } }
		public Rectangle DrawBounds { get { return drawBounds; } }
		public Rectangle ClipedBounds { get { return clipedBounds; } }
		public int Pos { get { return pos; } set { pos = value; } }
		public int[] Widths { get { return widths; } }
		public bool IsCliped { get { return isCliped; } }
		public bool MeasureTrailingSpaces { get { return measureTrailingSpaces; } }
		public void DrawString(IntPtr hdc) {
			if(Text.Length == 0) return;
			DrawStringLines(hdc);
		}
		public int LineCount { get { return this.lines != null ? lines.Count : 1; } }
		public int MaxDrawWidth { 
			get { 
				int width = this.lines.Width;
				return width < DrawBounds.Width ? width : DrawBounds.Width;
			}
		}
		protected int GetItalicOffset(char ch) {
			if(FontCache == null) 
				return 0;
			else return FontCache.GetItalicOffset(ch);
		}
		public int GetItalicOffset(int index) {
			return GetItalicOffset(Text[index]);
		}
		bool IsNoWrap { get { return isNoWrap; } }
		void CreateLines() {
			lines = new TextLines(this);
			if(IsNoWrap) {
				if(Text.IndexOf(FontCache.ReturnChar) > -1) 
					CreateReturnLines();
				else CreateNoWrapLines();
			} else CreateWrapLines();
		}
		bool CanAddOneMoreLine(int lineTop) {
			if(!trimming && IsNoWrap)
				return this.drawTop < DrawBounds.Height;
			else
				return (this.drawTop + FontHeight <= DrawBounds.Height) || this.lines.Count == 0;
		}
		void CreateWrapLines() {
			while(Pos < Text.Length && CanAddOneMoreLine(this.drawTop)) {
				while(Pos < Text.Length && FontCache.IsReturn(Text[Pos])) {
					AddNewLineToList();
				}
				if(Pos >= Text.Length || !CanAddOneMoreLine(this.drawTop)) break;
				if(this.widths[Pos] > DrawBounds.Width) break;
				int wordLength = GetNextWord(Pos, Text.Length);
				int newLineWidth = GetTextWidth(this.linepos, this.pos - this.linepos + wordLength);
				if(newLineWidth > DrawBounds.Width) {
					if(Pos - this.linepos > 0) {
						AddNewLineToList();
					} 
					else {
						int lineWidth = widths[Pos];
						for(int i = 1; i < wordLength; i ++) {
							if(lineWidth + widths[Pos + i] + GetItalicOffset(Pos + i) > DrawBounds.Width) { 
								this.pos += i;
								break;
							}
							lineWidth += widths[Pos + i];
						}
						AddNewLineToList();
						if(wordLength == 1) break;
					}
				} 
				else {
					this.pos += wordLength;
				}
			}
			if(Pos - this.linepos > 0)
				AddNewLineToList();
			this.pos = 0;
		}
		void CreateReturnLines() {
			while(Pos < Text.Length && CanAddOneMoreLine(this.drawTop)) {
				if(FontCache.IsReturn(Text[Pos]))
					AddNewLineToList();
				else this.pos ++;
			}
			if(Pos - this.linepos > 0)
				AddNewLineToList();
			this.pos = 0;
		}
		void CreateNoWrapLines() {
			lines.Add(0, Text.Length);
			lines[0].UpdateTrimmingLine(Format.Trimming, drawBounds.Width);
		}
		bool IsTrimmingElipsis { 
			get {
				return trimming && (Format.Trimming == StringTrimming.EllipsisCharacter
					|| Format.Trimming == StringTrimming.EllipsisPath
					|| Format.Trimming == StringTrimming.EllipsisWord);
			} 
		}
		void AddNewLineToList() {
			int length = Pos - this.linepos;
			if (length > 0 && this.trimming) {
				int wordLength = GetNextWord(Pos - 1, Text.Length);
				if (wordLength - 1 > 0) {
					length += wordLength - 1;
					this.pos += wordLength - 1;
				}
			}
			if(length > 0 && !MeasureTrailingSpaces) {
				while(length > 0 && CanRemoveSymbol(this.linepos + length - 1))
					length --;
				while(Pos < Text.Length && CanRemoveSymbol(Pos)) 
					this.pos ++;
			}
			if(Pos < Text.Length && FontCache.IsReturn(Text[Pos]))
				this.pos ++;
			lines.Add(linepos, length);
			if (this.trimming) {
				lines[lines.Count - 1].UpdateTrimmingLine(Format.Trimming, drawBounds.Width);
			}
			this.linepos = Pos;
			this.drawTop += FontHeight;
		}
		bool CanRemoveSymbol(int pos) {
			if(char.IsWhiteSpace(Text[pos]) && !FontCache.IsReturn(Text[pos])  &&
				(this.hotPrefixes == null || !this.hotPrefixes[pos])) 
				return true;
			else return false;
		}
		public int GetNextWord(int startIndex, int textLength) {
			if (char.IsWhiteSpace(Text[startIndex])) return 1;
			int newPos = startIndex + 1;
			while (newPos < textLength && !char.IsWhiteSpace(Text[newPos]) && !FontCache.IsReturn(Text[newPos]))
				newPos ++;
			return newPos - startIndex;
		}
		public int GetTextWidth(int startIndex, int length) {
			if(length == 0) return 0;
			int width = 0;
			for(int i = 0; i < length; i ++)
				width += widths[startIndex + i];
			return width + GetItalicOffset(startIndex + length - 1);
		}
		int[] GetLineWidths(int length) {
			return GetLineWidths(Pos, length);
		}
		int[] GetLineWidths(int startIndex, int length) {
			int[] linewidths = new int[length];
			for(int i = 0; i < length; i ++)
				linewidths[i] = Widths[startIndex + i];
			return linewidths;
		}
		void DrawStringLines(IntPtr hdc) {
			this.isCliped |= DrawBounds.Height < FontHeight * lines.Count;
			this.drawBounds = CorrectRectangleTop(DrawBounds, FontHeight * lines.Count);
			for(int i = 0; i < this.lines.Count; i ++) {
				Rectangle r = new Rectangle(DrawBounds.X, DrawBounds.Y + i * FontHeight, DrawBounds.Width, FontHeight);
				DrawStringLine(hdc, lines[i], r); 
			}
		}
		void DrawStringLine(IntPtr hdc, TextLine line, Rectangle bounds) {
			bounds = CorrectRectangleLeft(bounds, line.LineWidth);
			DrawStringLine(hdc, line.Position, line.Length, bounds.X, bounds.Y, this.highLight);
			bounds.X += line.TextWidth;
			if(line.HasElipsis) {
				DrawText(hdc, line.ElipsisText, bounds.X, bounds.Y, line.ElipsisWidths);
				bounds.X += line.ElipsisWidth;
			}
			if(line.Length2 > 0)
				DrawStringLine(hdc, line.Position2, line.Length2, bounds.X, bounds.Y, this.highLight);
		}
		void DrawStringLine(IntPtr hdc, int startPos, int length, int x, int y, TextHighLight highLight) {
			if(highLight == null || !highLight.IsTextHighLighted(startPos, length))
				DrawStringLine(hdc, startPos, length, x, y);
			else {
				if(startPos < highLight.StartIndex) 
					DrawStringLine(hdc, ref startPos, ref length, ref x, y, highLight.StartIndex - startPos);
				int len = length < highLight.EndIndex - startPos ? length : highLight.EndIndex - startPos;
				DrawHighLightStringLine(hdc, ref startPos, ref length, ref x, y, len);
				if(length > 0) 
					DrawStringLine(hdc, startPos, length, x, y);
			}
		}
		protected virtual void DrawHighLightStringLine(IntPtr hdc, ref int startPos, ref int length, ref int x, int y, int highLightLen) {
			Rectangle hBounds = new Rectangle(x, y, GetTextWidth(startPos, highLightLen), FontHeight);
			if(!ClipedBounds.IsEmpty && hBounds.Right >= ClipedBounds.Right) {
				hBounds.Width = Math.Max(0, ClipedBounds.Right - hBounds.X);
			}
			if(highLight.BackColor != Color.Empty) {
				IntPtr brush = Win32Util.CreateSolidBrush(highLight.BackColor);
				Win32Util.FillRect(hdc, hBounds, brush);
				Win32Util.DeleteObject(brush);
			}
			int prevColor = Win32Util.GetTextColor(hdc);
			Win32Util.SetTextColor(hdc, highLight.ForeColor);
			DrawStringLine(hdc, ref startPos, ref length, ref x, y, highLightLen);
			Win32Util.SetTextColor(hdc, prevColor);
		}
		void DrawStringLine(IntPtr hdc, ref int startPos, ref int length, ref int x, int y, int highLightLen) {
			DrawStringLine(hdc, startPos, highLightLen, x, y);
			x += GetTextWidth(startPos, highLightLen);
			startPos += highLightLen;
			length -= highLightLen;
		}
		void DrawStringLine(IntPtr hdc, int startPos, int length, int x, int y) {
			int[] lineWidths = GetLineWidths(startPos, length);
			bool[] hotkeys = null;
			if(this.hotPrefixes != null) {
				hotkeys = new bool[lineWidths.Length];
				for(int i = 0; i < length; i ++)
					hotkeys[i] = this.hotPrefixes[startPos + i];
			}
			string lineText = startPos == 0 && length == Text.Length ? Text : Text.Substring(startPos, length);
			DrawStringLine(hdc, lineText, x, y, lineWidths, hotkeys);
		}
		void DrawStringLine(IntPtr hdc, string lineText, int x, int y, int[] lineWidths, bool[] hotkeys) {
			if(GetHotKeysCount(hotkeys, lineText.Length) == 0) 
				DrawStringLine(hdc, lineText, x, y, lineWidths);
			else DrawStringLineWithHotkeys(hdc, lineText, x, y, lineWidths, hotkeys);
		}
		protected virtual void DrawStringLineWithHotkeys(IntPtr hdc, string lineText, int textX, int textY, int[] lineWidths, bool[] hotkeys) {
			string text;
			int[] textWidths;
			int x;
			GetHotKeyInfo(lineText, textX, lineWidths, hotkeys, out text, out textWidths, out x);
			DrawStringLine(hdc, text, x, textY, textWidths);
			IntPtr oldFontHandle = IntPtr.Zero;
			if(FontCache != null)
				oldFontHandle = Win32Util.SelectObject(hdc, FontCache.FontUnderlineHandle);
			x = textX;
			textWidths = new int[1];
			for(int i = 0; i < lineText.Length; i ++) {
				if(hotkeys[i]) {
					textWidths[0] = lineWidths[i];
					DrawStringLine(hdc, lineText[i].ToString(), x, textY, textWidths);
				} 
				x += lineWidths[i];
			}
			if(oldFontHandle != IntPtr.Zero)
				Win32Util.SelectObject(hdc, oldFontHandle);
		}
		protected void DrawStringLine(IntPtr hdc, string lineText, int x, int y, int[] lineWidths) {
			if(lineText.Length  == 0) return;
			lineText = ReplaceTabsWithSpaces(lineText);
			DrawText(hdc, lineText, x, y, lineWidths);
		}
		protected virtual void DrawText(IntPtr hdc, string text, int x, int y, int[]textWidths) {
			Win32Util.ExtTextOut(hdc, x, y, isCliped, ClipedBounds, text, textWidths);
		}
		void GetHotKeyInfo(string text, int x, int[] lineWidths, bool[] hotkeys, 
			out string newText, out int[] newLineWidths, out int newX) {
			newX = x;
			newLineWidths = new int[lineWidths.Length - GetHotKeysCount(hotkeys, text.Length)];
			char[] temp = new char[text.Length];
			int index = 0;
			for(int i = 0; i < text.Length; i ++) {
				if(!hotkeys[i]) {
					temp[index] = text[i];
					newLineWidths[index++] = lineWidths[i];
				} else {
					if(index == 0) 
						newX += lineWidths[i];
					else newLineWidths[index - 1] += lineWidths[i];
				}
			}
			newText = new string(temp, 0, index);
		}
		int GetHotKeysCount(bool[] hotkeys, int length) {
			if(hotkeys == null) return 0;
			int count = 0;
			for(int i = 0; i < length; i ++)
				if(hotkeys[i]) count ++;
			return count;
		}
		protected virtual bool IsFontUnderline { get { return FontCache != null ? FontCache.Underline : false; } }
		string ReplaceTabsWithSpaces(string st) {
			return st.Replace(FontCache.TabStopChar, FontCache.SpaceChar);
		}
		Rectangle CorrectRectangleLeft(Rectangle bounds, int lineWidth) {
			if(Format.Alignment != StringAlignment.Near) {
				if(Format.Alignment == StringAlignment.Far)
					bounds.X += bounds.Width - lineWidth;
				else bounds.X += (bounds.Width - lineWidth) / 2;
			}
			return bounds;
		}
		Rectangle CorrectRectangleTop(Rectangle bounds) {
			return CorrectRectangleTop(bounds, FontHeight);
		}
		Rectangle CorrectRectangleTop(Rectangle bounds, int height) {
			if(bounds.Height < height && trimming) return bounds;
			if(bounds.Height != FontHeight  && Format.LineAlignment != StringAlignment.Near) {
				if(Format.LineAlignment == StringAlignment.Far)
					bounds.Y += bounds.Height - height;
				else bounds.Y += (bounds.Height - height) / 2;
			}
			return bounds;
		}
		string ValidateHotPrefix(string text) {
			this.hotPrefixes = null;
			if(format.HotkeyPrefix == HotkeyPrefix.None) return text;
			if(format.HotkeyPrefix == HotkeyPrefix.Show && !IsFontUnderline)
				this.hotPrefixes = new bool[text.Length];
			int i = 0;
			bool hotPrefix = false;
			while(i < text.Length) {
				if(text[i] == '&' && !hotPrefix) {
					hotPrefix = true;
					text = text.Remove(i, 1);
				} else {
					if(this.hotPrefixes != null && text[i] != '&')
						this.hotPrefixes[i] = hotPrefix;
					hotPrefix = false;
					i ++;
				}
			}
			return text;
		}
		Rectangle ApplyTextUtilsOffsets(Rectangle bounds) {
			bounds.X += TextUtils.LeftOffset;
			bounds.Y += TextUtils.TopOffset;
			bounds.Width -= TextUtils.RightOffset + TextUtils.LeftOffset;
			bounds.Height -= TextUtils.BottomOffset + TextUtils.TopOffset;
			return bounds;
		}
	}
	internal class FontsCache : IDisposable {
		Dictionary<GraphicsUnit, ConstDpiFontsCache> dpiHash;
		class GraphicsUnitComparer : IEqualityComparer<GraphicsUnit> {
			public bool Equals(GraphicsUnit x, GraphicsUnit y) {
				return x == y;
			}
			public int GetHashCode(GraphicsUnit obj) {
				return ((int)obj).GetHashCode();
			}
		}
		public FontsCache() {
			this.dpiHash = new Dictionary<GraphicsUnit, ConstDpiFontsCache>(new GraphicsUnitComparer());
		}
		~FontsCache() {
			((IDisposable)this).Dispose();
		}
		public FontCache this[Graphics graphics, Font font] { get { return GetFontCacheByFont(graphics, font); } }
		public void DrawString(Graphics graphics, string text, Font font, Color foreColor, Rectangle drawBounds, Rectangle clipBounds, StringFormat stringFormat, TextHighLight hightLight) {
			if(graphics == null || text == null || font == null || stringFormat == null) return;
			this[graphics, font].DrawString(graphics, foreColor, text, drawBounds, clipBounds, stringFormat, hightLight);
		}
		public Size GetStringSize(Graphics graphics, string text, Font font, StringFormat stringFormat) {
			return this[graphics, font].GetStringSize(graphics, text, stringFormat);
		}
		public Size GetStringSize(Graphics graphics, string text, Font font, StringFormat stringFormat, int maxWidth) {
			return this[graphics, font].GetStringSize(graphics, text, stringFormat, maxWidth);
		}
		public int[] GetMeasureString(Graphics graphics, string text, Font font, StringFormat stringFormat) {
			return this[graphics, font].GetMeasureString(graphics, text, stringFormat);
		}
		public int GetStringHeight(Graphics graphics, string text, Font font, int width, StringFormat stringFormat) {
			return this[graphics, font].GetStringHeight(graphics, text, width, stringFormat);
		}
		public int GetFontHeight(Graphics graphics, Font font) {
			return this[graphics, font].Height;
		}
		public int GetFontAscentHeight(Graphics graphics, Font font) {
			return this[graphics, font].AscentHeight;
		}
		FontCache GetFontCacheByFont(Graphics graphics, Font font) {
			ConstDpiFontsCache dpiCache;
			if(!dpiHash.TryGetValue(graphics.PageUnit, out dpiCache)) {
				dpiCache = new ConstDpiFontsCache();
				dpiHash.Add(graphics.PageUnit, dpiCache);
			}
			return dpiCache[graphics, font];
		}
		void IDisposable.Dispose() {
			Clear();
		}
		void Clear() {
			foreach(IDisposable cache in this.dpiHash.Values) {
				cache.Dispose();
			}
			this.dpiHash.Clear();
		}
	}
	internal class ConstDpiFontsCache : IDisposable {
		Hashtable fonts;
		public ConstDpiFontsCache() {
			this.fonts = new Hashtable();
		}
		public FontCache this[Graphics graphics, Font font] { get { return GetFontCacheByFont(graphics, font); } }
		public void DrawString(Graphics graphics, string text, Font font, Color foreColor, Rectangle drawBounds, Rectangle clipBounds, StringFormat stringFormat, TextHighLight hightLight) {
			if(graphics == null || text == null || font == null || stringFormat == null) return;
			this[graphics, font].DrawString(graphics, foreColor, text, drawBounds, clipBounds, stringFormat, hightLight);
		}
		public Size GetStringSize(Graphics graphics, string text, Font font, StringFormat stringFormat) {
			return this[graphics, font].GetStringSize(graphics, text, stringFormat);
		}
		public Size GetStringSize(Graphics graphics, string text, Font font, StringFormat stringFormat, int maxWidth) {
			return this[graphics, font].GetStringSize(graphics, text, stringFormat, maxWidth);
		}
		public int[] GetMeasureString(Graphics graphics, string text, Font font, StringFormat stringFormat) {
			return this[graphics, font].GetMeasureString(graphics, text, stringFormat);
		}
		public int GetStringHeight(Graphics graphics, string text, Font font, int width, StringFormat stringFormat) {
			return this[graphics, font].GetStringHeight(graphics, text, width, stringFormat);
		}
		public int GetFontHeight(Graphics graphics, Font font) {
			return this[graphics, font].Height;
		}
		public int GetFontAscentHeight(Graphics graphics, Font font) {
			return this[graphics, font].AscentHeight;
		}
		FontCache GetFontCacheByFont(Graphics graphics, Font font) {
			FontCache fontCache = this.fonts[font] as FontCache;
			return fontCache != null ? fontCache : AddFontCache(graphics, font);
		}
		FontCache AddFontCache(Graphics graphics, Font font) {
			FontCache cache = new FontCache(graphics, font);
			this.fonts[font] = cache;
			return cache;
		}
		void IDisposable.Dispose() {
			Clear();
		}
		void Clear() {
			foreach(FontCache cache in this.fonts.Values) {
				cache.Dispose();
			}
			this.fonts.Clear();
		}
	}
	internal class Win32Util {
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto), ComVisible(false)]
		public class LOGFONT {
			public int lfHeight;
			public int lfWidth;
			public int lfEscapement;
			public int lfOrientation;
			public int lfWeight;
			public byte lfItalic;
			public byte lfUnderline;
			public byte lfStrikeOut;
			public byte lfCharSet;
			public byte lfOutPrecision;
			public byte lfClipPrecision;
			public byte lfQuality;
			public byte lfPitchAndFamily;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=0x20)]
			public string lfFaceName;
		}
		private const int ETO_CLIPPED = 0x0004;
		[StructLayout(LayoutKind.Sequential)]
			public struct RECT {
			public int left;
			public int top;
			public int right;
			public int bottom;
		}
		[StructLayout(LayoutKind.Sequential)]
		public struct POINT { 
			public int x; 
			public int y; 
		}
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
		public struct TEXTMETRIC {
			public int tmHeight;
			public int tmAscent;
			public int tmDescent;
			public int tmInternalLeading;
			public int tmExternalLeading;
			public int tmAveCharWidth;
			public int tmMaxCharWidth;
			public int tmWeight;
			public int tmOverhang;
			public int tmDigitizedAspectX;
			public int tmDigitizedAspectY;
			public char tmFirstChar;
			public char tmLastChar;
			public char tmDefaultChar;
			public char tmBreakChar;
			public byte tmItalic;
			public byte tmUnderlined;
			public byte tmStruckOut;
			public byte tmPitchAndFamily;
			public byte tmCharSet;
		}
		[StructLayout(LayoutKind.Sequential)]
		public struct OUTLINETEXTMETRIC {
			public uint otmSize;
			public TEXTMETRIC otmTextMetrics;
			public byte otmFiller;
			public PANOSE otmPanoseNumber;
			public uint otmfsSelection;
			public uint otmfsType;
			public int otmsCharSlopeRise;
			public int otmsCharSlopeRun;
			public int otmItalicAngle;
			public uint otmEMSquare;
			public int otmAscent;
			public int otmDescent;
			public uint otmLineGap;
			public uint otmsCapEmHeight;
			public uint otmsXHeight;
			public RECT otmrcFontBox;
			public int otmMacAscent;
			public int otmMacDescent;
			public uint otmMacLineGap;
			public uint otmusMinimumPPEM;
			public POINT otmptSubscriptSize;
			public POINT otmptSubscriptOffset;
			public POINT otmptSuperscriptSize;
			public POINT otmptSuperscriptOffset;
			public uint otmsStrikeoutSize;
			public int otmsStrikeoutPosition;
			public int otmsUnderscoreSize;
			public int otmsUnderscorePosition;
			public uint otmpFamilyName;
			public uint otmpFaceName;
			public uint otmpStyleName;
			public uint otmpFullName;
		}
		[StructLayout(LayoutKind.Sequential)]
		public struct PANOSE {
			public byte bFamilyType;
			public byte bSerifStyle;
			public byte bWeight;
			public byte bProportion;
			public byte bContrast;
			public byte bStrokeVariation;
			public byte bArmStyle;
			public byte bLetterform;
			public byte bMidline;
			public byte bXHeight;
		}
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public struct KerningPair {
			public char wFirst;
			public char wSecond;
			public int iKernAmount;
		}
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public struct ABC {
			public int abcA;
			public int abcB;
			public int abcC;
		}
		[System.Security.SuppressUnmanagedCodeSecurity]
		public class Win32API {
			[DllImport("USER32.dll", CharSet = CharSet.Auto)]
			public static extern int DrawText(IntPtr hdc, String text, int textLen, ref RECT gdiRect, int format);
			[DllImport("gdi32.dll")]
			public static extern IntPtr CreateSolidBrush(int color);
			[DllImport("gdi32.dll")]
			public static extern bool DeleteObject(IntPtr hObject);	
			[DllImport("User32.dll")]
			public static extern int FillRect(IntPtr hdc, ref RECT r, IntPtr brush);
			[DllImport("gdi32.dll")]
			public static extern int GetTextColor(IntPtr hdc);
			[DllImport("gdi32.dll")]
			public static extern int SetBkMode(IntPtr hdc, int mode);
			[DllImport("gdi32.dll")]
			public static extern IntPtr SelectObject(IntPtr hdc, IntPtr hgdiObj);
			[DllImport("gdi32.dll")]
			public static extern int SetTextColor(IntPtr hdc, int color);
			[DllImport("gdi32.dll")]
			public static extern int SetBkColor(IntPtr hdc, int color);
			[DllImport("gdi32.dll", CharSet = CharSet.Auto)]
			public static extern int ExtTextOut(IntPtr hdc, int x, int y, int options,
				ref RECT clip, string str, int len, 
				[In, MarshalAs(UnmanagedType.LPArray)] int[] widths);
			[DllImport("gdi32.dll", CharSet=CharSet.Auto)]
			public static extern bool GetTextMetrics(IntPtr hdc, out TEXTMETRIC lptm);
			[DllImport("gdi32.dll", CharSet=CharSet.Auto)]
			public static extern uint GetOutlineTextMetrics(IntPtr hdc, uint strSize, IntPtr lptm);
			[DllImport("gdi32.dll", CharSet=CharSet.Auto)]
			public static extern bool GetCharWidth(IntPtr hdc, uint firstChar, uint lastChar, 
				[Out, MarshalAs(UnmanagedType.LPArray)] int[] widths); 
			[DllImport("gdi32.dll", CharSet=CharSet.Auto)]
			public static extern bool GetCharABCWidths(IntPtr hdc, uint firstChar, uint lastChar, 
				[Out, MarshalAs(UnmanagedType.LPArray)] ABC[] widths); 
			[DllImport("Gdi32.dll", CharSet=CharSet.Auto)]
			public static extern int GetKerningPairs(IntPtr hdc, int nNumPairs, 
				[In, Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex=1)]
				KerningPair[] kerningPairs);
			[DllImport("gdi32.dll")]
			public static extern int GetDeviceCaps(IntPtr hdc, int nIndex);
			[DllImport("GDI32.dll")]
			public static extern IntPtr GetStockObject(int fnObject);		
			[DllImport("gdi32.dll")]
			public static extern IntPtr CreateFont(
				int nHeight,			   
				int nWidth,				
				int nEscapement,		   
				int nOrientation,		  
				int fnWeight,			  
				int fdwItalic,		   
				int fdwUnderline,		
				int fdwStrikeOut,		
				int fdwCharSet,		  
				int fdwOutputPrecision,  
				int fdwClipPrecision,	
				int fdwQuality,		  
				int fdwPitchAndFamily,   
				string lpszFace		   
				);
		}
		public static IntPtr CreateSolidBrush(IntPtr hdc) {
			return Win32API.CreateSolidBrush(GetTextColor(hdc));
		}
		public static IntPtr CreateSolidBrush(int color) {
			return Win32API.CreateSolidBrush(color);
		}
		public static IntPtr CreateSolidBrush(Color color) {
			return Win32API.CreateSolidBrush(ColorTranslator.ToWin32(color));
		}
		public static void DeleteObject(IntPtr hObject) {
			Win32API.DeleteObject(hObject);
		}
		public static void FillRect(IntPtr hdc, Rectangle bounds, IntPtr hBrush) {
			RECT rect = GetRect(bounds);
			Win32API.FillRect(hdc, ref rect, hBrush);
		}
		public static int GetTextColor(IntPtr hdc) {
			return Win32API.GetTextColor(hdc);
		}
		public const int TRANSPARENT = 1;
		public static void SetBkMode(IntPtr hdc, int mode) {
			Win32API.SetBkMode(hdc, mode);
		}
		public static IntPtr SelectObject(IntPtr hdc, IntPtr handle) {
			return Win32API.SelectObject(hdc, handle);
		}
		public static void SetTextColor(IntPtr hdc, Color color) {
			Win32API.SetTextColor(hdc, ColorTranslator.ToWin32(color));
		}
		public static void SetTextColor(IntPtr hdc, int color) {
			Win32API.SetTextColor(hdc, color);
		}
		public static void SetBkColor(IntPtr hdc, Color color) {
			Win32API.SetBkColor(hdc, ColorTranslator.ToWin32(color));
		}
		public static void ExtTextOut(IntPtr hdc, int x, int y, bool isCliped, Rectangle clip, string str, int[] spacings) {
			RECT rect = GetRect(clip);
			if(!isCliped)
				rect.top = y;
			int format = isCliped ? ETO_CLIPPED : 0;
			Win32API.SetBkColor(hdc, 1);
			IntPtr saveBrush = SelectObject(hdc, Win32API.GetStockObject(0)); 
			Win32API.ExtTextOut(hdc, x, y, format, ref rect, str, str.Length, spacings);
			SelectObject(hdc, saveBrush);
		}
		public static TEXTMETRIC GetTextMetrics(IntPtr hdc) {
			TEXTMETRIC lptm;
			bool rc = Win32API.GetTextMetrics(hdc, out lptm);
			return lptm;
		}
		public static bool GetOutlineTextMetrics(IntPtr hdc, ref OUTLINETEXTMETRIC lptm) {
			uint size = Win32API.GetOutlineTextMetrics(hdc, 0, IntPtr.Zero);
			if (size == 0) return false;
			IntPtr buffer = Marshal.AllocHGlobal((int)size);
			bool res = true;
			try {
				res = Win32API.GetOutlineTextMetrics(hdc, size, buffer) > 0;
				if(res)
					lptm = (OUTLINETEXTMETRIC)Marshal.PtrToStructure(buffer, typeof(OUTLINETEXTMETRIC));
			}
			finally {
				Marshal.FreeHGlobal(buffer);
			}
			return res;
		}
		public static int[] GetCharWidth(IntPtr hdc, uint firstChar, uint lastChar) {
			uint charsCount = lastChar - firstChar + 1;
			int[]  widths = new int[charsCount];
			bool res = Win32API.GetCharWidth(hdc, firstChar, lastChar, widths);
			return widths;
		}
		public static int[] GetItalicOffset(IntPtr hdc, uint firstChar, uint lastChar) {
			uint charsCount = lastChar - firstChar + 1;
			int[]  widths = new int[charsCount];
			ABC[] abcWidths = new ABC[charsCount];
			bool res = Win32API.GetCharABCWidths(hdc, firstChar, lastChar, abcWidths);
			for(int i = 0; i < widths.Length; i ++) {
				if(res)
					widths[i] = abcWidths[i].abcC < 0 ? -abcWidths[i].abcC : 0;
				else widths[i] = 0;
			}
			return widths;
		}
		public static KerningPair[] GetKerningPairs(IntPtr hdc) {
			int count = Win32API.GetKerningPairs(hdc, 0, null);
			if(count == 0) return null;
			KerningPair[] pairs = new KerningPair[count];
			Win32API.GetKerningPairs(hdc, count, pairs);
			return pairs;
		}
		const int LOGPIXELSX = 88;
		const int LOGPIXELSY = 90;
		public static int GetLogicPixelPerInchX(IntPtr hdc) {
			return Win32API.GetDeviceCaps(hdc, LOGPIXELSX);
		}
		public static int GetLogicPixelPerInchY(IntPtr hdc) {
			return Win32API.GetDeviceCaps(hdc, LOGPIXELSY);
		}
		static RECT GetRect(Rectangle bounds) {
			RECT rect;
			rect.top = bounds.Top;
			rect.left = bounds.Left;
			rect.bottom = bounds.Bottom;
			rect.right = bounds.Right;
			return rect;
		}
	}
}
