#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System.Reflection;
using System.Runtime.InteropServices;
using System;
using System.Drawing.Design;
using System.Windows.Forms.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.Globalization;
using DevExpress.Utils.Drawing.Helpers;
namespace DevExpress.Utils {
	public enum EditorShowMode { Default, MouseDown, MouseUp, Click, MouseDownFocused };
	public class ControlUtils {
		static int columnResizeEdgeSize = 4;
		public static int ColumnResizeEdgeSize { 
			get { return columnResizeEdgeSize; } 
			set { columnResizeEdgeSize = Math.Max(2, value); } 
		}
#if DXWhidbey
		public static bool EnableComponentNotifications(bool newValue, IComponent component) {
			if(component == null) return false;
			return EnableComponentNotifications(newValue, component.Site);
		}
		public static bool EnableComponentNotifications(bool newValue, ISite site) {
			if(site == null) return false;
			IDesignerHost host = site.GetService(typeof(IDesignerHost)) as IDesignerHost;
			if(host == null) return false;
			IDesignerLoaderService srv = host.GetService(typeof(IDesignerLoaderService)) as IDesignerLoaderService;
			if(srv == null) return false;
			MethodInfo mi = srv.GetType().GetMethod("EnableComponentNotification", BindingFlags.NonPublic | BindingFlags.Instance);
			if(mi != null) return (bool)mi.Invoke(srv, new object[] { newValue });
			return false;
		}
#else
		public static bool EnableComponentNotifications(bool newValue, IComponent component) { return false; }
		public static bool EnableComponentNotifications(bool newValue, ISite site) { return false; }
#endif
		[System.Runtime.InteropServices.DllImport("USER32.dll")]
		static extern short GetAsyncKeyState(int vKey);
		public static MouseButtons MouseButtons {
			get {
				MouseButtons ms = MouseButtons.None;
				if(GetAsyncKeyState(1) != 0) ms |= MouseButtons.Left;
				if(GetAsyncKeyState(2) != 0) ms |= MouseButtons.Right;
				if(GetAsyncKeyState(4) != 0) ms |= MouseButtons.Middle;
				return ms;
			}
		}
		public static Point CalcLocation(Point bottomLocation, Point topLocation, Size popupSize, bool allowXcheck) {
			Point res = CalcLocation(bottomLocation, topLocation, popupSize);
			if(!allowXcheck) res.X = bottomLocation.X;
			return res;
		}
		static bool useVirtualScreenForDropDown = false;
		public static bool UseVirtualScreenForDropDown {
			get { return useVirtualScreenForDropDown; }
			set { useVirtualScreenForDropDown = value; }
		}
		public static Point CalcLocation(Point bottomLocation, Point topLocation, Size popupSize) {
			Point location = bottomLocation;
			Rectangle rect = UseVirtualScreenForDropDown ? SystemInformation.VirtualScreen : SystemInformation.WorkingArea;
			if(SystemInformation.MonitorCount > 1) {
				rect = SystemInformation.WorkingArea;
				Screen scrBottom = Screen.FromPoint(bottomLocation), scrTop;
				if(bottomLocation == topLocation) 
					scrTop = scrBottom;
				else
					scrTop = Screen.FromPoint(topLocation);
				if(scrBottom.Equals(scrTop)) rect = UseVirtualScreenForDropDown ? scrTop.Bounds : scrTop.WorkingArea;
				else {
					rect = UseVirtualScreenForDropDown ? scrBottom.Bounds : scrBottom.WorkingArea;
				}
			}
			int bottom = bottomLocation.Y + popupSize.Height;
			int top = topLocation.Y - popupSize.Height;
			int maxBottomOutsize = bottom - rect.Bottom;
			int maxTopOutsize = rect.Top - top;
			if(maxBottomOutsize > 0 && maxBottomOutsize > maxTopOutsize) {
				location = topLocation;
				location.Y -= popupSize.Height;
			}
			if(location.X + popupSize.Width > rect.Right) {
				location.X = (rect.Right - popupSize.Width);
			}
			if(location.X < rect.Left) location.X = rect.Left;
			return location;
		}
		public static Point CalcFormLocation(Point location, Size formSize) {
			Rectangle rect = Screen.FromPoint(location).WorkingArea;
			if(location.X + formSize.Width > rect.Right) location.X = (rect.Right - formSize.Width);
			if(location.X < rect.Left) location.X = rect.Left;
			if(location.Y + formSize.Height > rect.Bottom) location.Y = (rect.Bottom - formSize.Height);
			if(location.Y < rect.Top) location.Y = rect.Top;
			return location;
		}
		public static void SuspendRedraw(Control control) {
			if(control == null || !control.IsHandleCreated) return;
			SetRedraw(control, false);
		}
		public static void ResumeRedraw(Control control) {
			if(control == null || !control.IsHandleCreated) return;
			SetRedraw(control, true);
			SendMessage(control.Handle, WM_NCPAINT, 0, 0);
			control.Invalidate(true);
		}
		const int WM_SETREDRAW = 0x000B, WM_NCPAINT = 0x0085;
		static void SetRedraw(Control control, bool value) {
			SendMessage(control.Handle, WM_SETREDRAW, (uint)(value ? 1 : 0), 0);
		}
		[System.Runtime.InteropServices.DllImport("user32.dll")]
		static extern int SendMessage(IntPtr hWnd, int Msg, uint wParam, uint lParam);
		[StructLayout(LayoutKind.Sequential)]
			class NONCLIENTMETRICS {
			public int cbSize = Marshal.SizeOf(typeof(NONCLIENTMETRICS));
			public int iBorderWidth = 0;
			public int iScrollWidth = 0;
			public int iScrollHeight = 0;
			public int iCaptionWidth = 0;
			public int iCaptionHeight = 0;
			[MarshalAs(UnmanagedType.Struct)]
			public LOGFONT lfCaptionFont = null;
			public int iSmCaptionWidth = 0;
			public int iSmCaptionHeight = 0;
			[MarshalAs(UnmanagedType.Struct)]
			public LOGFONT lfSmCaptionFont = null;
			public int iMenuWidth = 0;
			public int iMenuHeight = 0;
			[MarshalAs(UnmanagedType.Struct)]
			public LOGFONT lfMenuFont = null;
			[MarshalAs(UnmanagedType.Struct)]
			public LOGFONT lfStatusFont = null;
			[MarshalAs(UnmanagedType.Struct)]
			public LOGFONT lfMessageFont = null;
		}
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
			public class LOGFONT {
			public int lfHeight = 0;
			public int lfWidth = 0;
			public int lfEscapement = 0;
			public int lfOrientation = 0;
			public int lfWeight = 0;
			public byte lfItalic = 0;
			public byte lfUnderline = 0;
			public byte lfStrikeOut = 0;
			public byte lfCharSet = 0;
			public byte lfOutPrecision = 0;
			public byte lfClipPrecision = 0;
			public byte lfQuality = 0;
			public byte lfPitchAndFamily = 0;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=0x20)]
			public string lfFaceName = null;
		}
		[System.Runtime.InteropServices.DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		static extern bool SystemParametersInfo(int uiAction, int uiParam, [In, Out] NONCLIENTMETRICS pvParam, int fWinIni);
		const int SPI_GETNONCLIENTMETRICS = 0x0029;
		static Font GetCaptionFontBackup() {
			int fontHeight = SystemInformation.CaptionHeight;
			if(fontHeight >= 8) {
				fontHeight = fontHeight * 3 / 5;
			}
			Font font = new Font("Arial", fontHeight, FontStyle.Bold, GraphicsUnit.Pixel);
			return font;
		}
		public static Font GetCaptionFont() {
			try {
				NONCLIENTMETRICS metrics = new NONCLIENTMETRICS();
				bool getFontResult = SystemParametersInfo(SPI_GETNONCLIENTMETRICS, metrics.cbSize, metrics, 0);
				return Font.FromLogFont(metrics.lfCaptionFont);
			} catch {
				return GetCaptionFontBackup();
			}
		}
		public static bool IsSymbolFont(Font font) {
			const int SYMBOL_CHARSET = 2;
			return GetFontCharSet(font) == SYMBOL_CHARSET;
		}
		public static byte GetFontCharSet(Font font) {
			LOGFONT lf = new LOGFONT();
			IntPtr hfont = font.ToHfont();
			NativeMethods.GetObject(hfont, Marshal.SizeOf(lf), lf);
			NativeMethods.DeleteObject(hfont);			
			return lf.lfCharSet;
		}
	}
	public class ColorUtils {
		public static Color GetRealColor(Color clr) {
			Graphics g = Graphics.FromHwnd(IntPtr.Zero);
			Color res = g.GetNearestColor(clr);
			g.Dispose();
			return res;
		}
		public static Color OffsetColor(Color clr, int dR, int dG, int dB) {
			dR = Math.Min(255, clr.R + dR);
			dG = Math.Min(255, clr.G + dG);
			dB = Math.Min(255, clr.B + dB);
			return Color.FromArgb(dR, dG, dB);
		}
		public static Color FlatBarBorderColor {
			get {
				Color res = SystemColors.ControlDark;
				res = Color.FromArgb(GetDarkValue(res.R), GetDarkValue(res.G), GetDarkValue(res.B));
				return GetRealColor(res);
			}
		}
		public static Color FlatBarBackColor {
			get { 
				Color res = SystemColors.Control;
				res = Color.FromArgb(GetLightValue(res.R), GetLightValue(res.G), GetLightValue(res.B));
				return GetRealColor(res);
			}
		}
		public static Color FlatBarItemPressedBackColor {
			get { return GetRealColor(GetLightColor(14, 44, 40));
			}
		}
		public static Color FlatBarItemHighLightBackColor {
			get { return   GetRealColor(GetLightColor(-2, 30, 72));
			}
		}
		public static Color FlatBarItemDownedColor {
			get { return GetRealColor(GetLightColor(11, 9, 73)); }
		}
		protected class ColorInfo {
			public int BtnFaceColorPart, HighlightColorPart, WindowColorPart;
		}
		protected static int GetDarkValue(int val) {
			return (val * 8) / 10;
		}
		protected static int GetLightValue(byte val) {
			return val + ((255 - val) * 16) / 100;
		}
		protected static int GetLightIndex(ColorInfo info, byte btnFaceValue, byte highlightValue, byte windowValue) {
			int res = (btnFaceValue * info.BtnFaceColorPart) / 100 +
				(highlightValue * info.HighlightColorPart) / 100 +
				(windowValue * info.WindowColorPart) / 100;
			if(res < 0) res = 0;
			if(res > 255) res = 255;
			return res;
		}
		static ColorInfo colorInfo = new ColorInfo();
		public static Color GetLightColor(int btnFaceColorPart, int highlightColorPart, int windowColorPart) {
			colorInfo.BtnFaceColorPart = btnFaceColorPart;
			colorInfo.HighlightColorPart = highlightColorPart;
			colorInfo.WindowColorPart = windowColorPart;
			Color btnFace = SystemColors.Control,
				highlight = SystemColors.Highlight,
				window = SystemColors.Window;
			Color res;
			if(btnFace == Color.White || btnFace == Color.Black)
				res = highlight;
			else {
				res = Color.FromArgb(
					GetLightIndex(colorInfo, btnFace.R, highlight.R, window.R), 
					GetLightIndex(colorInfo, btnFace.G, highlight.G, window.G), 
					GetLightIndex(colorInfo, btnFace.B, highlight.B, window.B)); 
			}
			return res;
		}
		public static Color FlatTabBackColor {
			get {
				Color clr = SystemColors.Control;
				int r = clr.R, g = clr.G, b = clr.B;
				int max = Math.Max(Math.Max(r, g), b);
				int delta = 0x23;
				int maxDelta = (255 - (max + delta));
				if(maxDelta > 0) maxDelta = 0;
				r += (delta + maxDelta);
				g += (delta + maxDelta);
				b += (delta + maxDelta);
				return Color.FromArgb(r, g, b);
			}
		}
	}
#if DXWhidbey
	public class DXMouseEventArgs : HandledMouseEventArgs {
		HandledMouseEventArgs original = null;
		protected DXMouseEventArgs(HandledMouseEventArgs e) : base(e.Button, e.Clicks, e.X, e.Y, e.Delta, e.Handled) {
			this.original = e;
			Sync();
		}
		public DXMouseEventArgs(MouseButtons buttons, int clicks, int x, int y, int delta, bool handled) : base(buttons, clicks, x, y, delta, handled) { }
		public DXMouseEventArgs(MouseButtons buttons, int clicks, int x, int y, int delta) : this(buttons, clicks, x, y, delta, false) { }
		public static DXMouseEventArgs GetMouseArgs(Control control, EventArgs eventArgs) {
			DXMouseEventArgs ee = eventArgs as DXMouseEventArgs;
			if(ee == null) {
				HandledMouseEventArgs he = eventArgs as HandledMouseEventArgs;
				MouseEventArgs e = eventArgs as MouseEventArgs;
				if(e == null) {
					Point loc = Control.MousePosition;
					if(control != null && control.IsHandleCreated) loc = control.PointToClient(loc);
					e = new MouseEventArgs(Control.MouseButtons, 1, loc.X, loc.Y, 0);
				}
				if(he != null) 
					ee = new DXMouseEventArgs(he);
				else
					ee = new DXMouseEventArgs(e.Button, e.Clicks, e.X, e.Y, e.Delta);
			}
			return ee;
		}
		public new bool Handled {
			get {
				Sync();
				return base.Handled; 
			}
			set {
				base.Handled = value;
				Sync();
			}
		}
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void Sync() { if(original != null) original.Handled = base.Handled; }
		public static DXMouseEventArgs GetMouseArgs(MouseEventArgs eventArgs) {
			return GetMouseArgs(null, eventArgs);
		}
	}
#else
	public class DXMouseEventArgs : MouseEventArgs {
		bool _handled;
		public DXMouseEventArgs(MouseButtons buttons, int clicks, int x, int y, int delta) : base(buttons, clicks, x, y, delta) {
			this._handled = false;
		}
		public static DXMouseEventArgs GetMouseArgs(Control control, EventArgs eventArgs) {
			DXMouseEventArgs ee = eventArgs as DXMouseEventArgs;
			if(ee == null) {
				MouseEventArgs e = eventArgs as MouseEventArgs;
				if(e == null) {
					Point loc = Control.MousePosition;
					if(control != null && control.IsHandleCreated) loc = control.PointToClient(loc);
					e = new MouseEventArgs(Control.MouseButtons, 1, loc.X, loc.Y, 0);
				}
				ee = new DXMouseEventArgs(e.Button, e.Clicks, e.X, e.Y, e.Delta);
			}
			return ee;
		}
		public static DXMouseEventArgs GetMouseArgs(MouseEventArgs eventArgs) {
			return GetMouseArgs(null, eventArgs);
		}
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void Sync() { }
		public virtual bool Handled { get { return _handled; } set { _handled = value; } }
	}
#endif
	[AttributeUsage(System.AttributeTargets.Property)]
	public class ImageListAttribute : Attribute {
		public ImageListAttribute(object imageList) {
			this.imageList = imageList;
		}
		public ImageListAttribute(string name) {
			this.name = name;
		}
		public string Name {
			get {return name;}
			set {name = value;}
		}
		protected object imageList;
		string name;
	}
	public class Permissions {
		public static void UnmanagedCode() {
		}
		public static void UnmanagedCodeDemand() {
		}
		public static void Request() {
			UnmanagedCode();
		}
	}
	public class OSVersionHelper {
		public static bool Is64BitOS() {
			return Marshal.SizeOf(typeof(IntPtr)) == 8;
		}
	}
	public interface ILockService {
		bool CanChangeComponents(ICollection components);
		bool CanDeleteComponents(ICollection components);
	}
	class RootNS {
	}
}
namespace DevExpress.Utils.Registrator {
	public delegate void CheckTypeHandler(Type type);
	public class RegistratorHelper {
		public RegistratorHelper(CheckTypeHandler handler, Type baseType, IDesignerHost designerHost) {
			if(handler == null) return;
			RegisterUserItems(handler, baseType, designerHost);
		}
		string[] systemNames = new string[] {"mscorlib", "system.windows.forms", "microsoft.vsdesigner", "system.data",
												"system.drawing", "system", "system.design", "envdte", "microsoft.visualbasic", "office",
										"microsoft.visualstudio", "vslangproj", "system.xml", AssemblyInfo.SRAssemblyUtils.ToLower(), AssemblyInfo.SRAssemblyGrid.ToLower(),
										AssemblyInfo.SRAssemblyEditors.ToLower(), AssemblyInfo.SRAssemblyData.ToLower(), AssemblyInfo.SRAssemblyNavBar.ToLower()};
		protected virtual void RegisterUserItems(CheckTypeHandler handler, Type baseType, IDesignerHost designerHost) {
			try {
				try {
					if(designerHost != null) designerHost.GetType(designerHost.RootComponentClassName);
				}
				catch {
				}
				Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
				if(assemblies == null || assemblies.Length == 0) return;
				foreach(Assembly asm in assemblies) {
					if(Array.IndexOf(systemNames, asm.GetName().Name.ToLower(CultureInfo.InvariantCulture)) != -1) continue;
					Type[] types = asm.GetTypes();
					for(int n = 0; n < types.Length; n++) {
						Type type = types[n];
						if(baseType == null || type.IsSubclassOf(baseType)) {
							handler(type);
						}
					}
				}
			}
			catch {
			}
		}
	}
}
