#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Collections.Specialized;
using DevExpress.LookAndFeel;
using DevExpress.LookAndFeel.Helpers;
using DevExpress.Utils;
using DevExpress.Utils.Drawing;
using DevExpress.Skins;
using DevExpress.XtraEditors.Drawing;
using System.Reflection;
using DevExpress.Skins.XtraForm;
using DevExpress.Utils.Drawing.Helpers;
using System.Drawing.Imaging;
using DevExpress.Utils.Controls;
using System.Diagnostics;
using DevExpress.LookAndFeel.Design;
using DevExpress.Data;
namespace DevExpress.XtraEditors {
	[Description("Allows an end-user to resize neighboring controls."),
	 ToolboxTabName(AssemblyInfo.DXTabNavigation),
	  ToolboxBitmap(typeof(RootNS), "SplitterControl")
	]
	public class SplitterControl : Splitter, ISupportLookAndFeel {
		UserLookAndFeel lookAndFeel;
		AppearanceObject appearance;
		ObjectPainter painter;
		bool isLoading = true;
		public SplitterControl() {
			SetStyle(ControlConstants.DoubleBuffer | ControlStyles.ResizeRedraw | ControlStyles.SupportsTransparentBackColor, true);
			this.lookAndFeel = new ControlUserLookAndFeel(this);
			this.lookAndFeel.StyleChanged += new EventHandler(OnLookAndFeelStyleChanged);
			this.appearance = new AppearanceObject();
			this.appearance.Changed += new EventHandler(OnAppearance_Changed);
			UpdatePainter();
			this.isLoading = false;
			SetBoundsCore(Location.X, Location.Y, Width, Height, BoundsSpecified.Size);
		}
		protected override void Dispose(bool disposing) {
			if(disposing) {
				this.appearance.Changed -= new EventHandler(OnAppearance_Changed);
				if(this.lookAndFeel != null) {
					this.lookAndFeel.StyleChanged -= new EventHandler(OnLookAndFeelStyleChanged);
					this.lookAndFeel.Dispose();
					this.lookAndFeel = null;
				}
			}
			base.Dispose(disposing);
		}
		bool ISupportLookAndFeel.IgnoreChildren { get { return false; } }
		protected virtual void UpdatePainter() {
			this.painter = SplitterHelper.GetPainter(LookAndFeel);
		}
		protected ObjectPainter Painter { get { return painter; } }
		[Description("Gets or sets the splitter's background color."), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color BackColor {
			get { return GetColor(Appearance.BackColor, DefaultAppearance.BackColor); }
			set {
				if(value == Color.Transparent) return;
				Appearance.BackColor = value;
			}
		}
		void ResetAppearance() { Appearance.Reset(); }
		bool ShouldSerializeAppearance() { return Appearance.ShouldSerialize(); }
		[Description("Gets the appearance settings used to paint the splitter."), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public AppearanceObject Appearance { get { return appearance; } }
		bool ShouldSerializeLookAndFeel() { return LookAndFeel.ShouldSerialize(); }
		[Description("Provides access to the settings which control the splitter's look and feel. "), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public virtual UserLookAndFeel LookAndFeel { get { return lookAndFeel; } }
		void OnLookAndFeelStyleChanged(object sender, System.EventArgs e) {
			this.defaultAppearance = null;
			UpdatePainter();
			OnAppearance_Changed(sender, e);
			SetBoundsCore(Location.X, Location.Y, Width, Height, BoundsSpecified.Size);
		}
		protected void OnAppearance_Changed(object sender, EventArgs e) {
			if(IsHandleCreated) {
				OnBackColorChanged(e);
				Invalidate();
			}
		}
		AppearanceDefault defaultAppearance = null;
		protected AppearanceDefault DefaultAppearance {
			get {
				if(defaultAppearance == null)
					defaultAppearance = CreateDefaultAppearance();
				return defaultAppearance;
			}
		}
		protected virtual AppearanceDefault CreateDefaultAppearance() {
			AppearanceDefault result;
			if(Painter != null)
				result = Painter.DefaultAppearance.Clone() as AppearanceDefault;
			else
				result = new AppearanceDefault(SystemColors.ControlText, SystemColors.Control);
			if(LookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin) {
				SkinElement element = CommonSkins.GetSkin(LookAndFeel)[IsVertical ? CommonSkins.SkinSplitter : CommonSkins.SkinSplitterHorz];
				if(element != null) {
					element.Apply(result);
				}
			}
			return result;
		}
		protected override void OnPaint(PaintEventArgs e) {
			using(GraphicsCache cache = new GraphicsCache(e)) {
				Form frm = Parent as Form;
				if(frm != null && frm.IsMdiContainer) { 
					using(SolidBrush sb = new SolidBrush(CommonSkins.GetSkin(LookAndFeel).GetSystemColor(SystemColors.Control))) {
						cache.FillRectangle(sb, ClientRectangle);
					}
				}
				ObjectPainter.DrawObject(cache, Painter, CreateViewInfo());
			}
		}
		bool IsVertical {
			get { return Dock == DockStyle.Left || Dock == DockStyle.Right; }
		}
		protected virtual SplitterInfoArgs CreateViewInfo() {
			SplitterInfoArgs res = new SplitterInfoArgs(IsVertical);
			AppearanceObject paintAppearance = new AppearanceObject(Appearance, DefaultAppearance);
			res.SetAppearance(paintAppearance);
			res.Bounds = ClientRectangle;
			if(Enabled && IsHandleCreated && res.Bounds.Contains(PointToClient(MousePosition))) {
				res.State = ObjectState.Hot;
			}
			return res;
		}
		protected override void OnMouseEnter(EventArgs e) {
			Invalidate();
			base.OnMouseEnter(e);
		}
		protected override void OnMouseLeave(EventArgs e) {
			Invalidate();
			base.OnMouseLeave(e);
		}
		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
			Size prevSize = Size;
			if(!isLoading) {
				Size size = ObjectPainter.CalcObjectMinBounds(null, Painter, CreateViewInfo()).Size;
				if(!size.IsEmpty) {
					if(IsVertical)
						width = size.Width;
					else
						height = size.Height;
				}
			}
			base.SetBoundsCore(x, y, width, height, specified);
			if(!isLoading && Visible && Parent != null) {
				bool equals = (IsVertical ? Width == prevSize.Width : Height == prevSize.Height);
				if(!equals) {
					Parent.PerformLayout(this, "Size");
				}
			}
		}
		Color GetColor(Color color, Color defColor) {
			if(color == Color.Empty) return defColor;
			return color;
		}
	}
	[System.Runtime.InteropServices.ComVisible(false)]
	public interface IDXControl {
		void OnAppearanceChanged(object sender);
		void OnLookAndFeelStyleChanged(object sender);
		Control Control { get; }
	}
	public class ControlHelper : IDisposable {
		IDXControl owner;
		UserLookAndFeel lookAndFeel;
		AppearanceObject appearance;
		public ControlHelper(IDXControl owner, bool isForm) {
			this.owner = owner;
			if(isForm)
				this.lookAndFeel = new FormUserLookAndFeel(this.owner.Control as XtraForm);
			else
				this.lookAndFeel = new ControlUserLookAndFeel(this.owner.Control);
			this.lookAndFeel.StyleChanged += new EventHandler(OnLookAndFeelStyleChanged);
			this.appearance = new AppearanceObject();
			this.appearance.Changed += new EventHandler(OnAppearance_Changed);
			OnAppearance_Changed(this, EventArgs.Empty);
		}
		public static void SuspendLayout(Control control) {
			if(control is XtraForm)
				SuspendFormLayout((XtraForm)control);
			else
				control.SuspendLayout();
		}
		public static void ResumeLayout(Control control, bool performLayout) {
			if(control is XtraForm)
				ResumeFormLayout((XtraForm)control, performLayout);
			else
				control.ResumeLayout(performLayout);
		}
		public static void ResumeLayout(Control control) {
			if(control is XtraForm)
				ResumeFormLayout((XtraForm)control);
			else
				control.ResumeLayout();
		}
		static void ResumeFormLayout(XtraForm form, bool performLayout) {
			form.ResumeLayout(performLayout);
		}
		static void ResumeFormLayout(XtraForm form) {
			form.ResumeLayout();
		}
		static void SuspendFormLayout(XtraForm form) {
			form.SuspendLayout();
		}
		public IDXControl Owner { get { return owner; } }
		public virtual void Dispose() {
			this.appearance.Changed -= new EventHandler(OnAppearance_Changed);
			if(this.lookAndFeel != null) {
				this.lookAndFeel.StyleChanged -= new EventHandler(OnLookAndFeelStyleChanged);
				this.lookAndFeel.Dispose();
				this.lookAndFeel = null;
			}
		}
		public AppearanceObject Appearance { get { return appearance; } }
		public UserLookAndFeel LookAndFeel { get { return lookAndFeel; } }
		void OnLookAndFeelStyleChanged(object sender, System.EventArgs e) {
			Owner.OnLookAndFeelStyleChanged(sender);
		}
		protected void OnAppearance_Changed(object sender, EventArgs e) {
			Owner.OnAppearanceChanged(sender);
		}
	}
	[HiddenToolboxItem]
	public class XtraUserControl : UserControl, IDXControl, ISupportLookAndFeel {
		ControlHelper helper;
		ObjectPainter painter = null;
		public XtraUserControl() {
#if DEBUGTEST
			Debug.Assert(!IsHandleCreated);
			base.DestroyHandle();
#endif
			SetStyle(ControlStyles.SupportsTransparentBackColor, true);
			this.helper = new ControlHelper(this, false);
		}
		internal bool IsRootInDesigner { 
			get {
				if(!DesignMode) return false;
				return Parent != null && Parent.GetType().Name == "OverlayControl";
			} 
		}
		protected override void Dispose(bool disposing) {
			if(disposing) {
				this.helper.Dispose();
			}
			base.Dispose(disposing);
		}
		bool ISupportLookAndFeel.IgnoreChildren { get { return false; } }
		protected ControlHelper Helper { get { return helper; } }
		void ResetAppearance() { Appearance.Reset(); }
		bool ShouldSerializeAppearance() { return Appearance.ShouldSerialize(); }
		[Description("Gets the appearance settings used to paint the user control."), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public AppearanceObject Appearance {
			get {
				return Helper == null ? AppearanceObject.Dummy : Helper.Appearance;
			}
		}
		bool ShouldSerializeLookAndFeel() { return LookAndFeel.ShouldSerialize(); }
		[Description("Provides access to the settings which control the user control's look and feel. "), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public virtual UserLookAndFeel LookAndFeel { get { return Helper.LookAndFeel; } }
		void IDXControl.OnLookAndFeelStyleChanged(object sender) {
			this.defaultAppearance = null;
			this.painter = null;
			OnAppearance_Changed(sender);
		}
		void IDXControl.OnAppearanceChanged(object sender) {
			OnAppearance_Changed(sender);
		}
		Control IDXControl.Control { get { return this; } }
		internal Font GetBaseFont() { return base.Font; }
		protected void OnAppearance_Changed(object sender) {
			if(IsHandleCreated) {
				OnBackColorChanged(EventArgs.Empty);
				Invalidate();
			}
			if(!base.Font.Equals(Appearance.Font))
				base.Font = Appearance.Font;
		}
		[Description("Gets or sets font settings applied to the control."),
DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Font Font {
			get { return Appearance.Font; }
			set { Appearance.Font = value; }
		}
		public override void ResetBackColor() { BackColor = Color.Empty; }
		[Description("Gets or sets the user control's background color."), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color BackColor {
			get { return GetColor(Appearance.BackColor, DefaultAppearance.BackColor); }
			set {
				if(GetDesigner() != null && GetDesigner().Loading) return;
				Appearance.BackColor = value;
			}
		}
		public override void ResetForeColor() { ForeColor = Color.Empty; }
		[Description("Gets or sets the control's foreground color."), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color ForeColor {
			get { return LookAndFeelHelper.CheckTransparentForeColor(LookAndFeel, GetColor(Appearance.ForeColor, DefaultAppearance.ForeColor), this); }
			set { Appearance.ForeColor = value; }
		}
		Color GetColor(Color color, Color defColor) {
			if(color == Color.Empty) return defColor;
			return color;
		}
		AppearanceDefault defaultAppearance = null;
		protected AppearanceDefault DefaultAppearance {
			get {
				if(defaultAppearance == null)
					defaultAppearance = CreateDefaultAppearance();
				return defaultAppearance;
			}
		}
		protected virtual AppearanceDefault CreateDefaultAppearance() {
			AppearanceDefault res = new AppearanceDefault(SystemColors.ControlText, SystemColors.Control);
			if(LookAndFeel != null && LookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin) {
				SkinElement form = GetSkin();
				form.Apply(res);
				LookAndFeelHelper.CheckColors(LookAndFeel, res, this);
			}
			return res;
		}
		SkinElement GetSkin() {
			return CommonSkins.GetSkin(LookAndFeel)[CommonSkins.SkinGroupPanelNoBorder];
		}
		protected ObjectPainter Painter {
			get {
				if(painter == null) painter = CreatePainter();
				return painter;
			}
		}
		protected ObjectPainter CreatePainter() {
			switch(LookAndFeel.ActiveStyle) {
				case ActiveLookAndFeelStyle.Skin: return new ControlSkinPainter(LookAndFeel);
			}
			return new ControlPainter();
		}
		class ControlPainter : StyleObjectPainter {
			public override void DrawObject(ObjectInfoArgs e) {
				GetStyle(e).DrawBackground(e.Cache, e.Bounds);
			}
		}
		class ControlSkinPainter : SkinCustomPainter {
			public ControlSkinPainter(ISkinProvider provider) : base(provider) { }
			protected override SkinElementInfo CreateInfo(ObjectInfoArgs e) {
				SkinElementInfo info = new SkinElementInfo(CommonSkins.GetSkin(Provider)[CommonSkins.SkinGroupPanelNoBorder], e.Bounds);
				return info;
			}
		}
		bool loadFired = false;
		protected override void OnLoad(EventArgs e) {
			if(this.loadFired) return;
			OnFirstLoad();
			this.loadFired = true;
			base.OnLoad(e);
		}
		protected override void OnForeColorChanged(EventArgs e) {
			this.defaultAppearance = null;
			base.OnForeColorChanged(e);
		}
		protected override void OnBackColorChanged(EventArgs e) {
			this.defaultAppearance = null;
			base.OnBackColorChanged(e);
		}
		protected override void OnParentChanged(EventArgs e) {
			this.defaultAppearance = null;
			base.OnParentChanged(e);
			OnBackColorChanged(e);
		}
		protected virtual void OnFirstLoad() { }
		protected override void OnPaint(PaintEventArgs e) {
			if(BackgroundImage == null) {
				using(GraphicsCache cache = new GraphicsCache(e)) {
					StyleObjectInfoArgs info = new StyleObjectInfoArgs();
					info.Bounds = ClientRectangle;
					info.SetAppearance(new AppearanceObject(Appearance, DefaultAppearance));
					ObjectPainter.DrawObject(cache, Painter, info);
				}
			}
			RaisePaintEvent(this, e);
		}
		IDesignerHost GetDesigner() {
			if(Site == null) return null;
			return Site.GetService(typeof(IDesignerHost)) as IDesignerHost;
		}
	}
	public class XtraForm : Form, ISupportLookAndFeel, IDXControl, IMouseWheelContainer {
		static bool suppressDeactivation;
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		public static bool SuppressDeactivation { get { return suppressDeactivation; } set { suppressDeactivation = value; } }
		bool allowMdiBar = false;
		ControlHelper helper;
		bool allowFormSkin;
		FormPainter painter = null;
		Form prevMdiChild = null;
		Size minimumClientSize;
		Size maximumClientSize;
		string textMdiTab;
		static XtraForm() { DevExpress.Utils.Design.DXAssemblyResolverEx.Init(); }
		public XtraForm() {
#if DEBUGTEST
			Debug.Assert(!IsHandleCreated);
			base.DestroyHandle();
#endif
			this.allowFormSkin = true;
			this.helper = new ControlHelper(this, true);
			UpdateSkinPainter();
			this.minimumClientSize = Size.Empty;
			this.maximumClientSize = Size.Empty;
		}
		protected override void Dispose(bool disposing) {
			base.Dispose(disposing);
#if !DXWhidbey
			if(disposing)
				UnsubscribeRememberedMdiParent();
#endif
			if(disposing) {
				if(painter != null) painter.Dispose();
				this.painter = null;
				if(helper != null) this.helper.Dispose();
				helper = null;
			}
		}
		int isInitializing = 0;
		bool forceInitialized = false;
		protected internal virtual bool ShouldShowMdiBar {
			get {
				if(!IsMdiContainer || ActiveMdiChild == null || ActiveMdiChild.WindowState != FormWindowState.Maximized) return false;
				if(MainMenuStrip != null || Menu != null) return false;
				return AllowMdiBar;
			}
		}
		protected bool IsInitializing {  get { return !forceInitialized && (this.isInitializing != 0 || IsLayoutSuspendedCore); } }
		protected bool IsApplyingLocalizableResourcesInDesignTime {
			get { return DesignMode && StackTraceHelper.CheckStackFrame("ApplyResources", typeof(ComponentResourceManager)); }
	}
#region MouseWheel
		void IMouseWheelContainer.BaseOnMouseWheel(MouseEventArgs e) {
			base.OnMouseWheel(e);
		}
		Control IMouseWheelContainer.Control { get { return this; } }
		protected override void OnMouseWheel(MouseEventArgs e) {
			MouseWheelHelper.DoMouseWheel(this, e);
		}
		public static bool ProcessSmartMouseWheel(Control control, MouseEventArgs e) {
			return MouseWheelHelper.ProcessSmartMouseWheel(control, e);
		}
		public static bool IsAllowSmartMouseWheel(Control control, MouseEventArgs e) {
			return MouseWheelHelper.IsAllowSmartMouseWheel(control, e);
		}
		public static bool IsAllowSmartMouseWheel(Control control) {
			return MouseWheelHelper.IsAllowSmartMouseWheel(control);
		}
#endregion MouseWheel
		public new void SuspendLayout() {
			base.SuspendLayout();
			isInitializing ++;
		}
		void CheckForceLookAndFeelChangedCore() {
			if(this.isInitializing != 0) return;
			if(LayoutSuspendCountCore == 1 && this.shouldUpdateLookAndFeelOnResumeLayout) {
				this.forceInitialized = true;
				try {
					OnLookAndFeelChangedCore();
				}
				finally {
					this.forceInitialized = false;
				}
			}
		}
		public new void ResumeLayout() { ResumeLayout(true); }
		public new void ResumeLayout(bool performLayout) {
			if(--this.isInitializing == 0) {
				CheckMinimumSize();
				CheckMaximumSize();
				CheckForceLookAndFeelChangedCore();
			}
			base.ResumeLayout(performLayout);
		}
		void CheckMaximumSize() {
			if(this.maximumSize == null) return;
			Size msize = (Size)maximumSize;
			this.maximumSize = null;
			if(!msize.IsEmpty) {
				if(msize.Width > 0) msize.Width = Math.Max(msize.Width, Size.Width);
				if(msize.Height > 0) msize.Height = Math.Max(msize.Height, Size.Height);
			}
			base.MaximumSize = msize;
		}
		void CheckMinimumSize() {
			if(this.minimumSize == null) return;
			Size msize = (Size)minimumSize;
			this.minimumSize = null;
			if(!msize.IsEmpty) {
				if(msize.Width > 0) msize.Width = Math.Min(msize.Width, Size.Width);
				if(msize.Height > 0) msize.Height = Math.Min(msize.Height, Size.Height);
			}
			base.MinimumSize = msize;
		}
		protected override void OnLayout(LayoutEventArgs levent) {
			if(this.shouldUpdateLookAndFeelOnResumeLayout) {
				CheckForceLookAndFeelChangedCore();
			}
			base.OnLayout(levent);
		}
		protected override void ScaleCore(float x, float y) {
			MaximumClientSize = new Size((int)Math.Round(MaximumClientSize.Width * x), (int)Math.Round(MaximumClientSize.Height * y));
			base.ScaleCore(x, y);
			MinimumClientSize = new Size((int)Math.Round(MinimumClientSize.Width * x), (int)Math.Round(MinimumClientSize.Height * y));
		}
		PropertyInfo piLayout = null;
		internal bool IsLayoutSuspendedCore {
			get {
				if(piLayout == null) piLayout = typeof(Control).GetProperty("IsLayoutSuspended", BindingFlags.Instance | BindingFlags.NonPublic);
				if(piLayout != null) return (bool)piLayout.GetValue(this, null);
				return false;
			}
		}
		FieldInfo fiLayoutSuspendCount = null;
		internal byte LayoutSuspendCountCore {
			get {
				if(fiLayoutSuspendCount == null) fiLayoutSuspendCount = typeof(Control).GetField("layoutSuspendCount", BindingFlags.Instance | BindingFlags.NonPublic);
				if(fiLayoutSuspendCount != null) return (byte)fiLayoutSuspendCount.GetValue(this);
				return 1;
			}
		}
		protected internal FormPainter FormPainter { get { return painter; } }
		Control IDXControl.Control { get { return this; } }
		bool shouldUpdateLookAndFeelOnResumeLayout = false;
		void IDXControl.OnLookAndFeelStyleChanged(object sender) {
			this.defaultAppearance = null;
			OnAppearance_Changed(sender);
			OnLookAndFeelChangedCore();
		}
		protected override void OnParentChanged(EventArgs e) {
			this.defaultAppearance = null;
			OnAppearance_Changed(null);
			base.OnParentChanged(e);
		}
		void OnLookAndFeelChangedCore() {
			if(IsInitializing) {
				shouldUpdateLookAndFeelOnResumeLayout = true;
				return;
			}
			this.shouldUpdateLookAndFeelOnResumeLayout = false;
			Size clientSize = ClientSizeFromSize(Size);
			CheckUpdateSkinPainter();
			OnMinimumClientSizeChanged();
			OnMaximumClientSizeChanged();
			FieldInfo fiBounds = typeof(Form).GetField("restoredWindowBounds", BindingFlags.NonPublic | BindingFlags.Instance);
			FieldInfo fiBoundsSpec = typeof(Form).GetField("restoredWindowBoundsSpecified", BindingFlags.NonPublic | BindingFlags.Instance);
			FieldInfo fiBounds2 = typeof(Form).GetField("restoreBounds", BindingFlags.NonPublic | BindingFlags.Instance);
			Rectangle restoredWinBounds = (Rectangle)fiBounds.GetValue(this);
			Rectangle restoreBounds = (Rectangle)fiBounds2.GetValue(this);
			BoundsSpecified restoredWinBoundsSpec = (BoundsSpecified)fiBoundsSpec.GetValue(this);
			int frmStateExWindowBoundsWidthIsClientSize, frmStateExWindowBoundsHeightIsClientSize;
			GetFormStateExWindowBoundsIsClientSize(out frmStateExWindowBoundsWidthIsClientSize, out frmStateExWindowBoundsHeightIsClientSize);
			Size = SizeFromClientSize(clientSize);
			if((restoredWinBoundsSpec & BoundsSpecified.Width) != 0 && (restoredWinBoundsSpec & BoundsSpecified.Height) != 0) restoreBounds.Size = SizeFromClientSize(restoredWinBounds.Size);
			if(WindowState != FormWindowState.Normal && IsHandleCreated) {
				fiBounds.SetValue(this, restoredWinBounds);
				fiBounds2.SetValue(this, restoreBounds);
				SetFormStateExWindowBoundsIsClientSize(frmStateExWindowBoundsWidthIsClientSize, frmStateExWindowBoundsHeightIsClientSize);
			}
		}
		void GetFormStateExWindowBoundsIsClientSize(out int width, out int height) {
			FieldInfo formStateExInfo = typeof(Form).GetField("formStateEx", BindingFlags.NonPublic | BindingFlags.Instance);
			FieldInfo formStateExWindowBoundsWidthIsClientSizeInfo = typeof(Form).GetField("FormStateExWindowBoundsWidthIsClientSize", BindingFlags.NonPublic | BindingFlags.Static);
			FieldInfo formStateExWindowBoundsHeightIsClientSizeInfo = typeof(Form).GetField("FormStateExWindowBoundsHeightIsClientSize", BindingFlags.NonPublic | BindingFlags.Static);
			BitVector32.Section widthSection = (BitVector32.Section)formStateExWindowBoundsWidthIsClientSizeInfo.GetValue(this);
			BitVector32.Section heightSection = (BitVector32.Section)formStateExWindowBoundsHeightIsClientSizeInfo.GetValue(this);
			BitVector32 formState = (BitVector32)formStateExInfo.GetValue(this);
			width = formState[widthSection];
			height = formState[heightSection];
		}
		void SetFormStateExWindowBoundsIsClientSize(int width, int height) {
			FieldInfo formStateExInfo = typeof(Form).GetField("formStateEx", BindingFlags.NonPublic | BindingFlags.Instance);
			FieldInfo formStateExWindowBoundsWidthIsClientSizeInfo = typeof(Form).GetField("FormStateExWindowBoundsWidthIsClientSize", BindingFlags.NonPublic | BindingFlags.Static);
			FieldInfo formStateExWindowBoundsHeightIsClientSizeInfo = typeof(Form).GetField("FormStateExWindowBoundsHeightIsClientSize", BindingFlags.NonPublic | BindingFlags.Static);
			BitVector32.Section widthSection = (BitVector32.Section)formStateExWindowBoundsWidthIsClientSizeInfo.GetValue(this);
			BitVector32.Section heightSection = (BitVector32.Section)formStateExWindowBoundsHeightIsClientSizeInfo.GetValue(this);
			BitVector32 formState = (BitVector32)formStateExInfo.GetValue(this);
			formState[widthSection] = width;
			formState[heightSection] = height;
			formStateExInfo.SetValue(this, formState);
		}
		bool IsFormStateClientSizeSet() {
			FieldInfo fi1 = typeof(Form).GetField("FormStateSetClientSize", BindingFlags.NonPublic | BindingFlags.Static);
			FieldInfo fiFormState = typeof(Form).GetField("formState", BindingFlags.NonPublic | BindingFlags.Instance);
			BitVector32.Section bi1 = (BitVector32.Section)fi1.GetValue(this);
			BitVector32 state = (BitVector32)fiFormState.GetValue(this);
			return state[bi1] == 1;
		}
		protected override void OnSizeChanged(EventArgs e) {
			if(FormPainter != null && FormBorderStyle != FormBorderStyle.None && (ControlBox || !string.IsNullOrEmpty(Text)) && WindowState == FormWindowState.Normal) {
				FieldInfo fiClientWidth = typeof(Control).GetField("clientWidth", BindingFlags.GetField | BindingFlags.NonPublic | BindingFlags.Instance);
				FieldInfo fiClientHeight = typeof(Control).GetField("clientHeight", BindingFlags.GetField | BindingFlags.NonPublic | BindingFlags.Instance);
				Size sz = this.ClientSizeFromSize(base.Size);
				fiClientWidth.SetValue(this, sz.Width);
				fiClientHeight.SetValue(this, sz.Height);
			}
			base.OnSizeChanged(e);
		}
		protected override void OnLoad(EventArgs e) {
			base.OnLoad(e);
			OnMinimumClientSizeChanged();
			OnMaximumClientSizeChanged();
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override RightToLeft RightToLeft {
			get { return base.RightToLeft; }
			set { base.RightToLeft = value; }
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override bool RightToLeftLayout {
			get { return base.RightToLeftLayout; }
			set { base.RightToLeftLayout = value; }
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public string TextMdiTab {
			get { return textMdiTab; }
			set {
				if(textMdiTab == value) return;
				textMdiTab = value;
				OnTextMdiTabChanged();
			}
		}
		protected virtual void OnTextMdiTabChanged() {
			if(MdiParent == null) return;
			OnTextChanged(EventArgs.Empty);
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Size MinimumClientSize {
			get { return minimumClientSize; }
			set {
				if(MinimumClientSize == value) return;
				minimumClientSize = value;
				OnMinimumClientSizeChanged();
			}
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Size MaximumClientSize {
			get { return maximumClientSize; }
			set {
				if(MaximumClientSize == value) return;
				maximumClientSize = value;
				OnMaximumClientSizeChanged();
			}
		}
		protected virtual void OnMinimumClientSizeChanged() {
			if(IsInitializing) return;
			MinimumSize = GetConstrainSize(MinimumClientSize);
		}
		protected virtual void OnMaximumClientSizeChanged() {
			if(IsInitializing) return;
			MaximumSize = GetConstrainSize(MaximumClientSize);
		}
		protected virtual Size GetConstrainSize(Size clientSize) {
			if(clientSize == Size.Empty) return Size.Empty;
			return SizeFromClientSize(clientSize);
		}
		protected virtual Size ClientSizeFromSize(Size formSize) {
			if(formSize == Size.Empty)
				return Size.Empty;
			Size sz = SizeFromClientSize(Size.Empty);
			return new Size(formSize.Width - sz.Width, formSize.Height - sz.Height);
		}
		public override Size MinimumSize {
			get {
				if(inScaleControl && minimumSize.HasValue)
					return minimumSize.Value;
				return base.MinimumSize; 
			}
			set {
				minimumSize = value;
				if(IsInitializing || IsApplyingLocalizableResourcesInDesignTime) {
					return;
				}
				Size maxSize = MaximumSize;
				base.MinimumSize = value;
				if(maxSize != MaximumSize)
					MaximumClientSize = ClientSizeFromSize(MaximumSize);
			}
		}
		public override Size MaximumSize {
			get {
				if(inScaleControl && maximumSize.HasValue)
					return maximumSize.Value;
				return base.MaximumSize; 
			}
			set {
				maximumSize = value;
				if(IsInitializing || IsApplyingLocalizableResourcesInDesignTime) {
					return;
				}
				Size minSize = MinimumSize;
				base.MaximumSize = value;
				if(MinimumSize != minSize)
					MinimumClientSize = ClientSizeFromSize(MinimumSize);
			}
		}
		protected override void OnMinimumSizeChanged(EventArgs e) {
			base.OnMinimumSizeChanged(e);
			MinimumClientSize = ClientSizeFromSize(MinimumSize);
		}
		protected override void OnMaximumSizeChanged(EventArgs e) {
			base.OnMaximumSizeChanged(e);
			MaximumClientSize = ClientSizeFromSize(MaximumSize);
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		protected internal virtual bool SupportAdvancedTitlePainting { get { return true; } }
		protected void CheckUpdateSkinPainter() {
			bool needReset = UpdateSkinPainter();
			if(IsHandleCreated) {
				if(this.painter != null) this.painter.ResetDefaultAppearance();
				if(needReset) {
					if(FormPainter == null || !FormPainter.IsAllowNCDraw)
						EnableTheme();
					else
						DisableTheme();
					if(!DevExpress.Utils.WXPaint.WXPPainter.Default.ThemesEnabled) {
						RecreateHandle();
					}
					else {
						DevExpress.Skins.XtraForm.FormPainter.ForceFrameChanged(this);
						Refresh();
					}
					OnSkinPainterChanged();
				}
				if(FormPainter == null)
					Refresh();
			}
		}
		protected virtual void OnSkinPainterChanged() {
		}
		void IDXControl.OnAppearanceChanged(object sender) {
			OnAppearance_Changed(sender);
		}
		bool ISupportLookAndFeel.IgnoreChildren { get { return false; } }
		int inWmWindowPosChanged = 0;
		protected override void WndProc(ref Message msg) {
			if((msg.Msg == MSG.WM_ACTIVATE || msg.Msg == MSG.WM_NCACTIVATE) && SuppressDeactivation) {
				msg.WParam = new IntPtr(1);
			}
			if(msg.Msg == MSG.WM_DWMCOMPOSITIONCHANGED)
				CheckUpdateSkinPainter();
			if(painter != null) {
				bool res = painter.DoWndProc(ref msg);
				if(res) return;
				if(msg.Msg == MSG.WM_WINDOWPOSCHANGED) {
					this.inWmWindowPosChanged++;
				}
				base.WndProc(ref msg);
				if(msg.Msg == MSG.WM_WINDOWPOSCHANGED) {
					this.inWmWindowPosChanged--;
				}
				return;
			}
			base.WndProc(ref msg);
		}
		void DoBase(ref Message msg) {
			base.WndProc(ref msg);
		}
		protected bool IsCustomPainter { get { return painter != null; } }
		protected override void OnStyleChanged(EventArgs e) {
			if(UpdateSkinPainter() && this.clientSizeSet) {
				ClientSize = ClientSize;
				this.clientSizeSet = false;
			}
			base.OnStyleChanged(e);
		}
		bool clientSizeSet = false;
		protected override void SetClientSizeCore(int x, int y) {
			this.clientSizeSet = false;
			UpdateSkinPainter();
			FieldInfo fiWidth = typeof(Control).GetField("clientWidth", BindingFlags.Instance | BindingFlags.NonPublic);
			FieldInfo fiHeight = typeof(Control).GetField("clientHeight", BindingFlags.Instance | BindingFlags.NonPublic);
			FieldInfo fi1 = typeof(Form).GetField("FormStateSetClientSize", BindingFlags.NonPublic | BindingFlags.Static),
				fiFormState = typeof(Form).GetField("formState", BindingFlags.NonPublic | BindingFlags.Instance);
			if(IsCustomPainter && fiWidth != null && fiHeight != null && fiFormState != null && fi1 != null) {
				this.clientSizeSet = true;
				this.Size = SizeFromClientSize(new Size(x, y));
				fiWidth.SetValue(this, x);
				fiHeight.SetValue(this, y);
				BitVector32.Section bi1 = (BitVector32.Section)fi1.GetValue(this);
				BitVector32 state = (BitVector32)fiFormState.GetValue(this);
				state[bi1] = 1;
				fiFormState.SetValue(this, state);
				this.OnClientSizeChanged(EventArgs.Empty);
				state[bi1] = 0;
				fiFormState.SetValue(this, state);
			}
			else {
				base.SetClientSizeCore(x, y);
			}
		}
		[DllImport("user32.dll")]
		static extern bool DrawMenuBar(IntPtr hWnd);
		protected override void OnMdiChildActivate(EventArgs e) {
			base.OnMdiChildActivate(e);
			BeginInvoke(new MethodInvoker(DrawMenuBarCore));
			if(prevMdiChild != null)
				prevMdiChild.TextChanged -= new EventHandler(OnMdiChildTextChanged);
			if(ActiveMdiChild != null)
				ActiveMdiChild.TextChanged += new EventHandler(OnMdiChildTextChanged);
			prevMdiChild = ActiveMdiChild;
			OnTextChanged(e);
		}
		void DrawMenuBarCore() {
			DrawMenuBar(Handle);
		}
		protected virtual void OnMdiChildTextChanged(object sender, EventArgs e) {
			OnTextChanged(e);
			FormPainter.InvalidateNC(this);
		}
		protected override Size SizeFromClientSize(Size clientSize) {
			if((!ControlBox && !AllowSkinForEmptyText && (Text == "" || Text == null)) || painter == null)
			return base.SizeFromClientSize(clientSize);
			return painter.CalcSizeFromClientSize(clientSize);
		}
		protected override Rectangle GetScaledBounds(Rectangle bounds, SizeF factor, BoundsSpecified specified) {
			Rectangle rect = base.GetScaledBounds(bounds, factor, specified);
			if(!IsCustomPainter)
				return rect;
			Size sz = SizeFromClientSize(Size.Empty);
			if(!GetStyle(ControlStyles.FixedWidth) && ((specified & BoundsSpecified.Width) != BoundsSpecified.None)) {
				int clientWidth = bounds.Width - sz.Width;
				rect.Width = ((int)Math.Round((double)(clientWidth * factor.Width))) + sz.Width;
			}
			if(!GetStyle(ControlStyles.FixedHeight) && ((specified & BoundsSpecified.Height) != BoundsSpecified.None)) {
				int clientHeight = bounds.Height - sz.Height;
				rect.Height = ((int)Math.Round((double)(clientHeight * factor.Height))) + sz.Height;
			}
			return rect;
		}
		bool inScaleControl = false;
		protected override void ScaleControl(SizeF factor, BoundsSpecified specified) {
			this.inScaleControl = true;
			if(!IsCustomPainter) {
				base.ScaleControl(factor, specified);
				this.inScaleControl = false;
				return;
			}
			Size minSize = MinimumSize;
			Size maxSize = MaximumSize;
			Size sz = SizeFromClientSize(Size.Empty);
			base.ScaleControl(factor, specified);
			if(minSize != Size.Empty) {
				minSize -= sz;
				minSize = new Size((int)Math.Round(minSize.Width * factor.Width), (int)Math.Round(minSize.Height * factor.Height)) + sz;
			}
			if(maxSize != Size.Empty) {
				maxSize -= sz;
				maxSize = new Size((int)Math.Round(maxSize.Width * factor.Width), (int)Math.Round(maxSize.Height * factor.Height)) + sz;
			}
			MinimumSize = minSize;
			MaximumSize = maxSize;
			this.inScaleControl = false;
		}
		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
			if((inWmWindowPosChanged == 1 || inWmWindowPosChanged == 2) && IsCustomPainter) {
				try {
					FieldInfo fi1 = typeof(Form).GetField("FormStateExWindowBoundsWidthIsClientSize", BindingFlags.NonPublic | BindingFlags.Static),
						fiFormState = typeof(Form).GetField("formStateEx", BindingFlags.NonPublic | BindingFlags.Instance),
						fiBounds = typeof(Form).GetField("restoredWindowBounds", BindingFlags.NonPublic | BindingFlags.Instance);
					if(fi1 != null && fiFormState != null && fiBounds != null) {
						Rectangle restoredWindowBounds = (Rectangle)fiBounds.GetValue(this);
						BitVector32.Section bi1 = (BitVector32.Section)fi1.GetValue(this);
						BitVector32 state = (BitVector32)fiFormState.GetValue(this);
						if(state[bi1] == 1) {
							width = restoredWindowBounds.Width + painter.Margins.Left + painter.Margins.Right;
							height = restoredWindowBounds.Height + painter.Margins.Top + painter.Margins.Bottom;
						}
					}
				}
				catch {
				}
			}
			base.SetBoundsCore(x, y, width, height, specified);
		}
		internal bool creatingHandle = false;
		protected override void CreateHandle() {
			bool shouldPatchSize = !this.creatingHandle;
			this.creatingHandle = true;
			if(shouldPatchSize)
			if(WindowState != FormWindowState.Minimized)
				Size = SizeFromClientSize(ClientSize);
			if(!IsHandleCreated)
			base.CreateHandle();
			this.creatingHandle = false;
		}
		internal bool IsMaximizedBoundsSet {
			get { return !MaximumSize.IsEmpty || !MaximizedBounds.IsEmpty; }
		}
		int lockRedraw = 0;
		bool isWindowActive = false;
		XtraForm suspendRedrawMdiParent = null;
		protected internal virtual bool IsSuspendRedraw { get { return lockRedraw != 0; } }
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SuspendRedraw() {
			if(lockRedraw++ == 0) this.isWindowActive = painter == null ? false : painter.IsWindowActive;
			suspendRedrawMdiParent = MdiParent as XtraForm;
			if(suspendRedrawMdiParent != null) suspendRedrawMdiParent.SuspendRedraw();
		}
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void ResumeRedraw() {
			if(lockRedraw < 1) return;
			if(--lockRedraw == 0) {
				if(painter != null && painter.IsWindowActive != this.isWindowActive) painter.InvalidateNC();
			}
			if(suspendRedrawMdiParent != null) {
				suspendRedrawMdiParent.ResumeRedraw();
				if(suspendRedrawMdiParent.lockRedraw == 0)
					suspendRedrawMdiParent = null;
			}
		}
		protected ControlHelper Helper { get { return helper; } }
		bool ShouldSerializeAppearance() { return Appearance.ShouldSerialize(); }
		void ResetAppearance() { Appearance.Reset(); }
		[Description("Gets the appearance settings used to paint the form."), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public AppearanceObject Appearance { get { return Helper == null ? AppearanceObject.Dummy : Helper.Appearance; } }
		bool ShouldSerializeLookAndFeel() { return LookAndFeel.ShouldSerialize(); }
		[Description("Provides access to the settings that control the look and feel of all the DevExpress controls contained within the form."), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public UserLookAndFeel LookAndFeel { get { return Helper == null ? null : Helper.LookAndFeel; } }
		protected void OnAppearance_Changed(object sender) {
			if(IsHandleCreated) {
				OnBackColorChanged(EventArgs.Empty);
				Invalidate();
			}
			if(!base.Font.Equals(Appearance.Font)) {
				this.useBaseFont = true;
				base.Font = Appearance.Font;
				this.useBaseFont = false;
			}
		}
		[DefaultValue(false), Description("Gets or sets whether an MDI bar is allowed for this form. This property is in effect for a parent MDI form, when the title bar skinning feature is enabled.")]
		public virtual bool AllowMdiBar {
			get { return allowMdiBar; }
			set {
				if(allowMdiBar == value) return;
				allowMdiBar = value;
				if(FormPainter != null && IsHandleCreated) {
					FormPainter.ForceFrameChanged();
					Refresh();
				}
			}
		}
		[Browsable(false), DefaultValue(true), Description("Gets or sets whether the form's title bar and borders are painted using the current skin. This member supports the internal infrastructure and is not intended to be used directly from your code."), Category("Appearance")]
		public virtual bool AllowFormSkin {
			get { return allowFormSkin; }
			set {
				if(AllowFormSkin == value) return;
				allowFormSkin = value;
				UpdateSkinPainter();
				if(IsHandleCreated) RecreateHandle();
			}
		}
		protected virtual object GetSkinPainterType() {
			if(!GetAllowSkin()) return null;
			return typeof(FormPainter);
		}
		object currentPainterType = null;
		Size? minimumSize = null, maximumSize = null;
		protected bool UpdateSkinPainter() { return UpdateSkinPainter(false); }
		protected bool UpdateSkinPainter(bool fromHandleCreated) {
			bool needReset = false;
			object painterType = GetSkinPainterType();
			if(currentPainterType == painterType || object.Equals(currentPainterType, painterType)) {
				if(painter != null) {
					this.painter.ResetDefaultAppearance();
					if(!fromHandleCreated)
						this.painter.ForceFrameChanged();
				}
				return false;
			}
			this.currentPainterType = IsInitializing ? typeof(bool) : painterType;
			if(painter != null) {
				painter.Dispose();
				painter = null;
				needReset = true;
			}
			if(painterType != null) {
				if(!IsInitializing) {
				this.painter = CreateFormBorderPainter();
				this.painter.BaseWndProc += new FormPainter.WndProcMethod(DoBase);
				needReset = true;
			}
				else {
					shouldUpdateLookAndFeelOnResumeLayout = true;
				}
			}
			return needReset;
							}
		protected override void OnClientSizeChanged(EventArgs e) {
			base.OnClientSizeChanged(e);
		}
		protected virtual FormPainter CreateFormBorderPainter() {
			return new FormPainter(this, LookAndFeel);
		}
		protected internal virtual bool AllowSkinForEmptyText { get { return false; } }
		protected virtual bool GetAllowMdiFormSkins() {
			return SkinManager.AllowMdiFormSkins;
		}
		protected virtual bool GetAllowSkin() {
			if(Disposing || IsDisposed)
				return false;
			if(DesignMode && !UserLookAndFeelDefault.EnableDesignTimeFormSkin)
				return false;
			if(MdiParent != null && !GetAllowMdiFormSkins())
				return false;
			if(!AllowFormSkin)
				return false;
			if(!SkinManager.AllowFormSkins) {
				if(!DesignMode || (DesignMode && !UserLookAndFeelDefault.EnableDesignTimeFormSkin))
					return false;
			}
			if(FormBorderStyle == FormBorderStyle.None)
				return false;
			if(!ControlBox && !AllowSkinForEmptyText && (Text == "" || Text == null)) {
				if(IsInitializing && LookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin) {
					this.shouldUpdateLookAndFeelOnResumeLayout = true;
				} else
					return false;
			}
			return LookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin;
		}
		[System.Runtime.InteropServices.DllImport("USER32.dll")]
		static extern bool DestroyMenu(IntPtr menu);
		[DllImport("user32.dll")]
		static internal extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);
		protected override void OnHandleCreated(EventArgs e) {
			base.OnHandleCreated(e);
			UpdateSkinPainter(true);
			BeginInvoke(new MethodInvoker(UpdateTheme));
			GetSystemMenu(Handle, true);
			UpdateWindowThemeCore();
		}
		[DllImport("uxtheme.dll", ExactSpelling = true, CharSet = CharSet.Unicode)]
		static extern int SetWindowTheme(IntPtr hWnd, String pszSubAppName, String pszSubIdList);
		protected virtual void UpdateWindowThemeCore() {
			if(NativeVista.IsVista && !DesignMode && FormPainter != null) {
				SetWindowTheme(Handle, "", "");
			}
		}
		internal virtual MdiClient GetMdiClient() {
			foreach(Control ctrl in Controls) {
				if(ctrl is MdiClient) return ctrl as MdiClient;
			}
			return null;
		}
		bool themeEnabled = true;
		void EnableTheme() {
			if(themeEnabled) return;
			DevExpress.Utils.WXPaint.WXPPainter.Default.ResetWindowTheme(this);
			Region = null;
			this.themeEnabled = true;
		}
		void DisableTheme() {
			DevExpress.Utils.WXPaint.WXPPainter.Default.DisableWindowTheme(this);
			this.themeEnabled = false;
			UpdateWindowThemeCore();
		}
		void UpdateTheme() {
			if(FormPainter != null && FormPainter.IsAllowNCDraw) {
				DisableTheme();
			}
			else {
				EnableTheme();
			}
		}
		protected override CreateParams CreateParams { get { return GetCreateParams(base.CreateParams); } }
		protected virtual CreateParams GetCreateParams(CreateParams par) {
			if(IsDisposed || Disposing) return par;
			if(GetAllowSkin()) {
			}
			else {
				UpdateSkinPainter();
			}
			if(this.creatingHandle && IsFormStateClientSizeSet() && IsCustomPainter) {
				par.Width = Width;
				par.Height = Height;
			}
			return par;
		}
		public override void ResetBackColor() { BackColor = Color.Empty; }
		[Description("Gets or sets the background color of the form's client region. "), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color BackColor {
			get { return GetColor(Appearance.BackColor, DefaultAppearance.BackColor); }
			set {
				if(GetDesigner() != null && GetDesigner().Loading) return;
				Appearance.BackColor = value;
			}
		}
		protected Image GetSkinBackgroundImage() {
			if(GetSkin() == null) return null;
			if(GetSkin().Image == null) return null;
			return GetSkin().Image.Image;
		}
		protected SkinImageStretch GetSkinBackgroundImageStretch() {
			if(GetSkin() == null) return SkinImageStretch.NoResize;
			if(GetSkin().Image == null) return SkinImageStretch.NoResize;
			return GetSkin().Image.Stretch;
		}
		[Description("Specifies the background image for the form."),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Image BackgroundImage {
			get { return base.BackgroundImage == null ? GetSkinBackgroundImage() : base.BackgroundImage; }
			set {
				if(value != null && value == GetSkinBackgroundImage()) value = null;
				base.BackgroundImage = value;
			}
		}
		protected bool ShouldSerializeBackgroundImageStore() {
			return base.BackgroundImage != null;
		}
		protected bool ShouldSerializeBackgroundImageLayoutStore() {
			return base.BackgroundImage != null;
		}
		[EditorBrowsable(EditorBrowsableState.Never), Browsable(false)]
		public Image BackgroundImageStore {
			get { return base.BackgroundImage; }
			set { base.BackgroundImage = value; }
		}
		[EditorBrowsable(EditorBrowsableState.Never), Browsable(false)]
		public ImageLayout BackgroundImageLayoutStore {
			get { return base.BackgroundImageLayout; }
			set { base.BackgroundImageLayout = value; }
		}
		[Description("Gets or sets the background image layout as defined in the ImageLayout enumeration."), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override ImageLayout BackgroundImageLayout {
			get {
				if(base.BackgroundImage == null) {
					if(GetSkinBackgroundImageStretch() == SkinImageStretch.NoResize) return ImageLayout.None;
					if(GetSkinBackgroundImageStretch() == SkinImageStretch.Stretch) return ImageLayout.Stretch;
					if(GetSkinBackgroundImageStretch() == SkinImageStretch.Tile) return ImageLayout.Tile;
				}
				return base.BackgroundImageLayout;
			}
			set {
				base.BackgroundImageLayout = value;
			}
		}
		IDesignerHost GetDesigner() {
			if(Site == null) return null;
			return Site.GetService(typeof(IDesignerHost)) as IDesignerHost;
		}
		public override void ResetForeColor() { ForeColor = Color.Empty; }
		[Description("Gets or sets the foreground color of the form's client region. "), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Color ForeColor {
			get { return LookAndFeelHelper.CheckTransparentForeColor(LookAndFeel, GetColor(Appearance.ForeColor, DefaultAppearance.ForeColor), null); }
			set { Appearance.ForeColor = value; }
		}
		bool useBaseFont = false;
		[Description("Gets or sets the form's font. "), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Font Font {
			get {
				if(useBaseFont)
					return base.Font;
				return Appearance.Font; 
			}
			set { Appearance.Font = value; }
		}
		internal Font GetBaseFont() { return base.Font; }
		Color GetColor(Color color, Color defColor) {
			if(color == Color.Empty) return defColor;
			return color;
		}
		AppearanceDefault defaultAppearance = null;
		protected AppearanceDefault DefaultAppearance {
			get {
				if(defaultAppearance == null)
					defaultAppearance = CreateDefaultAppearance();
				return defaultAppearance;
			}
		}
		protected virtual AppearanceDefault CreateDefaultAppearance() {
			AppearanceDefault res = new AppearanceDefault(SystemColors.ControlText, SystemColors.Control);
			if(LookAndFeel != null && LookAndFeel.ActiveStyle == ActiveLookAndFeelStyle.Skin) {
				SkinElement form = GetSkin();
				form.Apply(res);
				CheckFormAppearanceAsMdiTabPage(res);
				LookAndFeelHelper.CheckColors(LookAndFeel, res, this);
			}
			return res;
		}
		protected void CheckFormAppearanceAsMdiTabPage(AppearanceDefault defaultAppearance) {
			SkinElement tabPane;
			if(IsMdiTab(out tabPane) && tabPane != null) {
				tabPane.ApplyForeColorAndFont(defaultAppearance);
				Color solidColor1 = tabPane.Color.GetSolidImageCenterColor();
				if(solidColor1 != Color.Empty) {
					defaultAppearance.BackColor = solidColor1;
					Color solidColor2 = tabPane.Color.GetSolidImageCenterColor2();
					if(solidColor2 != Color.Empty) {
						defaultAppearance.BackColor2 = solidColor2;
						defaultAppearance.GradientMode = tabPane.Color.SolidImageCenterGradientMode;
					}
				}
			}
		}
		protected bool IsMdiTab(out SkinElement tabPane) {
			tabPane = null;
			MdiClient mdiClient = GetMdiClient(MdiParent);
			if(mdiClient != null) {
				object subclasser = GetMdiClientSubclasser(mdiClient);
				if(subclasser != null) {
					if(subclasser.GetType().Name.EndsWith("XtraMdiClientSubclasser"))
						tabPane = TabSkins.GetSkin(LookAndFeel)[TabSkins.SkinTabPane];
					if(subclasser.GetType().Name.EndsWith("DocumentManagerMdiClientSubclasser"))
						tabPane = DockingSkins.GetSkin(LookAndFeel)[DockingSkins.SkinDocumentGroupTabPane] ??
							TabSkins.GetSkin(LookAndFeel)[TabSkins.SkinTabPane];
					return true;
				}
			}
			return false;
		}
		static MdiClient GetMdiClient(Form mdiParent) {
			if(mdiParent == null)
				return null;
			if(!mdiParent.IsMdiContainer)
				return null;
			foreach(Control control in mdiParent.Controls) {
				MdiClient clientCandidate = control as MdiClient;
				if(clientCandidate != null) {
					return clientCandidate;
				}
			}
			return null;
		}
		static object GetMdiClientSubclasser(MdiClient mdiClient) {
			Assembly barsAssembly = Array.Find(AppDomain.CurrentDomain.GetAssemblies(), IsBarsAssembly);
			if(barsAssembly != null) {
				Type subclasserType = barsAssembly.GetType("DevExpress.Utils.Mdi.MdiClientSubclasser");
				MethodInfo miFromMdiClient = subclasserType.GetMethod("FromMdiClient", BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance);
				return miFromMdiClient.Invoke(null, new object[] { mdiClient });
			}
			return null;
		}
		static bool IsBarsAssembly(Assembly asm) {
			return string.Equals(asm.GetName().Name, AssemblyInfo.SRAssemblyBars);
		}
		SkinElement GetSkin() {
			if(LookAndFeel.ActiveStyle != ActiveLookAndFeelStyle.Skin) return null;
			return CommonSkins.GetSkin(LookAndFeel)[CommonSkins.SkinForm];
		}
		protected override void OnPaint(PaintEventArgs e) {
			if(FormPainter != null && FormPainter.RequirePaint) {
				FormPainter.Draw(e);
				return;
			}
			base.OnPaint(e);
		}
		protected override void OnPaintBackground(PaintEventArgs e) {
			if(FormPainter != null && FormPainter.RequirePaintBackground) {
				FormPainter.DrawBackground(e);
				return;
			}
			base.OnPaintBackground(e);
		}
		internal CreateParams GetCreateParams() {
			return CreateParams;
		}
	}
}
namespace DevExpress.XtraEditors.Drawing {
	public class SplitterInfoArgs : StyleObjectInfoArgs {
		bool isVertical = false;
		bool collapsableCore = false;
		bool invertedCore = false;
		Rectangle buttonCore;
		public SplitterInfoArgs(bool isVertical)
			: this(isVertical, false) {
		}
		public SplitterInfoArgs(bool isVertical, bool collapsable) {
			this.isVertical = isVertical;
			this.collapsableCore = collapsable;
		}
		public Rectangle Button { get { return buttonCore; } set { buttonCore = value; } }
		public bool IsVertical { get { return isVertical; } set { isVertical = value; } }
		public bool IsHorizontal { get { return !IsVertical; } set { isVertical = !value; } }
		public bool IsCollapsable { get { return collapsableCore; } set { collapsableCore = value; } }
		public bool Inverted { get { return invertedCore; } set { invertedCore = value; } }
	}
	public class SplitterPainter : StyleObjectPainter {
		ImageCollection verticalImgCore;
		ImageCollection horizontalImgCore;
		protected ImageCollection VerticalSplitGlyphs {
			get {
				if(verticalImgCore == null) {
					Image img = ResourceImageHelper.CreateImageFromResources(
							"DevExpress.Utils.Images.flatSplitterGlyphV.png", typeof(SplitterPainter).Assembly
						);
					verticalImgCore = new ImageCollection();
					verticalImgCore.ImageSize = new Size(img.Width, img.Height / 2);
					verticalImgCore.AddImageStripVertical(img);
				}
				return verticalImgCore;
			}
		}
		protected ImageCollection HorizontalSplitGlyphs {
			get {
				if(horizontalImgCore == null) {
					Image img = ResourceImageHelper.CreateImageFromResources(
							"DevExpress.Utils.Images.flatSplitterGlyphH.png", typeof(SplitterPainter).Assembly
						);
					horizontalImgCore = new ImageCollection();
					horizontalImgCore.ImageSize = new Size(img.Width / 2, img.Height);
					horizontalImgCore.AddImageStrip(img);
				}
				return horizontalImgCore;
			}
		}
		public override Rectangle CalcObjectMinBounds(ObjectInfoArgs e) {
			SplitterInfoArgs ee = e as SplitterInfoArgs;
			Size size = new Size(4, 4);
			return new Rectangle(Point.Empty, size);
		}
		public override void DrawObject(ObjectInfoArgs e) {
			SplitterInfoArgs ee = e as SplitterInfoArgs;
			ee.Appearance.DrawBackground(e.Cache, e.Bounds);
			HatchStyle style = HatchStyle.Percent50;
			Rectangle r = e.Bounds;
			r.Inflate(-1, -1);
			if(ee.State == ObjectState.Hot) {
				e.Graphics.FillRectangle(e.Cache.GetSolidBrush(DevExpress.Utils.ColorUtils.FlatBarItemHighLightBackColor), e.Bounds);
			}
			using(HatchBrush brush = new HatchBrush(style, SystemColors.ControlDark, ee.Appearance.BackColor)) {
				e.Graphics.FillRectangle(brush, r);
			}
			if(ee.IsCollapsable) {
				Image img = GetCollapsableGlyph(r.Width < r.Height, ee.Inverted);
				Rectangle imgRect = CalcCollapsableGlyphRect(r, img);
				if(!imgRect.IsEmpty) e.Graphics.DrawImageUnscaled(img, imgRect);
			}
		}
		Rectangle CalcCollapsableGlyphRect(Rectangle splitter, Image img) {
			Rectangle imgRect = Rectangle.Empty;
			bool fVertical = splitter.Width < splitter.Height;
			if((fVertical && (splitter.Height >= img.Height)) || (!fVertical && (splitter.Width >= img.Width))) {
				imgRect = new Rectangle(
						splitter.Left + (splitter.Width - img.Width) / 2, splitter.Top + (splitter.Height - img.Height) / 2,
						img.Width, img.Height
					);
			}
			return imgRect;
		}
		Image GetCollapsableGlyph(bool vertical, bool inverted) {
			ImageCollection collection = vertical ? VerticalSplitGlyphs : HorizontalSplitGlyphs;
			return collection.Images[inverted ? 0 : 1];
		}
	}
	public class SplitterLineHelper {
		[ThreadStatic]
		static SplitterLineHelper _default;
		public static SplitterLineHelper Default {
			get {
				if(_default == null) _default = new SplitterLineHelper();
				return _default;
			}
		}
		public void DrawReversibleSplitter(IntPtr handle, Rectangle bounds) {
			DrawReversibleObject(handle, bounds, CreateSplitBrush());
		}
		public void DrawReversibleLine(IntPtr handle, Rectangle bounds) {
			DrawReversibleObject(handle, bounds, CreateLineBrush());
		}
		public void DrawReversibleLine(IntPtr handle, Point startPoint, Point endPoint) {
			Rectangle bounds;
			if(startPoint.Y == endPoint.Y) 
				bounds = new Rectangle(startPoint.X, startPoint.Y, endPoint.X - startPoint.X, 2);
			else
				bounds = new Rectangle(startPoint.X, startPoint.Y, 2, endPoint.Y - startPoint.Y);
			DrawReversibleObject(handle, bounds, CreateLineBrush());
		}
		public void DrawReversibleFrame(IntPtr handle, Rectangle bounds) {
			DrawReversibleFrame(handle, bounds, CreateLineBrush());
		}
		protected void DrawReversibleObject(IntPtr handle, Rectangle bounds, IntPtr splitBrush) {
			Rectangle r = bounds;
			IntPtr dc = GetDCEx(handle, IntPtr.Zero, DCX_CACHE | DCX_LOCKWINDOWUPDATE);
			IntPtr saveBrush = SelectObject(dc, splitBrush);
			PatBlt(dc, r.X, r.Y, r.Width, r.Height, PATINVERT);
			SelectObject(dc, saveBrush);
			DeleteObject(splitBrush);
			ReleaseDC(handle, dc);
		}
		const int frameWidth = 2;
		protected void DrawReversibleFrame(IntPtr handle, Rectangle bounds, IntPtr splitBrush) {
			Rectangle r = bounds;
			IntPtr dc = GetDCEx(handle, IntPtr.Zero, DCX_CACHE | DCX_LOCKWINDOWUPDATE);
			IntPtr saveBrush = SelectObject(dc, splitBrush);
			PatBlt(dc, r.X, r.Y, frameWidth, r.Height, PATINVERT);
			PatBlt(dc, r.X + frameWidth, r.Y, r.Width - frameWidth * 2, frameWidth, PATINVERT);
			PatBlt(dc, r.X + frameWidth, r.Bottom - frameWidth, r.Width - frameWidth * 2, frameWidth, PATINVERT);
			PatBlt(dc, r.Right - frameWidth, r.Y, frameWidth, r.Height, PATINVERT);
			SelectObject(dc, saveBrush);
			DeleteObject(splitBrush);
			ReleaseDC(handle, dc);
		}
		#region API
		IntPtr CreateLineBrush() {
			short[] grayPattern = new short[8];
			for(int i = 0; i < 2; i++)
				grayPattern[i] = 0xff; 
			IntPtr hBitmap = CreateBitmap(2, 2, 1, 1, grayPattern);
			LOGBRUSH lb = new LOGBRUSH(BS_PATTERN, 0, hBitmap);
			IntPtr brush = CreateBrushIndirect(lb);
			DeleteObject(hBitmap);
			return brush;
		}
		IntPtr CreateSplitBrush() {
			short[] grayPattern = new short[8];
			for(int i = 0; i < 8; i++)
				grayPattern[i] = (short)(0x5555 << (i & 1));
			IntPtr hBitmap = CreateBitmap(8, 8, 1, 1, grayPattern);
			LOGBRUSH lb = new LOGBRUSH(BS_PATTERN, 0, hBitmap);
			IntPtr brush = CreateBrushIndirect(lb);
			DeleteObject(hBitmap);
			return brush;
		}
		[StructLayout(LayoutKind.Sequential)]
		class LOGBRUSH {
			public LOGBRUSH(int style, int color, IntPtr hatch) {
				this.Style = style;
				this.Color = color;
				this.Hatch = hatch;
			}
			public int Style;
			public int Color;
			public IntPtr Hatch;
		}
		[DllImport("GDI32.dll", CharSet = CharSet.Auto)]
		static extern bool PatBlt(IntPtr hdc, int left, int top, int width, int height, int rop);
		[DllImport("GDI32.dll", CharSet = CharSet.Auto)]
		static extern IntPtr CreateBrushIndirect(LOGBRUSH lb);
		[DllImport("GDI32.dll", CharSet = CharSet.Auto)]
		static extern IntPtr CreateBitmap(int width, int height, int planes, int bitsPerPixel, short[] lpvBits);
		[DllImport("GDI32.dll")]
		static extern bool DeleteObject(IntPtr hObject);
		[DllImport("GDI32.dll")]
		static extern IntPtr SelectObject(IntPtr hdc, IntPtr hObject);
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		static extern int ReleaseDC(IntPtr hWnd, IntPtr hDC);
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		static extern IntPtr GetDCEx(IntPtr hwnd, IntPtr clip, int flags);
		const int DCX_CACHE = 0x00000002,
				  BS_PATTERN = 3,
				  PATINVERT = 0x005A0049,
				  DCX_LOCKWINDOWUPDATE = 0x00000400
									   ;
		#endregion
	}
	public class SkinSplitterPainter : SkinCustomPainter {
		public SkinSplitterPainter(ISkinProvider provider) : base(provider) { }
		protected override SkinElementInfo CreateInfo(ObjectInfoArgs e) {
			SplitterInfoArgs ee = e as SplitterInfoArgs;
			SkinElement skinElement = CommonSkins.GetSkin(Provider)[ee.IsVertical ? CommonSkins.SkinSplitter : CommonSkins.SkinSplitterHorz];
			SkinElementInfo info = new SkinElementInfo(skinElement, e.Bounds);
			info.State = e.State;
			info.ImageIndex = 0;
			if(info.State == ObjectState.Hot) info.ImageIndex = 1;
			if(info.Element.Glyph != null && info.Element.Glyph.ImageCount == 6) {
				if(ee.IsCollapsable) info.GlyphIndex = (ee.Inverted ? 2 : 1) * 2 + info.ImageIndex;
			}
			return info;
		}
	}
	public class SplitterHelper {
		public static ObjectPainter GetPainter(UserLookAndFeel lookAndFeel) {
			switch(lookAndFeel.ActiveStyle) {
				case ActiveLookAndFeelStyle.Skin:
					return new SkinSplitterPainter(lookAndFeel);
			}
			return new SplitterPainter();
		}
	}
	public interface IMouseWheelSupport {
		void OnMouseWheel(MouseEventArgs e);
	}
	public interface IMouseWheelContainer {
		void BaseOnMouseWheel(MouseEventArgs e);
		Control Control { get; }
	}
	public class MouseWheelHelper {
		public static bool SmartMouseWheelProcessing = true;
		public static bool ProcessSmartMouseWheel(Control control, MouseEventArgs e) {
			if(!SmartMouseWheelProcessing) return false;
			XtraForm form = control.FindForm() as XtraForm;
			if(form == null || !control.IsHandleCreated) return false;
			Point screen = control.PointToScreen(e.Location);
			IMouseWheelContainer wheelContainer = form as IMouseWheelContainer;
			Point p = wheelContainer.Control.PointToClient(screen);
			HandledMouseEventArgs eh = new HandledMouseEventArgs(e.Button, e.Clicks, p.X, p.Y, e.Delta);
			ProcessMouseWheel(wheelContainer, eh);
			if(e is HandledMouseEventArgs) {
				((HandledMouseEventArgs)e).Handled = true;
			}
			return true;
		}
		public static bool IsAllowSmartMouseWheel(Control control, MouseEventArgs e) {
			return ProcessSmartMouseWheel(control, e);
		}
		public static bool IsAllowSmartMouseWheel(Control control) {
			if(!SmartMouseWheelProcessing) return false;
			return (control.FindForm() is IMouseWheelContainer);
		}
		public static void DoMouseWheel(IMouseWheelContainer container, MouseEventArgs e) {
			if(SmartMouseWheelProcessing) {
				ProcessMouseWheel(container, e);
			}
			container.BaseOnMouseWheel(e);
		}
		static void ProcessMouseWheel(IMouseWheelContainer container, MouseEventArgs e) {
			container.BaseOnMouseWheel(e);
			HandledMouseEventArgs ee = e as HandledMouseEventArgs;
			if(ee != null && ee.Handled) return;
			Control controlInPosition = FindControl(e.Location, container.Control);
			if(controlInPosition != container.Control)
				ProcessControlMouseWheel(e, controlInPosition);
		}
		static void ProcessControlMouseWheel(MouseEventArgs e, Control controlInPosition) {
			IMouseWheelSupport control = controlInPosition as IMouseWheelSupport;
			if(control != null) control.OnMouseWheel(e);
		}
		static Control FindControl(Point point, Control parent) {
			for(int n = 0; n < parent.Controls.Count; n++) {
				Point screenPoint = parent.IsHandleCreated ? parent.PointToScreen(point) : point;
				Control ctrl = parent.Controls[n];
				if(CanIgnoreControl(ctrl)) continue;
				if(ctrl.Bounds.Contains(point)) {
					Control res = FindControl(ctrl.IsHandleCreated ? ctrl.PointToClient(screenPoint) : point, ctrl);
					return res == null ? ctrl : res;
				}
			}
			return parent;
		}
		static bool CanIgnoreControl(Control ctrl) {
			if(!ctrl.Visible) return true;
			if(ctrl is ScrollBarBase) return true;
			return false;
		}
	}
}
namespace DevExpress.Utils.Mdi {
	public static class MdiClientSubclasserService {
		static readonly IDictionary clients = new Hashtable();
		public static void Register(MdiClient mdiClient, NativeWindow subclasser) {
			lock(clients.SyncRoot) {
				if(mdiClient != null)
					clients.Add(mdiClient, subclasser);
			}
		}
		public static void Unregister(MdiClient mdiClient) {
			lock(clients.SyncRoot) {
				if(mdiClient != null)
					clients.Remove(mdiClient);
			}
		}
		public static NativeWindow FromMdiClient(MdiClient client) {
			lock(clients.SyncRoot) {
				return clients[client] as NativeWindow;
			}
		}
		public static MdiClient GetMdiClient(Form mdiParent) {
			if(mdiParent == null)
				return null;
			if(!mdiParent.IsMdiContainer)
				return null;
			foreach(Control control in mdiParent.Controls) {
				MdiClient clientCandidate = control as MdiClient;
				if(clientCandidate != null) {
					return clientCandidate;
				}
			}
			return null;
		}
	}
	public interface ISysCommandListener {
		void PreviewMessage(IntPtr hWnd, int cmd);
	}
}
