#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{                                                                   }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.Collections;
using DevExpress.Utils.Win.Hook;
using DevExpress.Utils.Win;
using DevExpress.Utils.Drawing.Helpers;
namespace DevExpress.XtraEditors.Controls {
	public class PopupController {
		public static IPopupServiceControl CreateController() {
			return new WinPopupController();
		}
	}
	public class WinPopupController : IPopupServiceControl {
		void IPopupServiceControl.UpdateTopMost(IntPtr handle) {
			SetWindowPos(handle, (IntPtr)HWND_TOPMOST, 0,0,0,0, SWP_NOACTIVATE | SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOMOVE);
		}
		bool IPopupServiceControl.SetVisibleCore(Control control, bool newVisible) {
#if DXWhidbey
			if(newVisible) {
				(this as IPopupServiceControl).UpdateTopMost(control.Handle);
				ShowWindow(control.Handle, 8);
			}
			return true;
#else
			if(!newVisible)
				return false;
			else {
				(this as IPopupServiceControl).UpdateTopMost(control.Handle);
				ShowWindow(control.Handle, 8);
				return true;
			}
#endif
		}
		bool IPopupServiceControl.SetSimpleVisibleCore(Control control, IntPtr parentForm, bool newVisible) {
#if DXWhidbey
			if(newVisible) {
				SetWindowPos(control.Handle, parentForm, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
				if(parentForm != IntPtr.Zero)
					ReparentWindow(control, parentForm);
				ShowWindow(control.Handle, 8);
			}
			return true;
#else
			if(!newVisible)
				return false;
			else {
				SetWindowPos(control.Handle, parentForm, 0,0,100,100, SWP_NOACTIVATE | SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
				ShowWindow(control.Handle, 8);
				return true;
			}
#endif
		}
		public void ReparentWindow(Control control, IntPtr parentForm) {
			if(!control.IsHandleCreated) return;
			parentForm = NativeMethods.GetDesktopWindow(); 
			NativeMethods.SetWindowLong(control.Handle, -8, parentForm);
		}
		public void EmulateFormFocus(IntPtr formHandle) {
			DevExpress.Utils.Drawing.Helpers.NativeMethods.SendMessage(formHandle, 0x86, 1, (IntPtr) (-1)); 
			DevExpress.Utils.Drawing.Helpers.NativeMethods.RedrawWindow(formHandle, IntPtr.Zero, IntPtr.Zero, 0x401);
		}
		public virtual void PopupClosed(IPopupControl popup) {
		}
		public virtual void PopupShowing(IPopupControl popup) {
		}
		protected const int MA_NOACTIVATE = 3;
		protected const int WM_MOUSEACTIVATE = 0x0021,
			WM_LBUTTONDOWN   = 0x0201;
		bool IPopupServiceControl.WndProc(ref System.Windows.Forms.Message m) {
			Control control = Control.FromHandle(m.HWnd);
			switch(m.Msg) {
				case WM_MOUSEACTIVATE : 
					m.Result = (IntPtr)MA_NOACTIVATE;
					return true;
				case WM_LBUTTONDOWN : 
					if(control is ListBox) return true;
					break;
			}
			return false;
		}
		bool IPopupServiceControl.IsDummy { get { return false; } 
		}
		[System.Runtime.InteropServices.DllImport("USER32.dll")]
		static extern bool ShowWindow(IntPtr hWnd, int cmdShow);
		const int SWP_NOSIZE = 0x0001,
			SWP_NOMOVE = 0x0002,
			SWP_NOACTIVATE = 0x0010,
			SWP_SHOWWINDOW = 0x0040,
			HWND_TOPMOST = -1;
		[System.Runtime.InteropServices.DllImport("USER32.dll")]
		static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter,
			int X, int Y, int cx, int cy, uint uFlags);
		[System.Runtime.InteropServices.DllImport("user32.dll")]
		static extern int SendMessage(IntPtr hWnd, int Msg, uint wParam, uint lParam);
	}
	public class HookPopup : IHookController, IDisposable {
		IPopupControl popup;
		public HookPopup(IPopupControl popup) {
			this.popup = popup;
			HookManager.DefaultManager.AddController(this);
		}
		public virtual void Dispose() {
			HookManager.DefaultManager.RemoveController(this);
		}
		public IPopupControl Popup { get { return popup; } }
		public IPopupControlEx PopupEx { get { return Popup as IPopupControlEx; } }
		protected Control GetParent(Control control) {
			Control parent = control;
			while(parent.Parent != null) parent = parent.Parent;
			return parent;
		}
		protected void CheckMouseDown(Control control, Point mousePosition) {
			Control popup = Popup as Control;
			IPopupControl pc = Popup;
			if(pc == null || !popup.Created || !popup.Visible || pc.PopupWindow == null || !pc.PopupWindow.Visible) return;
			Control parent = GetParent(pc.PopupWindow);
			if(parent.Contains(control) || parent == control || popup == control || popup.Contains(control)) return;
			if(IsPopupMenu(control)) return;
			if(!pc.AllowMouseClick(control, mousePosition)) pc.ClosePopup();
		}
		protected bool IsPopupMenu(Control control) {
			if(control is Shadow) return true;
			if(control != null && (control.GetType().Name == "SubMenuBarControl" || control.GetType().Name == "PopupMenuBarControl")) return true; 
			return false;
		}
		protected void ClosePopups() {
			Popup.ClosePopup();
		}
		const int WM_ACTIVATEAPP = 0x1C,WM_MINCLICK = 0x201, WM_MAXCLICK = 0x208, WM_ACTIVATE = 0x6, 
			WM_MINNCCLICK = 0xA1, WM_MAXNCCLICK = 0x0a9, WM_LBUTTONUP = 0x0202, WM_RBUTTONUP = 0x0205, WM_MBUTTONUP = 0x0208, WA_INACTIVE = 0;
		static int[] upMessages = new int[] { WM_LBUTTONUP, WM_RBUTTONUP, WM_MBUTTONUP};
		protected Control GetControl(IntPtr hwnd) {
			Control control = Control.FromHandle(hwnd);
			if(control == null) control = Control.FromChildHandle(hwnd);
			return control;
		}
		bool IHookController.InternalPreFilterMessage(int Msg, Control wnd, IntPtr HWnd, IntPtr WParam, IntPtr LParam) {
			return PreFilterMessageCore(Msg, HWnd, WParam);
		}
		protected virtual bool PreFilterMessageCore(int Msg, IntPtr HWnd, IntPtr WParam) {
			if((Msg >= WM_MINCLICK && Msg <= WM_MAXCLICK) || (Msg >= WM_MINNCCLICK && Msg <= WM_MAXNCCLICK)) {
				if(Array.IndexOf(upMessages, Msg) != -1) return false;
				CheckMouseDown(GetControl(HWnd), Control.MousePosition);
			}
			if(Msg == WM_ACTIVATEAPP) {
				if(WParam.ToInt32() == 0) ClosePopups();
			}
			if(Msg == 0x18 && WParam == IntPtr.Zero) { 
				if(PopupEx != null) PopupEx.WindowHidden(GetControl(HWnd));
			}
			if(Msg == WM_ACTIVATE) {
				if(PopupEx != null && (WParam.ToInt32() & 0xffff) != WA_INACTIVE) PopupEx.CheckClosePopup(GetControl(HWnd));
			}
			return false;
		}
		bool IHookController.InternalPostFilterMessage(int Msg, Control wnd, IntPtr HWnd, IntPtr WParam, IntPtr LParam) {
			return false;
		}
		IntPtr IHookController.OwnerHandle {
			get {
				Control popup = Popup as Control;
				return popup == null || !popup.IsHandleCreated ? IntPtr.Zero : popup.Handle;
			}
		}
	}
	public class HookPopupController : WinPopupController {
		ArrayList popups;
		protected ArrayList Popups { get { return popups; } }
		public HookPopupController() {
			popups = new ArrayList();
		}
		protected virtual HookPopup FindPopup(IPopupControl popup) {
			for(int n = 0; n < Popups.Count; n++) {
				HookPopup ppp = Popups[n] as HookPopup;
				if(ppp.Popup == popup) return ppp;
			}
			return null;
		}
		public override void PopupClosed(IPopupControl popup) {
			HookPopup ppp = FindPopup(popup);
			if(ppp != null) {
				Popups.Remove(ppp);
				ppp.Dispose();
			}
		}
		public override void PopupShowing(IPopupControl popup) {
			Popups.Add(new HookPopup(popup));
		}
	}
}
