#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.ComponentModel;
using System.Collections;
using System.Windows.Forms;
using DevExpress.Utils.Controls;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Columns;
using DevExpress.Utils.Serializing;
using DevExpress.Utils;
using DevExpress.Data;
using DevExpress.XtraEditors;
namespace DevExpress.XtraGrid.Columns {
	public class OptionsColumn : ViewBaseOptions {
		bool allowMove, allowSize, readOnly, allowFocus, fixedWidth, allowShowHide,
			showInCustomizationForm, showInExpressionEditor, allowEdit, showCaption, allowIncrementalSearch, tabStop;
		DefaultBoolean allowMerge, allowSort, allowGroup;
		public OptionsColumn() {
			this.tabStop = true;
			this.allowShowHide = true;
			this.showCaption = true;
			this.allowIncrementalSearch = true;
			this.allowEdit = true;
			this.allowMove = true;
			this.allowSize = true;
			this.allowFocus = true;
			this.readOnly = false;
			this.fixedWidth = false;
			this.showInCustomizationForm = true;
			this.showInExpressionEditor = true;
			this.allowMerge = DefaultBoolean.Default;
			this.allowGroup = DefaultBoolean.Default;
			this.allowSort = DefaultBoolean.Default;
		}
		[Description("Gets or sets whether the column's caption is displayed within the column header."), DefaultValue(true), XtraSerializableProperty()]
		public bool ShowCaption {
			get { return showCaption; }
			set {
				if(ShowCaption == value) return;
				bool prevValue = ShowCaption;
				showCaption = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowCaption", prevValue, ShowCaption));
			}
		}
		[Description("Gets or sets whether the column can be focused via the TAB key."), DefaultValue(true), XtraSerializableProperty()]
		public bool TabStop {
			get { return tabStop; }
			set {
				if(TabStop == value) return;
				bool prevValue = TabStop;
				tabStop = value;
				OnChanged(new BaseOptionChangedEventArgs("TabStop", prevValue, TabStop));
			}
		}
		[Description("Gets or sets whether end-users are allowed to invoke editors for the column's cells."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowEdit {
			get { return allowEdit; }
			set {
				if(AllowEdit == value) return;
				bool prevValue = AllowEdit;
				allowEdit = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowEdit", prevValue, AllowEdit));
			}
		}
		[Description("Gets or sets whether incremental searches can be performed on the column."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowIncrementalSearch {
			get { return allowIncrementalSearch; }
			set {
				if(AllowIncrementalSearch == value) return;
				bool prevValue = AllowIncrementalSearch;
				allowIncrementalSearch = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowIncrementalSearch", prevValue, AllowIncrementalSearch));
			}
		}
		[Description("Gets or sets whether end-users can drag the column's header."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowMove {
			get { return allowMove; }
			set {
				if(AllowMove == value) return;
				bool prevValue = AllowMove;
				allowMove = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowMove", prevValue, AllowMove));
			}
		}
		[Description("Gets or sets whether the column can be hidden or restored by an end-user."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowShowHide {
			get { return allowShowHide; }
			set {
				if(AllowShowHide == value) return;
				bool prevValue = AllowShowHide;
				allowShowHide = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowShowHide", prevValue, AllowShowHide));
			}
		}
		[Description("Gets or sets whether end-users can drag the column's header to the group panel to group data against the column's values."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public DefaultBoolean AllowGroup {
			get { return allowGroup; }
			set {
				if(!Enum.IsDefined(typeof(DefaultBoolean), value)) {
					throw new InvalidEnumArgumentException("value", (int)value, typeof(DefaultBoolean));
				}
				if(AllowGroup == value) return;
				DefaultBoolean prevValue = AllowGroup;
				allowGroup = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowGroup", prevValue, AllowGroup));
			}
		}
		[Description("Gets or sets whether end-users can sort data by the column's values."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public DefaultBoolean AllowSort {
			get { return allowSort; }
			set {
				if(!Enum.IsDefined(typeof(DefaultBoolean), value)) {
					throw new InvalidEnumArgumentException("value", (int)value, typeof(DefaultBoolean));
				}
				if(AllowSort == value) return;
				DefaultBoolean prevValue = AllowSort;
				allowSort = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowSort", prevValue, AllowSort));
			}
		}
		[Description("Gets or sets whether neighboring cells with identical values can be merged within the current column."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public DefaultBoolean AllowMerge {
			get { return allowMerge; }
			set {
				if(!Enum.IsDefined(typeof(DefaultBoolean), value)) {
					throw new InvalidEnumArgumentException("value", (int)value, typeof(DefaultBoolean));
				}
				if(AllowMerge == value) return;
				DefaultBoolean prevValue = AllowMerge;
				allowMerge = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowMerge", prevValue, AllowMerge));
			}
		}
		[Description("Gets or sets whether end-users can drag the column header's right edge to change the column's width."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowSize {
			get { return allowSize; }
			set {
				if(AllowSize == value) return;
				bool prevValue = AllowSize;
				allowSize = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowSize", prevValue, AllowSize));
			}
		}
		[Description("Gets or sets whether end-users can move focus to the column using either the mouse or keyboard."), DefaultValue(true), XtraSerializableProperty()]
		public bool AllowFocus {
			get { return allowFocus; }
			set {
				if(AllowFocus == value) return;
				bool prevValue = AllowFocus;
				allowFocus = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFocus", prevValue, AllowFocus));
			}
		}
		[Description("Gets or sets whether end-users are prevented from editing the column's cell values."), DefaultValue(false), XtraSerializableProperty()]
		public bool ReadOnly {
			get { return readOnly; }
			set {
				if(ReadOnly == value) return;
				bool prevValue = ReadOnly;
				readOnly = value;
				OnChanged(new BaseOptionChangedEventArgs("ReadOnly", prevValue, ReadOnly));
			}
		}
		[Description("Gets or sets whether the column's width remains the same when the column's auto width feature is enabled."), DefaultValue(false), XtraSerializableProperty()]
		public bool FixedWidth {
			get { return fixedWidth; }
			set {
				if(FixedWidth == value) return;
				bool prevValue = FixedWidth;
				fixedWidth = value;
				OnChanged(new BaseOptionChangedEventArgs("FixedWidth", prevValue, FixedWidth));
			}
		}
		[Description("Gets or sets whether the column's header is displayed within the customization form when the column is hidden."), DefaultValue(true), XtraSerializableProperty()]
		public bool ShowInCustomizationForm {
			get { return showInCustomizationForm; }
			set {
				if(ShowInCustomizationForm == value) return;
				bool prevValue = ShowInCustomizationForm;
				showInCustomizationForm = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowInCustomizationForm", prevValue, ShowInCustomizationForm));
			}
		}
		[Description("Gets or sets whether the current column is available in the field list in the Expression Editor."), DefaultValue(true), XtraSerializableProperty()]
		public bool ShowInExpressionEditor {
			get { return showInExpressionEditor; }
			set {
				if(ShowInExpressionEditor == value) return;
				bool prevValue = ShowInExpressionEditor;
				showInExpressionEditor = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowInExpressionEditor", prevValue, ShowInExpressionEditor));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				OptionsColumn opt = options as OptionsColumn;
				if(opt == null) return;
				this.allowShowHide = opt.AllowShowHide;
				this.showCaption = opt.ShowCaption;
				this.allowMerge = opt.allowMerge;
				this.allowEdit = opt.AllowEdit;
				this.allowIncrementalSearch = opt.AllowIncrementalSearch;
				this.allowMove = opt.AllowMove;
				this.allowGroup = opt.AllowGroup;
				this.allowSize = opt.AllowSize;
				this.allowSort = opt.AllowSort;
				this.readOnly = opt.ReadOnly;
				this.allowFocus = opt.AllowFocus;
				this.fixedWidth = opt.FixedWidth;
				this.tabStop = opt.TabStop;
				this.showInCustomizationForm = opt.ShowInCustomizationForm;
				this.showInExpressionEditor = opt.ShowInExpressionEditor;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class OptionsColumnFilter : ViewBaseOptions {
		DefaultBoolean immediateUpdatePopupDateFilterOnCheck, immediateUpdatePopupDateFilterOnDateChange, allowFilterModeChanging,
			filterBySortField;
		bool allowFilter, allowAutoFilter, immediateUpdateAutoFilter, showEmptyDateFilter;
		AutoFilterCondition autoFilterCondition;
		FilterPopupMode filterPopupMode;
		public OptionsColumnFilter() {
			this.filterBySortField = DefaultBoolean.Default;
			this.showEmptyDateFilter = false;
			this.autoFilterCondition = AutoFilterCondition.Default;
			this.filterPopupMode = FilterPopupMode.Default;
			this.allowFilter = true;
			this.immediateUpdatePopupDateFilterOnDateChange = DefaultBoolean.Default;
			this.immediateUpdatePopupDateFilterOnCheck = DefaultBoolean.Default;
			this.allowAutoFilter = true;
			this.immediateUpdateAutoFilter = true;
			this.allowFilterModeChanging = DefaultBoolean.Default;
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				OptionsColumnFilter opt = options as OptionsColumnFilter;
				if(opt == null) return;
				this.filterBySortField = opt.FilterBySortField;
				this.showEmptyDateFilter = opt.ShowEmptyDateFilter;
				this.immediateUpdatePopupDateFilterOnDateChange = opt.ImmediateUpdatePopupDateFilterOnDateChange;
				this.immediateUpdatePopupDateFilterOnCheck = opt.ImmediateUpdatePopupDateFilterOnCheck;
				this.autoFilterCondition = opt.AutoFilterCondition;
				this.filterPopupMode = opt.FilterPopupMode;
				this.allowFilter = opt.AllowFilter;
				this.allowAutoFilter = opt.AllowAutoFilter;
				this.immediateUpdateAutoFilter = opt.ImmediateUpdateAutoFilter;
				this.allowFilterModeChanging = opt.AllowFilterModeChanging;
			}
			finally {
				EndUpdate();
			}
		}
		[Description("Gets or sets whether the filter button in Grid Views (field filter button in Layout Views) is displayed within the column header. "), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowFilter {
			get { return allowFilter; }
			set {
				if(AllowFilter == value) return;
				bool prevValue = AllowFilter;
				allowFilter = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFilter", prevValue, AllowFilter));
			}
		}
		[Description("Gets or sets the type of the comparison operator used to create filter conditions for the current column via the auto filter row."), DefaultValue(AutoFilterCondition.Default), XtraSerializableProperty()]
		public virtual AutoFilterCondition AutoFilterCondition {
			get { return autoFilterCondition; }
			set {
				if(AutoFilterCondition == value) return;
				AutoFilterCondition prevValue = AutoFilterCondition;
				autoFilterCondition = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoFilterCondition", prevValue, AutoFilterCondition));
			}
		}
		[Description("Gets or sets whether the column's values can be filtered using the auto filter row."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowAutoFilter {
			get { return allowAutoFilter; }
			set {
				if(AllowAutoFilter == value) return;
				bool prevValue = AllowAutoFilter;
				allowAutoFilter = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowAutoFilter", prevValue, AllowAutoFilter));
			}
		}
		[Description("Gets or sets whether the column's filter condition is updated as soon as an end-user modifies the contents of the auto filter row's cell."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ImmediateUpdateAutoFilter {
			get { return immediateUpdateAutoFilter; }
			set {
				if(ImmediateUpdateAutoFilter == value) return;
				bool prevValue = ImmediateUpdateAutoFilter;
				immediateUpdateAutoFilter = value;
				OnChanged(new BaseOptionChangedEventArgs("ImmediateUpdateAutoFilter", prevValue, ImmediateUpdateAutoFilter));
			}
		}
		[Description("Gets or sets whether the column's Filter Dropdown List displays values of the GridColumn.FieldNameSortGroup field  or values of the GridColumn.FieldName field."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean FilterBySortField {
			get { return filterBySortField; }
			set {
				if(FilterBySortField == value) return;
				DefaultBoolean prevValue = FilterBySortField;
				filterBySortField = value;
				OnChanged(new BaseOptionChangedEventArgs("FilterBySortField", prevValue, FilterBySortField));
			}
		}
		[Description("This option is in effect for date-time columns whose filter dropdown list contains an embedded calendar with check boxes to select common date intervals.It specifies whether toggling the check boxes filters grid records immediately. "), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean ImmediateUpdatePopupDateFilterOnCheck {
			get { return immediateUpdatePopupDateFilterOnCheck; }
			set {
				if(ImmediateUpdatePopupDateFilterOnCheck == value) return;
				DefaultBoolean prevValue = ImmediateUpdatePopupDateFilterOnCheck;
				immediateUpdatePopupDateFilterOnCheck = value;
				OnChanged(new BaseOptionChangedEventArgs("ImmediateUpdatePopupDateFilterOnCheck", prevValue, ImmediateUpdatePopupDateFilterOnCheck));
			}
		}
		[Description("This option is in effect for date-time columns whose filter dropdown list contains an embedded calendar with check boxes to select common date intervals.It specifies whether selecting a date or date range via the embedded calendar filters grid records immediately."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean ImmediateUpdatePopupDateFilterOnDateChange {
			get { return immediateUpdatePopupDateFilterOnDateChange; }
			set {
				if(ImmediateUpdatePopupDateFilterOnDateChange == value) return;
				DefaultBoolean prevValue = ImmediateUpdatePopupDateFilterOnDateChange;
				immediateUpdatePopupDateFilterOnDateChange = value;
				OnChanged(new BaseOptionChangedEventArgs("ImmediateUpdatePopupDateFilterOnDateChange", prevValue, ImmediateUpdatePopupDateFilterOnDateChange));
			}
		}
		[Description("Gets or sets the display mode for the current column's filter dropdown."), DefaultValue(FilterPopupMode.Default), XtraSerializableProperty()]
		public virtual FilterPopupMode FilterPopupMode {
			get { return filterPopupMode; }
			set {
				if(FilterPopupMode == value) return;
				FilterPopupMode prevValue = FilterPopupMode;
				filterPopupMode = value;
				OnChanged(new BaseOptionChangedEventArgs("FilterPopupMode", prevValue, FilterPopupMode));
			}
		}
		[Description("Gets or sets whether a Filter Dropdown List has a filter used to select records that contain null values in a date-time column."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowEmptyDateFilter {
			get { return showEmptyDateFilter; }
			set {
				if(ShowEmptyDateFilter == value) return;
				bool prevValue = ShowEmptyDateFilter;
				showEmptyDateFilter = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowEmptyDateFilter", prevValue, ShowEmptyDateFilter));
			}
		}
		[Description("Gets or sets whether a dedicated menu item is available in a column header menu that allows an end-user to change the filter mode for columns."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean AllowFilterModeChanging {
			get { return allowFilterModeChanging; }
			set {
				if(AllowFilterModeChanging == value) return;
				DefaultBoolean prevValue = AllowFilterModeChanging;
				allowFilterModeChanging = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFilterModeChanging", prevValue, AllowFilterModeChanging));
			}
		}
	}
}
namespace DevExpress.XtraGrid.Views.Base {
	public class ColumnViewOptionsFind : ViewBaseOptions {
		int findDelay;
		bool allowFindPanel, showCloseButton, alwaysVisible, clearFindOnClose, highlightFindResults;
		FindMode findMode;
		string findFilterColumns;
		public ColumnViewOptionsFind() {
			this.highlightFindResults = true;
			this.clearFindOnClose = true;
			this.findDelay = 1000;
			this.alwaysVisible = false;
			this.allowFindPanel = true;
			this.showCloseButton = true;
			this.findFilterColumns = "*";
		}
		[Category("Data"), Description("Gets or sets whether data searching starts automatically, or must be started manually."), DefaultValue(FindMode.Default)]
		public FindMode FindMode {
			get { return findMode; }
			set {
				if(FindMode == value) return;
				FindMode prevValue = FindMode;
				findMode = value;
				OnChanged(new BaseOptionChangedEventArgs("FindMode", prevValue, FindMode));
			}
		}
		[Description("Gets or sets the delay in milliseconds, after which a data search is initiated (if automatic search mode is active)."), DefaultValue(1000), XtraSerializableProperty()]
		public virtual int FindDelay {
			get { return findDelay; }
			set {
				if(value < 100) value = 100;
				if(FindDelay == value) return;
				int prevValue = FindDelay;
				findDelay = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFindFilter", prevValue, FindDelay));
			}
		}
		[Description("Gets or sets whether the Find Panel can be invoked by an end-user."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowFindPanel {
			get { return allowFindPanel; }
			set {
				if(AllowFindPanel == value) return;
				bool prevValue = AllowFindPanel;
				allowFindPanel = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFindPanel", prevValue, AllowFindPanel));
			}
		}
		[Description("Gets or sets whether the search string is highlighted within located records."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool HighlightFindResults {
			get { return highlightFindResults; }
			set {
				if(HighlightFindResults == value) return;
				bool prevValue = HighlightFindResults;
				highlightFindResults = value;
				OnChanged(new BaseOptionChangedEventArgs("HighlightFindResults", prevValue, HighlightFindResults));
			}
		}
		[Description("Gets or sets whether the Find Panel's search (filter) string is cleared when the Find Panel is hidden."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ClearFindOnClose {
			get { return clearFindOnClose; }
			set {
				if(ClearFindOnClose == value) return;
				bool prevValue = ClearFindOnClose;
				clearFindOnClose = value;
				OnChanged(new BaseOptionChangedEventArgs("ClearFindOnClose", prevValue, ClearFindOnClose));
			}
		}
		[Description("Gets whether the close ('x') button is displayed within the Find Panel."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowCloseButton {
			get { return showCloseButton; }
			set {
				if(ShowCloseButton == value) return;
				bool prevValue = ShowCloseButton;
				showCloseButton = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowCloseButton", prevValue, ShowCloseButton));
			}
		}
		[Description("Gets or sets whether the Find Panel is always visible."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AlwaysVisible {
			get { return alwaysVisible; }
			set {
				if(AlwaysVisible == value) return;
				bool prevValue = AlwaysVisible;
				alwaysVisible = value;
				OnChanged(new BaseOptionChangedEventArgs("AlwaysVisible", prevValue, AlwaysVisible));
			}
		}
		[Description("Gets or sets the field names against which searches are performed by the Find Panel."), DefaultValue("*"), XtraSerializableProperty()]
		public virtual string FindFilterColumns {
			get { return findFilterColumns; }
			set {
				if(value == null) value = "";
				if(FindFilterColumns == value) return;
				string prevValue = FindFilterColumns;
				findFilterColumns = value;
				OnChanged(new BaseOptionChangedEventArgs("FindFilterColumns", prevValue, FindFilterColumns));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				ColumnViewOptionsFind opt = options as ColumnViewOptionsFind;
				if(opt == null) return;
				this.findMode = opt.FindMode;
				this.alwaysVisible = opt.AlwaysVisible;
				this.allowFindPanel = opt.AllowFindPanel;
				this.showCloseButton = opt.ShowCloseButton;
				this.findFilterColumns = opt.FindFilterColumns;
				this.highlightFindResults = opt.HighlightFindResults;
				this.findDelay = opt.FindDelay;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class ColumnViewOptionsFilter : ViewBaseOptions {
		bool allowMRUFilterList, allowColumnMRUFilterList, useNewCustomFilterDialog, showAllTableValuesInFilterPopup,
			allowFilterEditor, allowFilterIncrementalSearch, showAllTableValuesInCheckedFilterPopup,
			allowMultiSelectInCheckedFilterPopup, filterEditorUseMenuForOperandsAndOperators;
		int mruFilterListCount, mruFilterListPopupCount, mruColumnFilterListCount, columnFilterPopupMaxRecordsCount,
			columnFilterPopupRowCount, maxCheckedListItemCount;
		FilterEditorViewMode defaultFilterEditorView;
		FilterControlAllowAggregateEditing filterEditorAggregateEditing;
		public ColumnViewOptionsFilter() {
			this.columnFilterPopupMaxRecordsCount = -1;
			this.columnFilterPopupRowCount = 20;
			this.allowMRUFilterList = allowColumnMRUFilterList = allowFilterEditor = true;
			this.allowFilterIncrementalSearch = true;
			this.allowMultiSelectInCheckedFilterPopup = true;
			this.mruFilterListCount = 10;
			this.mruFilterListPopupCount = 7;
			this.mruColumnFilterListCount = 5;
			this.showAllTableValuesInFilterPopup = false;
			this.showAllTableValuesInCheckedFilterPopup = true;
			this.useNewCustomFilterDialog = false;
			this.maxCheckedListItemCount = 25;
			this.defaultFilterEditorView = FilterEditorViewMode.Visual;
			this.filterEditorAggregateEditing = FilterControlAllowAggregateEditing.No;
			this.filterEditorUseMenuForOperandsAndOperators = true;
		}
		[Browsable(false), Description("This member supports the internal infrastructure, and is not intended to be used directly from your code."), DefaultValue(FilterControlAllowAggregateEditing.No), XtraSerializableProperty()]
		public virtual FilterControlAllowAggregateEditing FilterEditorAggregateEditing {
			get { return filterEditorAggregateEditing; }
			set {
				if(FilterEditorAggregateEditing == value) return;
				FilterControlAllowAggregateEditing prevValue = FilterEditorAggregateEditing;
				filterEditorAggregateEditing = value;
				OnChanged(new BaseOptionChangedEventArgs("FilterEditorAggregateEditing", prevValue, FilterEditorAggregateEditing));
			}
		}
		[Description("Gets or sets whether a menu or combobox editor is used to select operands and operators in a Filter Editor. This property is not supported when filters are edited in a text form (see ColumnViewOptionsFilter.DefaultFilterEditorView)."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool FilterEditorUseMenuForOperandsAndOperators {
			get { return filterEditorUseMenuForOperandsAndOperators; }
			set {
				if(FilterEditorUseMenuForOperandsAndOperators == value) return;
				bool prevValue = FilterEditorUseMenuForOperandsAndOperators;
				filterEditorUseMenuForOperandsAndOperators = value;
				OnChanged(new BaseOptionChangedEventArgs("FilterEditorUseMenuForOperandsAndOperators", prevValue, FilterEditorUseMenuForOperandsAndOperators));
			}
		}
		[Description(""), DefaultValue(FilterEditorViewMode.Visual), XtraSerializableProperty()]
		public virtual FilterEditorViewMode DefaultFilterEditorView {
			get { return defaultFilterEditorView; }
			set {
				if(DefaultFilterEditorView == value) return;
				FilterEditorViewMode prevValue = DefaultFilterEditorView;
				defaultFilterEditorView = value;
				OnChanged(new BaseOptionChangedEventArgs("DefaultFilterView", prevValue, DefaultFilterEditorView));
			}
		}
		[Description("Gets or sets whether checked filter dropdown lists must display unique values from all the records in the View's data source or only from those records that meet the current filter criteria. This option is in effect for checked filter dropdown lists. "), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowAllTableValuesInCheckedFilterPopup {
			get { return showAllTableValuesInCheckedFilterPopup; }
			set {
				if(ShowAllTableValuesInCheckedFilterPopup == value) return;
				bool prevValue = ShowAllTableValuesInCheckedFilterPopup;
				showAllTableValuesInCheckedFilterPopup = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowAllTableValuesInCheckedFilterPopup", prevValue, ShowAllTableValuesInCheckedFilterPopup));
			}
		}
		[Description("Gets or sets whether filter dropdown lists must display unique values from all the records in the View's data source or only from those records that meet the current filter criteria. This option is in effect for regular filter dropdown lists."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowAllTableValuesInFilterPopup {
			get { return showAllTableValuesInFilterPopup; }
			set {
				if(ShowAllTableValuesInFilterPopup == value) return;
				bool prevValue = ShowAllTableValuesInFilterPopup;
				showAllTableValuesInFilterPopup = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowAllTableValuesInFilterPopup", prevValue, ShowAllTableValuesInFilterPopup));
			}
		}
		[Description("Gets or sets whether the advanced custom filter dialog is used instead of the standard one."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool UseNewCustomFilterDialog {
			get { return useNewCustomFilterDialog; }
			set {
				if(UseNewCustomFilterDialog == value) return;
				bool prevValue = UseNewCustomFilterDialog;
				useNewCustomFilterDialog = value;
				OnChanged(new BaseOptionChangedEventArgs("UseNewCustomFilterDialog", prevValue, UseNewCustomFilterDialog));
			}
		}
		[Description("Gets or sets the maximum number of records whose field values can be displayed within filter dropdowns."), DefaultValue(-1), XtraSerializableProperty()]
		public virtual int ColumnFilterPopupMaxRecordsCount {
			get { return columnFilterPopupMaxRecordsCount; }
			set {
				if(value < 0) value = -1;
				if(ColumnFilterPopupMaxRecordsCount == value) return;
				int prevValue = ColumnFilterPopupMaxRecordsCount;
				columnFilterPopupMaxRecordsCount = value;
				OnChanged(new BaseOptionChangedEventArgs("ColumnFilterPopupMaxRecordsCount", prevValue, ColumnFilterPopupMaxRecordsCount));
			}
		}
		[Description("Gets or sets the maximum number of items that regular filter dropdowns in columns, can display simultaneously."), DefaultValue(20), XtraSerializableProperty()]
		public virtual int ColumnFilterPopupRowCount {
			get { return columnFilterPopupRowCount; }
			set {
				if(value < 4) value = 4;
				if(value > 100) value = 100;
				if(ColumnFilterPopupRowCount == value) return;
				int prevValue = ColumnFilterPopupRowCount;
				columnFilterPopupRowCount = value;
				OnChanged(new BaseOptionChangedEventArgs("ColumnFilterPopupRowCount", prevValue, ColumnFilterPopupRowCount));
			}
		}
		[Description("Gets or sets the capacity of the list that stores the recently used filter conditions for the View."), DefaultValue(10), XtraSerializableProperty()]
		public virtual int MRUFilterListCount {
			get { return mruFilterListCount; }
			set {
				if(value < 0) value = 0;
				if(MRUFilterListCount == value) return;
				int prevValue = MRUFilterListCount;
				mruFilterListCount = value;
				OnChanged(new BaseOptionChangedEventArgs("MRUFilterListCount", prevValue, MRUFilterListCount));
			}
		}
		[Description("Gets or sets the capacity of the lists that store the recently used filter items for the View's columns."), DefaultValue(5), XtraSerializableProperty()]
		public virtual int MRUColumnFilterListCount {
			get { return mruColumnFilterListCount; }
			set {
				if(value < 0) value = 0;
				if(MRUColumnFilterListCount == value) return;
				int prevValue = MRUColumnFilterListCount;
				mruColumnFilterListCount = value;
				OnChanged(new BaseOptionChangedEventArgs("MRUColumnFilterListCount", prevValue, MRUColumnFilterListCount));
			}
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		Description("Gets or sets the maximum number of items in a column's filter dropdown when the items are displayed as a check list."), DefaultValue(25), 
		Obsolete("The MaxCheckedListItemCount property is now obsolete and isn't used at all. Instead, use the View.OptionsFilter.ColumnFilterPopupMaxRecordsCount property.")]
		public virtual int MaxCheckedListItemCount {
			get { return maxCheckedListItemCount; }
			set {
				if(value < 1) value = 1;
				if(value > 10000) value = 10000;
				if(MaxCheckedListItemCount == value) return;
				int prevValue = MaxCheckedListItemCount;
				maxCheckedListItemCount = value;
				OnChanged(new BaseOptionChangedEventArgs("MaxCheckedListItemCount", prevValue, MaxCheckedListItemCount));
			}
		}
		[Description("Gets or sets the maximum number of items the View's MRU Filter List can display at once, in effect its height."), DefaultValue(7), XtraSerializableProperty()]
		public virtual int MRUFilterListPopupCount {
			get { return mruFilterListPopupCount; }
			set {
				if(value < 0) value = 0;
				if(MRUFilterListPopupCount == value) return;
				int prevValue = MRUFilterListPopupCount;
				mruFilterListPopupCount = value;
				OnChanged(new BaseOptionChangedEventArgs("MRUFilterListPopupCount", prevValue, MRUFilterListPopupCount));
			}
		}
		[Description("Gets or sets whether recently used filter items are displayed in the Filter dropdowns for columns."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowColumnMRUFilterList {
			get { return allowColumnMRUFilterList; }
			set {
				if(AllowColumnMRUFilterList == value) return;
				bool prevValue = AllowColumnMRUFilterList;
				allowColumnMRUFilterList = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowColumnMRUFilterList", prevValue, AllowColumnMRUFilterList));
			}
		}
		[Description("Gets or sets whether the View's MRU Filter List is enabled."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowMRUFilterList {
			get { return allowMRUFilterList; }
			set {
				if(AllowMRUFilterList == value) return;
				bool prevValue = AllowMRUFilterList;
				allowMRUFilterList = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowMRUFilterList", prevValue, AllowMRUFilterList));
			}
		}
		[Description("Gets or sets whether the Filter Editor can be used to build complex filter criteria."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowFilterEditor {
			get { return allowFilterEditor; }
			set {
				if(AllowFilterEditor == value) return;
				bool prevValue = AllowFilterEditor;
				allowFilterEditor = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFilterEditor", prevValue, AllowFilterEditor));
			}
		}
		[Description("Gets or sets whether incremental searching is supported in columns' Regular and Checked Filter Dropdown Lists."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowFilterIncrementalSearch {
			get { return allowFilterIncrementalSearch; }
			set {
				if(AllowFilterIncrementalSearch == value) return;
				bool prevValue = AllowFilterIncrementalSearch;
				allowFilterIncrementalSearch = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFilterIncrementalSearch", prevValue, AllowFilterIncrementalSearch));
			}
		}
		[Description("Gets or sets whether multiple items can be simultaneously selected (highlighted) in a Checked Filter Dropdown List."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowMultiSelectInCheckedFilterPopup {
			get { return allowMultiSelectInCheckedFilterPopup; }
			set {
				if(AllowMultiSelectInCheckedFilterPopup == value) return;
				bool prevValue = AllowMultiSelectInCheckedFilterPopup;
				allowMultiSelectInCheckedFilterPopup = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowMultiSelectInCheckedFilterPopup", prevValue, AllowMultiSelectInCheckedFilterPopup));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				ColumnViewOptionsFilter opt = options as ColumnViewOptionsFilter;
				if(opt == null) return;
				this.useNewCustomFilterDialog = opt.UseNewCustomFilterDialog;
				this.showAllTableValuesInFilterPopup = opt.ShowAllTableValuesInFilterPopup;
				this.showAllTableValuesInCheckedFilterPopup = opt.ShowAllTableValuesInCheckedFilterPopup;
				this.allowColumnMRUFilterList = opt.AllowColumnMRUFilterList;
				this.allowMRUFilterList = opt.AllowMRUFilterList;
				this.allowFilterEditor = opt.AllowFilterEditor;
				this.allowFilterIncrementalSearch = opt.AllowFilterIncrementalSearch;
				this.allowMultiSelectInCheckedFilterPopup = opt.AllowMultiSelectInCheckedFilterPopup;
				this.mruColumnFilterListCount = opt.MRUColumnFilterListCount;
				this.mruFilterListCount = opt.MRUFilterListCount;
				this.mruFilterListPopupCount = opt.MRUFilterListPopupCount;
				this.columnFilterPopupMaxRecordsCount = opt.ColumnFilterPopupMaxRecordsCount;
				this.columnFilterPopupRowCount = opt.ColumnFilterPopupRowCount;
				this.defaultFilterEditorView = opt.DefaultFilterEditorView;
				this.filterEditorAggregateEditing = opt.FilterEditorAggregateEditing;
				this.filterEditorUseMenuForOperandsAndOperators = opt.FilterEditorUseMenuForOperandsAndOperators;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class ColumnViewOptionsSelection : ViewBaseOptions {
		bool multiSelect;
		public ColumnViewOptionsSelection() {
			this.multiSelect = false;
		}
		[Description("Gets or sets whether multiple rows (cards) can be selected."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool MultiSelect {
			get { return multiSelect; }
			set {
				if(MultiSelect == value) return;
				bool prevValue = MultiSelect;
				multiSelect = value;
				OnChanged(new BaseOptionChangedEventArgs("MultiSelect", prevValue, MultiSelect));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				ColumnViewOptionsSelection opt = options as ColumnViewOptionsSelection;
				if(opt == null) return;
				this.multiSelect = opt.MultiSelect;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class ColumnViewOptionsView : ViewBaseOptions {
		ShowFilterPanelMode showFilterPanelMode;
		GridAnimationType animationType;
		bool showViewCaption;
		public ColumnViewOptionsView() {
			this.showFilterPanelMode = ShowFilterPanelMode.Default;
			this.showViewCaption = false;
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				ColumnViewOptionsView opt = options as ColumnViewOptionsView;
				if(opt == null) return;
				this.showViewCaption = opt.ShowViewCaption;
				this.showFilterPanelMode = opt.ShowFilterPanelMode;
				this.animationType = opt.AnimationType;
			}
			finally {
				EndUpdate();
			}
		}
		[Description("Gets or sets whether the View Caption is displayed above the View."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowViewCaption {
			get { return showViewCaption; }
			set {
				if(ShowViewCaption == value) return;
				bool prevValue = ShowViewCaption;
				showViewCaption = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowViewCaption", prevValue, ShowViewCaption));
			}
		}
		[Description("Gets or sets a value which specifies when the Filter Panel is shown."), DefaultValue(ShowFilterPanelMode.Default), XtraSerializableProperty()]
		public ShowFilterPanelMode ShowFilterPanelMode {
			get { return showFilterPanelMode; }
			set {
				if(!Enum.IsDefined(typeof(ShowFilterPanelMode), value)) {
					throw new InvalidEnumArgumentException("value", (int)value, typeof(ShowFilterPanelMode));
				}
				if(ShowFilterPanelMode == value) return;
				ShowFilterPanelMode prevValue = ShowFilterPanelMode;
				showFilterPanelMode = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowFilterPanelMode", prevValue, ShowFilterPanelMode));
			}
		}
		[Description("Gets or sets animation mode, which identifies cells within which animation is enabled."), DefaultValue(GridAnimationType.Default), XtraSerializableProperty()]
		public GridAnimationType AnimationType {
			get { return animationType; }
			set {
				object prevValue = AnimationType;
				animationType = value;
				OnChanged(new BaseOptionChangedEventArgs("AnimationType", prevValue, AnimationType));
			}
		}
		public GridAnimationType GetAnimationType() {
			if(AnimationType == GridAnimationType.Default) return GridAnimationType.AnimateFocusedItem;
			return AnimationType;
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		Obsolete(ObsoleteText.SRColumnView_OptionsViewShowFilterPanel)]
		public virtual bool ShowFilterPanel {
			get { return ShowFilterPanelMode != ShowFilterPanelMode.Never; }
			set {
				if(ShowFilterPanel == value) return;
				ShowFilterPanelMode = value ? ShowFilterPanelMode.Default : ShowFilterPanelMode.Never;
			}
		}
	}
	public class ColumnViewOptionsBehavior : ViewBaseOptions {
		EditorShowMode editorShowMode;
		DefaultBoolean allowDeleteRows, allowAddRows;
		bool autoSelectAllInEditor, editable, readOnly,
			immediateUpdateRowPosition, keepFocusedRowOnUpdate,
			autoPopulateColumns, focusLeaveOnTab;
		CacheRowValuesMode cacheValuesOnRowUpdating;
		ColumnView view;
		public ColumnViewOptionsBehavior(ColumnView view) {
			this.editorShowMode = EditorShowMode.Default;
			this.view = view;
			this.cacheValuesOnRowUpdating = CacheRowValuesMode.CacheAll;
			this.autoPopulateColumns = true;
			this.keepFocusedRowOnUpdate = true;
			this.immediateUpdateRowPosition = true;
			this.autoSelectAllInEditor = true;
			this.editable = true;
			this.readOnly = false;
			this.focusLeaveOnTab = false;
			this.allowDeleteRows = this.allowAddRows = DefaultBoolean.Default;
		}
		public ColumnViewOptionsBehavior() : this(null) { }
		protected ColumnView View { get { return view; } }
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Obsolete(ObsoleteText.SRColumnView_OptionsBehaviorShowEditorOnMouseUp)]
		public virtual bool ShowEditorOnMouseUp {
			get { return EditorShowMode == DevExpress.Utils.EditorShowMode.MouseUp; }
			set {
				if(ShowEditorOnMouseUp == value) return;
				EditorShowMode = value ? DevExpress.Utils.EditorShowMode.MouseUp : DevExpress.Utils.EditorShowMode.Default;
			}
		}
		[Description("Gets or sets whether the grid caches data for a row that is about to be saved to a data source. "), DefaultValue(CacheRowValuesMode.CacheAll), XtraSerializableProperty()]
		public virtual CacheRowValuesMode CacheValuesOnRowUpdating {
			get { return cacheValuesOnRowUpdating; }
			set {
				if(CacheValuesOnRowUpdating == value) return;
				CacheRowValuesMode prevValue = CacheValuesOnRowUpdating;
				cacheValuesOnRowUpdating = value;
				OnChanged(new BaseOptionChangedEventArgs("CacheValuesOnRowUpdating", prevValue, CacheValuesOnRowUpdating));
			}
		}
		[Description("Gets or sets whether pressing the TAB key moves focus to the next control in the tab order, if moving focus via the TAB key within the View is prohibited (or cannot be performed)."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool FocusLeaveOnTab {
			get { return focusLeaveOnTab; }
			set {
				if(focusLeaveOnTab == value) return;
				bool prevFocusLeaveOnTab = FocusLeaveOnTab;
				focusLeaveOnTab = value;
				OnChanged(new BaseOptionChangedEventArgs("FocusLeaveOnTab", prevFocusLeaveOnTab, FocusLeaveOnTab));
			}
		}
		[Description("Gets or sets whether the Data Navigator's Delete button is enabled for the current View."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean AllowDeleteRows {
			get { return allowDeleteRows; }
			set {
				if(AllowDeleteRows == value) return;
				DefaultBoolean prevAllowDeleteRows = AllowDeleteRows;
				allowDeleteRows = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowDeleteRows", prevAllowDeleteRows, AllowDeleteRows));
			}
		}
		[Description("Gets or sets whether the Data Navigator's Append button is enabled for the current View."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean AllowAddRows {
			get { return allowAddRows; }
			set {
				if(AllowAddRows == value) return;
				DefaultBoolean prevAllowAddRows = AllowAddRows;
				allowAddRows = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowAddRows", prevAllowAddRows, AllowAddRows));
			}
		}
		[Description("Gets or sets a value which specifies how a cell editor is activated by the mouse."), DefaultValue(EditorShowMode.Default), XtraSerializableProperty()]
		public virtual EditorShowMode EditorShowMode {
			get { return editorShowMode; }
			set {
				if(EditorShowMode == value) return;
				EditorShowMode prevValue = EditorShowMode;
				editorShowMode = value;
				OnChanged(new BaseOptionChangedEventArgs("EditorShowMode", prevValue, EditorShowMode));
			}
		}
		[Description("Gets or sets whether a row's position is immediately updated according to the current sorting, grouping and filtering settings after the row's data has been modified."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ImmediateUpdateRowPosition {
			get { return immediateUpdateRowPosition; }
			set {
				if(ImmediateUpdateRowPosition == value) return;
				bool prevValue = ImmediateUpdateRowPosition;
				immediateUpdateRowPosition = value;
				OnChanged(new BaseOptionChangedEventArgs("ImmediateUpdateRowPosition", prevValue, ImmediateUpdateRowPosition));
			}
		}
		[Description("Gets or sets whether the currently focused row retains focus when records are added to or removed from the underlying data source."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool KeepFocusedRowOnUpdate {
			get { return keepFocusedRowOnUpdate; }
			set {
				if(KeepFocusedRowOnUpdate == value) return;
				bool prevValue = KeepFocusedRowOnUpdate;
				keepFocusedRowOnUpdate = value;
				OnChanged(new BaseOptionChangedEventArgs("KeepFocusedRowOnUpdate", prevValue, KeepFocusedRowOnUpdate));
			}
		}
		[Description("Gets or sets whether activating a cell editor using the ENTER or F2 keys leads to the entire editor's content being selected."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AutoSelectAllInEditor {
			get { return autoSelectAllInEditor; }
			set {
				if(AutoSelectAllInEditor == value) return;
				bool prevValue = AutoSelectAllInEditor;
				autoSelectAllInEditor = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoSelectAllInEditor", prevValue, AutoSelectAllInEditor));
			}
		}
		[Description("Gets or sets whether columns should be created automatically for all fields in the underlying data source when the View doesn't contain any columns."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AutoPopulateColumns {
			get { return autoPopulateColumns; }
			set {
				if(AutoPopulateColumns == value) return;
				bool prevValue = AutoPopulateColumns;
				autoPopulateColumns = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoPopulateColumns", prevValue, AutoPopulateColumns));
			}
		}
		[Description("Gets or sets whether end-users are allowed to invoke cell editors."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool Editable {
			get { return editable; }
			set {
				if(Editable == value) return;
				bool prevValue = Editable;
				editable = value;
				OnChanged(new BaseOptionChangedEventArgs("Editable", prevValue, Editable));
			}
		}
		[Description("Gets or sets whether the current View is read-only."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ReadOnly {
			get { return readOnly; }
			set {
				if(ReadOnly == value) return;
				bool prevValue = ReadOnly;
				readOnly = value;
				OnChanged(new BaseOptionChangedEventArgs("ReadOnly", prevValue, ReadOnly));
			}
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Obsolete(ObsoleteText.SRColumnView_OptionsBehaviorShowAllTableValuesInFilterPopup)]
		public virtual bool ShowAllTableValuesInFilterPopup {
			get { return view.OptionsFilter.ShowAllTableValuesInFilterPopup; }
			set { view.OptionsFilter.ShowAllTableValuesInFilterPopup = value; }
		}
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Obsolete(ObsoleteText.SRColumnView_OptionsBehaviorUseNewCustomFilterDialog)]
		public virtual bool UseNewCustomFilterDialog {
			get { return view.OptionsFilter.UseNewCustomFilterDialog; }
			set { view.OptionsFilter.UseNewCustomFilterDialog = value; }
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				ColumnViewOptionsBehavior opt = options as ColumnViewOptionsBehavior;
				if(opt == null) return;
				this.cacheValuesOnRowUpdating = opt.CacheValuesOnRowUpdating;
				this.editorShowMode = opt.EditorShowMode;
				this.autoPopulateColumns = opt.AutoPopulateColumns;
				this.keepFocusedRowOnUpdate = opt.KeepFocusedRowOnUpdate;
				this.immediateUpdateRowPosition = opt.ImmediateUpdateRowPosition;
				this.autoSelectAllInEditor = opt.AutoSelectAllInEditor;
				this.editable = opt.Editable;
				this.allowAddRows = opt.AllowAddRows;
				this.allowDeleteRows = opt.AllowDeleteRows;
				this.readOnly = opt.ReadOnly;
			}
			finally {
				EndUpdate();
			}
		}
	}
}
