#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Data;
using System.Collections;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using DevExpress.Utils;
using DevExpress.Utils.Design;
using DevExpress.XtraGrid;
namespace DevExpress.XtraGrid.Design { 
	public enum DesignerGroupType { Regular, Main, Repository, Appearance, Printing, Layout }
	public class GridDesignerGroupCollection : DesignerGroupCollection {
		public DesignerGroup AddOrClear(DesignerGroupType gtype, string caption, string description, Image image) {
			return AddOrClear(gtype, caption, description, image, false);
		}
		public DesignerGroup AddOrClear(DesignerGroupType gtype, string caption, string description, Image image, bool defaultExpanded) {
			DesignerGroup group = null;
			if(gtype != DesignerGroupType.Regular) {
				group = GetGroupByType(gtype);
				if(group != null) group.Clear();
			}
			if(group == null) group = Add(caption, description, image, defaultExpanded);
			group.Tag = gtype;
			return group;
		}
		public DesignerGroup GetGroupByType(DesignerGroupType gtype) {
			foreach(DesignerGroup group in this) {
				if(group.Tag == null) continue;
				if((DesignerGroupType)group.Tag == gtype) return group;
			}
			return null;
		}
		protected override void OnClear() {
			for(int n = Count - 1; n >= 0; n --) RemoveAt(n);
		}
		protected override void OnRemoveComplete(int index, object item) {
			DesignerGroup group = item as DesignerGroup;
			if(group != null) group.Dispose();
		}
	}
	public class BaseGridDesigner : BaseDesigner {
		[ThreadStatic]
		static ImageCollection largeImages = null;
		[ThreadStatic]
		static ImageCollection smallImages = null;
		static ImageCollection LargeImages { 
			get { 
				if(largeImages == null) 
					largeImages = DevExpress.Utils.Controls.ImageHelper.CreateImageCollectionFromResources("DevExpress.XtraGrid.Images.icons32x32.png", typeof(BaseGridDesigner).Assembly, new Size(32, 32));
				return largeImages; 
			} 
		}
		static ImageCollection SmallImages { 
			get { 
				if(smallImages == null) 
					smallImages = DevExpress.Utils.Controls.ImageHelper.CreateImageCollectionFromResources("DevExpress.XtraGrid.Images.icons16x16.png", typeof(BaseGridDesigner).Assembly, new Size(16, 16));
				return smallImages; 
			} 
		}
		public new GridDesignerGroupCollection Groups { get { return base.Groups as GridDesignerGroupCollection; } }
		protected override object LargeImageList { get { return LargeImages; } }
		protected override object SmallImageList { get { return SmallImages; } }
		protected override DesignerGroupCollection CreateGroupCollection() {
			return new GridDesignerGroupCollection();
		}
		protected virtual DesignerGroup CreateDefaultMainGroup() {
			DesignerGroup group = Groups.AddOrClear(DesignerGroupType.Main, "Main", "Main Grid settings (adjust the view, columns, bands, and specify in-place editors and summaries).", null, true);
			group.Add("Views", "Adjust the settings of the views and add new levels.", "DevExpress.XtraGrid.Frames.LevelStyle", GetDefaultLargeImage(0), GetDefaultSmallImage(0), true);
			group.Add("Feature Browser", "Locate relevant options by features.", "DevExpress.XtraGrid.FeatureBrowser.FeatureBrowserGridMainFrame", GetDefaultLargeImage(3), GetDefaultSmallImage(3));
			group.Add("Layout", "Customize the current view's layout and preview its data.", "DevExpress.XtraGrid.Frames.Layouts", GetDefaultLargeImage(4), GetDefaultSmallImage(4), null, false);
			return group;
		}
		protected virtual DesignerGroup CreateDefaultRepositoryGroup() {
			DesignerGroup group = Groups.AddOrClear(DesignerGroupType.Repository, "Repository", "Access all the views and in-place editors used by the Grid.", null, true);
			group.Add("View Repository", "Manage views.", "DevExpress.XtraGrid.Frames.ViewsEditor", GetDefaultLargeImage(6), GetDefaultSmallImage(6), null, false);
			group.Add("In-place Editor Repository", "Adjust the editors used for in-place editing.", "DevExpress.XtraGrid.Frames.PersistentRepositoryGridEditor", GetDefaultLargeImage(7), GetDefaultSmallImage(7), true, false);
			return group;
		}
		protected virtual DesignerGroup CreateDefaultStyleGroup() {
			DesignerGroup group = Groups.AddOrClear(DesignerGroupType.Appearance, "Appearance", "Adjust the appearance of the current view (style conditions, painting schemes).", null, true);
			group.Add("Appearances", "Manage the appearances for the current view.", "DevExpress.XtraGrid.Frames.AppearancesDesigner", GetDefaultLargeImage(8), GetDefaultSmallImage(8), null, false);
			group.Add("Format Conditions", "Manage the format conditions for the current view.", "DevExpress.XtraGrid.Frames.FormatCondition", GetDefaultLargeImage(9), GetDefaultSmallImage(9), null, false);
			group.Add("Style Schemes", "Apply a painting scheme to the current view.", "DevExpress.XtraGrid.Frames.SchemeDesigner", GetDefaultLargeImage(10), GetDefaultSmallImage(10), null, false);
			return group;
		}
		protected virtual DesignerGroup CreateDefaultPrintingGroup() {
			DesignerGroup group = Groups.AddOrClear(DesignerGroupType.Printing, "Printing", "Printing option management for the current view.", null);
			group.Add("Print Appearances", "Adjust the print appearances of the current view.", "DevExpress.XtraGrid.Frames.PrintAppearancesDesigner", GetDefaultLargeImage(12), GetDefaultSmallImage(12), null, false);
			return group;
		}
	}
	public class GridViewDesigner : BaseGridDesigner {
		protected override void CreateGroups() {
			base.CreateGroups();
			DesignerGroup group = CreateDefaultMainGroup();
			group.Insert(1, "Columns", "Adjust the Column collection of the current view, assign in-place editors to columns and specify total summaries.", "DevExpress.XtraGrid.Frames.ColumnDesigner", GetDefaultLargeImage(1), GetDefaultSmallImage(1), null);
			group.Add("Group Summary Items", "Manage group summaries.", "DevExpress.XtraGrid.Frames.GroupSummary", GetDefaultLargeImage(5), GetDefaultSmallImage(5), null, false);
			CreateDefaultStyleGroup();
			CreateDefaultRepositoryGroup();
			group = CreateDefaultPrintingGroup();
			group.Add("Printing Settings", "Adjust the printing settings of the current view.", "DevExpress.XtraGrid.Frames.GridViewPrinting", GetDefaultLargeImage(11), GetDefaultSmallImage(11), null, false);
		}
	}
	public class BandedGridViewDesigner : GridViewDesigner {
		protected override void CreateGroups() {
			base.CreateGroups();
			DesignerGroup group = Groups.GetGroupByType(DesignerGroupType.Main);
			if(group != null) {
				group.Insert(2, "Bands", "Adjust the Band collection of the current view and customize the layout of bands.", "DevExpress.XtraGrid.Frames.BandDesigner", GetDefaultLargeImage(2), GetDefaultSmallImage(2), null);
			}
			group = Groups.GetGroupByType(DesignerGroupType.Printing);
			if(group != null) {
				group.Clear();
				Groups.RemoveAt(Groups.IndexOf(group));
			}
			group = CreateDefaultPrintingGroup();
			group.Add("Printing Settings", "Adjust the printing settings of the current view.", "DevExpress.XtraGrid.Frames.BandedViewPrinting", GetDefaultLargeImage(11), GetDefaultSmallImage(11), null, false);
		}
	}
	public class EmptyViewDesigner : BaseGridDesigner {
		protected override void CreateGroups() {
			base.CreateGroups();
			CreateDefaultMainGroup();
			CreateDefaultStyleGroup();
		}
	}
	public class ControlGridDesigner : BaseGridDesigner {
		protected override void CreateGroups() {
			base.CreateGroups();
			DesignerGroup group = Groups.AddOrClear(DesignerGroupType.Main, "Main", "Main Grid settings (adjust the view, columns, bands, and specify in-place editors and summaries).", null, true);
			group.Add("Views", "Adjust the settings of the views and add new levels.", "DevExpress.XtraGrid.Frames.LevelStyle", GetDefaultLargeImage(0), GetDefaultSmallImage(0), true);
		}
	}
	public class CardViewDesigner : BaseGridDesigner {
		protected override void CreateGroups() {
			base.CreateGroups();
			DesignerGroup group = CreateDefaultMainGroup();
			group.Insert(1, "Columns", "Adjust the Column collection of the current view and assign in-place editors to columns.", "DevExpress.XtraGrid.Frames.ColumnDesigner", GetDefaultLargeImage(1), GetDefaultSmallImage(1), null);
			CreateDefaultStyleGroup();
			CreateDefaultRepositoryGroup();
			group = CreateDefaultPrintingGroup();
			group.Add("Printing Settings", "Adjust the printing settings of the current view.", "DevExpress.XtraGrid.Frames.CardViewPrinting", GetDefaultLargeImage(11), GetDefaultSmallImage(11));
		}
	}
}
