#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.Utils.Menu;
using DevExpress.LookAndFeel;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Filtering;
using DevExpress.XtraGrid.Localization;
using System.Reflection;
namespace DevExpress.XtraGrid.FilterEditor {
	[ToolboxItem(false)]
	public class FilterBuilder : DevExpress.XtraEditors.XtraForm {
		protected IFilterControl fcMain = null;
		protected PanelControl controlsPanel;
		protected SimpleButton sbOK;
		protected SimpleButton sbCancel;
		protected SimpleButton sbApply;
		private System.ComponentModel.Container components = null;
		private PanelControl panelControl2;
		private PanelControl panelControl1;
		ColumnView view;
		bool allowClose = true;
		internal IFilterControl GetIFilterControl() { return fcMain; }
		FilterBuilder() {
			InitializeComponent();
			CreateButtonCaptions();
			this.MinimumSize = new Size(300, 200);
		}
		public FilterBuilder(FilterColumnCollection columns, IDXMenuManager manager, UserLookAndFeel lookAndFeel, ColumnView view, FilterColumn fColumn)
			: this() {
			this.view = view;
			CreateFilterControl();
			fcMain.SetFilterColumnsCollection(columns, manager);
			fcMain.SetDefaultColumn(fColumn);
			fcMain.FilterCriteria = view.ActiveFilterCriteria;
			fcMain.ShowOperandTypeIcon = view.OptionsFilter.UseNewCustomFilterDialog;
			fcMain.FilterTextChanged += new FilterTextChangedEventHandler(fcMain_FilterTextChanged);
			if(view.GridControl != null && !view.GridControl.FormsUseDefaultLookAndFeel) {
				fcMain.LookAndFeel.Assign(lookAndFeel);
				SetLookAndFeel(view.ElementsLookAndFeel);
			}
		}
		static object ConstructorInfoInvoker(string constructorAssembly, string constructorClass) {
			try {
				Type constructorType = DevExpress.Xpo.Helpers.XPTypeActivator.GetType(constructorAssembly, constructorClass);
				if(constructorType != null) {
					ConstructorInfo constructorInfoObj = constructorType.GetConstructor(Type.EmptyTypes);
					if(constructorInfoObj != null) {
						return constructorInfoObj.Invoke(null);
					}
				}
			}
			catch { }
			return null;
		}
		public IFilterControl FilterConrol { get { return fcMain; } }
		private void CreateFilterControl() {
			if(view.OptionsFilter.DefaultFilterEditorView != FilterEditorViewMode.Visual) {
				view.SetCursor(Cursors.WaitCursor);
				fcMain = ConstructorInfoInvoker(
					AssemblyInfo.SRAssemblyRichEdit + ", Version=" + AssemblyInfo.Version,
					"DevExpress.XtraFilterEditor.FilterEditorControl") as IFilterControl;
				view.ResetDefaultCursor();
			}
			if(fcMain != null) 
				fcMain.SetViewMode(view.OptionsFilter.DefaultFilterEditorView);
			else 
				fcMain = new GridFilterControl();
			fcMain.UseMenuForOperandsAndOperators = view.OptionsFilter.FilterEditorUseMenuForOperandsAndOperators;
			fcMain.AllowAggregateEditing = view.OptionsFilter.FilterEditorAggregateEditing;
			OnFilterControlCreated(fcMain);
			((Control)fcMain).Dock = DockStyle.Fill;
			((Control)fcMain).Parent = this;
			((Control)fcMain).BringToFront();
		}
		protected virtual void OnFilterControlCreated(IFilterControl filterControl) {
		}
		protected virtual void CreateButtonCaptions() {
			sbOK.Text = GridLocalizer.Active.GetLocalizedString(GridStringId.FilterBuilderOkButton);
			sbCancel.Text = GridLocalizer.Active.GetLocalizedString(GridStringId.FilterBuilderCancelButton);
			sbApply.Text = GridLocalizer.Active.GetLocalizedString(GridStringId.FilterBuilderApplyButton);
			this.Text = GridLocalizer.Active.GetLocalizedString(GridStringId.FilterBuilderCaption);
		}
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		private void InitializeComponent() {
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FilterBuilder));
			this.controlsPanel = new DevExpress.XtraEditors.PanelControl();
			this.sbOK = new DevExpress.XtraEditors.SimpleButton();
			this.panelControl2 = new DevExpress.XtraEditors.PanelControl();
			this.sbCancel = new DevExpress.XtraEditors.SimpleButton();
			this.panelControl1 = new DevExpress.XtraEditors.PanelControl();
			this.sbApply = new DevExpress.XtraEditors.SimpleButton();
			((System.ComponentModel.ISupportInitialize)(this.controlsPanel)).BeginInit();
			this.controlsPanel.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.panelControl2)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.panelControl1)).BeginInit();
			this.SuspendLayout();
			this.controlsPanel.BorderStyle = DevExpress.XtraEditors.Controls.BorderStyles.NoBorder;
			this.controlsPanel.Controls.Add(this.sbOK);
			this.controlsPanel.Controls.Add(this.panelControl2);
			this.controlsPanel.Controls.Add(this.sbCancel);
			this.controlsPanel.Controls.Add(this.panelControl1);
			this.controlsPanel.Controls.Add(this.sbApply);
			resources.ApplyResources(this.controlsPanel, "controlsPanel");
			this.controlsPanel.Name = "controlsPanel";
			this.sbOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			resources.ApplyResources(this.sbOK, "sbOK");
			this.sbOK.Name = "sbOK";
			this.sbOK.Click += new System.EventHandler(this.sbOK_Click);
			this.panelControl2.BorderStyle = DevExpress.XtraEditors.Controls.BorderStyles.NoBorder;
			resources.ApplyResources(this.panelControl2, "panelControl2");
			this.panelControl2.Name = "panelControl2";
			this.sbCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			resources.ApplyResources(this.sbCancel, "sbCancel");
			this.sbCancel.Name = "sbCancel";
			this.sbCancel.Click += new System.EventHandler(this.sbCancel_Click);
			this.panelControl1.BorderStyle = DevExpress.XtraEditors.Controls.BorderStyles.NoBorder;
			resources.ApplyResources(this.panelControl1, "panelControl1");
			this.panelControl1.Name = "panelControl1";
			resources.ApplyResources(this.sbApply, "sbApply");
			this.sbApply.Name = "sbApply";
			this.sbApply.Click += new System.EventHandler(this.sbApply_Click);
			resources.ApplyResources(this, "$this");
			this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
			this.CancelButton = this.sbCancel;
			this.Controls.Add(this.controlsPanel);
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "FilterBuilder";
			this.ShowInTaskbar = false;
			((System.ComponentModel.ISupportInitialize)(this.controlsPanel)).EndInit();
			this.controlsPanel.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.panelControl2)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.panelControl1)).EndInit();
			this.ResumeLayout(false);
		}
		#endregion
		void SetLookAndFeel(DevExpress.LookAndFeel.UserLookAndFeel lookAndFeel) {
			this.LookAndFeel.Assign(lookAndFeel);
		}
		private void sbOK_Click(object sender, System.EventArgs e) {
			ApplyFilter();
		}
		private void sbApply_Click(object sender, System.EventArgs e) {
			ApplyFilter();
		}
		private void sbCancel_Click(object sender, EventArgs e) {
			allowClose = true;
		}
		void fcMain_FilterTextChanged(object sender, FilterTextChangedEventArgs e) {
			sbOK.Enabled = e.IsValid;
			sbApply.Enabled = e.IsValid;
		}
		protected virtual void ApplyFilter() {
			try {
				ViewFilter newFilter = new ViewFilter(view, fcMain.FilterCriteria);
				view.ActiveFilter.Assign(newFilter, view.Columns);
				allowClose = true;
			}
			catch(Exception e) {
				XtraMessageBox.Show(LookAndFeel, e.Message, GridLocalizer.Active.GetLocalizedString(GridStringId.WindowErrorCaption), MessageBoxButtons.OK, MessageBoxIcon.Error);
				allowClose = false;
			}
		}
		private void fcMain_KeyPress(object sender, KeyPressEventArgs e) {
			if(e.KeyChar == (char)Keys.Escape) {
				allowClose = true;
				this.Close();
			}
		}
		protected override void OnFormClosing(FormClosingEventArgs e) {
			base.OnFormClosing(e);
			e.Cancel = !allowClose;
		}
	}
}
