#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Data;
using System.Collections;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using DevExpress.Data;
using DevExpress.Data.Details;
using DevExpress.Utils;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Drawing;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Scrolling;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Base.ViewInfo;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;
using DevExpress.XtraPrinting;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Drawing;
using DevExpress.XtraGrid.Views.Printing;
using DevExpress.XtraEditors.Repository;
using System.Collections.Generic;
using DevExpress.XtraPrinting.NativeBricks;
using DevExpress.XtraGrid.Printing;
namespace DevExpress.XtraGrid.Views.Printing {
	public class PrintInfoArgs {
		public BaseView View;
		public int Indent;
		public int Y;
		public IBrickGraphics Graph;
		public ViewPrintWrapper PrintWrapper;
		public PrintInfoArgs(BaseView view) : this(view, null, null, 0) { }
		public PrintInfoArgs(BaseView view, ViewPrintWrapper printWrapper, IBrickGraphics graph, int y) {
			this.PrintWrapper = printWrapper;
			this.Indent = printWrapper == null ? 0 : printWrapper.PrintIndent;
			this.View = view;
			this.Y = y;
			this.Graph = graph;
		}
	}
	public class BrickCache {
		Hashtable bricks;
		public BrickCache() {
			this.bricks = new Hashtable();
		}
		public BrickStyle this[string name] { get { return Bricks[name] as BrickStyle; } }
		protected Hashtable Bricks { get { return bricks; } }
		public void Add(string name, AppearanceObject appearance, BorderSide borderSide, Color borderColor, int borderWidth) {
			BrickStyle brick = Create(appearance, borderSide, borderColor, borderWidth);
			Bricks[name] = brick;
		}
		public void Add(string name, AppearanceObject appearance) {
			Add(name, appearance, BorderSide.None, Color.Transparent, 1);
		}
		public void Clear() {
			Bricks.Clear();
		}
		public BrickStyle Create(AppearanceObject appearance, BorderSide borderSide, Color borderColor, int borderWidth) {
			if(borderColor == Color.Empty) borderColor = Color.Gray;
			BrickStringFormat format = new BrickStringFormat(appearance.GetStringFormat());
			format.PrototypeKind = BrickStringFormatPrototypeKind.GenericTypographic;
			format.Value.FormatFlags |= StringFormatFlags.LineLimit;
			BrickStyle brick = new BrickStyle(borderSide, borderWidth, borderColor, appearance.GetBackColor(),
				appearance.GetForeColor(), appearance.GetFont(), format);
			brick.TextAlignment = TextAlignmentFromTextOptions(appearance.TextOptions);
			return brick;
		}
		TextAlignment TextAlignmentFromTextOptions(TextOptions to) {
			return DevExpress.XtraPrinting.Native.TextAlignmentConverter.ToTextAlignment(to.HAlignment, to.VAlignment);
		}
	}
	public class CancelPrintRowEventArgs : PrintRowEventArgs {
		bool cancelCore = false;
		public CancelPrintRowEventArgs(IPrintingSystem ips, Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y) : base(ips, g, graph, rowHandle, level, x, y) { }
		public CancelPrintRowEventArgs(IPrintingSystem ips, Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y, bool hasRowFooter) : base(ips, g, graph, rowHandle, level, x, y, hasRowFooter) { }
		public bool Cancel {
			get { return cancelCore; }
			set { cancelCore = value; }
		}
	}
	public class PrintRowEventArgs : EventArgs {
		Graphics gCore;
		IBrickGraphics graphCore;
		IPrintingSystem psCore;
		int rowHandleCore;
		int levelCore;
		int xCore, yCore;
		bool hasFooter;
		public PrintRowEventArgs(IPrintingSystem ips, Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y, bool hasRowFooter) : this(ips, g, graph, rowHandle, level, x, y) {
			hasFooter = hasRowFooter;	
		}
		public PrintRowEventArgs(IPrintingSystem ips, Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y) {
			gCore = g;
			graphCore = graph;
			rowHandleCore = rowHandle;
			levelCore = level;
			yCore = y;
			xCore = x;
			psCore = ips;
		}
		public int X { get { return xCore; } set { xCore = value; } }
		public int Y { get { return yCore; } set { yCore = value; } }
		public int Level { get { return levelCore; } }
		public int RowHandle { get { return rowHandleCore; } }
		public Graphics Graphics { get { return gCore; } }
		public IBrickGraphics BrickGraphics { get { return graphCore; } }
		public IPrintingSystem PS { get { return psCore; } }
		public bool HasFooter { get { return hasFooter; } }
	}
	public class GridViewPrintInfo : ColumnViewPrintInfo {
		ArrayList columns;
		protected int headerRowHeight;
		int prevLevel = -1, prevRowHandle, currentRowHeight, 
			groupFooterHeight, footerPanelHeight;
		GridColumnsInfo columnsInfo;
		protected int fMaxRowWidth;
		GridRowInfo currentRowInfo;
		public GridViewPrintInfo(PrintInfoArgs args) : base(args) {
			this.columnsInfo = new GridColumnsInfo();
			this.columns = new ArrayList();
			this.currentRowHeight = 0;
			this.fMaxRowWidth = 0;
			this.currentRowInfo = null;
		}
		protected override BaseViewAppearanceCollection CreatePrintAppearance() { return new GridViewPrintAppearances(View);	}
		public new GridViewPrintAppearances AppearancePrint { get { return base.AppearancePrint as GridViewPrintAppearances; } }
		protected override bool PrintSelectedRowsOnly { get { return View.OptionsPrint.PrintSelectedRowsOnly; } }
		public GridColumnsInfo ColumnsInfo { get { return columnsInfo; } }
		public ArrayList Columns { get { return columns; } }
		protected new GridViewInfo ViewViewInfo { get { return base.ViewViewInfo as GridViewInfo; } }
		public new GridView View { get { return base.View as GridView; } }
		protected int CurrentRowHeight { get { return currentRowHeight; } }
		protected virtual int HeaderY { get { return 0; } }
		public override void PrintViewHeader(IBrickGraphics graph) {
			int width = 0;
			foreach (PrintColumnInfo col in Columns) if (width < col.Bounds.Right) width = col.Bounds.Right;
			PrintViewHeaderCore(graph, width);
		}
		public override void PrintHeader(DevExpress.XtraPrinting.IBrickGraphics graph) {
			if (!View.OptionsPrint.PrintHeader) return;
			Point indent = new Point(Indent, HeaderY);
			Rectangle r = Rectangle.Empty;
			bool usePrintStyles = View.OptionsPrint.UsePrintStyles;
			SetDefaultBrickStyle(graph, Bricks["HeaderPanel"]);
			foreach (PrintColumnInfo col in Columns) {
				if (!usePrintStyles) {
					AppearanceObject temp = new AppearanceObject();
					AppearanceHelper.Combine(temp, new AppearanceObject[] { col.Column.AppearanceHeader, View.Appearance.HeaderPanel, AppearancePrint.HeaderPanel });
					SetDefaultBrickStyle(graph, Bricks.Create(temp, BorderSide.All, temp.BorderColor, 1));
				}
				r = col.Bounds;
				r.Offset(indent);
				string caption = col.Column.GetTextCaption();
				if (!ColumnsInfo[Columns.IndexOf(col)].Column.OptionsColumn.ShowCaption) caption = string.Empty;
				ITextBrick itb = DrawTextBrick(graph, caption, r);
				if (AppearancePrint.HeaderPanel.TextOptions.WordWrap == WordWrap.NoWrap && View.OptionsPrint.UsePrintStyles) {
					using (Graphics g = this.View.GridControl.CreateGraphics()) {
						SizeF s = g.MeasureString(itb.Text, itb.Font, 1000, itb.StringFormat.Value);
						if (s.Width + 5 >= r.Width) {
							itb.Text = "";
							itb.TextValue = "";
						}
					}
				}
			}
		}
		int rowCount = -1;
		int currentPrintedRow = -1;
		Graphics g = null;
		public override void PrintNextRow(DevExpress.XtraPrinting.IBrickGraphics graph) {
			if(CancelPending) return;
			if(rowCount < 0) { PreparePrintRows(out rowCount, out g); currentPrintedRow = 0; }
			Y = 0;
			PrintRowCore(graph, g, currentPrintedRow);
			currentPrintedRow++;
			if(currentPrintedRow == rowCount) {
				AfterPrintRows(g);
				PrintFooterPanel(graph);
				PrintFilterInfo(graph);
			}
		}
		public override int GetDetailCount() {
			if(rowCount < 0) { PreparePrintRows(out rowCount, out g); currentPrintedRow = 0; }
			return rowCount;
		}
		public override void PrintRows(DevExpress.XtraPrinting.IBrickGraphics graph) {
			try {
				PreparePrintRows(out rowCount, out g);
				for(int n = 0; n < rowCount; n++) {
					if(CancelPending) break;
					PrintRowCore(graph, g, n);
				}
			}
			finally {
				AfterPrintRows(g);
			}
			if(CancelPending) return;
			PrintFooterPanel(graph);
			PrintFilterInfo(graph);
		}
		private void AfterPrintRows(Graphics g) {
			CompleteCellMerging();
			ViewViewInfo.GInfo.ReleaseGraphics();
			ViewViewInfo.GInfo.Cache.Paint = DevExpress.Utils.Paint.XPaint.Graphics;
			PrintWrapper.EndPrint();
			g.Dispose();
		}
		void CompleteCellMerging() {
			foreach(PrintMergeInfo pmi in printMergedInfoTable.Values) {
				PanelBrick pb = pmi.Brick as PanelBrick;
				if(pb != null && pb.Merged) ProcessMergedCell(pb);
				((IVisualBrick)pmi.Brick).Sides = CalcBrickBorders();
				Graph.DrawBrick(pmi.Brick, pmi.Brick.Rect);
			}
		}
		BorderSide CalcBrickBorders() {
			BorderSide border = BorderSide.None;
			if(View.OptionsPrint.PrintHorzLines) border |= BorderSide.Top | BorderSide.Bottom;
			if(View.OptionsPrint.PrintVertLines) border |= BorderSide.Left | BorderSide.Right;
			return border;
		}
		void PreparePrintRows(out int rowCount, out Graphics g) {
			MakeRowList();
			if(Rows.Count > 0) Rows.Add(GridControl.InvalidRowHandle);
			rowCount = Rows.Count;
			g = null;
			PrintWrapper.BeginPrint();
			g = View.GridControl.CreateGraphics();
			ViewViewInfo.GInfo.AddGraphics(g);
			ViewViewInfo.GInfo.Cache.Paint = new DevExpress.Utils.Paint.XPrintPaint();
			printMergedInfoTable = new Dictionary<GridColumn, PrintMergeInfo>();
		}
		int printedRows = 0;
		void PrintRowCore(DevExpress.XtraPrinting.IBrickGraphics graph, Graphics g, int n) {
			if(rowCount > 0) {
				ReportProgress((int)((n / (float)rowCount) * 100f));
			}
			int rh = (int)Rows[n];
			if(RaiseBeforePrintRow(g, graph, rh, View.GetRowLevel(rh), Indent, Y)) return;
			if(++printedRows % 10 == 0 && !AllowProcessMergedInfo) {
				BrickGraphics graph2 = (BrickGraphics)graph;
				graph2.Modifier = BrickModifier.None;
				graph2.Modifier = BrickModifier.Detail;
				Y = 0;
			}
			PrintRow(g, graph, rh, View.GetRowLevel(rh));
		}
		protected bool RaiseBeforePrintRow(Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y) {
			CancelPrintRowEventArgs prea = new CancelPrintRowEventArgs(PS, g, graph, rowHandle, level, x, y);
			View.RaiseBeforePrintRow(prea);
			this.Y = prea.Y;
			this.Indent = prea.X;
			return prea.Cancel;
		}
		protected void RaiseAfterPrintRow(Graphics g, IBrickGraphics graph, int rowHandle, int level, int x, int y, bool hasRowFooter) {
			CancelPrintRowEventArgs prea = new CancelPrintRowEventArgs(PS, g, graph, rowHandle, level, x, y, hasRowFooter);
			View.RaiseAfterPrintRow(prea);
			this.Y = prea.Y;
			this.Indent = prea.X;
		}
		bool? isSameCellEditor = null;
		bool IsSameCellEditor {
			get {
				if(isSameCellEditor == null) isSameCellEditor = View.IsSameColumnEditor;
				return isSameCellEditor.Value;
			}
		}
		bool IsAllowCacheDataRow = true;
		GridDataRowInfo cachedDataRow = null;
		GridRowInfo CreateRowInfo(int rowHandle, int level) {
			if(rowHandle >= 0 && IsAllowCacheDataRow) {
				if(cachedDataRow != null) {
					cachedDataRow.RowHandle = rowHandle;
					return cachedDataRow;
				}
			}
			GridRowInfo res = ViewViewInfo.CreateRowInfo(new GridRow(rowHandle, 0, level, 0, null, false));
			if(rowHandle >= 0 && IsAllowCacheDataRow) {
				cachedDataRow = res as GridDataRowInfo;
				CreateRowCellInfo(cachedDataRow);
			}
			return res;
		}
		void CreateRowCellInfo(GridDataRowInfo dataRow) {
			foreach(PrintColumnInfo colInfo in Columns) {
				GridCellInfo cell = new GridCellInfo();
				cell.ColumnInfo = new GridColumnInfoArgs(null);
				cell.ColumnInfo.Column = colInfo.Column;
				cell.RowInfo = dataRow;
				UpdateCellEditorInfo(cell, dataRow.RowHandle);
				dataRow.Cells.Add(cell);
			}
		}
		void UpdateCellEditorInfo(GridDataRowInfo dataRow) {
			for(int n = 0; n < dataRow.Cells.Count; n++) {
				UpdateCellEditorInfo(dataRow.Cells[n], dataRow.RowHandle);
			}
		}
		void UpdateCellEditorInfo(GridCellInfo cell, int rowHandle) {
			RepositoryItem item = View.GetRowCellRepositoryItem(rowHandle, cell.Column);
			if(cell.Editor == null || cell.Editor.GetType() != item.GetType()) {
				cell.Editor = item;
				cell.ViewInfo = null; 
			}
			if(cell.ViewInfo == null) {
				cell.ViewInfo = item.CreateViewInfo();
				if(!cell.Column.DisplayFormat.IsEmpty) {
					cell.ViewInfo.Format = cell.Column.DisplayFormat;
				}
			}
		}
		protected virtual void PrintRow(Graphics g, IBrickGraphics graph, int rowHandle, int level) {
			if (prevLevel != -1) {
				bool hasRowFooter = prevLevel > level ||
					(View.GroupFooterShowMode == GroupFooterShowMode.VisibleAlways &&
					prevLevel == level && View.IsGroupRow(prevRowHandle));
				if (hasRowFooter) PrintRowFooter(prevRowHandle, rowHandle);
				RaiseAfterPrintRow(g, graph, prevRowHandle, View.GetRowLevel(prevRowHandle), Indent, Y, hasRowFooter);
			}
			if(rowHandle == GridControl.InvalidRowHandle) return;
			bool isGroup = View.IsGroupRow(rowHandle);
			currentRowHeight = CalcRowHeight(g, rowHandle, isGroup);
			currentRowInfo = CreateRowInfo(rowHandle, level);
			prevLevel = level;
			if(isGroup) {
				CompleteCellMerging();
				printMergedInfoTable.Clear();
				PrintGroupRow(rowHandle, level);
			}
			else {
				PrintDataRow(rowHandle, level);
				PrintMasterDataRow(rowHandle, level);
			}
			prevRowHandle = rowHandle;
		}
		object dataRowHeight = null, groupRowHeight;
		protected int CalcDefaultRowHeight(bool dataRow) {
			if(dataRow) {
				if(dataRowHeight == null) {
					AppearanceObject[] apps = new AppearanceObject[] { AppearancePrint.Row, AppearancePrint.OddRow, AppearancePrint.EvenRow };
					dataRowHeight = ViewViewInfo.CalcMinEditorHeight(apps);
				}
				return (int)dataRowHeight;
			}
			if(groupRowHeight == null) {
				groupRowHeight = AppearancePrint.GroupRow.CalcDefaultTextSize(g).Height;
			}
			return (int)groupRowHeight;
		}
		protected virtual int CalcRowHeight(Graphics g, int rowHandle, bool isGroup) {
			int lineHeight = View.OptionsPrint.PrintHorzLines ? 2 : 0;
			if(isGroup) {
				return ViewViewInfo.CalcRowHeight(g, rowHandle, CalcDefaultRowHeight(false) + 2 + lineHeight, -1, false, ColumnsInfo);
			}
			return ViewViewInfo.CalcRowHeight(g, rowHandle, CalcDefaultRowHeight(true), -1, false, ColumnsInfo) + lineHeight;
		}
		protected virtual void PrintMasterDataRow(int rowHandle, int level) {
			if(!View.OptionsPrint.PrintDetails) return;
			if(!View.IsMasterRow(rowHandle)) return;
			bool expanded = View.GetMasterRowExpanded(rowHandle);
			bool printAll = View.OptionsPrint.ExpandAllDetails;
			if(!expanded && !printAll) return;
			if(!printAll) {
				PrintDetail(View.GetVisibleDetailView(rowHandle), level);
				return;
			}
			GridDetailInfo[] detailInfos = View.GetDetailInfo(rowHandle, true);
			for(int n = 0; detailInfos != null && n < detailInfos.Length; n++) {
				if(!View.CanExpandMasterRowEx(rowHandle, detailInfos[n].RelationIndex)) continue;
				BaseView defaultView;
				string levelName;
				BaseView newGV = View.CreateLevelDefault(rowHandle, detailInfos[n].RelationIndex, out defaultView, out levelName);
				if(newGV.IsSupportPrinting) {
					newGV.BeginUpdate();
					MasterRowInfo mi = new MasterRowInfo(View.DataController, rowHandle, null);
					DetailInfo di = mi.CreateDetail(View.DataController.GetDetailList(rowHandle, detailInfos[n].RelationIndex), detailInfos[n].RelationIndex);
					View.ApplyLevelDefaults(newGV, defaultView, di, levelName);
					newGV.ParentView = null;
					newGV.ParentView = View;
					PrintDetail(newGV, level);
				}
				newGV.ParentView = null;
				newGV.Dispose();
			}
		}
		protected virtual void PrintDetail(BaseView view, int rowLevel) {
			if(!view.IsSupportPrinting) return;
			PrintWrapper.PrintDetail(view, rowLevel, ViewViewInfo.LevelIndent, View.DetailVerticalIndent);
			Y = View.DetailVerticalIndent;
		}
		protected virtual int RowLineCount { get { return 1; } }
		protected virtual void PrintDataRow(int rowHandle, int level) {
			GridDataRowInfo dataRow = this.currentRowInfo as GridDataRowInfo;
			if(dataRow.Cells.Count == 0) {
				CreateRowCellInfo(dataRow);
			}
			else {
				if(!IsSameCellEditor) UpdateCellEditorInfo(dataRow);
			}
			ViewViewInfo.UpdateRowData(currentRowInfo, false, true);
			ViewViewInfo.UpdateBeforePaint(this.currentRowInfo);
			if(View.OptionsPrint.UsePrintStyles)
				dataRow.BaseAppearance = ((ViewViewInfo.CalcRowState(dataRow) & GridRowCellState.Even) != 0) ? AppearancePrint.EvenRow : AppearancePrint.OddRow;
			Point indent = new Point(Indent, Y);
			int lastRight = 0;
			for(int n = 0; n < Columns.Count; n++) {
				PrintColumnInfo colInfo = Columns[n] as PrintColumnInfo;
				lastRight = PreparePrintRowCell(colInfo, indent, rowHandle, dataRow.Cells[n], level, lastRight);
			}
			Y += RowLineCount * this.currentRowHeight;
			PrintDataRowPreview(rowHandle, level);
		} 
		protected virtual int PreparePrintRowCell(PrintColumnInfo colInfo, Point indent, int rowHandle, GridCellInfo cell, int level, int lastRight) {
			Rectangle r = colInfo.Bounds;
			r.Offset(indent);
			r.Height = this.CurrentRowHeight;
			int levelIndent = level * ViewViewInfo.LevelIndent;
			if(colInfo.Column.VisibleIndex == 0) {
				r.X += levelIndent;
				r.Width -= levelIndent;
			}
			if(r.X < indent.X + levelIndent) {
				int delta = indent.X + levelIndent - r.X;
				r.X += delta;
				r.Width -= delta;
			}
			if(r.Width > 0 && r.Right > indent.X + levelIndent) {
				PrintRowCell(rowHandle, cell, r);
			} 
			return r.Right;
		}
		protected virtual void PrintDataRowPreview(int rowHandle, int level) {
			if(!View.OptionsPrint.PrintPreview) return;
			GridDataRowInfo dataRow = this.currentRowInfo as GridDataRowInfo;
			if(dataRow.PreviewText == null || dataRow.PreviewText.Length == 0) return;
			RectangleF r = Rectangle.Empty;
			r.Y = Y;
			r.X = Indent + level * ViewViewInfo.LevelIndent;
			r.Width = this.fMaxRowWidth - r.X;
			GInfo.AddGraphics(null);
			try {
				r.Height = (int)AppearancePrint.Preview.CalcTextSize(GInfo.Cache, dataRow.PreviewText, (int)r.Width - 2 - 3 - ViewViewInfo.PreviewIndent).Height + 3 + 2;
			}
			finally {
				GInfo.ReleaseGraphics();
			}
			Y += (int)r.Height;
			SetDefaultBrickStyle(Graph, Bricks["Preview"]);
			IPanelBrick brick = (IPanelBrick)DrawBrick(Graph, "PanelBrick", r);
			IList col = brick.Bricks;
			UsePanelBrickDisableDrawing = true;
			RectangleF panelRect = r;
			if(View.OptionsPrint.SplitCellPreviewAcrossPages) brick.Separable = true;
			r.Inflate(-1, -1);
			r.X += ViewViewInfo.PreviewIndent;
			r.Width -= ViewViewInfo.PreviewIndent;
			r.X -= panelRect.X;
			r.Y -= panelRect.Y;
			SetDefaultBrickStyle(Graph, Bricks["PreviewText"]);
			ITextBrick textBrick = DrawTextBrick(Graph, dataRow.PreviewText, r, true);
			col.Add(textBrick);
			if(View.OptionsPrint.SplitCellPreviewAcrossPages) textBrick.Separable = true;
			UsePanelBrickDisableDrawing = false;
		}
		CellMergeEventArgs mergeArgs = new CellMergeEventArgs(0, 0, null);
		protected int RaiseMergeEvent(int row1, int row2, GridColumn column) {
			GridColumnInfoArgs columnInfo = ColumnsInfo[column];
			mergeArgs.Setup(row1, row2, columnInfo == null ? null : columnInfo.Column);
			View.RaiseCellMerge(mergeArgs);
			return this.mergeArgs.Handled ? (this.mergeArgs.Merge ? 0 : -1) : 3;
		}
		Dictionary<GridColumn, PrintMergeInfo> printMergedInfoTable;
		class PrintMergeInfo {
			public PrintMergeInfo(Brick brick, int rowHandle) {
				brickCore = brick;
				rowHandleCore = rowHandle;
			}
			private Brick brickCore;
			public Brick Brick {
				get { return brickCore; }
				set { brickCore = value; }
			}
			private int rowHandleCore;
			public int RowHandle {
				get { return rowHandleCore; }
				set { rowHandleCore = value; }
			}
		}
		protected object GetTextValue(VisualBrick brick) {
			PanelBrick pb = brick as PanelBrick;
			IRichTextBrick irtb = brick as IRichTextBrick;
			if(irtb != null) return irtb.RtfText;
			if(pb != null) return pb.Value; else return brick.TextValue;
		}
		protected Brick UpdateMergedStatus(GridCellInfo cell, VisualBrick brick) {
			if(printMergedInfoTable.ContainsKey(cell.Column)) {
				VisualBrick savedBrick = (VisualBrick)printMergedInfoTable[cell.Column].Brick;
				bool canMerge = object.Equals(GetTextValue(brick), GetTextValue(savedBrick)) && (savedBrick.Rect.Height < View.OptionsPrint.MaxMergedCellHeight || View.OptionsPrint.MaxMergedCellHeight <= 0) && cell.Column.GetAllowMerge();
				int eventCanMerge = RaiseMergeEvent(printMergedInfoTable[cell.Column].RowHandle, cell.RowHandle, cell.Column);
				if(eventCanMerge < 0) canMerge = false;
				if(eventCanMerge == 0) canMerge = true;
				if(canMerge) {
					PanelBrick pb = savedBrick as PanelBrick;
					if(pb == null) {
						pb = new PanelBrick();
						pb.BackColor = savedBrick.BackColor;
						pb.BorderStyle = savedBrick.BorderStyle;
						pb.BorderWidth = savedBrick.BorderWidth;
						pb.BorderColor = savedBrick.BorderColor;
						pb.Bricks.Add(savedBrick);
						pb.Value =  GetTextValue(savedBrick);
						printMergedInfoTable[cell.Column] = new PrintMergeInfo(pb, cell.RowHandle);
					}
					pb.Rect = new RectangleF(savedBrick.Rect.Location, new SizeF(savedBrick.Rect.Width, savedBrick.Rect.Height + brick.Rect.Height));
					pb.Merged = true;
					foreach(Brick b in pb.Bricks) {
						TextBrick tb = b as TextBrick;
						if(tb != null) tb.StringFormat = new BrickStringFormat(tb.StringFormat, tb.StringFormat.FormatFlags & StringFormatFlags.NoClip);
					}
					foreach(VisualBrick temp in pb.Bricks)
						if(!(temp is SpanPanelBrick))
						   temp.Sides = temp.Sides & (BorderSide)5;
					return null;
				}
				else {
					printMergedInfoTable[cell.Column] = new PrintMergeInfo(brick, cell.RowHandle);
					ProcessMergedCell(savedBrick);
					return savedBrick;
				}
			}
			else {
				printMergedInfoTable.Add(cell.Column, new PrintMergeInfo(brick, cell.RowHandle));
				return null;
			}
		}
		protected void ProcessMergedCell(VisualBrick vb) {
			PanelBrick pb = vb as PanelBrick;
			if(pb != null) 
				pb.CenterChildControls();
		}
		protected virtual void PrintRowCell(int rowHandle, GridCellInfo cell, Rectangle r) {
			cell.State = GridRowCellState.Dirty;
			cell.State = ViewViewInfo.CalcRowEvenOddState(rowHandle);
			ViewViewInfo.UpdateCellAppearanceCore(cell);
			string displayText = View.GetRowCellDisplayTextCore(rowHandle, cell.Column, cell.ViewInfo, cell.CellValue);
			PrintCellHelperInfo info = new PrintCellHelperInfo(
				LineColor,
				PS,
				cell.CellValue,
				cell.Appearance,
				displayText,
				r,
				Graph,
				View.GetRowCellDefaultAlignment(rowHandle, cell.Column, cell.Appearance.HAlignment),
				View.OptionsPrint.PrintHorzLines,
				View.OptionsPrint.PrintVertLines,
				cell.ColumnInfo.Column.DisplayFormat.FormatString,
				CalcBrickBorders()
				);
			IVisualBrick brick = cell.Editor.GetBrick(info);
			if(AllowProcessMergedInfo) {
				brick.Rect = r;
				IVisualBrick mbrick = (IVisualBrick)UpdateMergedStatus(cell, (VisualBrick)brick);
				if(mbrick != null) {
					Graph.DrawBrick(mbrick, mbrick.Rect);
				}
			}
			else {
				Graph.DrawBrick(brick, r);
			}
		}
		protected virtual void PrintRowCellOld(int rowHandle, GridCellInfo cell, Rectangle r) {
			ViewViewInfo.UpdateCellAppearance(cell);
			DevExpress.XtraEditors.Repository.RepositoryItem ritem = View.GetRowCellRepositoryItem(rowHandle, cell.ColumnInfo.Column);
			object cellvalue = View.GetRowCellValue(rowHandle, cell.ColumnInfo.Column);
			PrintCellHelperInfo info = new PrintCellHelperInfo(
				LineColor,
				PS,
				cellvalue,
				cell.Appearance,
				View.GetRowCellDisplayText(rowHandle, cell.ColumnInfo.Column),
				r,
				Graph,
				View.GetRowCellDefaultAlignment(rowHandle, cell.Column, cell.Appearance.HAlignment),
				View.OptionsPrint.PrintHorzLines,
				View.OptionsPrint.PrintVertLines,
				cell.ColumnInfo.Column.DisplayFormat.FormatString
				);
			IVisualBrick brick = ritem.GetBrick(info);
			if(AllowProcessMergedInfo) {
				brick.Rect = r;
				IVisualBrick mbrick = (IVisualBrick)UpdateMergedStatus(cell, (VisualBrick)brick);
				if(mbrick != null) {
					mbrick.Sides = CalcBrickBorders();
					Graph.DrawBrick(mbrick, mbrick.Rect);
				}
			}
			else {
				brick.Sides = CalcBrickBorders();
				Graph.DrawBrick(brick, r);
			}
		}
		protected virtual bool AllowProcessMergedInfo {
			get { return View.OptionsView.AllowCellMerge; }
		}
		protected virtual void PrintGroupRow(int rowHandle, int level) {
			Rectangle r = Rectangle.Empty;
			r.X = Indent + level * ViewViewInfo.LevelIndent;
			r.Width = this.fMaxRowWidth - r.Left;
			r.Y = Y;
			r.Height = currentRowHeight;
			if(View.OptionsPrint.UsePrintStyles) SetDefaultBrickStyle(Graph, Bricks["GroupRow"]);
			else {
				AppearanceObject rowAppearance = View.GetLevelStyle(level);
				BrickStyle def = new BrickStyle(Bricks["GroupRow"].Sides, Bricks["GroupRow"].BorderWidth, rowAppearance.BorderColor, rowAppearance.BackColor, rowAppearance.ForeColor, rowAppearance.Font, Bricks["GroupRow"].StringFormat);
				def.TextAlignment = DevExpress.XtraPrinting.Native.TextAlignmentConverter.ToTextAlignment(rowAppearance.TextOptions.HAlignment, rowAppearance.TextOptions.VAlignment);
				SetDefaultBrickStyle(Graph, def);
			}
			ITextBrick textBrick = DrawTextBrick(Graph, View.GetGroupRowDisplayText(rowHandle), r, true);
			textBrick.Text = textBrick.Text.Replace(Environment.NewLine, "");
			textBrick.TextValue = View.GetGroupRowPrintValue(rowHandle);
			Y += r.Height;
		}
		protected virtual void PrintRowFooter(int prevRowHandle, int rowHandle) {
			if(!View.OptionsPrint.PrintGroupFooter) return;
			bool isExpanded = View.OptionsPrint.ExpandAllGroups ? true : View.GetRowExpanded(prevRowHandle);
			int fcount = ViewViewInfo.GetRowFooterCount(prevRowHandle, rowHandle, isExpanded);
			if(fcount == 0) return;
			bool showCurrentFooter = ViewViewInfo.IsShowCurrentRowFooter(prevRowHandle, rowHandle);
			int level = View.GetRowLevel(prevRowHandle);
			if(!View.IsGroupRow(prevRowHandle) || !showCurrentFooter) prevRowHandle = View.GetParentRowHandle(prevRowHandle);
			if(!showCurrentFooter) 
				level --;
			PrintRowFooterCore(prevRowHandle, level, fcount);
		}
		protected virtual void PrintRowFooterCore(int prevRowHandle, int level, int fcount) {
			int x = 0;
			Rectangle r = Rectangle.Empty;
			for(int n = 0; n < fcount; n++) {
				int parentHandle = View.GetParentRowHandle(prevRowHandle);
				if(View.OptionsPrint.ExpandAllGroups || View.GetRowExpanded(prevRowHandle) || View.GroupFooterShowMode == GroupFooterShowMode.VisibleAlways) {
					r.X = Indent + level * ViewViewInfo.LevelIndent;
					r.Width = this.fMaxRowWidth - r.Left;
					r.Y = Y;
					r.Height = this.groupFooterHeight * RowLineCount + 2;
					SetDefaultBrickStyle(Graph, Bricks["GroupFooter"]);
					IPanelBrick brick = (IPanelBrick)DrawBrick(Graph, "PanelBrick", r);
					brick.Sides = CalcBrickBorders();
					IList col = brick.Bricks;
					UsePanelBrickDisableDrawing = true;
					ITextBrick textBrick;
					Rectangle panelRect = r;
					brick.SeparableHorz = true;
					foreach(PrintColumnInfo colInfo in Columns) {
						if(View.IsShowRowFooterCell(prevRowHandle, colInfo.Column)) {
							x = colInfo.Bounds.X + Indent;
							if(x < panelRect.X) {
								r.Width = colInfo.Bounds.Width - (panelRect.X - x);
								r.X = panelRect.X;
							}
							else {
								r.X = x;
								r.Width = colInfo.Bounds.Width;
							}
							r.Y = colInfo.RowIndex * this.groupFooterHeight + 1 + Y;
							r.Height = colInfo.RowCount * this.groupFooterHeight;
							r = new Rectangle(r.X - panelRect.X, r.Y - panelRect.Y, r.Width, r.Height);
							textBrick = DrawTextBrick(Graph, View.GetRowFooterCellText(prevRowHandle, colInfo.Column), r);
							DictionaryEntry de = View.GetRowSummaryItem(prevRowHandle, colInfo.Column);
							GridGroupSummaryItem gsi = de.Key as GridGroupSummaryItem;
							if(gsi != null) {
								textBrick.TextValue = de.Value;
								textBrick.TextValueFormatString = gsi.DisplayFormat;
							}
							textBrick.Sides = CalcBrickBorders();
							col.Add(textBrick);
						}
					}
					UsePanelBrickDisableDrawing = false;
					string text = View.GetRowCellDisplayText(prevRowHandle, View.Columns[0]);
					Hashtable hash1 = View.GetGroupSummaryValues(prevRowHandle);
					Hashtable hash2 = View.GetGroupSummaryValues(View.GetParentRowHandle(prevRowHandle));
					Y += this.groupFooterHeight * RowLineCount + 2;
				}
				prevRowHandle = View.GetParentRowHandle(prevRowHandle);
				level--;
			}
		}
		protected override void MakeRowList() {
			if(PrintSelectedRowsOnly) {
				MakeSelectedRowList();
				return;
			}
			VisibleIndexCollection indexes = new VisibleIndexCollection(View.DataController, View.DataController.GroupInfo);
			indexes.BuildVisibleIndexes(View.DataController.VisibleListSourceRowCount, true, View.OptionsPrint.ExpandAllGroups);
			Rows.AddRange(indexes);
		}
		protected override void AddSelectedRows(int[] rows) {
			if(!View.OptionsPrint.ExpandAllGroups) {
				base.AddSelectedRows(rows);
				return;
			}
			Hashtable printedRows = new Hashtable();
			foreach(int row in rows) {
				if(printedRows.ContainsKey(row)) continue;
				Rows.Add(row);
				printedRows[row] = true;
				if(View.IsGroupRow(row)) {
					GroupRowInfo rowInfo = View.DataController.GroupInfo.GetGroupRowInfoByHandle(row);
					if(rowInfo == null) continue;
					VisibleIndexCollection vi = new VisibleIndexCollection(View.DataController, View.DataController.GroupInfo);
					vi.BuildVisibleIndexes(rowInfo, true);
					foreach(int r in vi) {
						printedRows[r] = true;
						Rows.Add(r);
					}
				}
			}
		}
		public override void PrintFooter(DevExpress.XtraPrinting.IBrickGraphics graph) {
		}
		const int FooterCellIndent = 0;
		public virtual void PrintFooterPanel(DevExpress.XtraPrinting.IBrickGraphics graph) {
			if(!View.OptionsPrint.PrintFooter) return;
			int maxFooterCount = 0;
			foreach(PrintColumnInfo colInfo in Columns) {
				if(!colInfo.Column.HasSummary) continue;
				maxFooterCount = Math.Max(colInfo.Column.Summary.ActiveCount, maxFooterCount);
			}
			if(maxFooterCount == 0) return;
			Rectangle r = Rectangle.Empty;
			int footerHeight = this.footerPanelHeight * RowLineCount * maxFooterCount + FooterCellIndent * (maxFooterCount) + RowLineCount * FooterCellIndent; 
			r.Y = Y;
			r.X = Indent;
			r.Height = footerHeight;
			r.Width = this.fMaxRowWidth;
			SetDefaultBrickStyle(graph, Bricks["FooterPanel"]);
			IPanelBrick brick = (IPanelBrick)DrawBrick(graph, "PanelBrick", r);
			ITextBrick textBrick;
			brick.SeparableHorz = true;
			IList col = brick.Bricks;
			UsePanelBrickDisableDrawing = true;
			BrickStyle cellStyle = Bricks["FooterPanel"].Clone() as BrickStyle;
			cellStyle.Padding = new PaddingInfo(2, 2, 0, 0);
			graph.DefaultBrickStyle = cellStyle;
			foreach(PrintColumnInfo colInfo in Columns) {
				if(!colInfo.Column.HasSummary) continue;
				for(int n = 0; n < colInfo.Column.Summary.ActiveCount; n++) {
					r.X = colInfo.Bounds.X + Indent;
					r.Y = colInfo.RowIndex * this.footerPanelHeight * maxFooterCount + FooterCellIndent * (n + 1) + this.footerPanelHeight * n +
						colInfo.RowIndex * (FooterCellIndent * maxFooterCount);
					r.Width = colInfo.Bounds.Width;
					r.Height = this.footerPanelHeight * colInfo.RowCount;
					r.X -= Indent;
					GridSummaryItem item = colInfo.Column.Summary.GetActiveItem(n);
					textBrick = DrawTextBrick(graph, item.GetDisplayText(item.SummaryValue, false), r);
					textBrick.TextValue = item.SummaryValue;
					textBrick.TextValueFormatString = item.DisplayFormat;
					col.Add(textBrick);
				}
			}
			Y += footerHeight;
			UsePanelBrickDisableDrawing = false;
		}
		Hashtable widthCache = new Hashtable();
		protected Hashtable WidthCache { get { return widthCache; } }
		protected virtual void UpdateWidthCache(bool push) {
			if(push) WidthCache.Clear();
			foreach(GridColumn column in View.VisibleColumns) {
				UpdateWidthCache(column, push);
			}
		}
		protected virtual void UpdateWidthCache(object obj, bool push) {
			GridColumn column = obj as GridColumn;
			if(column == null) return;
			if(push) {
				WidthCache[column] = new Point(column.Width, column.VisibleWidth);
			} else {
				if(!WidthCache.ContainsKey(column)) return;
				Point p = (Point)WidthCache[column];
				column.width = p.X;
				column.visibleWidth = p.Y;
			}
		}
		protected virtual void CreateInfo() {
			if(Graph == null) return;
			View.ViewInfo.RecalcColumnWidthes();
			this.groupFooterHeight = CalcStyleHeight(AppearancePrint.GroupFooter) + 4;
			this.footerPanelHeight = CalcStyleHeight(AppearancePrint.FooterPanel) + 4;
			UpdateWidthCache(true);
			try {
				CalcAutoWidth(Graph);
				CreateColumns();
			}
			finally {
				UpdateWidthCache(false);
			}
		}
		protected void CalcAutoWidth(IBrickGraphics graph) {
			if(!View.OptionsPrint.AutoWidth) return;
			ViewViewInfo.RecalcColumnWidthes(-1, true, MaximumWidth - 1);
		}
		protected virtual void CreateColumns() {
			this.headerRowHeight = Math.Max(CalcStyleHeight(AppearancePrint.HeaderPanel), View.ColumnPanelRowHeight) + 4;
			View.RefreshVisibleColumnsList();
			int row = 0;
			this.columnsInfo = ViewViewInfo.CreateVisibleColumnsInfo(ref row);
			CreatePrintColumnCollection();
		}
		protected virtual void CreatePrintColumnCollection() {
			this.fMaxRowWidth = 0;
			foreach(GridColumn col in View.VisibleColumns) {
				PrintColumnInfo colInfo = new PrintColumnInfo();
				colInfo.Bounds = new Rectangle(this.fMaxRowWidth, 0,  col.VisibleWidth, headerRowHeight);
				colInfo.Column = col;
				Columns.Add(colInfo);
				this.fMaxRowWidth += colInfo.Bounds.Width;
			}
		}
		protected override int MaxWidth { get { return fMaxRowWidth; } }
		protected override bool AllowPrintFilterInfo { get { return View.OptionsPrint.PrintFilterInfo; } }
		protected override Color LineColor { get { return AppearancePrint.Lines.BackColor; } }
		public override void Initialize() {
			base.Initialize();
			Bricks.Add("GroupRow", AppearancePrint.GroupRow, BorderSide.All, LineColor, 1);
			Bricks.Add("GroupFooter", AppearancePrint.GroupFooter, BorderSide.All, LineColor, 1);
			Bricks.Add("FooterPanel", AppearancePrint.FooterPanel, BorderSide.All, AppearancePrint.FooterPanel.BorderColor, 1);
			Bricks.Add("HeaderPanel", AppearancePrint.HeaderPanel, BorderSide.All, AppearancePrint.HeaderPanel.BorderColor, 1);
			BorderSide cell = BorderSide.None;
			if(View.OptionsPrint.PrintHorzLines) cell |= BorderSide.Bottom | BorderSide.Top;
			if(View.OptionsPrint.PrintVertLines) cell |= BorderSide.Left | BorderSide.Right;
			Bricks.Add("Preview", AppearancePrint.Preview, cell, LineColor, 1);
			Bricks.Add("PreviewText", AppearancePrint.Preview);
			ViewViewInfo.CalcConstants();
			CreateInfo();
		}
		protected override void UpdateAppearances() {
			base.UpdateAppearances();
			if(!View.OptionsPrint.UsePrintStyles) {
				AppearanceDefaultInfo[] info = View.BaseInfo.GetDefaultPrintAppearance();
				AppearancePrint.Combine(ViewViewInfo.PaintAppearance, info);
				if(View.IsSkinned) {
					AppearancePrint.HeaderPanel.Assign(Find("HeaderPanel", info));
					AppearancePrint.FilterPanel.Assign(Find("FilterPanel", info));
					AppearancePrint.GroupFooter.Assign(Find("GroupFooter", info));
					AppearancePrint.GroupRow.Assign(Find("GroupRow", info));
					AppearancePrint.FooterPanel.Assign(Find("FooterPanel", info));
				} else {
					AppearancePrint.Lines.BackColor = AppearancePrint.Lines.ForeColor = ViewViewInfo.PaintAppearance.HorzLine.BackColor;
				}
				if(!View.OptionsView.EnableAppearanceEvenRow) AppearancePrint.EvenRow.Assign(AppearancePrint.Row);
				if(!View.OptionsView.EnableAppearanceOddRow) AppearancePrint.OddRow.Assign(AppearancePrint.Row);
			} else {
				if(!View.OptionsPrint.EnableAppearanceEvenRow) AppearancePrint.EvenRow.Assign(AppearancePrint.Row);
				if(!View.OptionsPrint.EnableAppearanceOddRow) AppearancePrint.OddRow.Assign(AppearancePrint.Row);
			}
			AppearanceHelper.Combine(AppearancePrint.EvenRow, new AppearanceObject[] { AppearancePrint.EvenRow.Clone() as AppearanceObject, AppearancePrint.Row });
			AppearanceHelper.Combine(AppearancePrint.OddRow, new AppearanceObject[] { AppearancePrint.OddRow.Clone() as AppearanceObject, AppearancePrint.Row });
		}
	}
	public class PrintColumnInfo {
		public GridColumn Column;
		public Rectangle Bounds;
		public int RowCount, RowIndex;
		public PrintColumnInfo(GridColumn column) {
			this.Column = column;
			this.Bounds = Rectangle.Empty;
			this.RowCount = 1;
			this.RowIndex = 0;
		}
		public PrintColumnInfo() : this(null) {
		}
	}
}
