#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Card;
using DevExpress.Data;
using DevExpress.LookAndFeel;
using DevExpress.Utils;
using DevExpress.Utils.Menu;
using System.Collections;
using DevExpress.XtraGrid.Localization;
using DevExpress.XtraExport;
using DevExpress.XtraGrid.Export;
using DevExpress.XtraGrid.FilterEditor;
using DevExpress.XtraEditors.Repository;
using DevExpress.Data.Helpers;
using System.Collections.Generic;
using DevExpress.XtraGrid.GroupSummaryEditor;
using DevExpress.XtraGrid.Views.BandedGrid;
namespace DevExpress.XtraGrid.Menu {
	public class ViewMenu : DXPopupMenu {
		BaseView view;
		public ViewMenu(BaseView view) {
			this.view = view;
		}
		public override void Dispose() {
			this.view = null;
			base.Dispose();
		}
		public virtual void Init(object info) {
			CreateItems();
		}
		protected virtual BaseView ViewCore { get { return view; } }
		public BaseView View { get { return ViewCore; } }
		protected virtual void CreateItems() {
		}
		protected DXMenuItem CreateMenuItem(GridStringId id, Image image, bool enabled, bool beginGroup) {
			return CreateMenuItem(GridLocalizer.Active.GetLocalizedString(id), image, id, enabled, beginGroup);
		}
		protected DXMenuItem CreateMenuItem(GridStringId id, Image image, bool enabled) {
			return CreateMenuItem(id, image, enabled, false);
		}
		protected DXMenuItem CreateMenuItem(GridStringId id, Image image) {
			return CreateMenuItem(GridLocalizer.Active.GetLocalizedString(id), image, id, true);
		}
		protected DXMenuItem CreateMenuItem(string caption, Image image, object tag, bool enabled) {
			return CreateMenuItem(caption, image, tag, enabled, false);
		}
		protected virtual DXMenuItem CreateMenuItem(string caption, Image image, object tag, bool enabled, bool beginGroup) {
			DXMenuItem item = new DXMenuItem(caption, new EventHandler(OnMenuItemClick), image);
			item.Enabled = enabled;
			item.BeginGroup = beginGroup;
			item.Tag = tag;
			return item;
		}
		protected virtual DXSubMenuItem CreateSubMenuItem(GridStringId id, Image image) {
			return CreateSubMenuItem(GridLocalizer.Active.GetLocalizedString(id), image, id, true, false);
		}
		protected virtual DXSubMenuItem CreateSubMenuItem(string caption, Image image, object tag, bool enabled, bool beginGroup) {
			DXSubMenuItem item = new DXSubMenuItem(caption, new EventHandler(OnBeforePopup));
			item.Image = image;
			item.Enabled = enabled;
			item.BeginGroup = beginGroup;
			item.Tag = tag;
			return item;
		}
		protected DXMenuItem CreateMenuCheckItem(GridStringId id, bool check, Image image) {
			return CreateMenuCheckItem(GridLocalizer.Active.GetLocalizedString(id), check, image, id, true);
		}
		protected DXMenuItem CreateMenuCheckItem(GridStringId id, bool check, Image image, bool enabled) {
			return CreateMenuCheckItem(GridLocalizer.Active.GetLocalizedString(id), check, image, id, enabled, false);
		}
		protected DXMenuItem CreateMenuCheckItem(GridStringId id, bool check, Image image, bool enabled, bool beginGroup) {
			return CreateMenuCheckItem(GridLocalizer.Active.GetLocalizedString(id), check, image, id, enabled, beginGroup);
		}
		protected DXMenuItem CreateMenuCheckItem(string caption, bool check, Image image, object tag, bool enabled) {
			return CreateMenuCheckItem(caption, check, image, tag, enabled, false);
		}
		protected virtual DXMenuItem CreateMenuCheckItem(string caption, bool check, Image image, object tag, bool enabled, bool beginGroup) {
			DXMenuCheckItem item = new DXMenuCheckItem(caption, check);
			item.Image = image;
			item.Enabled = enabled;
			item.Click += new EventHandler(OnMenuItemClick);
			item.CheckedChanged += new EventHandler(OnMenuItemCheckedChanged);
			item.BeginGroup = beginGroup;
			item.Tag = tag;
			return item;
		}
		protected string GetHideShowCaptionForGroupPanel(bool p) {
			return string.Format("{0}", p ?
				GridLocalizer.Active.GetLocalizedString(GridStringId.MenuGroupPanelHide) :
				GridLocalizer.Active.GetLocalizedString(GridStringId.MenuGroupPanelShow));
		}
		protected virtual bool RaiseClickEvent(object sender, EventArgs e) {
			return false;
		}
		protected virtual void OnMenuItemCheckedChanged(object sender, EventArgs e) { }
		protected virtual void OnMenuItemClick(object sender, EventArgs e) { }
		protected virtual void OnBeforePopup(object sender, EventArgs e) { }
		public virtual void Show(Point pos) {
			if(View == null || View.GridControl == null) return;
			if(!View.IsDesignMode && View.GridControl != null && View.GridControl.MenuManager != null) {
				View.GridControl.MenuManager.ShowPopupMenu(this, View.GridControl, pos);
			} else
				MenuManagerHelper.GetMenuManager(View.ElementsLookAndFeel).ShowPopupMenu(this, View.GridControl, pos);
		}
	}
	public class GridViewMenu : ViewMenu {
		public GridViewMenu(GridView view) : base(view) { }
		public new GridView View { get { return base.View as GridView; } }
		public virtual GridMenuType MenuType { get { return GridMenuType.User; } }
		protected override bool RaiseClickEvent(object sender, EventArgs e) {
			if(View == null) return false;
			GridMenuItemClickEventArgs ee = e as GridMenuItemClickEventArgs;
			if(ee == null) ee = new GridMenuItemClickEventArgs(null, null, null, SummaryItemType.Custom, string.Empty, MenuType, sender as DXMenuItem);
			View.InvokeMenuItemClick(ee);
			return ee.Handled;
		}
	}
	public class GridViewFooterMenu : GridViewMenu {
		public GridViewFooterMenu(GridView view) : base(view) { }
		GridColumn column = null;
		GridSummaryItem summaryItem = null;
		bool isGroupSummary = false;
		bool allowMultiSummary = false;
		DXSubMenuItem addSummaryItem = null;
		public override void Init(object info) {
			GridHitInfo hitInfo = info as GridHitInfo;
			this.column = hitInfo.Column;
			if(hitInfo.HitTest == GridHitTest.Footer) {
				allowMultiSummary = true;
				if(hitInfo.FooterCell != null && hitInfo.FooterCell.SummaryItem != null)
					this.summaryItem = hitInfo.FooterCell.SummaryItem;
			}
			if(hitInfo.HitTest == GridHitTest.RowFooter) {
				this.isGroupSummary = true;
				this.summaryItem = FindSummary(View.GroupSummary);
			}
			base.Init(info);
		}
		public override GridMenuType MenuType { get { return GridMenuType.Summary; } }
		RepositoryItem ColumnEditor {
			get {
				if(Column == null) return null;
				return Column.RealColumnEdit;
			}
		}
		protected bool IsSumAvgAvailable {
			get {
				if(Column == null || ColumnEditor == null) return false;
				Type type = Column.ColumnType;
				Type underlyingType = Nullable.GetUnderlyingType(type);
				if(underlyingType != null)
					type = underlyingType;
				if(type.Equals(typeof(DateTime)) ||
				   type.Equals(typeof(Boolean))) return false;
				if(IsLookUpEditor) return false;
				return type.IsValueType;
			}
		}
		bool IsLookUpEditor {
			get {
				return (ColumnEditor is RepositoryItemImageComboBox ||
					ColumnEditor is RepositoryItemLookUpEdit ||
					ColumnEditor is RepositoryItemGridLookUpEdit);
			}
		}
		GridGroupSummaryItem FindSummary(GridSummaryItemCollection collection) {
			GridGroupSummaryItem res = null;
			foreach(GridGroupSummaryItem item in collection) {
				if(item.ShowInGroupColumnFooter == Column) {
					if(item.SummaryType != SummaryItemType.None || res == null) res = item; 
				}
			}
			return res;
		}
		public GridColumn Column { get { return column; } }
		public IFormatProvider Format {
			get {
				if(Column == null || Column.DisplayFormat == null) return null;
				return Column.DisplayFormat.Format;
			}
		}
		public GridSummaryItem SummaryItem { get { return summaryItem; } }
		bool IsEmpty { 
			get {
				if(Column == null || !Column.AllowSummaryMenu) return true;
				if(SummaryItem == null) return false;
				return SummaryItem.FieldName == string.Empty; 
			} 
		}
		public bool IsGroupSummary { get { return isGroupSummary; } }
		public SummaryItemType SummaryType { get { return SummaryItem == null ? SummaryItemType.None : SummaryItem.SummaryType; } }
		static SummaryItemType[] Summary = new SummaryItemType[] {
																	 SummaryItemType.Sum, SummaryItemType.Min, SummaryItemType.Max, SummaryItemType.Count, 
																	 SummaryItemType.Average, SummaryItemType.Custom, SummaryItemType.None};
		bool AllowAddNewSummaryItem { get { return allowMultiSummary && AllowNewSummaryType(SummaryItemType.None) && View.AllowAddNewSummaryItemMenu && Column.Summary.Count != 0; } }
		bool AllowClearSummaryItems { get { return allowMultiSummary && View.AllowAddNewSummaryItemMenu && Column.Summary.Count > 1; } }
		bool AllowSummary { get { return IsSumAvgAvailable && !IsEmpty; } }
		bool AllowMinMax { get { return !IsLookUpEditor && !IsEmpty; } }
		bool IsTypeExist(SummaryItemType type) {
			foreach(GridColumnSummaryItem item in column.Summary)
				if(item.SummaryType == type) return true;
			return false;
		}
		bool AllowSummaryItem(SummaryItemType type) { 
			if(IsTypeExist(type)) return false;
			switch(type) {
				case SummaryItemType.Average: 
				case SummaryItemType.Sum: return AllowSummary;
				case SummaryItemType.Max:
				case SummaryItemType.Min: return AllowMinMax;
			}
			return true;
		}
		bool AllowNewSummaryType(SummaryItemType currentType) {
			SummaryItemType[] types = new SummaryItemType[] { SummaryItemType.Count, SummaryItemType.Sum, SummaryItemType.Average, SummaryItemType.Max, SummaryItemType.Min };
			foreach(SummaryItemType type in types)
				if(type != currentType && AllowSummaryItem(type)) return true;
			return false;
		}
		protected override void CreateItems() {
			Items.Clear();
			if(summaryItem == null && !AllowNewSummaryType(SummaryItemType.None)) return;
			bool isEnabled = SummaryType != SummaryItemType.Custom;
			if(isEnabled && AllowAddNewSummaryItem) {
				addSummaryItem = CreateSubMenuItem(GridStringId.MenuFooterAddSummaryItem, null);
				Items.Add(addSummaryItem);
				if(AllowSummaryItem(SummaryItemType.Sum)) addSummaryItem.Items.Add(CreateMenuItem(GridStringId.MenuFooterSum, GridMenuImages.Footer.Images[0]));
				if(AllowSummaryItem(SummaryItemType.Min)) addSummaryItem.Items.Add(CreateMenuItem(GridStringId.MenuFooterMin, GridMenuImages.Footer.Images[1]));
				if(AllowSummaryItem(SummaryItemType.Max)) addSummaryItem.Items.Add(CreateMenuItem(GridStringId.MenuFooterMax, GridMenuImages.Footer.Images[2]));
				if(AllowSummaryItem(SummaryItemType.Count)) addSummaryItem.Items.Add(CreateMenuItem(GridStringId.MenuFooterCount, GridMenuImages.Footer.Images[3]));
				if(AllowSummaryItem(SummaryItemType.Average)) addSummaryItem.Items.Add(CreateMenuItem(GridStringId.MenuFooterAverage, GridMenuImages.Footer.Images[4]));
			}
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterSum, SummaryType == SummaryItemType.Sum, GridMenuImages.Footer.Images[0], AllowSummary && isEnabled, isEnabled && AllowAddNewSummaryItem));
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterMin, SummaryType == SummaryItemType.Min, GridMenuImages.Footer.Images[1], AllowMinMax && isEnabled));
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterMax, SummaryType == SummaryItemType.Max, GridMenuImages.Footer.Images[2], AllowMinMax && isEnabled));
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterCount, SummaryType == SummaryItemType.Count, GridMenuImages.Footer.Images[3], isEnabled));
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterAverage, SummaryType == SummaryItemType.Average, GridMenuImages.Footer.Images[4], AllowSummary && isEnabled));
			Items.Add(CreateMenuCheckItem(GridStringId.MenuFooterNone, SummaryType == SummaryItemType.None, null, isEnabled, true));
			if(isEnabled && AllowClearSummaryItems)
				Items.Add(CreateMenuItem(GridStringId.MenuFooterClearSummaryItems, null, true, true));
		}
		protected bool GetSummaryType(DXMenuItem item, ref SummaryItemType summaryType) {
			if(!(item.Tag is GridStringId) || Column == null) return false;
			GridStringId id = (GridStringId)item.Tag;
			summaryType = SummaryItemType.None;
			switch(id) {
				case GridStringId.MenuFooterSum : summaryType = SummaryItemType.Sum; break; 
				case GridStringId.MenuFooterMin : summaryType = SummaryItemType.Min; break; 
				case GridStringId.MenuFooterMax : summaryType = SummaryItemType.Max; break; 
				case GridStringId.MenuFooterCount : summaryType = SummaryItemType.Count; break; 
				case GridStringId.MenuFooterAverage : summaryType = SummaryItemType.Average; break; 
				case GridStringId.MenuFooterNone : summaryType = SummaryItemType.None; break; 
				default: 
					return false;
			}
			return true;
		}
		protected override void OnMenuItemClick(object sender, EventArgs e) {
			DXMenuItem item = sender as DXMenuItem;
			SummaryItemType summaryType = SummaryItemType.None;
			bool validItem = GetSummaryType(item, ref summaryType);
			GridMenuItemClickEventArgs ee = new GridMenuItemClickEventArgs(Column, IsGroupSummary ? View.GroupSummary : null, SummaryItem, summaryType, View.GetSummaryFormat(Column, summaryType), MenuType, item); 
			if(RaiseClickEvent(sender, ee)) return;
			if(validItem) {
				if(IsGroupSummary) {
					if(SummaryType != ee.SummaryType || (SummaryItem != null && SummaryItem.DisplayFormat != ee.SummaryFormat)) {
						if(SummaryItem != null) SummaryItem.SetSummary(ee.SummaryType, ee.SummaryFormat, Format);
						else
							View.GroupSummary.Add(ee.SummaryType, ee.Column.FieldName, ee.Column, ee.SummaryFormat, Format);
					}
				}
				else {
					if(addSummaryItem != null && item.Collection == addSummaryItem.Items) {
						this.summaryItem = Column.Summary.Add(ee.SummaryType, "", ee.SummaryFormat, Format);
					}
					if(summaryItem == null) {
						this.summaryItem = Column.Summary.FindEmptyOrCreate(ee.SummaryType, "", ee.SummaryFormat, Format);
					}
					if(SummaryItem.SummaryType != ee.SummaryType || SummaryItem.DisplayFormat != ee.SummaryFormat) {
						SummaryItem.SetSummary(ee.SummaryType, ee.SummaryFormat, Format);
					}
					ClearEmptySummaryItems(column);
				}
			}
			else {
				if(GridStringId.MenuFooterClearSummaryItems.Equals(item.Tag)) Column.Summary.Clear();
			}
		}
		static void ClearEmptySummaryItems(GridColumn column) {
			for(int i = column.Summary.Count - 1; i >= 0; i--)
				if(column.Summary[i].SummaryType == SummaryItemType.None)
					column.Summary.RemoveAt(i);
		}
	}
	public class GridViewBandMenu : GridViewColumnMenu {
		public GridViewBandMenu(GridView view) : base(view) { }
		GridBand band = null;
		protected override bool ColumnMenu { get { return false; } }
		public override void Init(object info) {
			this.band = info as GridBand;
			base.Init(info);
		}
	}
	public class GridViewColumnMenu : GridViewMenu {
		public GridViewColumnMenu(GridView view) : base(view) { }
		GridColumn column = null;
		public override void Init(object info) {
			this.column = info as GridColumn;
			base.Init(info);
		}
		protected virtual bool ColumnMenu { get { return true; } }
		protected override BaseView ViewCore { get { return Column != null ? Column.View : base.ViewCore; } }
		public override GridMenuType MenuType { get { return GridMenuType.Column; } }
		public GridColumn Column { get { return column; } }
		protected ColumnSortOrder SortOrder { get { return Column == null ? ColumnSortOrder.None : Column.SortOrder; } }
		protected int GroupIndex { get { return Column == null ? -1 : Column.GroupIndex; } }
		bool IsGroupColumn { get { return GroupIndex != -1;}}
		bool IsColumnFiltered {
			get { 
				if(!View.OptionsCustomization.AllowFilter) return false;
				if(Column.FilterInfo.Type != ColumnFilterType.None) return true;
				return false;
			}
		}
		protected override void CreateItems() {
			Items.Clear();
			if(Column != null) {
				if(IsGroupColumn) {
					Items.Add(CreateMenuItem(GridStringId.MenuGroupPanelFullExpand, GridMenuImages.GroupPanel.Images[0], View.SortInfo.GroupCount > 0));
					Items.Add(CreateMenuItem(GridStringId.MenuGroupPanelFullCollapse, GridMenuImages.GroupPanel.Images[1], View.SortInfo.GroupCount > 0));
				}
				Items.Add(CreateMenuCheckItem(GridStringId.MenuColumnSortAscending, SortOrder == ColumnSortOrder.Ascending, GridMenuImages.Column.Images[1], View.CanSortColumn(Column), IsGroupColumn));
				Items.Add(CreateMenuCheckItem(GridStringId.MenuColumnSortDescending, SortOrder == ColumnSortOrder.Descending, GridMenuImages.Column.Images[2], View.CanSortColumn(Column)));
				Items.Add(CreateMenuItem(GridStringId.MenuColumnClearSorting, null, View.CanSortColumn(Column) && !IsGroupColumn && Column.SortOrder != ColumnSortOrder.None));
				if (IsGroupColumn) CreateSummarySortInfo();
				Items.Add(CreateMenuItem(IsGroupColumn ? GridStringId.MenuColumnUnGroup : GridStringId.MenuColumnGroup, GridMenuImages.Column.Images[3], View.CanGroupColumn(Column), true));
			}
			Items.Add(CreateMenuCheckItem(GetHideShowCaptionForGroupPanel(View.OptionsView.ShowGroupPanel),
				View.OptionsView.ShowGroupPanel, GridMenuImages.Column.Images[4], GridStringId.MenuColumnGroupBox, true));
			if(Column != null) {
				if(IsGroupColumn && (Column.ColumnType == typeof(DateTime) || Column.ColumnType == typeof(DateTime?))
					&& !View.IsDesignMode) CreateGroupIntervalInfo();
				if(IsGroupColumn && !View.IsDesignMode) CreateGroupSummaryEditorInfo();
				if(Column.Visible || IsGroupColumn)
					Items.Add(CreateMenuItem(GridStringId.MenuColumnRemoveColumn, null,
						Column.OptionsColumn.AllowShowHide && !(!IsGroupColumn && View.VisibleColumns.Count == 1 && View.SortInfo.GroupCount != 0), true));
				if(!Column.Visible && !IsGroupColumn)
					Items.Add(CreateMenuItem(GridStringId.MenuColumnShowColumn, null, Column.OptionsColumn.AllowShowHide, true));
			}
			if(View.OptionsCustomization.AllowColumnMoving)
				Items.Add(CreateMenuItem(GetStringColumnCustomization(View), GridMenuImages.Column.Images[5], View.CustomizationForm == null, Column == null));
			if(Column != null && (!IsGroupColumn || View.OptionsView.ShowGroupedColumns))
				Items.Add(CreateMenuItem(GridStringId.MenuColumnBestFit, GridMenuImages.Column.Images[6], View.CanResizeColumn(Column)));
			if(!(View is DevExpress.XtraGrid.Views.BandedGrid.BandedGridView && View.OptionsView.ColumnAutoWidth)
				&& View.OptionsCustomization.AllowColumnResizing) {
				Items.Add(CreateMenuItem(GridStringId.MenuColumnBestFitAllColumns, null, View.VisibleColumns.Count > 0));
			}
			bool beginGroup = true;
			if(Column != null)  {
				if((View.IsDesignMode || object.Equals(View.GridControl.Tag, "Design")) && View.OptionsCustomization.AllowFilter) {
					Items.Add(CreateMenuCheckItem(GridStringId.MenuColumnFilter, View.IsColumnAllowFilter(Column), GridMenuImages.Column.Images[10], true, beginGroup));
					beginGroup = false;
				}
				if(IsColumnFiltered) {
					Items.Add(CreateMenuItem(GridStringId.MenuColumnClearFilter, GridMenuImages.Column.Images[7], true, beginGroup));
					beginGroup = false;
				}
			}
			if(GridCriteriaHelper.IsAllowFilterEditorForColumn(View, Column) && !View.IsDesignMode) {
				Items.Add(CreateMenuItem(GridStringId.MenuColumnFilterEditor, GridMenuImages.Column.Images[8], true, beginGroup));
				if(!View.OptionsFind.AlwaysVisible && View.OptionsFind.AllowFindPanel && (!View.IsDetailView || View.IsZoomedView))
					Items.Add(CreateMenuItem(
						View.IsFindPanelVisible ? GridStringId.MenuColumnFindFilterHide : GridStringId.MenuColumnFindFilterShow, null));
				if(View.OptionsMenu.ShowAutoFilterRowItem && ColumnMenu)
					Items.Add(CreateMenuItem(
						View.OptionsView.ShowAutoFilterRow ? GridStringId.MenuColumnAutoFilterRowHide : GridStringId.MenuColumnAutoFilterRowShow, 
						null));
			}
			if(Column != null && Column.AllowFilterModeChanging && !View.IsServerMode) {
				DXSubMenuItem parentItem = CreateSubMenuItem(GridStringId.MenuColumnFilterMode, null);
				Items.Add((DXMenuItem)parentItem);
				parentItem.Items.Add(CreateMenuCheckItem(GridStringId.MenuColumnFilterModeValue, Column.FilterMode == ColumnFilterMode.Value, null));
				parentItem.Items.Add(CreateMenuCheckItem(GridStringId.MenuColumnFilterModeDisplayText, Column.FilterMode == ColumnFilterMode.DisplayText, null));
			}
			if(Column != null && Column.ShowUnboundExpressionMenu && !View.IsDesignMode)
				Items.Add(CreateMenuItem(GridStringId.MenuColumnExpressionEditor, GridMenuImages.Column.Images[12], true, true));
		}
		internal static GridStringId GetStringColumnCustomization(ColumnView view) {
			if(view is DevExpress.XtraGrid.Views.BandedGrid.BandedGridView) return GridStringId.MenuColumnBandCustomization;
			return GridStringId.MenuColumnColumnCustomization;
		} 
		void CreateSummarySortInfo() {
			if(!View.OptionsMenu.ShowGroupSortSummaryItems) return;
			if(View.GroupSummary.ActiveCount == 0) return;
			if(View.GroupSummarySortInfo.Count > 0) {
				Items.Add(CreateMenuItem(GridStringId.MenuColumnResetGroupSummarySort, null));
			}
			DXSubMenuItem menu = new DXSubMenuItem(GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnSortGroupBySummaryMenu));
			menu.Tag = GridStringId.MenuColumnSortGroupBySummaryMenu;
			foreach(GridGroupSummaryItem item in View.GroupSummary) {
				if(item.SummaryType == SummaryItemType.None) continue;
				DXMenuCheckItem menuItemAsc = CreateSummarySortMenuItem(item, ColumnSortOrder.Ascending);
				DXMenuCheckItem menuItemDesc = CreateSummarySortMenuItem(item, ColumnSortOrder.Descending);
				menuItemAsc.BeginGroup = true && menu.Items.Count > 0;
				menu.Items.Add(menuItemAsc);
				menu.Items.Add(menuItemDesc);
			}
			if(menu.Items.Count > 0) Items.Add(menu);
		}
		string GetDisplaySummaryName(GridSummaryItem item) {
			string fname = item.FieldName;
			if(!string.IsNullOrEmpty(fname)) {
				GridColumn col = View.Columns[fname];
				if(col != null) return col.GetTextCaption();
				return MasterDetailHelper.SplitPascalCaseString(fname);
			}
			return Column.GetTextCaption();
		}
		string GetSummaryTypeDescription(SummaryItemType type) { 
			switch(type) {
				case SummaryItemType.Sum: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnSumSummaryTypeDescription);
				case SummaryItemType.Min: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnMinSummaryTypeDescription);
				case SummaryItemType.Max: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnMaxSummaryTypeDescription);
				case SummaryItemType.Count: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnCountSummaryTypeDescription);
				case SummaryItemType.Average: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnAverageSummaryTypeDescription);
				case SummaryItemType.Custom: return GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnCustomSummaryTypeDescription);
			}
			return string.Format("{0}", type);
		}
		DXMenuCheckItem CreateSummarySortMenuItem(GridGroupSummaryItem item, ColumnSortOrder sortOrder) {
			DXMenuCheckItem res = new DXMenuCheckItem(string.Format(GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnGroupSummarySortFormat),
				 GetDisplaySummaryName(item),
				 GetSummaryTypeDescription(item.SummaryType), GridLocalizer.Active.GetLocalizedString(sortOrder == ColumnSortOrder.Ascending ? GridStringId.MenuColumnSortAscending : GridStringId.MenuColumnSortDescending)));
			res.Tag = item;
			if(sortOrder == ColumnSortOrder.Ascending) {
				res.Click += new EventHandler(OnSummarySortAscending);
			}
			else {
				res.Click += new EventHandler(OnSummarySortDescending);
			}
			foreach(GroupSummarySortInfo info in View.GroupSummarySortInfo) {
				if(info.GroupColumn == Column && info.SummaryItem == item) 
					res.Checked = info.SortOrder == sortOrder;
			}
			return res;
		}
		void CreateGroupIntervalInfo() {
			if (!View.OptionsMenu.ShowDateTimeGroupIntervalItems) return;
			DXSubMenuItem menu = new DXSubMenuItem(GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnGroupIntervalMenu));
			menu.Tag = GridStringId.MenuColumnGroupIntervalMenu;
			Items.Add(menu);
			menu.Items.Add(CreateGroupIntervalItem(ColumnGroupInterval.Date & ColumnGroupInterval.Default, GridStringId.MenuColumnGroupIntervalDay));
			menu.Items.Add(CreateGroupIntervalItem(ColumnGroupInterval.DateMonth, GridStringId.MenuColumnGroupIntervalMonth));
			menu.Items.Add(CreateGroupIntervalItem(ColumnGroupInterval.DateYear, GridStringId.MenuColumnGroupIntervalYear));
			menu.Items.Add(CreateGroupIntervalItem(ColumnGroupInterval.DateRange, GridStringId.MenuColumnGroupIntervalSmart));
		}
		DXMenuItem CreateGroupIntervalItem(ColumnGroupInterval groupInterval, GridStringId gridStringId) {
			DXMenuCheckItem menuItem = new DXMenuCheckItem();
			menuItem.Caption = GridLocalizer.Active.GetLocalizedString(gridStringId);
			menuItem.Click += new EventHandler(OnDateTimeGroupIntervalMenuItemClick);
			menuItem.Tag = groupInterval;
			menuItem.Checked = groupInterval == Column.GroupInterval;
			return menuItem;
		}
		void CreateGroupSummaryEditorInfo() {
			if (!View.OptionsMenu.ShowGroupSummaryEditorItem) return;
			DXMenuItem menuItem = new DXMenuItem(GridLocalizer.Active.GetLocalizedString(GridStringId.MenuColumnGroupSummaryEditor));
			menuItem.Image = GridMenuImages.Column.Images[11];
			Items.Add(menuItem);
			menuItem.Click += new EventHandler(OnCreateGroupSummaryEditorClick);
		}
		void OnCreateGroupSummaryEditorClick(object sender, EventArgs e) {
			if(RaiseClickEvent(sender, new GridMenuItemClickEventArgs(Column, null, null, SummaryItemType.None, string.Empty, MenuType, sender as DXMenuItem))) return;
			new GroupSummaryEditorForm(View, Column).ShowDialog();
		}
		void OnDateTimeGroupIntervalMenuItemClick(object sender, EventArgs e) {
			if(RaiseClickEvent(sender, new GridMenuItemClickEventArgs(Column, null, null, SummaryItemType.None, string.Format("{0}", ((DXMenuItem)sender).Tag), MenuType, sender as DXMenuItem))) return;
			Column.GroupInterval =(ColumnGroupInterval)((DXMenuItem)sender).Tag;
		}
		void OnSummarySortDescending(object sender, EventArgs e) {
			GroupBySummary(sender, ((DXMenuItem)sender).Tag as GridGroupSummaryItem, ColumnSortOrder.Descending);
		}
		void OnSummarySortAscending(object sender, EventArgs e) {
			GroupBySummary(sender, ((DXMenuItem)sender).Tag as GridGroupSummaryItem, ColumnSortOrder.Ascending);
		}
		void GroupBySummary(object sender, GridGroupSummaryItem summaryItem, ColumnSortOrder sortOrder) {
			DXMenuCheckItem checkItem = sender as DXMenuCheckItem;
			if(checkItem == null || !checkItem.Checked) return;
			if(!IsGroupColumn) return;
			if(RaiseClickEvent(sender, new GridMenuItemClickEventArgs(Column, null, summaryItem, SummaryItemType.Custom, sortOrder.ToString(), MenuType, sender as DXMenuItem))) return;
			List<GroupSummarySortInfo> items = new List<GroupSummarySortInfo>();
			foreach(GroupSummarySortInfo info in View.GroupSummarySortInfo) {
				if(info.GroupColumn == Column || info.GroupLevel == Column.GroupIndex) continue;
				items.Add(info);
			}
			items.Add(new GroupSummarySortInfo(summaryItem, Column, sortOrder));
			View.GroupSummarySortInfo.ClearAndAddRange(items.ToArray());
		}
		void ChangeColumnSorting(GridColumn column, ColumnSortOrder sortOrder) {
			if(column == null) return;
			View.GroupSummarySortInfo.Clear();
			column.SortOrder = sortOrder;
		}
		protected override void OnMenuItemClick(object sender, EventArgs e) {
			DXMenuItem item = sender as DXMenuItem;
			if(RaiseClickEvent(sender, new GridMenuItemClickEventArgs(Column, null, null, SummaryItemType.Custom, "", MenuType, item))) return;
			if(!(item.Tag is GridStringId) || View == null) return;
			GridStringId id = (GridStringId)item.Tag;
			switch(id) {
				case GridStringId.MenuGroupPanelFullExpand : View.ExpandAllGroups(); break;
				case GridStringId.MenuGroupPanelFullCollapse : View.CollapseAllGroups(); break;
				case GridStringId.MenuColumnSortAscending: ChangeColumnSorting(Column, ColumnSortOrder.Ascending); break;
				case GridStringId.MenuColumnSortDescending : ChangeColumnSorting(Column, ColumnSortOrder.Descending); break;
				case GridStringId.MenuColumnClearSorting : ChangeColumnSorting(Column, ColumnSortOrder.None); break;
				case GridStringId.MenuColumnUnGroup : 
					if(Column != null) Column.UnGroup();
					break;
				case GridStringId.MenuColumnRemoveColumn:
					if(Column != null) {
						column.UnGroup();
						Column.Visible = false;
					}
					break;
				case GridStringId.MenuColumnShowColumn:
					if(Column != null) {
						Column.Visible = true;
					}
					break;
				case GridStringId.MenuColumnGroup: 
					if(Column != null) Column.Group();
					break;
				case GridStringId.MenuColumnColumnCustomization :
				case GridStringId.MenuColumnBandCustomization: 
					View.ColumnsCustomization();
					break;
				case GridStringId.MenuColumnBestFit : 
					if(Column != null) Column.BestFit();
					break;
				case GridStringId.MenuColumnFilter : 
					if(Column != null) {
						Column.OptionsFilter.AllowFilter = !Column.OptionsFilter.AllowFilter;
						View.LayoutChanged();
					}
					break;
				case GridStringId.MenuColumnClearFilter : 
					if(Column != null) Column.FilterInfo = ColumnFilterInfo.Empty;
					break;
				case GridStringId.MenuColumnBestFitAllColumns : 
					View.BestFitColumns();
					break;
				case GridStringId.MenuColumnGroupBox : 
					View.OptionsView.ShowGroupPanel = !View.OptionsView.ShowGroupPanel;
					break;
				case GridStringId.MenuColumnFilterEditor :
					View.ShowFilterEditor(Column);
					break;
				case GridStringId.MenuColumnAutoFilterRowHide:
				case GridStringId.MenuColumnAutoFilterRowShow:
					View.OptionsView.ShowAutoFilterRow = !View.OptionsView.ShowAutoFilterRow;
					break;
				case GridStringId.MenuColumnFindFilterHide:
					View.HideFindPanel();
					break;
				case GridStringId.MenuColumnFindFilterShow:
					View.ShowFindPanel();
					break;
				case GridStringId.MenuColumnExpressionEditor:
					View.ShowUnboundExpressionEditor(Column);
					break;
				case GridStringId.MenuColumnResetGroupSummarySort:
					View.GroupSummarySortInfo.Clear();
					break;
				case GridStringId.MenuColumnFilterModeValue:
					if(Column != null) {
						Column.ClearFilter();
						Column.FilterMode = ColumnFilterMode.Value;
					}
					break;
				case GridStringId.MenuColumnFilterModeDisplayText:
					if(Column != null) {
						Column.ClearFilter();
						Column.FilterMode = ColumnFilterMode.DisplayText;
					}
					break;
			}
		}
	}
	public class GridViewGroupPanelMenu : GridViewMenu {
		public GridViewGroupPanelMenu(GridView view) : base(view) { }
		protected override void CreateItems() {
			Items.Clear();
			Items.Add(CreateMenuItem(GridStringId.MenuGroupPanelFullExpand, GridMenuImages.GroupPanel.Images[0], View.SortInfo.GroupCount > 0));
			Items.Add(CreateMenuItem(GridStringId.MenuGroupPanelFullCollapse, GridMenuImages.GroupPanel.Images[1], View.SortInfo.GroupCount > 0));
			Items.Add(CreateMenuItem(GridStringId.MenuGroupPanelClearGrouping, GridMenuImages.GroupPanel.Images[2], View.SortInfo.GroupCount > 0, true));
			Items.Add(CreateMenuCheckItem(GetHideShowCaptionForGroupPanel(View.OptionsView.ShowGroupPanel),
				View.OptionsView.ShowGroupPanel, GridMenuImages.Column.Images[4], GridStringId.MenuColumnGroupBox, true));
		}
		public override GridMenuType MenuType { get { return GridMenuType.Group; } }
		protected override void OnMenuItemClick(object sender, EventArgs e) {
			if(RaiseClickEvent(sender, null)) return;
			DXMenuItem item = sender as DXMenuItem;
			if(!(item.Tag is GridStringId)) return;
			GridStringId id = (GridStringId)item.Tag;
			switch(id) {
				case GridStringId.MenuGroupPanelFullExpand : View.ExpandAllGroups(); break;
				case GridStringId.MenuGroupPanelFullCollapse : View.CollapseAllGroups(); break;
				case GridStringId.MenuGroupPanelClearGrouping : View.ClearGrouping(); break;
				case GridStringId.MenuColumnGroupBox:
					View.OptionsView.ShowGroupPanel = !View.OptionsView.ShowGroupPanel;
					break;
			}
		}
	}
	public class GridMenuImages {
		[ThreadStatic]
		static ImageCollection groupPanel = null;
		public static ImageCollection GroupPanel {
			get {
				if(groupPanel == null) groupPanel = DevExpress.Utils.Controls.ImageHelper.CreateImageCollectionFromResources("DevExpress.XtraGrid.Images.GridGroupMenu.bmp", typeof(GridMenuImages).Assembly, new Size(16, 16));
				return groupPanel;
			}
		}
		[ThreadStatic]
		static ImageCollection column = null;
		public static ImageCollection Column {
			get {
				if(column == null) column = DevExpress.Utils.Controls.ImageHelper.CreateImageCollectionFromResources("DevExpress.XtraGrid.Images.GridColumnMenu.bmp", typeof(GridMenuImages).Assembly, new Size(16, 16));
				return column;
			}
		}
		[ThreadStatic]
		static ImageCollection footer = null;
		public static ImageCollection Footer {
			get {
				if(footer == null) footer = DevExpress.Utils.Controls.ImageHelper.CreateImageCollectionFromResources("DevExpress.XtraGrid.Images.GridFooterMenu.bmp", typeof(GridMenuImages).Assembly, new Size(16, 16));
				return footer;
			}
		}
	}
}
