#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System;
using System.ComponentModel;
using System.Collections;
using System.Windows.Forms;
using DevExpress.XtraEditors.Controls;
using DevExpress.Utils.Controls;
using DevExpress.Utils.Serializing;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraEditors;
using DevExpress.Utils;
namespace DevExpress.XtraGrid.Views.Grid {
	public class GridOptionsBehavior : ColumnViewOptionsBehavior {
		public static bool DefaultCopyToClipboardWithColumnHeaders = true;
		bool autoUpdateTotalSummary, 
			keepGroupExpandedOnSorting, smartVertScrollBar, allowIncrementalSearch,
			autoExpandAllGroups, allowPartialRedrawOnScrolling, summariesIgnoreNullValues,
			copyToClipboardWithColumnHeaders;
		DefaultBoolean allowFixedGroups;
		public GridOptionsBehavior(ColumnView view) : base(view) {
			this.allowFixedGroups = DefaultBoolean.Default;
			this.summariesIgnoreNullValues = false;
			this.allowPartialRedrawOnScrolling = true;
			this.autoUpdateTotalSummary = true;
			this.autoExpandAllGroups = false;
			this.keepGroupExpandedOnSorting = true;
			this.smartVertScrollBar = true;
			this.allowIncrementalSearch = false;
			this.copyToClipboardWithColumnHeaders = DefaultCopyToClipboardWithColumnHeaders;
		}
		public GridOptionsBehavior() : this(null) { }
		protected new GridView View { get { return (GridView)base.View; } }
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Obsolete(ObsoleteText.SRGridView_OptionsBehaviorAllowOnlyOneMasterRowExpanded)]
		public virtual bool AllowOnlyOneMasterRowExpanded {
			get { return View.OptionsDetail.AllowOnlyOneMasterRowExpanded; }
			set { View.OptionsDetail.AllowOnlyOneMasterRowExpanded = value; }
		}
		[Description("Gets or sets whether column captions are copied to the clipboard, when CTRL+C is pressed or the CopyToClipboard method is called."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool CopyToClipboardWithColumnHeaders {
			get { return copyToClipboardWithColumnHeaders; }
			set {
				if(CopyToClipboardWithColumnHeaders == value) return;
				bool prevValue = CopyToClipboardWithColumnHeaders;
				copyToClipboardWithColumnHeaders = value;
				OnChanged(new BaseOptionChangedEventArgs("CopyToClipboardWithColumnHeaders", prevValue, CopyToClipboardWithColumnHeaders));
			}
		}
		[Description("Gets or sets whether the top visible group row is always displayed when scrolling through grouped data."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean AllowFixedGroups {
			get { return allowFixedGroups; }
			set {
				if(AllowFixedGroups == value) return;
				DefaultBoolean prevValue = AllowFixedGroups;
				allowFixedGroups = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFixedGroups", prevValue, AllowFixedGroups));
			}
		}
		[Description("Gets or sets a value specifying whether end-users can locate rows by typing the desired column value."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowIncrementalSearch {
			get { return allowIncrementalSearch; }
			set {
				if(AllowIncrementalSearch == value) return;
				bool prevValue = AllowIncrementalSearch;
				allowIncrementalSearch = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowIncrementalSearch", prevValue, AllowIncrementalSearch));
			}
		}
		[Description("Gets or sets whether null values must be ignored when calculating summaries."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool SummariesIgnoreNullValues {
			get { return summariesIgnoreNullValues; }
			set {
				if(SummariesIgnoreNullValues == value) return;
				bool prevValue = SummariesIgnoreNullValues;
				summariesIgnoreNullValues = value;
				OnChanged(new BaseOptionChangedEventArgs("SummariesIgnoreNullValues", prevValue, SummariesIgnoreNullValues));
			}
		}
		[Description("Gets or sets whether a View is partially repainted when it's scrolled vertically by a row."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowPartialRedrawOnScrolling {
			get { return allowPartialRedrawOnScrolling; }
			set {
				if(AllowPartialRedrawOnScrolling == value) return;
				bool prevValue = AllowPartialRedrawOnScrolling;
				allowPartialRedrawOnScrolling = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowPartialRedrawOnScrolling", prevValue, AllowPartialRedrawOnScrolling));
			}
		}
		[Description("Gets or sets a value specifying whether total summaries are automatically updated when changing column values."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AutoUpdateTotalSummary {
			get { return autoUpdateTotalSummary; }
			set {
				if(AutoUpdateTotalSummary == value) return;
				bool prevValue = AutoUpdateTotalSummary;
				autoUpdateTotalSummary = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoUpdateTotalSummary", prevValue, AutoUpdateTotalSummary));
			}
		}
		internal const string AutoExpandAllGroupsName = "AutoExpandAllGroups";
		[Description("Gets or sets whether all the groups within the View are automatically expanded."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AutoExpandAllGroups {
			get { return autoExpandAllGroups; }
			set {
				if(AutoExpandAllGroups == value) return;
				bool prevValue = AutoExpandAllGroups;
				autoExpandAllGroups = value;
				OnChanged(new BaseOptionChangedEventArgs(AutoExpandAllGroupsName, prevValue, AutoExpandAllGroups));
			}
		}
		[Description("Gets or sets a value specifying whether the group rows' expanded state is preserved when applying data sorting."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool KeepGroupExpandedOnSorting {
			get { return keepGroupExpandedOnSorting; }
			set {
				if(KeepGroupExpandedOnSorting == value) return;
				bool prevValue = KeepGroupExpandedOnSorting;
				keepGroupExpandedOnSorting = value;
				OnChanged(new BaseOptionChangedEventArgs("KeepGroupExpandedOnSorting", prevValue, KeepGroupExpandedOnSorting));
			}
		}
		[Description("Gets or sets a value specifying whether scrolling the View to the bottom places the last row to the top of the View. "), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool SmartVertScrollBar {
			get { return smartVertScrollBar; }
			set {
				if(SmartVertScrollBar == value) return;
				bool prevValue = SmartVertScrollBar;
				smartVertScrollBar = value;
				OnChanged(new BaseOptionChangedEventArgs("SmartVertScrollBar", prevValue, SmartVertScrollBar));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsBehavior opt = options as GridOptionsBehavior;
				if(opt == null) return;
				this.summariesIgnoreNullValues = opt.SummariesIgnoreNullValues;
				this.autoExpandAllGroups = opt.AutoExpandAllGroups;
				this.allowIncrementalSearch = opt.AllowIncrementalSearch;
				this.autoUpdateTotalSummary = opt.AutoUpdateTotalSummary;
				this.keepGroupExpandedOnSorting = opt.KeepGroupExpandedOnSorting;
				this.smartVertScrollBar = opt.SmartVertScrollBar;
				this.allowPartialRedrawOnScrolling = opt.AllowPartialRedrawOnScrolling;
				this.copyToClipboardWithColumnHeaders = opt.CopyToClipboardWithColumnHeaders;
				this.allowFixedGroups = opt.AllowFixedGroups;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public enum DetailExpandButtonMode { Default, AlwaysEnabled, CheckDefaultDetail, CheckAllDetails }
	public class GridOptionsDetail : ViewBaseOptions {
		bool allowExpandEmptyDetails, allowZoomDetail, enableMasterViewMode, autoZoomDetail, 
			enableDetailToolTip, showDetailTabs, smartDetailHeight, smartDetailExpand, allowOnlyOneMasterRowExpanded;
		DetailExpandButtonMode smartDetailExpandButtonMode;
		public GridOptionsDetail() {
			this.smartDetailExpand = true;
			this.enableMasterViewMode = true;
			this.smartDetailExpandButtonMode = DetailExpandButtonMode.Default;
			this.allowExpandEmptyDetails = false;
			this.allowZoomDetail = true;
			this.autoZoomDetail = false;
			this.enableDetailToolTip = false;
			this.showDetailTabs = true;
			this.smartDetailHeight = false;
			this.allowOnlyOneMasterRowExpanded = false;
		}
		[Description("Gets or sets a value specifying whether several master rows can be expanded simultaneously."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowOnlyOneMasterRowExpanded {
			get { return allowOnlyOneMasterRowExpanded; }
			set {
				if(AllowOnlyOneMasterRowExpanded == value) return;
				bool prevValue = AllowOnlyOneMasterRowExpanded;
				allowOnlyOneMasterRowExpanded = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowOnlyOneMasterRowExpanded", prevValue, AllowOnlyOneMasterRowExpanded));
			}
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), EditorBrowsable(EditorBrowsableState.Never), Obsolete(ObsoleteText.SRGridOptionsDetail_SmartDetailExpandButton)]
		public virtual bool SmartDetailExpandButton {
			get { return GetSmartDetailExpandButtonMode() != DetailExpandButtonMode.AlwaysEnabled; }
			set {
				if(SmartDetailExpandButton == value) return;
				SmartDetailExpandButtonMode = value ? DetailExpandButtonMode.Default : DetailExpandButtonMode.CheckDefaultDetail;
			}
		}
		protected internal virtual DetailExpandButtonMode GetSmartDetailExpandButtonMode() {
			DetailExpandButtonMode res = SmartDetailExpandButtonMode;
			if(res == DetailExpandButtonMode.Default) return DetailExpandButtonMode.CheckDefaultDetail;
			return res;
		}
		[Description("Gets or sets a  value which specifies the manner in which master-detail expand buttons are painted."), DefaultValue(DetailExpandButtonMode.Default), XtraSerializableProperty()]
		public virtual DetailExpandButtonMode SmartDetailExpandButtonMode {
			get { return smartDetailExpandButtonMode; }
			set {
				if(SmartDetailExpandButtonMode == value) return;
				DetailExpandButtonMode prevValue = SmartDetailExpandButtonMode;
				smartDetailExpandButtonMode = value;
				OnChanged(new BaseOptionChangedEventArgs("SmartDetailExpandButtonMode", prevValue, SmartDetailExpandButtonMode));
			}
		}
		[Description("Gets or sets a value specifying whether a View is automatically scrolled when one of its detail Views is expanded, so all rows in that detail View will be displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool SmartDetailExpand {
			get { return smartDetailExpand; }
			set {
				if(SmartDetailExpand == value) return;
				bool prevValue = SmartDetailExpand;
				smartDetailExpand = value;
				OnChanged(new BaseOptionChangedEventArgs("SmartDetailExpand", prevValue, SmartDetailExpand));
			}
		}
		[Description("Gets or sets a value specifying whether master rows can be expanded when details have no data."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowExpandEmptyDetails {
			get { return allowExpandEmptyDetails; }
			set {
				if(AllowExpandEmptyDetails == value) return;
				bool prevValue = AllowExpandEmptyDetails;
				allowExpandEmptyDetails = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowExpandEmptyDetails", prevValue, AllowExpandEmptyDetails));
			}
		}
		[Description("Gets or sets a value specifying whether the View's immediate details can be maximized to fit the entire grid control's area."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowZoomDetail {
			get { return allowZoomDetail; }
			set {
				if(AllowZoomDetail == value) return;
				bool prevValue = AllowZoomDetail;
				allowZoomDetail = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowZoomDetail", prevValue, AllowZoomDetail));
			}
		}
		[Description("Gets or sets a value specifying whether details are automatically maximized to fit the entire grid control's area when corresponding master rows are expanded."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AutoZoomDetail {
			get { return autoZoomDetail; }
			set {
				if(AutoZoomDetail == value) return;
				bool prevValue = AutoZoomDetail;
				autoZoomDetail = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoZoomDetail", prevValue, AutoZoomDetail));
			}
		}
		[Description("Gets or sets a value specifying whether the View's details can be displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableMasterViewMode {
			get { return enableMasterViewMode; }
			set {
				if(EnableMasterViewMode == value) return;
				bool prevValue = EnableMasterViewMode;
				enableMasterViewMode = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableMasterViewMode", prevValue, EnableMasterViewMode));
			}
		}
		[Description("Gets or sets a value specifying whether detail tooltips are enabled for detail Views that are opened via the current master View. This option must be applied to a master View."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnableDetailToolTip {
			get { return enableDetailToolTip; }
			set {
				if(EnableDetailToolTip == value) return;
				bool prevValue = EnableDetailToolTip;
				enableDetailToolTip = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableDetailToolTip", prevValue, EnableDetailToolTip));
			}
		}
		[Description("Gets or sets whether detail tabs are displayed in detail sections when master rows of the current master View are expanded. This option must be applied to a master View."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowDetailTabs {
			get { return showDetailTabs; }
			set {
				if(ShowDetailTabs == value) return;
				bool prevValue = ShowDetailTabs;
				showDetailTabs = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowDetailTabs", prevValue, ShowDetailTabs));
			}
		}
		[Description("Reserved for future use."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool SmartDetailHeight {
			get { return smartDetailHeight; }
			set {
				if(SmartDetailHeight == value) return;
				bool prevValue = SmartDetailHeight;
				smartDetailHeight = value;
				OnChanged(new BaseOptionChangedEventArgs("SmartDetailHeight", prevValue, SmartDetailHeight));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsDetail opt = options as GridOptionsDetail;
				if(opt == null) return;
				this.allowOnlyOneMasterRowExpanded = opt.AllowOnlyOneMasterRowExpanded;
				this.smartDetailExpandButtonMode = opt.SmartDetailExpandButtonMode;
				this.allowExpandEmptyDetails = opt.AllowExpandEmptyDetails;
				this.allowZoomDetail = opt.AllowZoomDetail;
				this.autoZoomDetail = opt.AutoZoomDetail;
				this.enableDetailToolTip = opt.EnableDetailToolTip;
				this.enableMasterViewMode = opt.EnableMasterViewMode;
				this.showDetailTabs = opt.ShowDetailTabs;
				this.smartDetailHeight = opt.SmartDetailHeight;
				this.smartDetailExpand = opt.SmartDetailExpand;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsCustomization : ViewBaseOptions {
		bool allowFilter, allowGroup, allowRowSizing, allowSort, allowColumnMoving, allowColumnResizing, allowQuickHideColumns;
		public GridOptionsCustomization() {
			this.allowFilter = true;
			this.allowGroup = true;
			this.allowRowSizing = false;
			this.allowSort = true;
			this.allowColumnMoving = true;
			this.allowColumnResizing = true;
			this.allowQuickHideColumns = true;
		}
		[Description("Gets or sets a value which specifies whether end-users can filter data using the filter dropdowns."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowFilter {
			get { return allowFilter; }
			set {
				if(AllowFilter == value) return;
				bool prevValue = AllowFilter;
				allowFilter = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowFilter", prevValue, AllowFilter));
			}
		}
		[Description("Gets or sets whether a column/band can be hidden by dragging it away from the Column Header Panel, even if the Customization Form is not open."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowQuickHideColumns {
			get { return allowQuickHideColumns; }
			set {
				if(AllowQuickHideColumns == value) return;
				bool prevValue = AllowQuickHideColumns;
				allowQuickHideColumns = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowQuickHideColumns", prevValue, AllowQuickHideColumns));
			}
		}
		[Description("Gets or sets whether end-users are allowed to change the width of grid columns."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowColumnResizing {
			get { return allowColumnResizing; }
			set {
				if(AllowColumnResizing == value) return;
				bool prevValue = AllowColumnResizing;
				allowColumnResizing = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowColumnResizing", prevValue, AllowColumnResizing));
			}
		}
		[Description("Gets or sets whether end-users are allowed to move columns by dragging their headers."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowColumnMoving {
			get { return allowColumnMoving; }
			set {
				if(AllowColumnMoving == value) return;
				bool prevValue = AllowColumnMoving;
				allowColumnMoving = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowColumnMoving", prevValue, AllowColumnMoving));
			}
		}
		[Description("Gets or sets a value specifying whether end-users can apply data grouping."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowGroup {
			get { return allowGroup; }
			set {
				if(AllowGroup == value) return;
				bool prevValue = AllowGroup;
				allowGroup = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowGroup", prevValue, AllowGroup));
			}
		}
		[Description("Gets or sets a value specifying whether end-users can resize data rows vertically."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowRowSizing {
			get { return allowRowSizing; }
			set {
				if(AllowRowSizing == value) return;
				bool prevValue = AllowRowSizing;
				allowRowSizing = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowRowSizing", prevValue, AllowRowSizing));
			}
		}
		[Description("Gets or sets a value specifying whether end-users can apply data sorting."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AllowSort {
			get { return allowSort; }
			set {
				if(AllowSort == value) return;
				bool prevValue = AllowSort;
				allowSort = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowSort", prevValue, AllowSort));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsCustomization opt = options as GridOptionsCustomization;
				if(opt == null) return;
				this.allowColumnResizing = opt.AllowColumnResizing;
				this.allowFilter = opt.AllowFilter;
				this.allowGroup = opt.AllowGroup;
				this.allowRowSizing = opt.AllowRowSizing;
				this.allowSort = opt.AllowSort;
				this.allowColumnMoving = opt.AllowColumnMoving;
				this.allowQuickHideColumns = opt.AllowQuickHideColumns;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsNavigation : ViewBaseOptions {
		bool autoFocusNewRow, autoMoveRowFocus, enterMoveNextColumn, useOfficePageNavigation,
			useTabKey;
		public GridOptionsNavigation() {
			this.useOfficePageNavigation = true;
			this.autoFocusNewRow = false;
			this.autoMoveRowFocus = true;
			this.enterMoveNextColumn = false;
			this.useTabKey = true;
		}
		[Description("Enables the Office 2007 navigation mode, where pressing the PAGE DOWN key moves focus to the bottom visible record."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool UseOfficePageNavigation {
			get { return useOfficePageNavigation; }
			set {
				if(UseOfficePageNavigation == value) return;
				bool prevValue = UseOfficePageNavigation;
				useOfficePageNavigation = value;
				OnChanged(new BaseOptionChangedEventArgs("UseOfficePageNavigation", prevValue, UseOfficePageNavigation));
			}
		}
		[Description("Gets or sets whether adding a new record to an underlying data source automatically moves focus to the corresponding grid row."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AutoFocusNewRow {
			get { return autoFocusNewRow; }
			set {
				if(AutoFocusNewRow == value) return;
				bool prevValue = AutoFocusNewRow;
				autoFocusNewRow = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoFocusNewRow", prevValue, AutoFocusNewRow));
			}
		}
		[Description("Gets or sets a value specifying whether horizontal navigation keys move focus to the next/previous row when the current row's last/first cell is focused."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AutoMoveRowFocus {
			get { return autoMoveRowFocus; }
			set {
				if(AutoMoveRowFocus == value) return;
				bool prevValue = AutoMoveRowFocus;
				autoMoveRowFocus = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoMoveRowFocus", prevValue, AutoMoveRowFocus));
			}
		}
		[Description("Gets or sets a value specifying whether the ENTER key can be used to move cell focus."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnterMoveNextColumn {
			get { return enterMoveNextColumn; }
			set {
				if(EnterMoveNextColumn == value) return;
				bool prevValue = EnterMoveNextColumn;
				enterMoveNextColumn = value;
				OnChanged(new BaseOptionChangedEventArgs("EnterMoveNextColumn", prevValue, EnterMoveNextColumn));
			}
		}
		[Description("Gets or sets a value specifying whether the TAB/SHIFT+TAB key combinations move focus to the next/previous row cell or to the next/previous control in tab order."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool UseTabKey {
			get { return useTabKey; }
			set {
				if(UseTabKey == value) return;
				bool prevValue = UseTabKey;
				useTabKey = value;
				OnChanged(new BaseOptionChangedEventArgs("UseTabKey", prevValue, UseTabKey));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsNavigation opt = options as GridOptionsNavigation;
				if(opt == null) return;
				this.autoFocusNewRow = opt.AutoFocusNewRow;
				this.autoMoveRowFocus = opt.AutoMoveRowFocus;
				this.enterMoveNextColumn = opt.EnterMoveNextColumn;
				this.useTabKey = opt.UseTabKey;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public enum GridMultiSelectMode { RowSelect, CellSelect }
	public class GridOptionsSelection : ColumnViewOptionsSelection {
		bool invertSelection, enableAppearanceFocusedRow, enableAppearanceFocusedCell, enableAppearanceHideSelection, useIndicatorForSelection;
		GridMultiSelectMode multiSelectMode;
		public GridOptionsSelection() {
			this.multiSelectMode = GridMultiSelectMode.RowSelect;
			this.invertSelection = false;
			this.enableAppearanceHideSelection = true;
			this.enableAppearanceFocusedRow = true;
			this.enableAppearanceFocusedCell = true;
			this.useIndicatorForSelection = true;
		}
		[Description("Gets or sets whether rows can be selected via the Row Indicator Panel when the grid is in multiple selection mode."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool UseIndicatorForSelection {
			get { return useIndicatorForSelection; }
			set {
				if(UseIndicatorForSelection == value) return;
				bool prevValue = useIndicatorForSelection;
				useIndicatorForSelection = value;
				OnChanged(new BaseOptionChangedEventArgs("UseIndicatorForSelection", prevValue, UseIndicatorForSelection));
			}
		}
		[Description("Gets or sets whether multiple cells or rows can be selected."), DefaultValue(GridMultiSelectMode.RowSelect), XtraSerializableProperty()]
		public virtual GridMultiSelectMode MultiSelectMode {
			get { return multiSelectMode; }
			set {
				if(MultiSelectMode == value) return;
				GridMultiSelectMode prevValue = multiSelectMode;
				multiSelectMode = value;
				OnChanged(new BaseOptionChangedEventArgs("MultiSelectMode", prevValue, MultiSelectMode));
			}
		}
		[Description("Gets or sets whether the appearance settings for focused cell are enabled."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableAppearanceFocusedCell {
			get { return enableAppearanceFocusedCell; }
			set {
				if(EnableAppearanceFocusedCell == value) return;
				bool prevValue = EnableAppearanceFocusedCell;
				enableAppearanceFocusedCell = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceFocusedCell", prevValue, EnableAppearanceFocusedCell));
			}
		}
		[Description("Gets or sets whether the appearance settings for focused row are enabled."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableAppearanceFocusedRow {
			get { return enableAppearanceFocusedRow; }
			set {
				if(EnableAppearanceFocusedRow == value) return;
				bool prevValue = EnableAppearanceFocusedRow;
				enableAppearanceFocusedRow = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceFocusedRow", prevValue, EnableAppearanceFocusedRow));
			}
		}
		[Description("Gets or sets whether the appearance settings used to paint the selected row(s) when the grid control is not focused are enabled."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableAppearanceHideSelection {
			get { return enableAppearanceHideSelection; }
			set {
				if(EnableAppearanceHideSelection == value) return;
				bool prevValue = EnableAppearanceHideSelection;
				enableAppearanceHideSelection = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceHideSelection", prevValue, EnableAppearanceHideSelection));
			}
		}
		[Description("Gets or sets a value specifying whether the focused style is applied to the focused cell only or to all but the focused row cell."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool InvertSelection {
			get { return invertSelection; }
			set {
				if(InvertSelection == value) return;
				bool prevValue = InvertSelection;
				invertSelection = value;
				OnChanged(new BaseOptionChangedEventArgs("InvertSelection", prevValue, InvertSelection));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsSelection opt = options as GridOptionsSelection;
				if(opt == null) return;
				this.enableAppearanceHideSelection = opt.EnableAppearanceHideSelection;
				this.multiSelectMode = opt.MultiSelectMode;
				this.useIndicatorForSelection = opt.UseIndicatorForSelection;
				this.invertSelection = opt.InvertSelection;
				this.enableAppearanceFocusedCell = opt.EnableAppearanceFocusedCell;
				this.enableAppearanceFocusedRow = opt.EnableAppearanceFocusedRow;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsView : ColumnViewOptionsView {
		bool autoCalcPreviewLineCount, allowCellMerge, columnAutoWidth, rowAutoHeight, 
			showChildrenInGroupPanel, showColumnHeaders, showDetailButtons, showGroupExpandCollapseButtons, 
			showFooter, showGroupedColumns, showGroupPanel, 
			showHorzLines, showIndicator, showPreview, 
			showVertLines, showAutoFilterRow, enableAppearanceEvenRow, enableAppearanceOddRow, showPreviewLines, allowHtmlDrawHeaders;
		NewItemRowPosition newItemRowPosition;
		GroupDrawMode groupDrawMode;
		FilterButtonShowMode headerFilterButtonShowMode;
		WaitAnimationOptions waitAnimationOptions;
		public GridOptionsView() {
			this.waitAnimationOptions = WaitAnimationOptions.Default;
			this.showGroupExpandCollapseButtons = true;
			this.allowHtmlDrawHeaders = false;
			this.headerFilterButtonShowMode = FilterButtonShowMode.Default;
			this.enableAppearanceEvenRow = this.enableAppearanceOddRow = false;
			this.allowCellMerge = false;
			this.groupDrawMode = GroupDrawMode.Default;
			this.newItemRowPosition = NewItemRowPosition.None;
			this.autoCalcPreviewLineCount = false;
			this.columnAutoWidth = true;
			this.rowAutoHeight = false;
			this.showChildrenInGroupPanel = false;
			this.showColumnHeaders = true;
			this.showDetailButtons = true;
			this.showAutoFilterRow = false;
			this.showFooter = false;
			this.showGroupedColumns = false;
			this.showGroupPanel = true;
			this.showHorzLines = true;
			this.showPreviewLines = true;
			this.showIndicator = true;
			this.showPreview = false;
			this.showVertLines = true;
		}
		[Description("Gets or sets the display style of column filter buttons."), DefaultValue(FilterButtonShowMode.Default), XtraSerializableProperty()]
		public virtual FilterButtonShowMode HeaderFilterButtonShowMode {
			get { return headerFilterButtonShowMode; }
			set {
				if(HeaderFilterButtonShowMode == value) return;
				FilterButtonShowMode prevValue = HeaderFilterButtonShowMode;
				headerFilterButtonShowMode = value;
				OnChanged(new BaseOptionChangedEventArgs("HeaderFilterButtonShowMode", prevValue, HeaderFilterButtonShowMode));
			}
		}
		protected internal virtual FilterButtonShowMode GetHeaderFilterButtonShowMode() {
			return HeaderFilterButtonShowMode == FilterButtonShowMode.Default ? FilterButtonShowMode.SmartTag : HeaderFilterButtonShowMode;
		}
		[Description("Gets or sets a value specifying how group rows should be painted."), DefaultValue(GroupDrawMode.Default), XtraSerializableProperty()]
		public virtual GroupDrawMode GroupDrawMode {
			get { return groupDrawMode; }
			set {
				if(GroupDrawMode == value) return;
				DevExpress.XtraGrid.Views.Grid.GroupDrawMode prevValue = GroupDrawMode;
				groupDrawMode = value;
				OnChanged(new BaseOptionChangedEventArgs("GroupDrawMode", prevValue, GroupDrawMode));
			}
		}
		[Description("Gets or sets how async data loading operations are indicated. This property is in effect in Instant Feedback Mode."), DefaultValue(WaitAnimationOptions.Default), XtraSerializableProperty()]
		public virtual WaitAnimationOptions WaitAnimationOptions {
			get { return waitAnimationOptions; }
			set {
				if(WaitAnimationOptions == value) return;
				WaitAnimationOptions prevValue = WaitAnimationOptions;
				waitAnimationOptions = value;
				OnChanged(new BaseOptionChangedEventArgs("WaitAnimationOptions", prevValue, WaitAnimationOptions));
			}
		}
		[Description("Gets or sets the new item row's position within the View."), DefaultValue(NewItemRowPosition.None), XtraSerializableProperty()]
		public virtual NewItemRowPosition NewItemRowPosition {
			get { return newItemRowPosition; }
			set {
				if(NewItemRowPosition == value) return;
				DevExpress.XtraGrid.Views.Grid.NewItemRowPosition prevValue = NewItemRowPosition;
				newItemRowPosition = value;
				OnChanged(new BaseOptionChangedEventArgs("NewItemRowPosition", prevValue, NewItemRowPosition));
			}
		}
		[Description("Gets or sets a value specifying whether the number of text lines within preview sections is calculated automatically depending on their contents."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AutoCalcPreviewLineCount {
			get { return autoCalcPreviewLineCount; }
			set {
				if(AutoCalcPreviewLineCount == value) return;
				bool prevValue = AutoCalcPreviewLineCount;
				autoCalcPreviewLineCount = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoCalcPreviewLineCount", prevValue, AutoCalcPreviewLineCount));
			}
		}
		[Description("Gets or sets whether HTML formatting can be applied to column and band captions."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowHtmlDrawHeaders {
			get { return allowHtmlDrawHeaders; }
			set {
				if(AllowHtmlDrawHeaders == value) return;
				bool prevValue = AllowHtmlDrawHeaders;
				allowHtmlDrawHeaders = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowHtmlDrawHeaders", prevValue, AllowHtmlDrawHeaders));
			}
		}
		[Description("Gets or sets a value specifying whether column widths are automatically modified so that the total columns' width matches the View's width."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ColumnAutoWidth {
			get { return columnAutoWidth; }
			set {
				if(ColumnAutoWidth == value) return;
				bool prevValue = ColumnAutoWidth;
				columnAutoWidth = value;
				OnChanged(new BaseOptionChangedEventArgs("ColumnAutoWidth", prevValue, ColumnAutoWidth));
			}
		}
		[Description("Gets or sets whether neighboring cells with identical values can merge."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool AllowCellMerge {
			get { return allowCellMerge; }
			set {
				if(AllowCellMerge == value) return;
				bool prevValue = AllowCellMerge;
				allowCellMerge = value;
				OnChanged(new BaseOptionChangedEventArgs("AllowCellMerge", prevValue, AllowCellMerge));
			}
		}
		[Description("Gets or sets a value specifying whether the height of each data row is automatically adjusted to completely display the contents of its cells."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool RowAutoHeight {
			get { return rowAutoHeight; }
			set {
				if(RowAutoHeight == value) return;
				bool prevValue = RowAutoHeight;
				rowAutoHeight = value;
				OnChanged(new BaseOptionChangedEventArgs("RowAutoHeight", prevValue, RowAutoHeight));
			}
		}
		[Description("Gets or sets a value specifying whether the View's group panel acts as a joined group panel."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowChildrenInGroupPanel {
			get { return showChildrenInGroupPanel; }
			set {
				if(ShowChildrenInGroupPanel == value) return;
				bool prevValue = ShowChildrenInGroupPanel;
				showChildrenInGroupPanel = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowChildrenInGroupPanel", prevValue, ShowChildrenInGroupPanel));
			}
		}
		[Description("Gets or sets a value specifying whether column headers are displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowColumnHeaders {
			get { return showColumnHeaders; }
			set {
				if(ShowColumnHeaders == value) return;
				bool prevValue = ShowColumnHeaders;
				showColumnHeaders = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowColumnHeaders", prevValue, ShowColumnHeaders));
			}
		}
		[Description("Gets or sets whether the auto filter row is displayed."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowAutoFilterRow {
			get { return showAutoFilterRow; }
			set {
				if(ShowAutoFilterRow == value) return;
				bool prevValue = ShowAutoFilterRow;
				showAutoFilterRow = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowAutoFilterRow", prevValue, ShowAutoFilterRow));
			}
		}
		[Description("Gets or sets a value specifying whether master rows display master-detail expand buttons."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowDetailButtons {
			get { return showDetailButtons; }
			set {
				if(ShowDetailButtons == value) return;
				bool prevValue = ShowDetailButtons;
				showDetailButtons = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowDetailButtons", prevValue, ShowDetailButtons));
			}
		}
		[Description("Gets or sets whether group expand buttons are visible."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowGroupExpandCollapseButtons {
			get { return showGroupExpandCollapseButtons; }
			set {
				if(ShowGroupExpandCollapseButtons == value) return;
				bool prevValue = ShowGroupExpandCollapseButtons;
				showGroupExpandCollapseButtons = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowGroupExpandCollapseButtons", prevValue, ShowGroupExpandCollapseButtons));
			}
		}
		[Description("Gets or sets a value specifying whether the view footer is displayed."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowFooter {
			get { return showFooter; }
			set {
				if(ShowFooter == value) return;
				bool prevValue = ShowFooter;
				showFooter = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowFooter", prevValue, ShowFooter));
			}
		}
		[Description("Gets or sets a value specifying whether grouping columns are displayed within the View."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowGroupedColumns {
			get { return showGroupedColumns; }
			set {
				if(ShowGroupedColumns == value) return;
				bool prevValue = ShowGroupedColumns;
				showGroupedColumns = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowGroupedColumns", prevValue, ShowGroupedColumns));
			}
		}
		[Description("Gets or sets a value specifying whether the group panel is displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowGroupPanel {
			get { return showGroupPanel; }
			set {
				if(ShowGroupPanel == value) return;
				bool prevValue = ShowGroupPanel;
				showGroupPanel = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowGroupPanel", prevValue, ShowGroupPanel));
			}
		}
		[Description("Gets or sets a value specifying whether horizontal grid lines are displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowHorzLines {
			get { return showHorzLines; }
			set {
				if(ShowHorzLines == value) return;
				bool prevValue = ShowHorzLines;
				showHorzLines = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowHorzLines", prevValue, ShowHorzLines));
			}
		}
		[Description("Gets or sets whether preview lines are displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowPreviewLines {
			get { return showPreviewLines; }
			set {
				if(ShowPreviewLines == value) return;
				bool prevValue = ShowPreviewLines;
				showPreviewLines = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowPreviewLines", prevValue, ShowPreviewLines));
			}
		}
		[Description("Gets or sets a value specifying whether the row indicator panel is displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowIndicator {
			get { return showIndicator; }
			set {
				if(ShowIndicator == value) return;
				bool prevValue = ShowIndicator;
				showIndicator = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowIndicator", prevValue, ShowIndicator));
			}
		}
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), EditorBrowsable(EditorBrowsableState.Never), Obsolete(ObsoleteText.SRGridOptionsView_ShowNewItemRow)]
		public bool ShowNewItemRow {
			get { return NewItemRowPosition != DevExpress.XtraGrid.Views.Grid.NewItemRowPosition.None; }
			set {
				NewItemRowPosition = (value ? DevExpress.XtraGrid.Views.Grid.NewItemRowPosition.Bottom : DevExpress.XtraGrid.Views.Grid.NewItemRowPosition.None);
			}
		}
		[Description("Gets or sets a value specifying whether preview sections are displayed."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowPreview {
			get { return showPreview; }
			set {
				if(ShowPreview == value) return;
				bool prevValue = ShowPreview;
				showPreview = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowPreview", prevValue, ShowPreview));
			}
		}
		[Description("Gets or sets a value specifying whether vertical grid lines are displayed."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowVertLines {
			get { return showVertLines; }
			set {
				if(ShowVertLines == value) return;
				bool prevValue = ShowVertLines;
				showVertLines = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowVertLines", prevValue, ShowVertLines));
			}
		}
		[Description("Gets or sets whether the odd rows are painted using the appearance settings provided by the GridViewAppearances.OddRow property."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnableAppearanceOddRow {
			get { return enableAppearanceOddRow; }
			set {
				if(EnableAppearanceOddRow == value) return;
				bool prevValue = EnableAppearanceOddRow;
				enableAppearanceOddRow = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceOddRow", prevValue, EnableAppearanceOddRow));
			}
		}
		[Description("Gets or sets whether the even rows are painted using the appearance settings provided by the GridViewAppearances.EvenRow property."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnableAppearanceEvenRow {
			get { return enableAppearanceEvenRow; }
			set {
				if(EnableAppearanceEvenRow == value) return;
				bool prevValue = EnableAppearanceEvenRow;
				enableAppearanceEvenRow = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceEvenRow", prevValue, EnableAppearanceEvenRow));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsView opt = options as GridOptionsView;
				if(opt == null) return;
				this.showGroupExpandCollapseButtons = opt.ShowGroupExpandCollapseButtons;
				this.headerFilterButtonShowMode = opt.HeaderFilterButtonShowMode;
				this.enableAppearanceEvenRow = opt.EnableAppearanceEvenRow;
				this.enableAppearanceOddRow = opt.EnableAppearanceOddRow;
				this.allowCellMerge = opt.AllowCellMerge;
				this.showAutoFilterRow = opt.ShowAutoFilterRow;
				this.autoCalcPreviewLineCount = opt.AutoCalcPreviewLineCount;
				this.columnAutoWidth = opt.ColumnAutoWidth;
				this.rowAutoHeight = opt.RowAutoHeight;
				this.showChildrenInGroupPanel = opt.ShowChildrenInGroupPanel;
				this.showColumnHeaders = opt.ShowColumnHeaders;
				this.showDetailButtons = opt.ShowDetailButtons;
				this.showFooter = opt.ShowFooter;
				this.showGroupedColumns = opt.ShowGroupedColumns;
				this.showGroupPanel = opt.ShowGroupPanel;
				this.showHorzLines = opt.ShowHorzLines;
				this.showPreviewLines = opt.ShowPreviewLines;
				this.showIndicator = opt.ShowIndicator;
				this.showPreview = opt.ShowPreview;
				this.showVertLines = opt.ShowVertLines;
				this.newItemRowPosition = opt.NewItemRowPosition;
				this.groupDrawMode = opt.GroupDrawMode;
				this.waitAnimationOptions = opt.WaitAnimationOptions;
				this.allowHtmlDrawHeaders = opt.AllowHtmlDrawHeaders;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsPrint : ViewPrintOptionsBase {
		bool autoWidth, expandAllDetails, expandAllGroups, printDetails, 
			printFilterInfo, printFooter, printGroupFooter, printHeader, 
			printHorzLines, printPreview, printVertLines, usePrintStyles,
			enableAppearanceEvenRow, enableAppearanceOddRow, splitCellPreviewAcrossPages,
			printSelectedRowsOnly;
		int maxMergedCellHeight = 600;
		public GridOptionsPrint() {
			this.printSelectedRowsOnly = false;
			this.enableAppearanceEvenRow = false;
			this.enableAppearanceOddRow = false;
			this.autoWidth = true;
			this.expandAllDetails = false;
			this.expandAllGroups = true;
			this.printDetails = false;
			this.printFilterInfo = false;
			this.printFooter = true;
			this.printGroupFooter = true;
			this.printHeader = true;
			this.printHorzLines = true;
			this.printPreview = false;
			this.printVertLines = true;
			this.usePrintStyles = false;
			this.splitCellPreviewAcrossPages = false;
		}
		[DefaultValue(600), XtraSerializableProperty()]
		public virtual int MaxMergedCellHeight {
			get { return maxMergedCellHeight; }
			set {
				if(MaxMergedCellHeight == value) return;
				int prevValue = maxMergedCellHeight;
				maxMergedCellHeight = value;
				OnChanged(new BaseOptionChangedEventArgs("MaxMergedCellHeight", prevValue, MaxMergedCellHeight));
			}
		}
		[Description("Gets or sets whether preview sections are allowed to be split across pages when printing/exporting."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool SplitCellPreviewAcrossPages {
			get { return splitCellPreviewAcrossPages; }
			set {
				if(SplitCellPreviewAcrossPages == value) return;
				bool prevValue = splitCellPreviewAcrossPages;
				splitCellPreviewAcrossPages = value;
				OnChanged(new BaseOptionChangedEventArgs("SplitCellPreviewAcrossPages", prevValue, SplitCellPreviewAcrossPages));
			}
		}
		[Description("Gets or sets whether odd rows in the print/export output are painted using the appearance settings provided by the GridViewPrintAppearances.OddRow property.This option is in effect when the GridOptionsPrint.UsePrintStyles property is set to true."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnableAppearanceOddRow {
			get { return enableAppearanceOddRow; }
			set {
				if(EnableAppearanceOddRow == value) return;
				bool prevValue = EnableAppearanceOddRow;
				enableAppearanceOddRow = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceOddRow", prevValue, EnableAppearanceOddRow));
			}
		}
		[Description("Gets or sets whether even rows in the print/export output are painted using the appearance settings provided by the GridViewPrintAppearances.EvenRow property.This option is in effect when the GridOptionsPrint.UsePrintStyles property is set to true."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool EnableAppearanceEvenRow {
			get { return enableAppearanceEvenRow; }
			set {
				if(EnableAppearanceEvenRow == value) return;
				bool prevValue = EnableAppearanceEvenRow;
				enableAppearanceEvenRow = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableAppearanceEvenRow", prevValue, EnableAppearanceEvenRow));
			}
		}
		[Description("Gets or sets whether widths of columns in the print/export output are automatically changed so that the View fits the page width."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool AutoWidth {
			get { return autoWidth; }
			set {
				if(AutoWidth == value) return;
				bool prevValue = AutoWidth;
				autoWidth = value;
				OnChanged(new BaseOptionChangedEventArgs("AutoWidth", prevValue, AutoWidth));
			}
		}
		[Description("Gets or sets whether the View is printed/exported with all master rows expanded."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ExpandAllDetails {
			get { return expandAllDetails; }
			set {
				if(ExpandAllDetails == value) return;
				bool prevValue = ExpandAllDetails;
				expandAllDetails = value;
				OnChanged(new BaseOptionChangedEventArgs("ExpandAllDetails", prevValue, ExpandAllDetails));
			}
		}
		[Description("Gets or sets whether the View is printed/exported with all group rows expanded."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ExpandAllGroups {
			get { return expandAllGroups; }
			set {
				if(ExpandAllGroups == value) return;
				bool prevValue = ExpandAllGroups;
				expandAllGroups = value;
				OnChanged(new BaseOptionChangedEventArgs("ExpandAllGroups", prevValue, ExpandAllGroups));
			}
		}
		[Description("Gets or sets to print/export details of master rows."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool PrintDetails {
			get { return printDetails; }
			set {
				if(PrintDetails == value) return;
				bool prevValue = PrintDetails;
				printDetails = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintDetails", prevValue, PrintDetails));
			}
		}
		[Description("Gets or sets whether to display the filter panel in the print/export output."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool PrintFilterInfo {
			get { return printFilterInfo; }
			set {
				if(PrintFilterInfo == value) return;
				bool prevValue = PrintFilterInfo;
				printFilterInfo = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintFilterInfo", prevValue, PrintFilterInfo));
			}
		}
		[Description("Gets or sets whether to display the view footer in the print/export output."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool PrintFooter {
			get { return printFooter; }
			set {
				if(PrintFooter == value) return;
				bool prevValue = PrintFooter;
				printFooter = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintFooter", prevValue, PrintFooter));
			}
		}
		[Description("Gets or sets whether to display group footers in the print/export output."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool PrintGroupFooter {
			get { return printGroupFooter; }
			set {
				if(PrintGroupFooter == value) return;
				bool prevValue = PrintGroupFooter;
				printGroupFooter = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintGroupFooter", prevValue, PrintGroupFooter));
			}
		}
		[Description("Gets or sets whether to display column headers in the print/export output."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool PrintHeader {
			get { return printHeader; }
			set {
				if(PrintHeader == value) return;
				bool prevValue = PrintHeader;
				printHeader = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintHeader", prevValue, PrintHeader));
			}
		}
		[Description("Gets or sets whether horizontal grid lines are printed/exported."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool PrintHorzLines {
			get { return printHorzLines; }
			set {
				if(PrintHorzLines == value) return;
				bool prevValue = PrintHorzLines;
				printHorzLines = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintHorzLines", prevValue, PrintHorzLines));
			}
		}
		[Description("Gets or sets whether to display the preview sections in the print/export output."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool PrintPreview {
			get { return printPreview; }
			set {
				if(PrintPreview == value) return;
				bool prevValue = PrintPreview;
				printPreview = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintPreview", prevValue, PrintPreview));
			}
		}
		[Description("Gets or sets whether only the focused/selected rows are printed/exported."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool PrintSelectedRowsOnly {
			get { return printSelectedRowsOnly; }
			set {
				if(PrintSelectedRowsOnly == value) return;
				bool prevValue = PrintSelectedRowsOnly;
				printSelectedRowsOnly = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintSelectedRowsOnly", prevValue, PrintSelectedRowsOnly));
			}
		}
		[Description("Gets or sets whether vertical grid lines are displayed in the print/export output."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool PrintVertLines {
			get { return printVertLines; }
			set {
				if(PrintVertLines == value) return;
				bool prevValue = PrintVertLines;
				printVertLines = value;
				OnChanged(new BaseOptionChangedEventArgs("PrintVertLines", prevValue, PrintVertLines));
			}
		}
		[Description("Gets or sets whether print appearances are used when the grid control is printed/exported."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool UsePrintStyles {
			get { return usePrintStyles; }
			set {
				if(UsePrintStyles == value) return;
				bool prevValue = UsePrintStyles;
				usePrintStyles = value;
				OnChanged(new BaseOptionChangedEventArgs("UsePrintStyles", prevValue, UsePrintStyles));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsPrint opt = options as GridOptionsPrint;
				if(opt == null) return;
				this.printSelectedRowsOnly = opt.PrintSelectedRowsOnly;
				this.enableAppearanceOddRow = opt.EnableAppearanceOddRow;
				this.enableAppearanceEvenRow = opt.EnableAppearanceEvenRow;
				this.autoWidth = opt.AutoWidth;
				this.expandAllDetails = opt.ExpandAllDetails;
				this.expandAllGroups = opt.ExpandAllGroups;
				this.printDetails = opt.PrintDetails;
				this.printFilterInfo = opt.PrintFilterInfo;
				this.printFooter = opt.PrintFooter;
				this.printGroupFooter = opt.PrintGroupFooter;
				this.printHeader = opt.PrintHeader;
				this.printHorzLines = opt.PrintHorzLines;
				this.printPreview = opt.PrintPreview;
				this.printVertLines = opt.PrintVertLines;
				this.usePrintStyles = opt.UsePrintStyles;
				this.maxMergedCellHeight = opt.MaxMergedCellHeight;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsMenu : ViewBaseOptions {
		bool enableColumnMenu, enableFooterMenu, enableGroupPanelMenu, showGroupSortSummaryItems, 
			showDateTimeGroupIntervalItems, showGroupSummaryEditorItem, showAutoFilterRowItem;
		DefaultBoolean showAddNewSummaryItem;
		public GridOptionsMenu() {
			this.enableColumnMenu = true;
			this.enableFooterMenu = true;
			this.enableGroupPanelMenu = true;
			this.showGroupSortSummaryItems = true;
			this.showDateTimeGroupIntervalItems = true;
			this.showGroupSummaryEditorItem = false;
			this.showAutoFilterRowItem = true;
			this.showAddNewSummaryItem = DefaultBoolean.Default;
		}
		[Description("Gets or sets whether the end-users can invoke Column Header Context Menu."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableColumnMenu {
			get { return enableColumnMenu; }
			set {
				if(EnableColumnMenu == value) return;
				bool prevValue = EnableColumnMenu;
				enableColumnMenu = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableColumnMenu", prevValue, EnableColumnMenu));
			}
		}
		[Description("Gets or sets whether a context menu for grouping columns contains commands that allow group rows to be sorted according to group summary values."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowGroupSortSummaryItems {
			get { return showGroupSortSummaryItems; }
			set {
				if(ShowGroupSortSummaryItems == value) return;
				bool prevValue = showGroupSortSummaryItems;
				showGroupSortSummaryItems = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowGroupSortSummaryItems", prevValue, ShowGroupSortSummaryItems));
			}
		}
		[Description("Gets or sets whether a menu that enables switching group modes for date-time columns is available at runtime."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowDateTimeGroupIntervalItems {
			get { return showDateTimeGroupIntervalItems; }
			set {
				if(ShowDateTimeGroupIntervalItems == value) return;
				bool prevValue = ShowDateTimeGroupIntervalItems;
				showDateTimeGroupIntervalItems = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowDateTimeGroupIntervalItems", prevValue, ShowDateTimeGroupIntervalItems));
			}
		}
		[Description("Gets or sets whether an end-user can invoke a Group Summary Editor at runtime, via a context menu."), DefaultValue(false), XtraSerializableProperty()]
		public virtual bool ShowGroupSummaryEditorItem {
			get { return showGroupSummaryEditorItem; }
			set {
				if(ShowGroupSummaryEditorItem == value) return;
				bool prevValue = ShowGroupSummaryEditorItem;
				showGroupSummaryEditorItem = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowGroupSummaryEditorItem", prevValue, ShowGroupSummaryEditorItem));
			}
		}
		[Description("Gets whether the \"Show Auto Filter Row\" check item is displayed within the Column Header Context Menu."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowAutoFilterRowItem {
			get { return showAutoFilterRowItem; }
			set {
				if(ShowAutoFilterRowItem == value) return;
				bool prevValue = ShowAutoFilterRowItem;
				showAutoFilterRowItem = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowAutoFilterRowItem", prevValue, ShowAutoFilterRowItem));
			}
		}
		[Description("Gets or sets whether or not the footer menu contains an item that allows adding multiple total summaries for a column at runtime."), DefaultValue(DefaultBoolean.Default), XtraSerializableProperty()]
		public virtual DefaultBoolean ShowAddNewSummaryItem {
			get { return showAddNewSummaryItem; }
			set {
				if(ShowAddNewSummaryItem == value) return;
				DefaultBoolean prevValue = ShowAddNewSummaryItem;
				showAddNewSummaryItem = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowAddNewSummaryItem", prevValue, ShowAddNewSummaryItem));
			}
		}
		[Description("Gets or sets whether end-users can invoke footer context menus. "), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableFooterMenu {
			get { return enableFooterMenu; }
			set {
				if(EnableFooterMenu == value) return;
				bool prevValue = EnableFooterMenu;
				enableFooterMenu = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableFooterMenu", prevValue, EnableFooterMenu));
			}
		}
		[Description("Gets or sets whether end-users can invoke the group panel context menu. "), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool EnableGroupPanelMenu {
			get { return enableGroupPanelMenu; }
			set {
				if(EnableGroupPanelMenu == value) return;
				bool prevValue = EnableGroupPanelMenu;
				enableGroupPanelMenu = value;
				OnChanged(new BaseOptionChangedEventArgs("EnableGroupPanelMenu", prevValue, EnableGroupPanelMenu));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsMenu opt = options as GridOptionsMenu;
				if(opt == null) return;
				this.showGroupSortSummaryItems = opt.ShowGroupSortSummaryItems;
				this.showDateTimeGroupIntervalItems = opt.ShowDateTimeGroupIntervalItems;
				this.showGroupSummaryEditorItem = opt.ShowGroupSummaryEditorItem;
				this.enableColumnMenu = opt.EnableColumnMenu;
				this.enableFooterMenu = opt.EnableFooterMenu;
				this.enableGroupPanelMenu = opt.EnableGroupPanelMenu;
				this.showAutoFilterRowItem = opt.ShowAutoFilterRowItem;
				this.showAddNewSummaryItem = opt.ShowAddNewSummaryItem;
			}
			finally {
				EndUpdate();
			}
		}
	}
	public class GridOptionsHint : ViewBaseOptions {
		bool showCellHints, showColumnHeaderHints, showFooterHints;
		public GridOptionsHint() {
			this.showCellHints = true;
			this.showColumnHeaderHints = true;
			this.showFooterHints = true;
		}
		[Description("Gets or sets a value specifying whether hints are displayed for cells with truncated content."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowCellHints {
			get { return showCellHints; }
			set {
				if(ShowCellHints == value) return;
				bool prevValue = ShowCellHints;
				showCellHints = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowCellHints", prevValue, ShowCellHints));
			}
		}
		[Description("Gets or sets whether tooltips are displayed for footer cells, when the text is clipped."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowFooterHints {
			get { return showFooterHints; }
			set {
				if(ShowFooterHints == value) return;
				bool prevValue = ShowFooterHints;
				showFooterHints = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowFooterHints", prevValue, ShowFooterHints));
			}
		}
		[Description("Gets or sets a value specifying whether hints are displayed for column headers displaying truncated captions."), DefaultValue(true), XtraSerializableProperty()]
		public virtual bool ShowColumnHeaderHints {
			get { return showColumnHeaderHints; }
			set {
				if(ShowColumnHeaderHints == value) return;
				bool prevValue = ShowColumnHeaderHints;
				showColumnHeaderHints = value;
				OnChanged(new BaseOptionChangedEventArgs("ShowColumnHeaderHints", prevValue, ShowColumnHeaderHints));
			}
		}
		public override void Assign(BaseOptions options) {
			BeginUpdate();
			try {
				base.Assign(options);
				GridOptionsHint opt = options as GridOptionsHint;
				if(opt == null) return;
				this.showFooterHints = opt.ShowFooterHints;
				this.showCellHints = opt.ShowCellHints;
				this.showColumnHeaderHints = opt.ShowColumnHeaderHints;
			}
			finally {
				EndUpdate();
			}
		}
	}
}
