#region Copyright (c) 2000-2011 Developer Express Inc.
/*
{*******************************************************************}
{                                                                   }
{       Developer Express .NET Component Library                    }
{       XtraGrid                                                    }
{                                                                   }
{       Copyright (c) 2000-2011 Developer Express Inc.              }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{   The entire contents of this file is protected by U.S. and       }
{   International Copyright Laws. Unauthorized reproduction,        }
{   reverse-engineering, and distribution of all or any portion of  }
{   the code contained in this file is strictly prohibited and may  }
{   result in severe civil and criminal penalties and will be       }
{   prosecuted to the maximum extent possible under the law.        }
{                                                                   }
{   RESTRICTIONS                                                    }
{                                                                   }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES           }
{   ARE CONFIDENTIAL AND PROPRIETARY TRADE                          }
{   SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER IS   }
{   LICENSED TO DISTRIBUTE THE PRODUCT AND ALL ACCOMPANYING .NET    }
{   CONTROLS AS PART OF AN EXECUTABLE PROGRAM ONLY.                 }
{                                                                   }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED      }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE        }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE       }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT  }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                      }
{                                                                   }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON       }
{   ADDITIONAL RESTRICTIONS.                                        }
{                                                                   }
{*******************************************************************}
*/
#endregion Copyright (c) 2000-2011 Developer Express Inc.

using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System;
using DevExpress.XtraEditors;
using DevExpress.LookAndFeel;
using DevExpress.XtraGrid.Views.Base;
namespace DevExpress.XtraGrid.Scrolling {
	[ToolboxItem(false)]
	public class VCrkScrollBar : DevExpress.XtraEditors.VScrollBar {
		ScrollInfo scrollInfo;
		public VCrkScrollBar(ScrollInfo scrollInfo) {
			SetStyle(ControlStyles.Selectable, false);
			TabStop = false;
			this.scrollInfo = scrollInfo;
		}
		protected override void OnMouseEnter(EventArgs e) {
			scrollInfo.SetScrollCursor(Cursors.Arrow);
			base.OnMouseEnter(e);
		}
		protected override void OnMouseMove(MouseEventArgs e) {
			scrollInfo.SetScrollCursor(Cursors.Arrow);
			base.OnMouseMove(e);
		}
		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
			if(specified == BoundsSpecified.None || specified == BoundsSpecified.All) return;
			base.SetBoundsCore(x, y, width, height, specified);
		}
		protected override void Dispose(bool disposing) {
			if(disposing)
				scrollInfo = null;
			base.Dispose(disposing);
		}
	}
	[ToolboxItem(false)]
	public class HCrkScrollBar : DevExpress.XtraEditors.HScrollBar {
		ScrollInfo scrollInfo;
		public HCrkScrollBar(ScrollInfo scrollInfo) {
			SetStyle(ControlStyles.Selectable, false);
			TabStop = false;
			this.scrollInfo = scrollInfo;
		}
		protected override void OnMouseMove(MouseEventArgs e) {
			scrollInfo.SetScrollCursor(Cursors.Arrow);
			base.OnMouseMove(e);
		}
		protected override void OnMouseEnter(EventArgs e) {
			scrollInfo.SetScrollCursor(Cursors.Arrow);
			base.OnMouseEnter(e);
		}
		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
			if(specified == BoundsSpecified.None || specified == BoundsSpecified.All) return;
			base.SetBoundsCore(x, y, width, height, specified);
		}
		protected override void Dispose(bool disposing) {
			if(disposing)
				scrollInfo = null;
			base.Dispose(disposing);
		}
	}
	public class ScrollInfo : IDisposable {
		HCrkScrollBar hScroll;
		VCrkScrollBar vScroll;
		private Rectangle clientRect;
		private bool hScrollVisible, vScrollVisible, prevShowDataNavigator;
		Rectangle hscrollRect, vscrollRect, navigatorRect;
		BaseView view;
		bool allowDataNavigator;
		public ScrollInfo(BaseView view) {
			this.clientRect = this.hscrollRect = this.vscrollRect = this.navigatorRect = Rectangle.Empty;
			this.allowDataNavigator = true;
			this.view = view;
			this.prevShowDataNavigator = false;
			this.hScroll = CreateHScroll();
			this.vScroll = CreateVScroll();
			this.HScroll.Visible = false;
			this.VScroll.Visible = false;
			this.HScroll.Anchor = AnchorStyles.None;
			this.VScroll.Anchor = AnchorStyles.None;
			this.VScroll.SmallChange = this.HScroll.SmallChange = 1;
			this.HScroll.LargeChange = this.VScroll.LargeChange = 1;
			this.HScroll.Value = this.HScroll.Maximum = this.VScroll.Value = this.VScroll.Maximum = 0;
			this.hScrollVisible = this.vScrollVisible = false;
		}
		bool UseWaitCursor { 
			get {
				if(view == null || view.GridControl == null) return false;
				return view.GridControl.UseWaitCursor;
			} 
		}
		internal void SetScrollCursor(Cursor cursor) {
			if(!UseWaitCursor)
			Cursor.Current = cursor;
		}
		protected virtual HCrkScrollBar CreateHScroll() {
			HCrkScrollBar scroll = new HCrkScrollBar(this);
			scroll.RightToLeft = RightToLeft.No;
			return scroll;
		}
		protected virtual VCrkScrollBar CreateVScroll() {
			VCrkScrollBar scroll = new VCrkScrollBar(this);
			scroll.RightToLeft = RightToLeft.No;
			return scroll;
		}
		public void CreateHandles() {
			IntPtr fakeHandle;
			if(!VScroll.IsHandleCreated) fakeHandle = VScroll.Handle;
			if(!HScroll.IsHandleCreated) fakeHandle = HScroll.Handle;
		}
		public event EventHandler VScroll_ValueChanged {
			add { VScroll.ValueChanged += value; }
			remove { VScroll.ValueChanged -= value; }
		}
		public event EventHandler HScroll_ValueChanged {
			add { HScroll.ValueChanged += value; }
			remove { HScroll.ValueChanged -= value; }
		}
		public event ScrollEventHandler HScroll_Scroll { 
			add { HScroll.Scroll += value; }
			remove { HScroll.Scroll -= value; }
		}
		public event ScrollEventHandler VScroll_Scroll { 
			add { VScroll.Scroll += value; }
			remove { VScroll.Scroll -= value; }
		}
		public virtual HCrkScrollBar HScroll { get { return hScroll; } }
		public virtual VCrkScrollBar VScroll { get { return vScroll; } }
		public virtual void Dispose() {
			if(HScroll != null) {
				HScroll.Dispose();
				VScroll.Dispose();
				this.hScroll = null;
				this.vScroll = null;
			}
		}
		protected virtual bool RealShowDataNavigator { 
			get { return ShowDataNavigator && Navigator != null; }
		}
		protected virtual bool ShowDataNavigator {
			get {
				return AllowDataNavigator && View.GridControl != null && View.GridControl.DefaultView == View;
			}
		}
		protected virtual UserLookAndFeel LookAndFeel { get { return View.ElementsLookAndFeel; } }
		public virtual BaseView View { get { return view; } }
		public virtual bool AllowDataNavigator { 
			get { return allowDataNavigator; }
			set {
				if(AllowDataNavigator == value) return;
				allowDataNavigator = value;
				LayoutChanged();
			}
		}
		protected virtual void UpdateNavigatorProperties() {
			if(Navigator == null) return;
			Navigator.BorderStyle = DevExpress.XtraEditors.Controls.BorderStyles.NoBorder;
			Navigator.LookAndFeel.Assign(View.ElementsLookAndFeel);
		}
		protected virtual void CalcRects() {
			this.hscrollRect = this.vscrollRect = this.navigatorRect = Rectangle.Empty;
			Rectangle r = Rectangle.Empty;
			int hscrollHeight = HScrollSize;
			if(HScrollVisible) {
				if(RealShowDataNavigator) {
					r.Size = Navigator.MinSize;
					r.Height = hscrollHeight;
					r.Location = new Point(ClientRect.X, ClientRect.Bottom - r.Height);
					if(r.Width > ClientRect.Width) r = Rectangle.Empty; 
					else {
					}
				}
				this.navigatorRect = r;
				r.Location = new Point(ClientRect.X + this.navigatorRect.Width, ClientRect.Bottom - hscrollHeight);
				r.Size = new Size(ClientRect.Width - (VScrollVisible ? ScrollSize : 0) - this.navigatorRect.Width, hscrollHeight);
				this.hscrollRect = r;
			}
			if(VScrollVisible) {
				r.Location = new Point(ClientRect.Right - ScrollSize, ClientRect.Y);
				r.Size = new Size(ScrollSize, ClientRect.Height  - (HScrollVisible ? hscrollHeight : 0));
				this.vscrollRect = r;
			}
		}
		int lockLayout = 0;
		public virtual void LayoutChanged() {
			if(lockLayout != 0) return;
			lockLayout ++;
			try {
				CalcRects();
				UpdateNavigatorProperties();
				UpdateScrollRects();
				UpdateVisiblity();
				if(ClientRect.IsEmpty) {
					VScroll.Visible = false;
					HScroll.Visible = false;
				}
				UpdateNavigatorVisiblitiy();
				this.prevShowDataNavigator = RealShowDataNavigator;
			}
			finally {
				lockLayout --;
			}
		}
		protected virtual void UpdateNavigatorVisiblitiy() {
			if(!ShowDataNavigator) return;
			ControlNavigator navigator;
			if(TryGetNavigator(out navigator)) {
				if(!HScroll.Visible || NavigatorRect.IsEmpty)
					navigator.Visible = false;
				else navigator.Visible = true;
			}
		}
		public virtual ControlNavigator Navigator {
			get { return View.GridControl == null || !View.GridControl.UseEmbeddedNavigator ? null : View.GridControl.EmbeddedNavigator; } 
		}
		bool TryGetNavigator(out ControlNavigator navigator) {
			navigator = null;
			if(View.GridControl != null)
				navigator = view.GridControl.EmbeddedNavigator;
			return navigator != null;
		}
		public virtual void UpdateLookAndFeel(UserLookAndFeel lookAndFeel) {
			if(lookAndFeel == null) return;
			VScroll.LookAndFeel.Assign(lookAndFeel);
			HScroll.LookAndFeel.Assign(lookAndFeel);
			if(Navigator != null) Navigator.LookAndFeel.Assign(lookAndFeel);
		}
		protected virtual int ScrollSize {
			get { return SystemInformation.VerticalScrollBarWidth; }
		}
		public int VScrollSize {
			get { return ScrollSize; }
		}
		public int HScrollSize {
			get { 
				int hscrollHeight = ScrollSize;
				if(RealShowDataNavigator) {
					hscrollHeight = Math.Max(Navigator.MinSize.Height, hscrollHeight);
				}
				return hscrollHeight;
			}
		}
		public Rectangle ClientRect {
			get { return clientRect; }
			set {
				if(ClientRect == value && RealShowDataNavigator == this.prevShowDataNavigator) return;
				clientRect = value;
				LayoutChanged();
			}
		}
		public virtual Rectangle NavigatorRect {
			get { return this.navigatorRect; }
		}
		public virtual void UpdateScrollRects() {
			if(!VScrollVisible) VScroll.Visible = false;
			if(!HScrollVisible) HScroll.Visible = false;
			if(HScroll.Anchor == AnchorStyles.None) {
				Rectangle r = HScrollRect;
				HScroll.Location = r.Location;
				HScroll.Size = r.Size;
			}
			if(VScroll.Anchor == AnchorStyles.None) {
				Rectangle r = VScrollRect;
				VScroll.Location = r.Location;
				VScroll.Size = r.Size;
			}
			if(ShowDataNavigator && Navigator != null) {
				Rectangle r = NavigatorRect;
				if(r.IsEmpty) 
					Navigator.Visible = false;
				else {
					Navigator.Location = r.Location;
					Navigator.Size = r.Size;
				}
			}
		}
		public Rectangle HScrollRect {
			get { return hscrollRect; }
		}
		public Rectangle VScrollRect {
			get { return vscrollRect; }
		}
		public bool HScrollVisible {
			get { return hScrollVisible; }
			set { 
				if(HScrollVisible == value) {
					UpdateVisiblity();
					return;
				}
				hScrollVisible = value;
				LayoutChanged();
			}
		}
		public int VScrollPosition { get { return VScroll.Value; } }
		public int HScrollPosition { get { return HScroll.Value; } }
		public bool VScrollVisible {
			get { return vScrollVisible; }
			set { 
				if(VScrollVisible == value) {
					UpdateVisiblity();
					return;
				}
				vScrollVisible = value;
				LayoutChanged();
			}
		}
		public void UpdateVisiblity() {
			if(HScroll.Visible != hScrollVisible) { 
				if(!hScrollVisible || !ClientRect.IsEmpty)
					HScroll.Visible = hScrollVisible;
			}
			if(VScroll.Visible != vScrollVisible) {
				if(!vScrollVisible || !ClientRect.IsEmpty) 
					VScroll.Visible = vScrollVisible;
			}
			UpdateNavigatorVisiblitiy();
		}
		public ScrollArgs HScrollArgs {
			get { return new ScrollArgs(HScroll); }
			set {
				value.AssignTo(HScroll);
			}
		}
		public ScrollArgs VScrollArgs {
			get { return new ScrollArgs(VScroll); }
			set {
				value.AssignTo(VScroll);
			}
		}
		public virtual void RemoveControls(Control container) {
			if(container == null) return;
			container.Controls.Remove(HScroll);
			container.Controls.Remove(VScroll);
		}
		public virtual void AddControls(Control container) {
			if(container == null) return;
			if(container.InvokeRequired) return;
			container.Controls.Add(HScroll);
			container.Controls.Add(VScroll);
		}
	}
}
